#
# $Id: Disk.pm,v 1.2.2.3.32.1 2004/11/06 01:04:40 avanish Exp $
#ident "$Source: /project/unixvm-cvs/src/sol/cmd/sdscvt/lib/Disk.pm,v $"
#
# $Copyrights: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

use strict;

use Registered;
use Spacemap;
use VxVM;

package Disk;
use Egettxt;

use vars qw( @ISA %DiskTab );

@ISA = qw ( Registered );

%DiskTab = ();
sub Registry { \%DiskTab }

my $DefaultPath = '/dev/dsk';
my $DefaultMultiPath = '/dev/ap/dsk';

sub GetVTOC {
	my $self = shift;

	$self->{'partitions'} = [];
	if (not open(VTOC, "/usr/sbin/prtvtoc " . $self->{'path'} . "s0|")) {
		Terminal('vmsolsds:87',
			'$prog: FATAL: prtvtoc $path failed: $strerror',
			{ 'path' => $self->{'path'}, 'strerror' => "$!" });
		return;
	}
	# XXX save this output???
	line: while (<VTOC>) {
		$self->{'secpercyl'} = $1 if /^\* *(\d+) sectors.cylinder/;
		$self->{'cylinders'} = $1 if /^\* *(\d+) cylinders/;
		$self->{'usableCylinders'} = $1 if /^\* *(\d+) accessible cyl/;
		next line if /^\*/;

		my ($part, $type, $flags, $first, $size, $last, $mp) = split;
		my $child = Partition->new($self->{'path'} . "s" . $part, 1);
		$child->{'part'} = $part;
		$child->{'type'} = $type;
		$child->{'flags'} = $flags;
		$child->{'first'} = $first;
		$child->{'size'} = $size;
		$child->{'last'} = $last;
		$child->{'skip'} = 0;  # will be initted by md parsing
		$child->{'extentlast'} = 0; # no extents are added till now
		$child->{'extentfirst'} = $last;
		push @{$self->{'partitions'}}, $child;
	}
	close VTOC;
}

sub Part {
	my $self = shift;
	my $part = shift;

	my @result = grep { $_->{'part'} == $part } @{$self->{'partitions'}};
	$result[0];
}

sub Protected {
	my $self = shift;
	grep { $_->{'protected'} } @{$self->{'partitions'}};
}

sub NameAndPath {
	my $name = shift;
	my $path;

	if ($name =~ m,^/,) {
		$path = $name;
		$name =~ s,^.*/,,;
	} elsif ($name =~ m,^m,) {
		$path = $DefaultMultiPath . '/' . $name;
	} else {
		$path = $DefaultPath . '/' . $name;
	}
	($name, $path);
}

sub RoundUp {
	my $self = shift;
	my $block = shift;

	my $excess = $block % $self->{'secpercyl'};
	$block += $self->{'secpercyl'} - $excess if ($excess);
	$block
}

sub RoundDown {
	my $self = shift;
	my $block = shift;

	my $excess = $block % $self->{'secpercyl'};
	$block -= $excess if ($excess);
	$block
}

sub new {
	my $type = shift;
	my $name = shift;

	$name =~ s/s\d+$//;
	($name, my $path) = Disk::NameAndPath($name);
	my $self = $type->Find($name);
	if ($self) {
		bless $self, $type;
	} else {
		$self = bless { 'name' => $name, 'path' => $path }, $type;
		$self->Register();
		$self->GetVTOC();
		$self;
	}
}

#
# Create (if necessary) and return a Spacemap representing the used/free
# space on the disk, based on partition information and MD usage.
#
sub Spacemap {
	my $self = shift;

	unless ($self->{'spacemap'}) {
		my $spacemap = Spacemap->new();
		$self->{'spacemap'} = $spacemap;
		foreach (@{$self->{'partitions'}}) {
			 #Before taking simple partition check for
			 #extents on partition
			 #with soft partition a chunck of partitiuon may
			 #be in use
			foreach (@{$_->{'extent'}}) {
				my @range = $_->OccupiedRange();
				$spacemap->Add(@range) if @range;
			}
			if (! @{$_->{'extent'}}) {
				# if no extent in check for
				# complete partition
				my @range = $_->OccupiedRange();
				$spacemap->Add(@range) if @range;
			}
		}
		# fake entry just beyond the end of the disk
		my $end = $self->{'usableCylinders'} * $self->{'secpercyl'};
		$spacemap->Add($end, $end);
	}
	$self->{'spacemap'}
}

#
# Alloc space on this disk.
#
sub Alloc {
	my $self = shift;

	$self->Spacemap->Alloc(@_);
}

#
# Estimate how many VxVM objects this will make
#
sub estVxVM {
	  1		# one DM
	+ 1		# plus up to one spacer per disk (see Partition)
}

#
# Convert each disk to a DM record
#
sub toVxVM {
	my $self = shift;
	my $dm = VxVM::DM->new($self->{'name'});

	$self->{'dm'} = $dm;
	$dm->{'disk'} = $self;
	$dm
}

package Partition;
use Egettxt;

use vars qw ( @ISA %PartitionTab );

@ISA = qw ( Registered );

%PartitionTab = ();
sub Registry { \%PartitionTab }

sub new {
	my $type = shift;
	my $name = shift;
	my $internal = shift;

	($name, my $path) = Disk::NameAndPath($name);
	my $self = $type->Find($name);
	if ($self) {
		bless $self, $type;
	} else {
		my $part = $1 if $name =~ /s(\d+)$/;
		$self = bless { 'name' => $name,
				'path' => $path,
				'part' => $part }, $type;
		$self->Register();
		$self->{'parent'} = Disk->new($name);
		# Make sure our slice was found in the VTOC (unless we're
		# being called from the VTOC parser itself!)
		if (not $internal and not defined $self->{'first'}) {
			Terminal('vmsolsds:88',
			'$prog: FATAL: Partition $name not found in VTOC',
				{ 'name' => $self->{'name'} });
			# Band-aid to quell other errors
			$self->{'first'} = 0;
			$self->{'size'} = 0;
		}
		$self;
	}
}

sub Match {
	my $self = shift;
	my $name = shift;

	($name, my $path) = Disk::NameAndPath($name);
	if ($name eq $self->{'name'} or $path eq $self->{'path'}) {
		1;
	} else {
		0;
	}
}

sub SetMD {
	my $self = shift;
	my $md = shift;

	# two metadevices on one slice?
	# need to change here
	die if (($self->{'md'}) && (! $self->{'extent'}));

	$self->{'md'} = $md;
}

sub SetExtent {
	my $self =  shift;
	my $extent = shift;

	push @{$self->{'extent'}}, $extent;
	my $last = $extent->{'offset'} + $extent->{'len'};
	$self->{'extentlast'} = $last if ($self->{'extentlast'} < $last);
$self->{'extentfirst'} = $extent->{'offset'} if ($self->{'extentfirst'} >
					$extent->{'offset'});
}

sub OccupiedRange {
	my $self = shift;

	my $first = $self->{'first'};
	my $md = $self->{'md'};
	if ($md and $md->Inuse) {
		$first += $self->{'skip'};
		return ($first, $self->{'last'});
	} else {
		return ();
	}
}

sub Vxname {
	my $self = shift;

	# XXX uuugly
	"sd-" . $self->{'name'}
}

sub DevOffset {
	my $self = shift;

	$self->{'first'} + $self->{'skip'};
}

sub PubOffset {
	my $self = shift;

	$self->DevOffset + $self->{'parent'}->{'devToPubOffset'}
}

sub PubSize {
	my $self = shift;

	$self->{'size'} - $self->{'skip'}
}

sub Inuse {
	my $self = shift;
	my $md = $self->{'md'};

	return ($self->{'inuse'} or ($md and $md->Inuse));
}

sub Use {
	my $self = shift;
	my $spacemap = $self->{'parent'}->Spacemap;
	my @range = ($self->{'first'}, $self->{'last'});

	if ($spacemap->Inuse(@range)) {
	Terminal('vmsolsds:89', '$prog: FATAL: $part used multiple times.' .
				'  Metadevice and filesystem?',
				{ 'part' => $self->{'name'} });
	} else {
		$spacemap->Add(@range);
	}
	$self->{'inuse'} = 1;
}

# Avoid destroying this partition during the vxdisksetup step, since it
# will be mounted.
sub Protect {
	my $self = shift;

	$self->{'protected'} = 1;
}

#
# XXX Return this disk's Rootness.  Lower scores mean rootier disks.
# XXX This is used to sort plain partitions when temporarily
# XXX unencapsulating partitions needed to boot during the conversion.
# XXX This version uses the lowness of the cXtYdZ numbers as an
# XXX indicator of Rootness.
#
sub Rootness {
	my $self = shift;

	$self->{'name'} =~ m,m?c(\d+)t([A-F0-9]+)d(\d+), || die;
	$1 * 1000000 + $2 * 1000 + $3
}

#
# Estimate how many VxVM objects this will make
#
sub estVxVM { 1	}

#
# All partitions translate into subdisks.  Partitions at device offset 0
# must have 1-sector spacers inserted before them because VxVM will force
# it to *offset* over the sector, not just prevent access to it.
#
sub toVxVM {
	my $self = shift;

	my @result = ();

	$self->Inuse || die "unused partition converted to subdisk?";

	my $spacer;
	my $offset = $self->PubOffset;
	my $len = $self->PubSize;

	# VxVM shifts DMs with offset 0 up 1 block.  That means the
	# offset required to get the original SDS offset may actually
	# shift us off the beginning of the disk, in which case we have
	# to add a spacer subdisk.
	my $spacerlen = 0;
	if ($offset < 0) {
		$spacerlen = -$offset;
		# XXX I'm sure I'll need this in a general purpose place
		my $a;
		my $disk;
		# Try to keep the spacer local, otherwise anything goes
		disk: foreach ($self->{'parent'}, Disk->All) {
			$a = $_->Alloc($spacerlen);
			$disk = $_, last disk if $a;
		}
		unless ($a) {
			Terminal('vmsolsds:90',
'$prog: FATAL: Unable to allocate spacer subdisk for partition $name
$prog: Try freeing one block on a disk coverted for this set',
			{ 'name' => $self->{'name'} });
			# bandaid to keep things running..
			$a = [ 0, 1 ];
		}

		$spacer = VxVM::Subdisk->new($self->Vxname . "-spc");

		$spacer->{'dm'} = $disk->{'dm'};
		$spacer->{'offset'} = $a->[0] + $disk->{'devToPubOffset'};
		$spacer->{'len'} = $spacerlen;
		$spacer->{'column'} = $self->{'column'};
		$spacer->{'coloffs'} = 0;
		push @result, $spacer;

		$offset += $spacerlen;
		$len -= $spacerlen;
	}

	my $sd = VxVM::Subdisk->new($self->Vxname);

	$sd->{'dm'} = $self->{'parent'}->{'dm'};
	$sd->{'offset'} = $offset;
	$sd->{'len'} = $len;
	$sd->{'column'} = $self->{'column'};
	$sd->{'coloffs'} = $spacerlen;
	$sd->{'clean'} = 1 if $self->{'force_clean'};
	# $sd->{'part'} = $self;
	push @result, $sd;

	@result
}

sub PreAssert {
	my $self = shift;

	my $hsp = $self->{'hotspared'};
	if ($hsp) {
		Terminal('vmsolsds:91',
	'$prog: FATAL: Partition $part has been hot spared by $newpart in $md' . "\n" . '$prog: Use metareplace to eliminate the hot sparing',
			{ 'part' => $self->{'name'}, 'newpart' => $hsp,
				'md' => $self->{'md'}->Name });
	}
}

sub Skip {
	my $self = shift;

	return $self->{'skip'};
}

sub Name {
	my $self = shift;

	return $self->{'name'};
}

#
# Static method to match partitions
#
sub IsPartition {
	my $s = shift;

	$s =~ /^m?c\d+t[A-F0-9]+d\d+s\d+/;
}

# Extent is part of partition. In earlier SDS, smallest possible
# unit was disk partition, now it is extent. Concat and Raid metadevice
# can take both disk partition or soft partition

package Partition::Extent;
use Egettxt;

use vars qw ( @ISA %ExtentTab );

@ISA = qw ( Partition );
%ExtentTab = ();
sub Registry { \%ExtentTab }

sub new {
	my $type = shift;
	my $parentname =shift;
	my $partition = shift;
	my $extentid = shift;
	my $offset = shift;
	my $len = shift;

	# Need to have unique name for extent. multiple extent of two
	# different partition or one soft partition is possible

	my $name = $parentname."_".$partition->{'name'}."_".$extentid;
	(my $diskname, my $path) = Disk::NameAndPath($partition->{'name'});

	my $part = $1 if $partition->{'name'} =~ /s(\d+)$/;
	my $self = bless { 'name' => $name,
		'diskname' => $diskname,
		'path' => $path,
		'partition' => $partition,
		'offset' => $offset,
		'len' => $len,
		'extentid' => $extentid}, $type;
	$self->Register();
	$self->{'parent'} = $partition;
	$partition->SetExtent($self);
	$self->{'first'} = $offset;
	$self->{'skip'}  = $offset;
	$self->{'size'} =$len;
	$self;
}

sub Match {
	my $self = shift;
	my $name = shift;

	($name, my $path) = Disk::NameAndPath($name);
	if ($name eq $self->{'name'} or $path eq $self->{'path'}) {
		1;
	} else {
		0;
	}
}


sub OccupiedRange {
	my $self = shift;

	my $first = $self->{'offset'};
	my $md = $self->{'md'};
	my $end;
	if ($md and $md->Inuse) {
	        my @parentrange = $self->{'parent'}->OccupiedRange();
	        # This is in refrence with parent partition so add here
	        $first += $self->{'parent'}->{'first'};
		$end = $first + $self->{'size'};
		#Make sure extent is within parttion :)
		die if (($first < $parentrange[0]) || ($end > $parentrange[1])); 
		return ($first, $first + $self->{'size'});
	} else {
		return ();
	}
}

sub Vxname {
	my $self = shift;

	# XXX uuugly
	"sd-" . $self->{'name'};
}

sub PubSize {
	my $self = shift;

	$self->{'$len'};
}

#
# All Extents translate into subdisks. Extents at device offset 0
# must have 1-sector spacers inserted before them because VxVM will force
# it to *offset* over the sector, not just prevent access to it.
#
sub toVxVM {
	my $self = shift;

	my @result = ();

	$self->Inuse || die "unused partition converted to subdisk?";

	my $spacer;
	my $offset = $self->{'offset'} + $self->{'parent'}->PubOffset;
	my $len = $self->{'len'};

	# Skiping the parent **skip** field
	# this is needed when soft partition 
	# is on disk slice starting with 0 but 
	# skip value for slice gives greater than 0 value
	# Needed for correction with pub offset
	# for other metadevice this value remain 0

	$offset -= $self->{'parent'}->{'skip'};

	# VxVM shifts DMs with offset 0 up 1 block.  That means the
	# offset required to get the original SDS offset may actually
	# shift us off the beginning of the disk, in which case we have
	# to add a spacer subdisk.
	my $spacerlen = 0;
	if ($offset < 0) {
		$spacerlen = -$offset;
		# XXX I'm sure I'll need this in a general purpose place
		my $a;
		my $disk;
		# Try to keep the spacer local, otherwise anything goes
		disk: foreach ($self->{'parent'}->{'parent'}, Disk->All) {
			$a = $_->Alloc($spacerlen);
			$disk = $_, last disk if $a;
		}
		unless ($a) {
			Terminal('vmsolsds:90',
'$prog: FATAL: Unable to allocate spacer subdisk for partition $name
$prog: Try freeing one block on a disk coverted for this set',
			{ 'name' => $self->{'name'} });
			# bandaid to keep things running..
			$a = [ 0, 1 ];
		}

		$spacer = VxVM::Subdisk->new($self->Vxname . "-spc");

		$spacer->{'dm'} = $disk->{'dm'};
		$spacer->{'offset'} = $a->[0] + $disk->{'devToPubOffset'};
		$spacer->{'len'} = $spacerlen;
		$spacer->{'column'} = $self->{'column'};
		$spacer->{'coloffs'} = 0;
		push @result, $spacer;

		$offset += $spacerlen;
		$len -= $spacerlen;
	}

	my $sd = VxVM::Subdisk->new($self->Vxname);

	$sd->{'dm'} = $self->{'parent'}->{'parent'}->{'dm'};
	$sd->{'offset'} = $offset;
	$sd->{'len'} = $len;
	$sd->{'column'} = $self->{'column'};
	$sd->{'coloffs'} = $spacerlen;
	$sd->{'clean'} = 1 if $self->{'force_clean'};
	# $sd->{'part'} = $self;
	push @result, $sd;

	@result
}

sub Name {
	my $self = shift;
	return $self->{'name'};
}

sub Inuse {
	my $self = shift;
	my $partition = $self->{'parent'};
	return ($partition->Inuse());
}

1;
