#
# $Id: MD.pm,v 1.2.2.4.24.2 2004/12/13 07:02:17 avanish Exp $
#ident "$Source: /project/unixvm-cvs/src/sol/cmd/sdscvt/lib/MD.pm,v $"
#
# $Copyrights: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

use strict;

use Registered;
use Disk;
use Egettxt;

package MD;
use Egettxt;

use vars qw( @ISA %MDTab $diskset );

@ISA = qw ( Registered );

$diskset = undef;

sub diskset_ext { $diskset ? ('.' . $diskset) : '' }

%MDTab = ();
sub Registry { \%MDTab }
sub RegistryKey { 'number' }

#
# External interface MD::Parse(number, @body) will return an object of
# the right type, with data extracted from `metastat -p' style command
# line arguments in @body
#
sub Parse {
	my $type = shift;
	$_ = shift;

	my @body = split;
	my $number = shift @body;
	$number =~ s,^$diskset/,,;
	if ($number =~ /^d(\d+)/) {
		$_ = $body[0];
		if (/^\d+$/) {
			$type = "MD::Concat"
		} elsif (/-p/) {
			$type = "MD::SoftPartition"
		} elsif (/-m/) {
			$type = "MD::Mirror"
		} elsif (/-t/) {
			$type = "MD::Trans"
		} elsif (/-r/) {
			$type = "MD::RAID"
		} else {
			die "unknown object type metastat $_";
		}
	} elsif ($number =~ /^hsp(\d+)/) {
		$type = "MD::Hotspare";
	} else {
		die "unrecognized object $number";
	}
	my $self = $type->new($number);
	$self->ParseMSP(@body);
}

sub new {
	my $type = shift;
	my($number) = @_;

	$number =~ s/^\D+//;
	$number = $number + 0;

	my $self = $type->SUPER::new($number);
	# re-bless here because we may have been created as a generic
	bless $self, $type;
}

# Most MD devices are being used.  Hotspares are the exception.
sub Inuse { 1 }

# Return a Partition which exactly maps to the metadevice, if any.
sub SimplePartition { undef }

sub ParseMS {
	my $self = shift;
	my $lines = shift;
	my $is_reloc = 0;

	line: foreach (@{$lines}) {
		last line if (/^\S/ and not /devices?:/);
		$self->{'state'} = $1 if /^    State: (\S+)/;
		$self->{'size'} = $1 if /^    Size: (\d+) blocks/;

		# find if it is new SVM version. Older SVM version does not
		# print reloc information
		$is_reloc = 1 if 
		/^\s+(Device)\s+(Start)\s+(Block)\s+(Dbase)\s+(State)\s+(Reloc)\s+(Hot)\s+(Spare)/;

		next line if /^\tDevice/;
		if (/^\t(c\d+t[A-F0-9]+d\d+s\d+)\s+(\d+)\s+(Yes|No)(\s+(\S+)\s+(\S+)\s+(\S+))?/) {
			my ($partname, $skip, $dbasep, $state, $reloc, $hsp) =
				($1, $2, $3, $5, $6, $7);
			my $part = Partition->Find($partname);
			# Don't check if parent is soft partition
			# multiple soft partition allowed on a disk slice
			if (ref($self) ne 'MD::SoftPartition') {
				die unless $part and $part->{'md'} == $self;
			}
			$part->{'skip'} = $skip;
			$part->{'dbase'} = $dbasep eq "Yes" ? 1 : 0;
			$part->{'state'} = $state if $state;
			#
			# If there's an entry in the Hot Spare column, we
			# have been most deceived.  The device listed by
			# `metastat -p' and at the beginning of this line
			# is not being used at all!  By now we probably
			# printed an error.   :(
			#
			# Since `metastat -p' is probably what the admin
			# used to back up the config, don't feel too bad
			# about forcing them to use metareplace to fix
			# this hotsparing, since a restore from the
			# `metastat -p' output would lead to data corruption.
			#
			$hsp = $reloc if ($is_reloc == 0);
			$part->{'hotspared'} = $hsp if $hsp;
		}
		if (/^\t([a-zA-Z]*\/?d\d+)\s+(\d+)\s+(Yes|No)(\s+(S+)\s+(\S+))?/) {
			#we are parsing soft partition in place of hard parition
			my  ($partname, $skip, $dbasep, $state, $hsp) =
							($1, $2, $3, $5, $6);
			if ($partname =~ /d(\d+)/) {
				my $sp = MD->Find($1) || die;
				die unless $sp and $sp->{'parent'} == $self;
				$sp->{'dbase'} = $dbasep eq "Yes" ? 1 : 0;
				$sp->{'state'} = $state if $state;
				$sp->{'skip'} = $skip;
			} else {
				die;
			}
		}
	}
}

sub Name {
	my $self = shift;

	($MD::diskset ne '' ? $MD::diskset . '/' : '') . 'd' . $self->{'number'}
}

sub Vxname {
	my $self = shift;

	return $self->{'force_name'} if $self->{'force_name'};

	if ($self->{'force_use_type'}) {
		if ($self->{'force_use_type'} eq 'root') {
			return $self->{'force_name'} = "rootvol";
		} elsif ($self->{'force_use_type'} eq 'swap') {
			return $self->{'force_name'} =
				VxVM->UniqueName("swapvol");
		}
	}
	"d" . $self->{'number'}
}

sub PreAssert {
	my $self = shift;

	if ($self->{'state'} and $self->{'state'} ne 'Okay') {
		Terminal('vmsolsds:96',
			'$prog: FATAL: $md in state $state: \
		 	Must be $okay for conversion',
			{ 'md' => $self->Name,
			  'state' => $self->{'state'},
			  'okay' => 'Okay' });
	}
}

sub Path {
	my $self = shift;

	my $disksetpath = $diskset ? $diskset . "/" : "";

	"/dev/md/" . $disksetpath . "dsk/d" . $self->{'number'}
}

# there's really no stripe object in SDS, but we represent a concat
# as a collection of them
package MD::_Stripe;
use Egettxt;

use vars qw( %InterlaceUnits );

sub new {
	my $type = shift;
	my $number = shift;
	my $parent = shift;

	my $self = bless {'partitions' => [],
			  'interlace' => 0,
			  'number' => $number,
			  'parent' => $parent }, $type;
	$self->SetInterlace('16k');
	$self;
}

%InterlaceUnits = (	'b', 512,
			'k', 1024,
			'm', 1024*1024
);

sub SetInterlace {
	my $self = shift;
	my $s = shift;

	my $factor = 1;
	$s =~ /^(\d+)([bkm])?$/ || die "unrecognized interlace unit in -i $s";
	$factor = $InterlaceUnits{$2} if $2;
	$self->{'interlace'} = $1 * $factor;
	$self;
}

sub count {
	my $self = shift;

	scalar(@{$self->{'partitions'}});
}

# when computing the size, the rounded size of the limiting (smallest)
# partition is multiplied by this.
sub sizeMultiplier {
	my $self = shift;

	$self->count;
}

sub Vxname {
	my $self = shift;
	"d" . $self->{'parent'}->{'number'} . "-s" . $self->{'number'}
}

sub VxStripeWidth {
	my $self = shift;

	$self->{'interlace'} / 512
}

sub Size {
	my $self = shift;

	# we're constrained by the smallest partition
	(my $s) = sort {$a <=> $b} map $_->PubSize, @{$self->{'partitions'}};

	# if it's really a stripe (more than one element)
	# round down to the interlace size
	if ($self->count > 1) {
		my $st_width = $self->VxStripeWidth;
		$s = int($s / $st_width) * $st_width;
	}

	$s * $self->sizeMultiplier;
}

# our container will want to round based on the secpercyl of the
# first partition in its first _Stripe
sub RoundDown {
	my $self = shift;
	my $blocks = shift;
	if ( ref($self->{'partitions'}->[0]) eq 'MD::SoftPartition') {
		my $extent = $self->{'partitions'}->[0]->{'extents'}->[0];
		$extent->{'parent'}->{'parent'}->RoundDown($blocks);
	} else {
		$self->{'partitions'}->[0]->{'parent'}->RoundDown($blocks);
	}
 }

sub estVxVM {
	3 # partitions are accounted for separately, so we always make 3
}

#
# Every ::_Stripe results in a subdisk, suitable for being concat'd by a
# plex in ::Concat.  If the ::_Stripe consists of one subdisk there's
# nothin to add.  Otherwise this always results in a striped plex in a
# volume wrapped in a sd.
#
# (subdisks) - dN-sM-p - dN-sM - dN-sM-s
#
sub toVxVM {
	my $self = shift;

	my @sds = map $_->toVxVM, @{$self->{'partitions'}};

	if (scalar(@sds) == 1) {
		return $sds[0];
	}

	my $basename = $self->Vxname;

	my $plex = VxVM::Plex->new($basename . "-p");
	$_->{'assoc'} = $plex for (@sds);
	$plex->{'aslist'} = \@sds;
	# if there are multiple columns, make this a stripe, else a concat
	if (grep { $_->{'column'} != 0 } @sds) {
		$plex->{'st_width'} = $self->VxStripeWidth;
		$plex->{'layout'} = 'stripe';
	} else {
		$plex->{'layout'} = 'concat';
	}

	my $vol = VxVM::Volume->new($basename);
	$plex->{'assoc'} = $vol;
	$vol->{'aslist'} = [ $plex ];
	$vol->{'use_type'} = $self->{'force_use_type'} || 'gen';
	$vol->{'len'} = $self->Size;

	my $sv = VxVM::Subdisk->new($basename . "-s");
	$vol->{'assoc'} = $sv;
	$sv->{'dm'} = $vol;
	$sv->{'offset'} = 0;
	$sv->{'len'} = $vol->{'len'};

	$sv
}

sub SimplePartition {
	my $self = shift;
	return $self->{'partitions'}->[0] if ($self->count == 1);
}

# Collections of partitions for a RAID share a lot of behavior with
# Concat/Stripe stripes, but not all..
package MD::_Stripe::_RAIDStripe;
use Egettxt;

use vars qw( @ISA );
@ISA= qw(MD::_Stripe);

# when computing the size, the rounded size of the limiting (smallest)
# partition is multiplied by this.
sub sizeMultiplier {
	my $self = shift;

	$self->count - 1;
}

sub Vxname {
	my $self = shift;

	"d" . $self->{'parent'}->{'number'} . "-p"
}

sub estVxVM { 1 }

#
# A ::_RAIDStripe is only ever contained inside a RAID.  We provide a
# plex to wrap up the partitions.
#
sub toVxVM {
	my $self = shift;

	my @sds = map $_->toVxVM, @{$self->{'partitions'}};

	my $plex = VxVM::Plex->new($self->Vxname);
	$_->{'assoc'} = $plex for (@sds);
	$plex->{'aslist'} = \@sds;
	$plex->{'st_width'} = $self->VxStripeWidth;
	$plex->{'layout'} = 'raid5';
	$plex->{'len'} = $self->Size;

	$plex
}

package MD::SoftPartition;
use Egettxt;

use vars qw( @ISA );

@ISA= qw(MD);

#
# Parse `metastat -p' output for a concat.  see metainit(1) and md.tab(4)
# for the format.
#
sub ParseMSP {
	my $self = shift;
	my (@body) = @_;
	my $i = 1;
	# -p
	if (scalar(@body) == 1) {
	}
	shift @body;
	my $diskslice = shift @body;

	# In this version we are not supporting soft partition on metadevice
	# so print error
	Die('vmsolsds:111',
'$prog: FATAL: This conversion utility does not support layout of soft
\t\tpartition on metadevice\n',) 
		unless ($diskslice =~ /c(\d+)t([A-F0-9]+)d(\d+)s(\d+)/);

	my $child = Partition->new($diskslice);
	$self->{'partition'} = $child;
	$child->SetMD($self);
	# Now the turn of Extents on Partition
	while (@body) {
		#Fails if soft partition had multiple extents and
		# Raid metadevice is on top
		Die('vmsolsds:110',
'$prog: FATAL: This conversion utility does not support layout of RAID5
\t\t   on soft partition with multiple extents\n',) if (($i > 1) && 
			(ref($self->{'parent'}) eq 'MD::RAID'));
		my ($offset, $blocks);
		die unless (shift @body eq "-o");
		$offset = shift @body;
		die unless (shift @body eq "-b");
		$blocks = shift @body;
		my $ext = Partition::Extent->new($self->Vxname, $child, $i,
							$offset, $blocks);
		push @{$self->{'extents'}}, $ext;
		$ext->{'column'} = $self->{'column'};
		$ext->SetMD($self);
		$i++;
	}
	$self->{'skip'} = 0;
	$self;
}

sub count {
	my $self = shift;
	scalar(@{$self->{'extents'}});
}

sub estVxVM {
	my $self = shift;
	my $nobj = 1;

	# need to set it properly
	$nobj += scalar(@{$self->{'extents'}});
	$nobj += 1 unless ((ref($self->{'parent'}) eq 'MD::Concat') ||
			   (ref($self->{'parent'}) eq 'MD::RAID'));
	$nobj
}

# addition function for pubsize used by raid volumes
sub PubSize {
	my $self =shift;
	my $size = 0;
	my $i =0;

	while ($self->{'extents'}->[$i]) {
		$size += $self->{'extents'}->[$i]->{'len'};
		$i++;
	}
	$size - $self->{'skip'};
}

sub Skip {
	my $self = shift;
	return $self->{'skip'} + $self->{'extents'}->[0]->{'skip'};
}

# Every ::Concat is a single plex concatenating all of the ::_Stripes
# it contains.  If it's not inside a MD::Mirror then wrap it in a volume
# so the user can get to it.
sub toVxVM {
	my $self = shift;

	my $basename = $self->Vxname;

	my @sds = map $_->toVxVM, @{$self->{'extents'}};

	# Set the skip if any
	$sds[0]->{'offset'} += $self->{'skip'};
	$sds[0]->{'len'} -= $self->{'skip'};
	if (((ref($self->{'parent'}) eq 'MD::Concat') ||
	    (ref($self->{'parent'}) eq 'MD::RAID')) &&
	    (scalar(@sds) == 1)) {
	    $self->{'path'} = $self->{'extents'}->[0]->{'path'};
	    return $sds[0];
	}

	my $len = 0;
	$len += $_ for map $_->{'len'}, @sds;

	# our size is rounded to the secpercyl of the first partition
	# in the first stripe.  genius, eh?
	#$len = $self->{'extents'}->[0]->RoundDown($len);

	my $plex = VxVM::Plex->new($basename . "-p");
	$_->{'assoc'} = $plex for (@sds);
	$plex->{'aslist'} = \@sds;
	$plex->{'layout'} = 'concat';
	$plex->{'len'} = $len;

	#
	# SDS doesn't distinguish between plexes and volumes.  This MD
	# might be used directly (no parent or in a Trans) or it might be
	# a mirror.  In VxVM terms it's a plex if it's a mirror, so we
	# short-circuit here rather than making MD::Mirror rip the volume
	# back off.
	#
	# return $plex if (ref($self->{'parent'}) eq 'MD::Mirror');

	my $vol = VxVM::Volume->new($basename);
	$plex->{'assoc'} = $vol;
	$vol->{'aslist'} = [ $plex ];
	$vol->{'use_type'} = $self->{'force_use_type'} || 'gen';
	$vol->{'len'} = $len;
	$vol->{'cvtpath'} = $self->Path;
	$self->{'vol'} = $vol;
	# for raid 5 volume if we have to make layered volume
	# we need to set path  as we set for partition
	if ((ref($self->{'parent'}) eq 'MD::Concat') ||
	    (ref($self->{'parent'}) eq 'MD::RAID')) {
		my $sv =VxVM::Subdisk->new($basename . "-s");
		$vol->{'assoc'} = $sv;
		$sv->{'dm'} = $vol;
		$sv->{'offset'} = 0;
		$sv->{'len'} = $vol->{'len'};
		$sv->{'column'} = $self->{'column'};
		$sv->{'coloffs'} = 0;
		$vol->{'use_type'} = 'gen';
		$self->{'path'} = "/dev/vx/dsk/".MD::diskset_ext.
						"/".$vol->{'name'};
		$self->{'skip'} = 0;
		return $sv;
	}
	$self;
}

sub SimplePartition {
	my $self = shift;
	return $self->{'stripes'}->[0]->SimplePartition if ($self->count == 1);
}

package MD::Concat;
use Egettxt;

use vars qw( @ISA );

@ISA= qw(MD);

#
# Parse `metastat -p' output for a concat.  see metainit(1) and md.tab(4)
# for the format.
#
sub ParseMSP {
	my $self = shift;
	my (@body) = @_;
	my ($i, $j);

	$self->{'stripes'} = [];
	my $nconcat = shift @body;
	concat: for ($i = 0; $i < $nconcat; ++$i) {
		my $nstripe = shift @body;
		unshift @body, $nstripe, last concat if $nstripe =~ /^-/;

		my $stripe = MD::_Stripe->new($i, $self);
		push @{$self->{'stripes'}}, $stripe;
		for ($j = 0; $j < $nstripe; ++$j) {
			my $child;
			# Is possible to have soft partition or simple
			# partition or mixture of both
			if ($body[0] =~ /c(\d+)t([A-F0-9]+)d(\d+)s(\d+)/) {
			       $child = Partition->new(shift @body);
			       $child->SetMD($self);
			    } else {
			       $child = MD::SoftPartition->new(shift @body);
			       $child->{'parent'} = $self;
			   }
			push @{$stripe->{'partitions'}}, $child;
			#$child->SetMD($self);
			$child->{'column'} = $j;
		}
		if (@body && $body[0] =~ /^-i/) {
			shift @body;
			$stripe->SetInterlace(shift @body);
		}
	}
	# If we get what we expected, keep parsing, otherwise fall through
	# to the error case.
	if ($i == $nconcat) {
		if (@body && $body[0] =~ /^-h/) {
			shift @body;
			$self->{'hotspare'} = MD::Hotspare->new(shift @body);
		}
	}

	# Unknown arguments are ultimately fatal but we can limp on.
	if (@body) {
		Terminal('vmsolsds:97',
			'$prog: FATAL: unknown metainit args for $md: $bad',
			{ 'md' => $self->Name, 'bad' => (join ' ', @body) });
	}

	$self;
}

sub count {
	my $self = shift;

	scalar(@{$self->{'stripes'}});
}

sub estVxVM {
	my $self = shift;

	my $nobj = 1;
	$nobj += $_->estVxVM for @{$self->{'stripes'}};
	$nobj += 1 unless (ref($self->{'parent'}) eq 'MD::Mirror');
	$nobj
}

# Every ::Concat is a single plex concatenating all of the ::_Stripes
# it contains.  If it's not inside a MD::Mirror then wrap it in a volume
# so the user can get to it.

sub toVxVM {
	my $self = shift;

	my $basename = $self->Vxname;

	my @sds = map $_->toVxVM, @{$self->{'stripes'}};

	my $len = 0;
	$len += $_ for map $_->{'len'}, @sds;

	# our size is rounded to the secpercyl of the first partition
	# in the first stripe.  genius, eh?
	$len = $self->{'stripes'}->[0]->RoundDown($len);

	my $plex = VxVM::Plex->new($basename . "-p");
	$_->{'assoc'} = $plex for (@sds);
	$plex->{'aslist'} = \@sds;
	$plex->{'layout'} = 'concat';
	$plex->{'len'} = $len;

	# SDS doesn't distinguish between plexes and volumes.  This MD
	# might be used directly (no parent or in a Trans) or it might be
	# a mirror.  In VxVM terms it's a plex if it's a mirror, so we
	# short-circuit here rather than making MD::Mirror rip the volume
	# back off.
	#
	return $plex if (ref($self->{'parent'}) eq 'MD::Mirror');

	my $vol = VxVM::Volume->new($basename);
	$plex->{'assoc'} = $vol;
	$vol->{'aslist'} = [ $plex ];
	$vol->{'use_type'} = $self->{'force_use_type'} || 'gen';
	$vol->{'len'} = $len;
	$vol->{'cvtpath'} = $self->Path;
	# XXX set prefered plex, read pol

	$self->{'vol'} = $vol
}

sub SimplePartition {
	my $self = shift;
	return $self->{'stripes'}->[0]->SimplePartition if ($self->count == 1);
}

package MD::Mirror;
use Egettxt;

use vars qw( @ISA );

@ISA= qw(MD);

#
# Parse `metastat -p' output for a mirror.  see metainit(1) and md.tab(4)
# for the format.
#
sub ParseMSP {
	my $self = shift;
	my (@body) = @_;

	# -m
	shift @body;

	$self->{'mirrors'} = [];
	while (@body && $body[0] !~ /^[-\d]/) {
		my $child = MD->new(shift @body);
		$child->{'parent'} = $self;
		push @{$self->{'mirrors'}}, $child;
	}
	$self->{'readpol'} = 'round-robin';
	$self->{'writepol'} = 'parallel';
	while (@body && $body[0] =~ /^-([rgS])$/) {
		shift @body;
		if ($1 eq 'r') {
			$self->{'readpol'} = 'prefer-first';
		} elsif ($1 eq 'g') {
			$self->{'readpol'} = 'geometric';
		} elsif ($1 eq 'S') {
			$self->{'writpol'} = 'serial';
		}
	}

	# unknown option before 'pass'
	die if $body[0] !~ /^\d+$/;

	$self->{'pass'} = shift @body;

	# more??
	die if @body;

	$self;
}

sub estVxVM { 1 }

#
# Mirror, my children!
#
sub toVxVM {
	my $self = shift;

	my $basename = $self->Vxname;

	# MD::Concat->toVxVM can see us up here, so it makes plexes
	# for us.
	my @plexes = map $_->toVxVM, @{$self->{'mirrors'}};

	# It set the len= attribute on those plexes, but sadly vxmake
	# will not honor it, and we will not inherit the rounding necessary
	# to emulate SDS.  So we do it manually here.
	(my $len) = sort {$a <=> $b} map($_->{'len'}, @plexes);

	my $vol = VxVM::Volume->new($basename);
	$_->{'assoc'} = $vol for (@plexes);
	$vol->{'aslist'} = \@plexes;
	$vol->{'use_type'} = $self->{'force_use_type'} || 'gen';
	$vol->{'len'} = $len;
	$vol->{'cvtpath'} = $self->Path;

	$self->{'vol'} = $vol
}

sub SimplePartition {
	my $self = shift;

	grep { $_ }  map $_->SimplePartition, @{$self->{'mirrors'}}
}

package MD::Trans;
use Egettxt;

use vars qw( @ISA );

@ISA= qw(MD);

#
# Parse `metastat -p' output for a trans.  see metainit(1) and md.tab(4)
# for the format.
#
sub ParseMSP {
	my $self = shift;
	my (@body) = @_;

	# -t
	shift @body;

	my $master = shift @body;
	if (Partition::IsPartition($master)) {
		$self->{'master'} = Partition->new($master);
		$self->{'master'}->SetMD($self);
		$self->{'master'}->{'column'} = 0;
		bless $self, 'MD::Trans::Flat';
	} else {
		$self->{'master'} = MD->new($master);
		$self->{'master'}->{'parent'} = $self;
	}
	if (@body) {
		my $log = shift @body;
		if (Partition::IsPartition($log)) {
			# record information for warning message, but
			# do not SetMD, that way it will not be "in use"
			$self->{'log'} = Partition->new($log);
		} else {
			$self->{'log'} = MD->new($log);
			$self->{'log'}->{'parent'} = $self;
		}
	}
	$self;
}

#
# Trans devices promote the "Detached" state to "Okay", since we don't
# care that the log is detached.
#
sub ParseMS {
	my $self = shift;

	$self->SUPER::ParseMS(@_);
	$self->{'state'} = 'Okay' if $self->{'state'} eq 'Detached';
}

#
# We're only going to build the master, so our name is its name
#
sub Vxname {
	my $self = shift;
	my $master = $self->{'master'};

	return $master->Vxname if $master;
	$self->SUPER::Vxname
}

sub estVxVM { 0 }

#
# We just ignore the log and let the user reallocate it under VxVM
#
sub toVxVM {
	my $self = shift;

	$self->{'master'}->{'force_use_type'} = $self->{'force_use_type'};
	$self->{'vol'} = $self->{'master'}->toVxVM;
	$self->{'vol'}->{'cvtpath'} = $self->Path;
	$self->{'vol'}
}

sub SimplePartition {
	my $self = shift;
	return $self->{'master'}->SimplePartition;
}

sub PreAssert {
	my $self = shift;

	$self->SUPER::PreAssert;
	if ($self->{'log'}) {
		Warning('vmsolsds:98',
			'$prog: Warning: Log $logmd in trans $md cannot \
			be used by VxVM and will not be converted',
			{ 'md' => $self->Name,
			  'logmd' => $self->{'log'}->Name });
	}
}

#
# Trans metadevices directly on slices (rather than other mds)
#
package MD::Trans::Flat;
use Egettxt;

use vars qw( @ISA );

@ISA= qw(MD::Trans);

#
# In the flat form we use our own name.
#
sub Vxname {
	my $self = shift;

	$self->MD::Vxname
}

sub estVxVM {
	2 # always one plex, one volume on the partition (counted separately)
}

#
# In the Flat form of the Trans there is just one underlying partition
# which we turn into a simple volume (rather than delegating to another
# metadevice for conversion).
#
# We just ignore the log and let the user reallocate it under VxVM
#
sub toVxVM {
	my $self = shift;

	my @sds = $self->{'master'}->toVxVM;

	my $plex = VxVM::Plex->new($self->Vxname . "-p");
	$_->{'assoc'} = $plex for (@sds);
	$plex->{'aslist'} = \@sds;
	$plex->{'layout'} = 'concat';

	my $vol = VxVM::Volume->new($self->Vxname);
	$plex->{'assoc'} = $vol;
	$vol->{'aslist'} = [ $plex ];
	$vol->{'use_type'} = $self->{'force_use_type'} || 'fsgen';
	$vol->{'len'} = $self->{'size'};

	$vol->{'cvtpath'} = $self->Path;

	$self->{'vol'} = $vol
}

sub SimplePartition {
	my $self = shift;
	return $self->{'master'};
}

#
# RAID Metadevices
#
package MD::RAID;
use Egettxt;

use vars qw( @ISA );

@ISA= qw(MD);
#
# Parse `metastat -p' output for a RAID.  see metainit(1) and md.tab(4)
# for the format.
#
sub ParseMSP {
	my $self = shift;
	my (@body) = @_;

	# -r
	shift @body;

	my $raid = MD::_Stripe::_RAIDStripe->new(0, $self);
	$self->{'raid'} = $raid;
	my $j = 0;
	while (@body && $body[0] !~ /^-/) {
		my $child;
		if ($body[0] =~ /c(\d+)t([A-F0-9]+)d(\d+)s(\d+)/) {
				$child = Partition->new(shift @body);
				$child->SetMD($self);
		} else {
				$child = MD::SoftPartition->new(shift @body);
				$child->{'parent'} = $self;
		}
		push @{$raid->{'partitions'}}, $child;
		$child->{'column'} = $j++;
	}
	$self->{'clobber'} = 'yes';
	$self->{'original'} = $raid->count();
	while (@body && $body[0] =~ /^-([kihow])$/) {
		shift @body;
		if ($1 eq 'k') {
			$self->{'clobber'} = 'no';
		} elsif ($1 eq 'i') {
			$raid->SetInterlace(shift @body);
		} elsif ($1 eq 'h') {
			$self->{'hotspare'} = MD::Hotspare->new(shift @body);
		} elsif ($1 eq 'o') {
			$self->{'original'} = shift @body;
		} elsif ($1 eq 'w') {
			# this one is undocumented by metainit(1)
			$self->{'simultaneous_writes'} = shift @body;
		} else {
			# put the bad option back, let the error below catch it
			unshift @body, "-$1";
			last;
		}
	}
	$self->{'extended'} = $raid->count() - $self->{'original'};

	# Unknown arguments are ultimately fatal but we can limp on.
	if (@body) {
		Terminal('vmsolsds:97',
			'$prog: FATAL: unknown metainit args for $md: $bad',
			{ 'md' => $self->Name, 'bad' => (join ' ', @body) });
	}

	$self;
}

sub PreAssert {
	my $self = shift;

	$self->SUPER::PreAssert;
	if ($self->{'extended'} != 0) {
		Terminal('vmsolsds:99',
			'$prog: FATAL: unable to convert extended RAID $md',
			{ 'md' => $self->Name });
	}
}

sub estVxVM {
	my $self = shift;

	my $nobj = 1;
	$nobj += $self->{'raid'}->estVxVM;
	$nobj;
}

sub toVxVM {
	my $self = shift;

	my $plex = $self->{'raid'}->toVxVM;

	my $vol = VxVM::Volume->new($self->Vxname);
	$plex->{'assoc'} = $vol;
	$vol->{'aslist'} = [ $plex ];
	$vol->{'use_type'} = 'raid5';
	$vol->{'read_pol'} = 'raid';
	$vol->{'cvtpath'} = $self->Path;

	#
	# To find the length of the MD RAID, round down to the only
	# "stripe"'s first partition's secpercyl.  Similar to the concat
	# rule.  However, we can't really use this length, because VxVM
	# requires rounding to st_width, which is almost never going to
	# be compatible.  Instead we let the VxVM RAID be a little too
	# long.  But here's the calculation for posterity:
	#
	#	my $len = $self->{'raid'}->RoundDown($plex->{'len'});
	#
	my $len = $plex->{'len'};

	#
	# ...but we must set the length explicitly on the volume because
	# vxmake will not honor the plex len= attribute, expanding it to
	# be as long as the LONGEST column, but SPARSE (brilliant, eh?)
	# and causing the volume to inherit this length and be unstartable.
	#
	$vol->{'len'} = $len;

	$self->{'vol'} = $vol
}

sub R5Trans {
	my $self = shift;
	my $raid = $self->{'raid'};
	my $basename = $self->Vxname;
	my $logfile = "$main::base_dir/tmp/logfile.$basename". MD::diskset_ext;

	"rm -f $logfile\n" .
	"r5trans -s " . $raid->VxStripeWidth . " -l " . $raid->Size . " -f $logfile " . (join " ", map($_->{'path'} . " " . $_->Skip, @{$raid->{'partitions'}})) . "\n"
}

package MD::Hotspare;
use Egettxt;

use vars qw( @ISA %HotspareTab );

%HotspareTab = ();
sub Registry { \%HotspareTab }

@ISA= qw(MD);
#
# Parse `metastat -p' output for a hot spare pool.
# see metainit(1) and md.tab(4) for the format.
#
sub ParseMSP {
	my $self = shift;
	my (@body) = @_;

	$self->{'partitions'} = [];
	foreach (@body) {
		my $child = Partition->new($_);
		push @{$self->{'partitions'}}, $child;
		$child->SetMD($self);
	}
	$self;
}

sub Inuse { 0 }

1;
