#
# $Id: Spacemap.pm,v 1.2.2.1 2003/06/06 16:44:11 rohit Exp $
#ident "$Source: /project/unixvm-cvs/src/sol/cmd/sdscvt/lib/Spacemap.pm,v $"
#
# $Copyrights: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

use strict;

package Spacemap;

sub new {
	my $type = shift;

	bless [], $type;
}

sub Inuse {
	my $self = shift;
	my ($start, $end) = @_;

	# there can only be about 7 distinct regions in use on a
	# disk under SDS anyway, so there's no need for a wizzy
	# algorithm to search this sorted list.
	my $found = 0;
	region: foreach (@{$self}) {
		last region if $_->[0] > $end;
		next region if $_->[1] < $start;
		$found = 1;
	}
	$found
}

sub Add {
	my $self = shift;
	my ($start, $end) = @_;

	$self->Inuse($start, $end) && die;
	my $i;
	region: for ($i = 0; $i < scalar(@{$self}); ++$i) {
		last region if $self->[$i]->[0] > $end;
	}
	splice @{$self}, $i, 0, [$start, $end];
}

# assume there's a dummy entry marking the end, so never allocate beyond
# the last element.
sub Alloc {
	my $self = shift;
	my $size = shift;

	# XXX can't use block 0 so start at 1
	my $freestart = 1;
	# XXX would be nicer to nibble from the smallest region
	my $available;
	region: foreach (@{$self}) {
		$available = $_->[0] - $freestart;
		last region if $available >= $size;
		$freestart = $_->[1] + 1;
	}
	if ($available && $available >= $size) {
		my $result = [ $freestart, $freestart + $size - 1 ];
		$self->Add(@{$result});
		return $result;
	} else {
		return undef;
	}
}

1;
