#!/bin/ksh
# $Id: es_install.sh,v 1.1.4.3.24.1 2004/07/29 21:58:34 hhorn Exp $
#ident "$Source: /project/unixvm-cvs/src/sol/cmd/vxvm/esd/es_install.sh,v $"

#
# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

# Script to install handler for EC_devfs sysevents and RCM events

# default values
SH=/usr/bin/perl

DEVFS_SCRIPT=/etc/vx/es_scripts/es_devfs.pl

CONFIG_DIR=/etc/sysevent/config/
DEVFS_CONFIG=EC_devfs,sysevent.conf

RCM_PATH=/etc/vx/es_scripts
RCM_SCRIPT=es_rcm.pl


# function to install sysevent handler
function install_sysevent
{
	#echo "Installing handler for EC_devfs sysevent"
	case `uname -r` in
		5.8)
			if [ ! -d $CONFIG_DIR ]
			then
				ugettxt -s ERROR "Sysevent script config directory missing" vxvmshm:3260
				return
			fi

			TEMP=$CONFIG_DIR/$DEVFS_CONFIG

			touch $TEMP
			if [ ! $? -eq 0 ]
			then
				ugettxt -s ERROR "Failed to create DEVFS sysevent script" vxvmshm:3158
				return
			fi

        		# copy the comment to a temporary file
			echo "# $Id: es_install.sh,v 1.1.4.3.24.1 2004/07/29 21:58:34 hhorn Exp $" > $TEMP
			echo "#ident "$Source: /project/unixvm-cvs/src/sol/cmd/vxvm/esd/es_install.sh,v $"" >> $TEMP
			echo "" >> $TEMP
			echo "# Copyright (c) 2000 VERITAS Software Corporation.  ALL RIGHTS RESERVED." >> $TEMP
			echo "# UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT" >> $TEMP
			echo "# LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE" >> $TEMP
			echo "# IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION" >> $TEMP
			echo "# OR DISCLOSURE." >> $TEMP
			echo "#" >> $TEMP
			echo "# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND" >> $TEMP
			echo "# TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE," >> $TEMP
			echo "# OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR" >> $TEMP
			echo "# EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE." >> $TEMP
			echo "#" >> $TEMP
			echo "#               RESTRICTED RIGHTS LEGEND" >> $TEMP
			echo "# USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS" >> $TEMP
			echo "# SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH" >> $TEMP
			echo "# (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND" >> $TEMP
			echo "# COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013." >> $TEMP
			echo "#               VERITAS SOFTWARE" >> $TEMP
			echo "# 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043" >> $TEMP
			echo "" >> $TEMP
			echo "# The format for an entry in a sysevent.conf file is:" >> $TEMP
			echo "# <class> <subclass> <vendor> <publisher> <user> <reserved1> <reserved2> <path> <arguments>" >> $TEMP
			echo "" >> $TEMP

			# add an entry to each file that specifies the binary to be executed
			echo "EC_devfs ESC_devfs_devi_add - - - - - $SH $DEVFS_SCRIPT \${di.path}" >>$TEMP

			# inform syseventd to restart
			pkill -HUP syseventd
			;;

		5.9)
        		syseventadm remove -c EC_devfs $SH $DEVFS_SCRIPT
			syseventadm add -c EC_devfs -s ESC_devfs_devi_add $SH $DEVFS_SCRIPT \${di.path}
			if [ ! $? -eq 0 ]
			then
				ugettxt -s ERROR "Failed to install DEVFS sysevent script" vxvmshm:3159
				return
			fi

			syseventadm restart
			;;

		5.10)
        		syseventadm remove -c EC_devfs $SH $DEVFS_SCRIPT
			syseventadm add -c EC_devfs -s ESC_devfs_devi_add $SH $DEVFS_SCRIPT \${di.path}
			if [ ! $? -eq 0 ]
			then
				ugettxt -s ERROR "Failed to install DEVFS sysevent script" vxvmshm:3159
				return
			fi

			syseventadm restart
			;;

		  *)	
			ugettxt -s ERROR "Unknown OS release version" vxvmshm:3312
			;;
	esac
}


# function to install RCM script
function install_rcmevent
{
	#echo "Installing RCM script"
	if [ ! -d $RCM_PATH ]
	then
		ugettxt -s ERROR "RCM scripts directory missing" vxvmshm:3221
		return
	fi

	# Add the following comment which makes rcm recognise the script as a shell script
	echo "#!/usr/bin/perl" > /usr/lib/rcm/scripts/$RCM_SCRIPT
	cat $RCM_PATH/$RCM_SCRIPT >> /usr/lib/rcm/scripts/$RCM_SCRIPT

	if [ ! $? -eq 0 ]
	then
		ugettxt -s ERROR "Failed to install RCM script" vxvmshm:3160
		return
	fi

	chmod +x /usr/lib/rcm/scripts/$RCM_SCRIPT
	pkill -HUP -x -u root rcm_daemon
}


# Main
case $1 in
	"sysevent")	
		install_sysevent	
		;;
	"rcmevent")	
		install_rcmevent	
		;;
	"all")
		install_sysevent	
		install_rcmevent	
		;;
	*)
		ugettxt -s INFO "Script to install handler for EC_devfs sysevent and RCM script" vxvmshm:3230
		echo "Usage:   $0 <sysevent | rcmevent | all>"
		;;
esac
