#!/usr/bin/perl
# $Id: es_rcm.pl,v 1.1.4.3 2003/10/31 01:13:56 davidl Exp $
#ident "$Source: /project/unixvm-cvs/src/sol/cmd/vxvm/esd/es_rcm.pl,v $"

#
# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

use strict;


# function to print the given error message to RCM
sub do_print_error
{
	my ($errmsg);

	$errmsg = shift(@_);
	print "rcm_failure_reason=$errmsg\n";
	exit(3);
}


# function to return information about the script
sub do_scriptinfo
{
	print "rcm_script_version=1\n";
	print "rcm_script_func_info=RCM script to inform departure of devices to ddl\n";
	exit (0);
}


# function to register list of devices being handled
sub do_register
{
	my($i, @ctlr, $str, $x, $c, $disk);

	# obtain list of controllers on the system
	open(FILE, "vxdmpadm getctlr all|") || 
			&do_print_error("Could not obtain list of controllers");
	$i = 0;
	@ctlr = ();
	while(<FILE>) {
		if ($i > 1) {
			$str = $_;
			$x = index($str, " ");
			$c = substr($str, 0, $x);
			push(@ctlr, $c);
		}
		$i = $i + 1;
	}
	close(FILE);

	# for each controller obtain the list of disks and print them
	while($#ctlr >= 0) {
		$c = pop(@ctlr);
	open(FILE, "vxdmpadm getsubpaths ctlr=$c|") || 
			&do_print_error("Could not obtain list of disks");
		$i = 0;
		while(<FILE>) {
			if ($i > 1) {
				$str = $_;
				$x = index($str, " ");
				$disk = substr($str, 0, $x);
				print "rcm_resource_name=/dev/dsk/$disk\n";
			}
			$i = $i + 1;
		}
		close(FILE);
	}

	exit(0);
}


# function to obtain the resource usage information for the given device
sub do_resourceinfo
{
	print "rcm_resource_usage_info=Device being used by VxVM\n";
	exit(0);
}


# function to check if the given device can be detached
sub do_queryremove
{
	my($path, $i, $disk, $force, $r);

	$path = shift(@ARGV);
	$i = rindex($path, "/");
	$disk = substr($path, $i + 1);
	$force = $ENV{RCM_ENV_FORCE};

	# if force = TRUE then return success, otherwise do a check
	if ($force eq "TRUE") {
		exit(0);
	}

	$r = `vxdmpadm -c disable path=$disk`;
	if ($? == 0) {
		exit(0);
	}
	else {
                print "rcm_failure_reason=The device cannot be disabled\n";
		exit(3);
	}
}

# function to perform operations prior to device detach
sub do_preremove
{
	my($path, $i, $disk, $force, $f, $r);

	$path = shift(@ARGV);
	$i = rindex($path, "/");
	$disk = substr($path, $i + 1);
	$force = $ENV{RCM_ENV_FORCE};

	if ($force eq "TRUE") {
        	$f = "-f";
	}
	else {
        	$f = "";
	}

	$r = `vxdmpadm $f disable path=$disk`;
	if ($? == 0) {
		exit(0);
	}
	else {
                print "rcm_failure_reason=The device cannot be disabled\n";
 		exit(3);
	}
}

# function to perform operations after the device detach operation has been completed
sub do_postremove
{
	# Nothing to be done
	exit(0);
}

# function to undo the operations done prior to device detach in preremove function
sub do_undoremove
{
	my($path, $i, $disk, $r);

	$path = shift(@ARGV);
	$i = rindex($path, "/");
	$disk = substr($path, $i + 1);

	$r = `vxdmpadm enable path=$disk`;
	if ($? == 0) {
		exit(0);
	}
	else {
                print "rcm_failure_reason=The device cannot be enabled\n";
		exit(3);
	}
}


# Main
my ($cmd, %dispatch);
$cmd = shift(@ARGV);

# dispatch table for RCM commands
%dispatch = (
	"scriptinfo"	=>	\&do_scriptinfo,
	"register"	=>	\&do_register,
	"resourceinfo"	=>	\&do_resourceinfo,
	"queryremove"	=>	\&do_queryremove,
	"preremove"	=>	\&do_preremove,
	"postremove"	=>	\&do_postremove,
	"undoremove"	=>	\&do_undoremove
);

if (defined($dispatch{$cmd})) {
	&{$dispatch{$cmd}};
} else {
	exit (2);
}
