#!/sbin/sh -
# $Id: vxadm_syslib.sh,v 1.53.2.18.10.21.10.3.2.2 2009/01/11 23:57:56 hhorn Exp $
# #ident "$Source: /project/unixvm-cvs/src/sol/cmd/vxvm/voladm/vxadm_syslib.sh,v $"

# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
xcmd()
{
	if [ ! -z "$VOL_VERBOSE" ]
	then
		echo "!$VOL_XCMD $@" >&2
	fi
	$VOL_XCMD "$@"
	[ $? = 0 ] || bail
}

inuse_check()
{
	mount_outfile=$tmpfile2

	mount > $mount_outfile

	inuse_list=
	not_inuse_list=

	shift $#	# Reset argument count to zero.
	set -- $dev_list
	while [ $# -gt 0 ]
	do
		device=$1
		daname=$2

# 		egrep -s /dev/dsk/$device $mount_outfile 2>/dev/null
#		egrep -s  "/dev(/|/ap/)dsk/$device" $mount_outfile 2>/dev/null
		egrep -s  "/dev(/|/ap/)(vx/dmp|dsk)/$device" $mount_outfile 2>/dev/null
		if [ $? -eq 0 ]
		then
			append_list inuse_list $device $daname
		else
			append_list not_inuse_list $device $daname
		fi
		shift $dev_num
	done

	if [ -n "$inuse_list" ]
	then
		if [ `list_count $inuse_list` -eq $dev_num ]
		then
			ewritemsg -f $disp_file -M vxvmshm:242 \
"One or more partitions of the following disk device are in use as
  a mounted file system. The disk cannot be initialized, but you can
  encapsulate the existing disk partitions as volumes.
  Output format: [Device_Name]"
		else
			ewritemsg -f $disp_file -M vxvmshm:243 \
"One or more partitions of the following disk devices are in use as
  a mounted file system. The disks cannot be initialized, but you can
  encapsulate the existing disk partitions as volumes.
  Output format: [Device_Name]"
		fi

		voladm_yorn_batch -h \
		-g $dev_num -p $dev_index -d $dev_index \
		-f $disp_file -M vxvmshm:1004 -m vxvmshm:1005 \
		"Encapsulate these devices?" Y \
		"Encapsulate this device?" y \
		inuse_list encap_list ignore_list

		if [ -n "$encap_list" ]
		then
			append_list ckinit_encap $encap_list
		fi
	fi

	# If any not-inuse disks were found, then set dev_list to
	# the not-inuse list and continue. Otherwise if no not-inuse
	# disks were found all other disks have been set-up for
	# encapsulation or have been ignored so return.
	if [ -n "$not_inuse_list" ]
	then
		dev_list="$not_inuse_list"
	fi
}

#
# get_mountdevice dir basevarname
#
# Locate dir in the vfstab file and set the following variables as
# appropriate:
#
#	basevarname_bdev	- the block device for the file system
#	basevarname_rdev	- the raw device for the file system
#	basevarname_disk	- the standard form disk (cNtNdN), if the
#				  root file system is on a regular disk
#	basevarname_vol		- the volume containing the file system,
#				  if the file system is on a volume
#	basevarname_dg		- thd disk group containing the volume
#
get_mountdevice()
{
	# On Solaris read the /etc/vfstab to get mount information.
	__bdev=
	__cdev=
	__disk=
	__vol=
	__dg=
	exec 9<&0 < $SYSROOT/etc/vfstab
	while read _special _fsckdev _mountp _fstype _fsckpass _automnt _mntopts
	do
		case ${_special} in
		'#'* | '')	#  Ignore comments, empty lines
				continue ;;
		'-')		#  Ignore no-action lines
				continue ;;
		esac

		if [ "${_mountp}" = "$1" ]
		then
			__bdev=$_special
			__cdev=$_fsckdev
			dogi_path_to_slice ${__bdev} slc
			#echo "Slice: $slc"
			slc_disk=`/usr/sbin/vxdmpadm getdmpnode nodename=$slc 2> /dev/null | tail +3 | awk '{print $1}'`
			if [ -z "$slc_disk" ]; then
				slc_disk=$slc
			fi
			dogi_slice_to_device "$slc_disk" __disk
			#echo "Disk: ${__disk}"
			#__disk=`expr "\${__bdev}" : '^/dev/dsk/\(.*\)s.\$'`
			__vol=`expr "\${__bdev}" : '^/dev/vx/dsk/\(.*\)'`
			case $__vol in
			*"/"*"/"*) __vol=;;
			*"/"*)	__dg="`expr "\$__vol" : '\(.*\)/.*'`"
				__vol="`expr "\$__vol" : '.*/\(.*\)'`";;
			*) __vol=;;
			esac
			break
		fi
	done
	exec <&9 9<&-
	# rootdisk will  be empty if the root is defined as a volume in
	# the /etc/vfstab file.

	eval "${2}_bdev=$__bdev ${2}_cdev=$__cdev ${2}_disk=$__disk"
	eval "${2}_vol=$__vol ${2}_dg=$__dg"
}

#
# get_dmpname pathname basename
#
#	basename_dmp - dmpnodename for the path $pathname
#
# get_dmpname - returns the dmpnodename of the path
#
get_dmpname()
{
	__dmp=
	__full=

	# DMP:
	# Check if rootdisk is part of a multipath
	# DMP device, then return name of the actual
	# DMP device rather than the subpath

	if [ -f /usr/sbin/vxdmpadm ]
	then
		dogi_whole_slice $1 __full
		dmpdisk=`/usr/sbin/vxdmpadm getdmpnode nodename=$__full 2> /dev/null | tail +3 | awk '{print $1}'`
		if [ -z "$dmpdisk" ]
		then
			dmpdisk=$1
		fi
		dogi_slice_to_device "$dmpdisk" __dmp
		if [ -z "$__dmp" ]
		then
			__dmp=$1
		fi

	else
		__dmp=$1
	fi
	eval "${2}_dmp=$__dmp"
}

#
# set_bootdisk - set parameters for the root file system
#
set_rootdisk()
{
	mpdev=
	rootslice=
	get_mountdevice / root
	# for backward compatibility, set rootdisk

	# DMP:
	# Check if rootdisk is part of a multipath
	# DMP device, then return name of the actual
	# DMP device rather than the subpath

	if [ -f /usr/sbin/vxdmpadm ]
	then
		dogi_whole_slice $root_disk rootslice
		rootdmpdisk=`/usr/sbin/vxdmpadm getdmpnode nodename=$rootslice 2> /dev/null | tail +3 | awk '{print $1}'`
		if [ -z "$rootdmpdisk" ]; then
			# This could be the subpaths of TPD metanode, so try
			# to get the name of TPD metanode before giving up.

			tpdmetanode=`/usr/sbin/vxdmpadm gettpdnode \
				nodename="$rootslice" 2> /dev/null | \
				tail +3 | awk '{print $1}'`
			rootdmpdisk=`/usr/sbin/vxdmpadm getdmpnode \
				nodename="$tpdmetanode" 2> /dev/null | \
				tail +3 | awk '{print $1}'`
			[ -z "$rootdmpdisk" ] && rootdmpdisk="$rootslice"
		fi
		dogi_slice_to_device "$rootdmpdisk" mpdev
		rootdisk=$mpdev
		if [ -z "$rootdisk" ]
		then
			rootdisk=$root_disk
		fi

	else
		rootdisk=$root_disk
	fi
}

get_drv_list() 
{
	dir_entry="`ls -ld \$1`"
	path=`expr "$dir_entry" : ".*\/devices\(.*\)"`
	[ -n "$path" ] || {
		file=$1
		export file; ugettxt -s ERROR >&2 \
			'get_drv_list: Invalid device file: $file' vxvmshm:1027 $file
		return 1
	}

	node=`basename $path`
	path=`dirname $path`

	while [ -n "$path" -a "X$node" != "X/" ]; do
		case $node in
			*@*)	drv=`expr $node : "\(.*\)@.*"`
				if [ "X$drv" = "XSUNW,ifp" ]
				then
					drv=`echo ifp`
				elif [ "X$drv" = "XSUNW,isptwo" -o \
					"X$drv" = "XQLGC,isp"  ]
				then
					drv=`echo isp`
				else
					drv=`echo $drv | sed 's/SUNW,//'`
				fi
				;;
			*)	drv=$node
				;;
		esac
		echo $drv
		node=`basename $path`
		path=`dirname $path`
	done
}

forceload_drv()
{
	file=$SYSROOT/etc/system
	cp $file $file.sav
	drv=$1

	# return if driver is already forceload'd 
	grep "^ *forceload: *drv/$drv *$" $file 2>&1 > /dev/null
	if [ $? -eq 0 ] ; then
		return 0
	fi

	# Don't forceload driver if it's not installed  
        mach_platform=`eval uname -i`
        mach_name=`eval uname -m`
        if [ ! -f /kernel/drv/$drv -a \
          ! -f /platform/$mach_platform/kernel/drv/$drv -a \
          ! -f /platform/$mach_name/kernel/drv/$drv ] ; then
		return 1
	fi

	end_line=`grep -n vxvm_END $file`
	if [ -z "$end_line" ] ; then
	sed \
	-e '$ a\
* vxvm_START (do not remove)\
* vxvm_END (do not remove)' $file > $file.sav 
	cp $file.sav $file
	end_line=`grep -n vxvm_END $file`
	fi

	end_line=`expr "$end_line" : "\(.*\):.*"`
	end_line=`expr $end_line - 1`
	sed \
	-e ''$end_line' a\
forceload: 'drv/$drv'' $file > $file.sav
	cp $file.sav $file

	return 0
}

#
# Routines to comment/uncomment "rootdev:" lines from /etc/system
#

# Common pattern variables
rpattern='^rootdev:'
block="*(vxvm rootability)*"
eblock='\*(vxvm rootability)\*'
cpattern="${eblock} rootdev:.*"

# The following is vxdmp driver configuration file and its associated property

dmp_conf_file="/kernel/drv/vxdmp.conf"
dmp_boot_device="dmp_tpd_root_device"

#
# Name:	comment_out_rootdev()
#
# Purpose: Any line matching $rpattern will be replaced by one line
# 	   composed of the matching line preceded by $block.
#
comment_out_rootdev()
{
	if [ $# -gt 1 ]
	then
		file=$1
	else
		file=$SYSROOT/etc/system
	fi

	# Return if no "rootdev" specs are found
	grep "${rpattern}" $file 2>&1 > /dev/null
	if [ $? -ne 0 ]; then
		return 0
	fi

	# Get the entries corresponding to rootdev: from /etc/system file and
	# put them into vxdmp.conf file.  This is required when the boot disk
	# is being overridden by Third Party Drivers like ATF or RDAC

	# First, remove the rootdev: string from the boot device.

	tpd_boot_device=`grep "${rpattern}" $file | sed -e "s/${rpattern}//g"`
	echo "$dmp_boot_device=\"${tpd_boot_device}\";" >> $dmp_conf_file

	# Comment each one 
	sed -e "/${rpattern}/s,.*,${block} &," $file > $file.cor.save

	mv $file.cor.save $file

	return 0
}

#
# Name: restore_commented_rootdev()
#
# Purpose: Any line matching $cpattern will be replaced by one line
# 	   composed of the matching line with $block removed.  
#
restore_commented_rootdev()
{
	if [ $# -gt 1 ]
	then
		file=$1
	else
		file=$SYSROOT/etc/system
	fi

	# Return if no "rootdev" specs are found
	grep "${cpattern}" $file 2>&1 > /dev/null
	if [ $? -ne 0 ]; then
		return 0
	fi

	# Uncomment each one 
	sed -e "/${cpattern}/s,${eblock} \(.*\),\1," $file > $file.rcr.save

	mv $file.rcr.save $file

	# Simply delete dmp_tpd_root_device property from dmp_conf_file

	sed -e "/^${dmp_boot_device}=/d" $dmp_conf_file > \
						$dmp_conf_file.rcr.save
	mv $dmp_conf_file.rcr.save $dmp_conf_file

	return 0
}

#
# Set OS specific variables
#
set_OS_variables()
{
	VOL_FULL_SLICE=s2
	VOL_FULL_VSLICE=c
	VOL_PRIV_SLICE=3
	VOL_PUB_SLICE=4
	if [ "X`uname -s`" = "XSunOS" -a "X`uname -p`" = "Xi386" ]
	then
		SOLARIS_X86=TRUE
		VOL_GHOST_LEN=4
		VOL_PRIV_OFFSET=4
	else
		VOL_GHOST_LEN=1
		VOL_PRIV_OFFSET=1
	fi
	PRIVLEN_NOCONFIG=160
	PRIVLEN_CONFIG=2048
	PRIVLEN_MAX=524288
	EDVTOC=vxedvtoc
	PRTVTOC=vxprtvtoc
        VOLD=vxconfigd
	CDS_ALIGN=16			# 8K
	CDS_PRIV_OFFSET=256		# 128K
	CDS_PRIV_SLICE=7


	DISK_RAWPATH="/dev/rdsk"
	DISK_BLKPATH="/dev/dsk"
	VOL_DISK_RAWPATH="/dev/vx/rdmp"
	VOL_DISK_BLKPATH="/dev/vx/dmp"
	VOL_AP_DISK_RAWPATH="/dev/ap/rdsk"
	VOL_AP_DISK_BLKPATH="/dev/ap/dsk"

	VOL_ROOTABLE_FILESYSTEMS="/ /usr /var"
	VOL_KEEPER_FILESYSTEMS="/ /opt /usr /usr/kvm /var /var/adm"

	VOL_VFSTAB_PATH="/etc/vfstab"

	export VOL_FULL_SLICE VOL_PRIV_SLICE VOL_PUB_SLICE VOL_PRIV_OFFSET 
	export EDVTOC PRTVTOC VOLD
	export VOL_GHOST_LEN
	export VOL_DISK_RAWPATH VOL_DISK_BLKPATH
	export VOL_AP_DISK_RAWPATH VOL_AP_DISK_BLKPATH
	export VOL_ROOTABLE_FILESYSTEMS
	export VOL_VFSTAB_PATH
	export SOLARIS_X86
}


#
# Set EFI specific variables
#
set_EFI_variables()
{
	VOL_FULL_SLICE=
	VOL_PRIV_OFFSET=34
	export VOL_FULL_SLICE VOL_PRIV_OFFSET
}

voladm_mounted_filesystems()
{
	fslist=`/sbin/mount | sed 's/ .*//'`
	if [ $# -gt 0 ]
	then
		eval "${1}=\"$fslist\""
	else
		echo $fslist
	fi
}

#
# voladm_mount_table - output (or copy to a file) the
# table of "filesystems to be moutned". On some unix systems,
# root (/) is not part of the vfstab, so it may need to
# be represented explicitly.

voladm_mount_table()
{
	if [ $# -gt 0 ]
	then
		xcmd cp $SYSROOT/$VOL_VFSTAB_PATH ${1}
	else
		cat $SYSROOT/$VOL_VFSTAB_PATH
	fi
}

voladm_disk_in_use()
{
	disk=`basename $1`

	allsubpaths=
	if [ -f /usr/sbin/vxdmpadm ]
	then
		allsubpaths=`/usr/sbin/vxdmpadm getsubpaths dmpnodename=$disk 2> /dev/null | \
		tail +3 | grep -v "=" | awk '{print $1}'`
	fi

	# check if mounted as disk name
	_diskdev=
	dogi_slice_to_device "$disk" _diskdev
	if [ "X$_diskdev" != "X" ] ; then
		mount | egrep "/dev(/|/ap/)(vx/dmp|dsk)/$_diskdev(c|s| )" \
						> /dev/null 2> /dev/null
		if [ $? -eq 0 ] ; then
			return 0
		fi	
	fi

	if [ "X${allsubpaths}" = X ] ; then
		return 1
	fi

	# check if mounted as dmp name
	for subpath in $allsubpaths
	do
		_diskdev=
		dogi_slice_to_device "$subpath" _diskdev
		if [ "X$_diskdev" = "X" ] ; then
			continue
		fi

		mount | egrep "/dev(/|/ap/)(vx/dmp|dsk)/$_diskdev(c|s| )" \
						> /dev/null 2> /dev/null
		if [ $? -eq 0 ] ; then
			return 0
		fi
	done
	return 1	
}

#
# is_std_disk - test if a disk is standard format
#
is_std_disk()
{
	_isd_base="m\{0,1\}c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*"
	_isd_slice=s[0-9][0-9]*
	case $1 in
	"/dev/dsk/"*s*)	expr "$1" : "^/dev/dsk/$_isd_base$_isd_slice\$";;
	"/dev/rdsk/"*s*) expr "$1" : "^/dev/rdsk/$_isd_base$_isd_slice\$";;
	"/dev/vx/dmp/"*s*)	expr "$1" : "^/dev/vx/dmp/$_isd_base$_isd_slice\$";;
	"/dev/vx/rdmp/"*s*) expr "$1" : "^/dev/vx/rdmp/$_isd_base$_isd_slice\$";;
	*s*)		expr "$1" : "^$_isd_base$_isd_slice";;
	*)		expr "$1" : "^$_isd_base$_isd_slice";;
	esac
}

chroot_to_vol()
{
	file=$SYSROOT/etc/system
	grep "vxvm_END" $file 1>&2 > /dev/null
	if [ $? -ne 0 ] ; then
		ugettxt -s ERROR \
		 "VxVM Information in /etc/system missing or corrupted." \
		 vxvmshm:986 >&2
		return 1
	fi

	# First, comment out any "old" rootdev specs
	comment_out_rootdev $file

	end_line=`grep -n vxvm_END $file`
	end_line=`expr "$end_line" : "\(.*\):.*"`
	end_line=`expr $end_line - 1`
	sed \
	-e ''$end_line' a\
rootdev:/pseudo/vxio@0:0\
set vxio:vol_rootdev_is_volume=1' $file > $file.sav
	mv $file.sav $file
	return 0
}

#
# This is used by some if the init.d scripts
#
set_pub_attr()
{

	#
	# Find the len and offset of the public region. 
	# If priv at start it is straightforward
	# else It should be the 
	# len of pub partition - len of priv partition - pub offset + priv
	# offset + priv offset because pub should enter the priv for the
	# ghost sd For an encapsulated root disk with priv carved out of swap,
	# the priv will be lying in the middle of pub. In this case return 
	# len of pub partition - pub offset
	#

	dogi_slice_rawpath $1 rawpath
	eval `vxparms 2>/dev/null`
	eval `vxtaginfo -s $rawpath $VOL_PUB_SLICE_TAG 2> /dev/null`
	pub_start=$PART_START
	pub_size=$PART_SIZE
	pub_end=`expr $pub_start + $pub_size`
	eval `vxtaginfo -s $rawpath $VOL_PRIV_SLICE_TAG 2> /dev/null`
	priv_start=$PART_START
	priv_size=$PART_SIZE
	eval `vxpartinfo -s $rawpath 2 2> /dev/null`
	full_start=$PART_START
	full_size=$PART_SIZE
	priv_end=`expr $priv_start + $priv_size`
	if [ $priv_start -eq 0 ] 
	then
		VOL_PUB_OFFSET=0
		VOL_PUB_LEN=$pub_size
	elif [ $priv_end -eq $full_size ] 
	then
		VOL_PUB_OFFSET=1
		if [ $pub_end -gt $priv_start ] ; then
			VOL_PUB_LEN=`expr $pub_size - $priv_size - \
			     $VOL_PUB_OFFSET + $VOL_PRIV_OFFSET`
		else
			VOL_PUB_LEN=`expr $pub_size - $VOL_PUB_OFFSET`
		fi
	else
		VOL_PUB_OFFSET=1
		VOL_PUB_LEN=`expr $pub_size - $VOL_PUB_OFFSET`
	fi
	export VOL_PUB_LEN VOL_PUB_OFFSET
}

#
# THESE ARE NOT USED
#

vol_add_slice()
{
	#
	# If device specified as cCtTdD then  add s2 
	#
	VOL_FULL_DISK=$1

	if vxcheckda $1
	then
		VOL_FULL_DISK=$VOL_FULL_DISK$VOL_FULL_SLICE
	fi
	export VOL_FULL_DISK
}

pipe_escape()
{
	echo $1 | sed -e 's/\|/\\|/g'
}
#
# add_cntrlr_expression -- add the given controller name to the
# expression to egrep out controller names. The second argument should
# be the variable name of the expression to be used.
#
add_cntrlr_expression()
{
	cntrlr=$1

	eval "expression=\${${2}}"

	if [ -z "$expression" ]
	then
		expression="${cntrlr}t"
	else
		expression="${expression}|${cntrlr}t"
	fi

	expression=`pipe_escape $expression`

	eval "${2}=$expression"
}

#
# add_device_expression -- add the given disk name to the
# expression to egrep out disk access names. The second argument should
# be the variable name of the expression to be used.
#
add_device_expression()
{
	dogi_whole_slice $1 device

	eval "expression=\${${2}}"

	if [ -z "$expression" ]
	then
		expression="${device}"
	else
		expression="${expression}|${device}"
	fi

	expression=`pipe_escape $expression`
	eval "${2}=$expression"
}

#
# add_enclr_expression -- add the given enclosure name to the
# expression to egrep out disk access names. The second argument should
# be the variable name of the expression to be used.
#
add_enclr_expression()
{
	enclr=$1

	eval "expression=\${${2}}"

	if [ -z "$expression" ]
	then
		expression="${enclr}_"
	else
		expression="${expression}|${enclr}_"
	fi

	expression=`pipe_escape $expression`
	eval "${2}=$expression"
}

create_enclr_expression()
{
	enclr=$1

	eval "expression=\${${2}}"

	if [ -z "$expression" ]
	then
		expression="${enclr}"
	else
		expression="${expression}|${enclr}"
	fi

	expression=`pipe_escape $expression`
	eval "${2}=$expression"
}

#
# extr_list_info -- extract information from a long listing of a device file.
#
extr_list_info()
{
	listing=$1
	diskname=$2

	cinfo=`expr "$listing" : '.* -> .*/devices/\(.*\)/[^/]*$'`
	cntrlr=`expr "$diskname" : "\(m\{0,1\}c[0-9][0-9]*\)t[0-9|A-F]*d[0-9][0-9]*${VOL_FULL_SLICE}"`

	eval "${3}=${cinfo}"
	eval "${4}=${cntrlr}"
}

extr_plninfo()
{
	info=$1
	ssa_id=`expr "$info" : '.*/SUNW,pln@[ab]\([0-9,a-fA-F]*\).*'`
	[ -n "$ssa_id" ] || return 1
	eval "${2}=${ssa_id}"
}

tchar="-"
twirl()
{
	/bin/echo "$tchar\b\c"
	case $tchar in
		"-") tchar="/";;
		"/") tchar="|";;
		"|") tchar="\\\\";;
		"\\\\") tchar="-";;
	esac
}

#
# controllers_on_system -- list all controllers on the system
#
controllers_on_system()
{
	xcmd cd $VOL_DISK_RAWPATH
	ls | egrep "c[0-9][0-9]*t?[0-9|A-F]*d[0-9][0-9]*${VOL_FULL_SLICE}" | \
	 while read disk
	 do
		 twirl >&2
		 cntr_info=`ls -l $disk`
		 case $cntr_info in
			 l*" -> "*)	: ;;
			 *)		: ;;
		 esac

		extr_list_info "$cntr_info" $disk cntr_info cntr
		list_member $cntr $ctl_temp && continue
		append_list ctl_temp $cntr

		if [ -f $VOL_CONFIG_DIR/vxvm.exclude ]
		then
			vxdmpadm checkexclude vxvm ctlr=$cntr:$cntr 2>/dev/null
			ret=$?
			if [ $ret -eq 0 -o $ret -eq 3 ]
			then
				cntr="$cntr" egettxt "<excluding $cntr>" vxvmshm:67 >&2
				continue
			fi
		fi

		if [ -f $CNTRL_EXCLUDE_FILE ]
		then
			if fgrep -x "$cntr" $VOL_CONFIG_DIR/cntrls.exclude \
			> /dev/null 2>&1
			then
				cntr="$cntr" egettxt "<excluding $cntr>" vxvmshm:67 >&2
				continue
		fi
		fi
		echo "$cntr $cntr_info"
	 done
}

#
# disks_on_controller -- given a controller number, list all disks
# that we can find on that controller to stdout.

disks_on_controller()
{
	cntrlr=$1

	(cd $VOL_DISK_RAWPATH;
	ls;
	) | egrep "m?${1}t?[0-9|A-F]*d[0-9][0-9]*${VOL_FULL_SLICE}"
}

#
# disks_on_enclosure -- given an enclosure name, list all disks
# that we can find on that enclosure to stdout.

disks_on_enclosure()
{
	enclr=$1
	vxdmpadm getdmpnode enclosure=$enclr | tail +3 | awk '{ print $1; }'
}

#
# dogi_node_to_path -- given a node name, return the
# subpath associated to the node

dogi_node_to_path()
{
	nodename=$1
	np=`vxdmpadm getsubpaths dmpnodename=$nodename 2> /dev/null | \
       	    awk '{if ($2=="ENABLED(A)") { path=$1; exit}} END {print path}'`
	if [ $# -gt 1 ]
	then
		eval "${2}=${np}";
	else
		echo "$np";
	fi
}

#
# expand_device_wildcard -- expand a disk specification into all
# matching devices. This is used in voladm_disk_device_list. It honors
# controller/device exclusion patterns.
#
expand_device_wildcard()
{
	list_var=$1
	eval "disklist=\${${list_var}}"
	excl_disks=$2
	excl_cntrlr=$3
	excl_enclr=$4
	disk_excluded_var=$5
	eval "disk_excl=\${$disk_excluded_var}"
	cntrlr_excluded_var=$6
	eval "cntrlr_excl=\${$cntrlr_excluded_var}"
	enclr_excluded_var=$7
	eval "enclr_excl=\${$enclr_excluded_var}"

	shift 7

	# The string "${VOL_FULL_SLICE}\$" must be appended to each disk 
	# device string in the egrep pattern list to ensure that only the 
	# specified disk(s) is filtered out.
	if [ -n "$excl_disks" ]
	then

           excl_disks=`echo $excl_disks | sed "s/\|/${VOL_FULL_SLICE}\$\|/g" | \
				sed "s/$/${VOL_FULL_SLICE}\$/"`

	fi


	if list_member allc $@
	then
		ewritemsg -s ERROR -M vxvmshm:767 "Input not recognized: \"allc\""
		return 1
	elif list_member allm $@
	then
		ewritemsg -s ERROR -M vxvmshm:768 "Input not recognized: \"allm\""
		return 1
	fi

	if list_member all $@
	then
		if [ $# -gt 1 ]
		then
			ewritemsg -s ERROR -M vxvmshm:63 \
			 "'all' should appear alone, enter ?? for help"
			return 1
		fi
		set -- allc allm allenclr
	fi

	while [ $# -gt 0 ]
	do
		spec=$1
		pattern=

		# Verify that it's valid
		slen=`strlen $spec`
		ldir=
		case "$spec" in
			*_)
			    mlen=`expr $spec : '.*_'`
			    [ $mlen -eq $slen ] && pattern="$spec*"
			    ldir=$VOL_DISK_RAWPATH
			    ;;
			*_*)
			    mlen=`expr $spec : '.*_[0-9][0-9]*'`
			    [ $mlen -eq $slen ] && pattern="$spec"
			    ldir=$VOL_DISK_RAWPATH
			    ;;
			c[0-9]*d[0-9]*)
			    mlen=`expr $spec : \
			     'c[0-9][0-9]*t\[0,1\}[0-9]*d[0-9][0-9]*'`
			    [ $mlen -eq $slen ] && pattern="$spec"
			    ldir=$VOL_DISK_RAWPATH
				if [ -z "$pattern" ]
				then
			    	mlen=`expr $spec : \
			     	'c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*'`
			    	[ $mlen -eq $slen ] && pattern="$spec"
				fi
			    ;;
			mc[0-9]*d[0-9]*)
			    mlen=`expr $spec : \
			     'mc[0-9][0-9]*t\{0,1\}[0-9]*d[0-9][0-9]*'`
			    [ $mlen -eq $slen ] && pattern="$spec"
			    ldir=$VOL_AP_DISK_RAWPATH
			    ;;

			c[0-9]*t[0-9]*)
			    mlen=`expr $spec : \
			     'c[0-9][0-9]*t\{0,1\}[0-9]*'`
			    [ $mlen -eq $slen ] && pattern="${spec}d*"
			    ldir=$VOL_DISK_RAWPATH
			    ;;
			mc[0-9]*t[0-9]*)
			    mlen=`expr $spec : \
			     'mc[0-9][0-9]*t\{0,1\}[0-9]*'`
			    [ $mlen -eq $slen ] && pattern="${spec}d*"
			    ldir=$VOL_AP_DISK_RAWPATH
			    ;;

			c[0-9]*)
			    mlen=`expr $spec : \
			     'c[0-9][0-9]*'`
			    [ $mlen -eq $slen ] && pattern="${spec}t*"
			    ldir=$VOL_DISK_RAWPATH
			    ;;
			mc[0-9]*)
			    mlen=`expr $spec : \
			     'mc[0-9][0-9]*'`
			    [ $mlen -eq $slen ] && pattern="${spec}t*"
			    ldir=$VOL_AP_DISK_RAWPATH
			    ;;
			allc)
			    pattern="c*"
			    ldir=$VOL_DISK_RAWPATH
			    ;;
			allm)
			    pattern="mc*"
			    ldir=$VOL_AP_DISK_RAWPATH
			    ;;
			allenclr)
			    pattern="*_[0-9]*"
			    ldir=$VOL_DISK_RAWPATH
			    ;;
			*) pattern="$spec"
			   ldir=`dirname \`vxdisk list $spec 2>/dev/null|grep "^pubpaths" |awk '{ print $3 }'|cut -d= -f2\``
			   ;;
		esac

		if [ -z "$pattern" ]
		then
			export spec; ewritemsg -s ERROR -M vxvmshm:766 \
"Input not recognized: \"$spec\""
			return 1
		fi

		case "$spec" in
			*_ | *_* | c[0-9]*d[0-9]* | mc[0-9]*d[0-9]* | c[0-9]*t[0-9|A-F]* | mc[0-9]*t[0-9]* | c[0-9]* | mc[0-9]* | allc | allm | allenclr)
				push_list disklist \
		 			`ls -1 ${ldir}/${pattern}${VOL_FULL_SLICE} 2>/dev/null | \
					tee $_vdskd_rawmatch |
					if [ -n "$excl_cntrlr" ]
					then
						egrep -v -e "$excl_cntrlr"
					else
						cat
					fi |
					tee $_vdskd_cntrlmatch |
					if [ -n "$excl_disks" ]
					then
						egrep -v -e "$excl_disks"
					else
						cat
					fi |
					tee $_vdskd_diskmatch |
					if [ -n "$excl_enclr" ]
					then
						egrep -v -e "$excl_enclr"
					else
						cat
					fi |
					tee $_vdskd_enclrmatch |
					sed "s%${ldir}/\(.*\)${VOL_FULL_SLICE}$%\1%"`
				;;
		 	*) 
				push_list disklist \
		 			`ls -1 ${ldir}/${pattern} 2>/dev/null | \
					tee $_vdskd_rawmatch |
					if [ -n "$excl_cntrlr" ]
					then
						egrep -v -e "$excl_cntrlr"
					else
						cat
					fi |
					tee $_vdskd_cntrlmatch |
					if [ -n "$excl_disks" ]
					then
						egrep -v -e "$excl_disks"
					else
						cat
					fi |
					tee $_vdskd_diskmatch |
					if [ -n "$excl_enclr" ]
					then
						egrep -v -e "$excl_enclr"
					else
						cat
					fi |
					tee $_vdskd_enclrmatch |
					sed "s%${ldir}/\(.*\)%\1%"`
				;;
		esac	

		# Add excluded controllers and disks to the lists
		append_list $cntrlr_excluded_var \
		 `comm -3 $_vdskd_rawmatch $_vdskd_cntrlmatch`
		append_list $disk_excluded_var \
		 `comm -3 $_vdskd_cntrlmatch $_vdskd_diskmatch`
		append_list $enclr_excluded_var \
		 `comm -3 $_vdskd_diskmatch $_vdskd_enclrmatch`

		shift
	done

	vx_really_in_config=
	for i in $disklist
	do
		doit vxdisk -s list $i >/dev/null 2>&1
		if [ $? -eq 0 ]
		then
			append_list vx_really_in_config $i
		fi
	done
	disklist=$vx_really_in_config

	eval "$list_var=\"$disklist\""
	return 0
}

extr_devicepath()
{
	wholepath=$1

	devicepath=`expr "$wholepath" : '.*\(/devices/.*\)'`

	eval "${2}=${devicepath}"
}

# usage: subdisk_to_partition diskgroup volume dm_name
# if an sd takes up a whole partition, return the blkpath to the partition.

subdisk_to_partition()
{
	_ckdg=$1
	_ckvol=$2
	_fdumpdm=$3
	[ -n "$_ckdg" -a -n "$_ckvol" -a -n "$_fdumpdm" ] || return

	vxprint -g $_ckdg -se \
		"sd_pl_offset=0 && sd_dmname=\"$_fdumpdm\" &&
		 assoc.assoc=\"$_ckvol\" && len >= assoc.assoc.len" \
		-F "%path %dev_offset %len" |
	{
	    found=
	    while [ -z "$found" ] && read path devoffset sdlen
	    do
		[ "X$path" != "X-" -a "X$devoffset" != "X-" ] || continue

		# get the diskoffset of the subdisk
		dogi_path_to_slice $path base_slice

		[ -n "$base_slice" ] || continue
		dogi_slice_rawpath $base_slice base_rawpath
		STAT_PSTARTSEC=
		STAT_DISK_TYPE=
		eval "`vxparms -s $base_rawpath 2> /dev/null`"
		[ -n "$STAT_PSTARTSEC" ] || continue
		[ "X$STAT_DISK_TYPE" = Xdisk ] || continue
		diskoffset=`expr $devoffset + $STAT_PSTARTSEC`

		# locate a partition that matches the beginning of that
		# subdisk, and that is not longer than the subdisk
		dogi_slice_to_device "$base_slice" base_device
		old_device=$base_device
		path=`vxdisk -g $_ckdg list $base_device 2>/dev/null | \
			egrep "^m?c[0-9]+t[0-9|A-F]+d[0-9]+" | \
			awk '$2 == "state=enabled" {print $1}'`
		path=`echo $path`
		if [ "X$path" != "X" ]
		then
			base_device=`echo $path | cut -d "s" -f 1`
		fi

		if [ -z "$base_device" ]
		then
			base_device=$old_device
		fi
		for slice_num in 1 0 2 3 4 5 6 7 c
		do
			dogi_device_slice $base_device $slice_num try_slice 
			dogi_slice_rawpath $try_slice try_rawpath
			STAT_PSTARTSEC=
			STAT_PNUMSEC=
			eval "`vxparms -s $try_rawpath 2>/dev/null`"
			[ -n "$STAT_PSTARTSEC" -a -n "$STAT_PNUMSEC" ] || continue
			[ $STAT_PSTARTSEC -eq $diskoffset ] || continue
			[ $STAT_PNUMSEC -le $sdlen ] || continue
			[ -n "$base_device" ] || continue

			# set the dump device
			dogi_slice_blkpath $try_slice dumpdev
			found=yes
			echo $dumpdev
			break
		done
	    done
	}
}


#
# These are the dogi_* routines used by the rest of the administrative
# scripts. These should mask out any system-specific DA-naming or
# disk-related dependencies.
#

#
# dogi_name_is_device -- returns true if the (single) argument is
# in the form of a device name.
#
dogi_name_is_device()
{
	#
	# return true if the name is a valid device name
	#
	name=`basename $1`
	case "$name" in
		*_*)
		     disknum=`expr $name : '.*_\([0-9][0-9]*\)'`
		     last=`expr $name : '.*_\(.*\)'`
		     [ "$disknum" = "$last" ]
		     ;;
		mc*)
		     name_len=`strlen $name`;
		     match_len=`expr $name : \
		      'mc[0-9][0-9]*t\{0,1\}[0-9]*d[0-9][0-9]*'`
		     [ "$name_len" -eq "$match_len" ]
		     ;;
		c*)
		    name_len=`strlen $name`;
		    match_len=`expr $name : \
		     'c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*'`
		    match_len2=`expr $name : \
		     'c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*'`
		    [ "$name_len" -eq "$match_len" ] || \
			[ "$name_len" -eq "$match_len2" ]
		    ;;
		vpath*)
		    name_len=`strlen $name`;
		    match_len=`expr $name : \
		    'vpath[0-9][0-9]*'`
		    [ "$name_len" -eq "$match_len" ]
		    ;;
		
                 emc*)
  
                   dmpnode=`vxdmpadm getdmpnode nodename=$name 2> /dev/null | \
                        tail +3 | awk '{print $1}'`
                   if [ -z "$dmpnode" ]
                   then
                        false
                   else
                        vxdisk list "$dmpnode" > /dev/null 2>&1
                        status=$?
                        if [ "$status" -ne 0 ]
                        then
                            vxdisk scandisks
                        fi
                        devicetag=`vxdisk list $dmpnode 2> /dev/null | \
                               grep '^devicetag' | sed 's/ //g' | \
                               awk -F: '{print $2}'`
                         name_len=`strlen $dmpnode`;
                         match_len=`expr $dmpnode : \
                           'c[0-9][0-9]*t\{0,1\}[0-9]*d[0-9][0-9]*'`
                         match_len2=`expr $dmpnode : \
                           'c[0-9][0-9]*t\{0,1\}[0-9|A-F]*d[0-9][0-9]*'`
                         match_len3=`expr $dmpnode : ${devicetag}'s[0-9][0-9]*'`
                         [ "$name_len" -eq "$match_len" ] || \
                         [ "$name_len" -eq "$match_len2" ] || \
			 [ "$name_len" -eq "$match_len3" ] 
                     fi
                   ;;

		*)
		   dmpnode=`vxdmpadm getdmpnode nodename=$name 2> /dev/null | \
			tail +3 | awk '{print $1}'`
		   if [ -z "$dmpnode" ]
		   then
			false
		   else
			vxdisk list "$dmpnode" > /dev/null 2>&1
			status=$?
			if [ "$status" -ne 0 ]
			then
				vxdisk scandisks
			fi
			devicetag=`vxdisk list $dmpnode 2> /dev/null | \
				grep '^devicetag' | sed 's/ //g' | \
				awk -F: '{print $2}'`
			name_len=`strlen $name`
			match_len=`expr $name : ${devicetag}'.*'`
			[ "$name_len" -eq "$match_len" ]
		   fi
		   ;;
	esac
}

#
# dogi_name_is_slice -- returns true if the (single) argument is in
# the form of a slice name.
#
dogi_name_is_slice()
{
	#
	# return true if the name looks like a slice of a disk
	#
	name=`basename $1`
	case "$name" in
		*_*)
		     name_len=`strlen $name`;
		     match_len1=`expr $name : \
		      '.*_[0-9][0-9]*'`
		     match_len2=`expr $name : \
		      '.*_[0-9][0-9]*s[0-9][0-9]*'`
		     [ "$name_len" -eq "$match_len1" ] || \
			[ "$name_len" -eq "$match_len2" ]
		     ;;
		mc*)
		     name_len=`strlen $name`;
		     match_len=`expr $name : \
		      'mc[0-9][0-9]*t\{0,1\}[0-9]*d[0-9][0-9]*s[0-9][0-9]*'`
		     [ "$name_len" -eq "$match_len" ]
		     ;;
		c*)
		    name_len=`strlen $name`;
		    if not dogi_is_efi_wd $name
		    then 
		    	match_len=`expr $name : \
		     	  'c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*s[0-9][0-9]*'`
			match_len2=`expr $name : \
			  'c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*s[0-9][0-9]*'`
		    else
		    	match_len=`expr $name : \
		     	'c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*'`
			match_len2=`expr $name : \
			  'c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*'`
		    fi
		    [ "$name_len" -eq "$match_len" ] || \
			[ "$name_len" -eq "$match_len2" ]
			
		    ;;

                emc*)
                   dmpnode=`vxdmpadm getdmpnode nodename=$name 2> /dev/null | \
                        tail +3 | awk '{print $1}'`
                   if [ -z "$dmpnode" ]
                   then
                        false
                   else
                        vxdisk list "$dmpnode" > /dev/null 2>&1
                        status=$?
                        if [ "$status" -ne 0 ]
                        then
                                vxdisk scandisks
                        fi
                   fi
 
                    devicetag=`vxdisk list $dmpnode | grep '^devicetag' | \
                                sed 's/ //g' | awk -F: '{print $2}'`
                    name_len=`strlen $dmpnode`;
                    if not dogi_is_efi_wd $dmpnode
                    then
                        match_len=`expr $dmpnode : \
                          'c[0-9][0-9]*t\{0,1\}[0-9]*d[0-9][0-9]*s[0-9][0-9]*'`
                        match_len2=`expr $dmpnode : \
                          'c[0-9][0-9]*t\{0,1\}[0-9|A-F]*d[0-9][0-9]*s[0-9][0-9]*'`
                        match_len3=`expr $dmpnode : ${devicetag}'s[0-9][0-9]*'`
 
                    else
                        match_len=`expr $dmpnode : \
                        'c[0-9][0-9]*t\{0,1\}[0-9]*d[0-9][0-9]*'`
                        match_len2=`expr $dmpnode : \
                          'c[0-9][0-9]*t\{0,1\}[0-9|A-F]*d[0-9][0-9]*'`
                        match_len3=`expr $dmpnode : ${devicetag}'s[0-9][0-9]*'`
                   fi
                    [ "$name_len" -eq "$match_len" ] || \
                    [ "$name_len" -eq "$match_len2" ]|| \
                    [ "$name_len" -eq "$match_len3" ]                   
                ;;
		vpath*)
		    name_len=`strlen $name`;
		    match_len=`expr $name : \
		     'vpath[0-9][0-9]*c'`
		    [ "$name_len" -eq "$match_len" ]
		    ;;
		*)
		   dmpnode=`vxdmpadm getdmpnode nodename=$name 2> /dev/null | \
			tail +3 | awk '{print $1}'`
		   if [ -z "$dmpnode" ]
		   then
			false
		   else
			vxdisk list "$dmpnode" > /dev/null 2>&1
			status=$?
			if [ "$status" -ne 0 ]
			then
				vxdisk scandisks
			fi
			devicetag=`vxdisk list $dmpnode | grep '^devicetag' | \
				sed 's/ //g' | awk -F: '{print $2}'`
			name_len=`strlen $name`
			if not dogi_is_efi_wd $name
			then
				match_len=`expr $name : ${devicetag}'s[0-9][0-9]*'`
			else
				match_len=`expr $name : ${devicetag}`
			fi
			[ "$name_len" -eq "$match_len" ]
		   fi
		   ;;
	esac
}

#
# dogi_slice_num -- echo the slice part of the complete slice name
#
#	If optional second argument is provided, will set a variable by
#	that name to the slice portion of the first argument
#
dogi_slice_num()
{
	name=`basename $1`
	case "$name" in
		*_*)
		     s=`expr $name : '.*_[0-9][0-9]*\(s[0-9][0-9]*\)'`
		     [ -z "$s" ] && { s=s2; }
		      ;;
		mc*)
		     s=`expr $name : \
		      'mc[0-9][0-9]*t\{0,1\}[0-9]*d[0-9][0-9]*\(s[0-9][0-9]*\)'`
		     ;;
		c*)
		    s=`expr $name : \
		     'c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*\(s[0-9][0-9]*\)'`
		    if [ X = X"$s" ]
		    then
		    	s=`expr $name : \
			     'c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*\(s[0-9][0-9]*\)'`
		    fi
		    ;;
		vpath*)
		    s=`expr $name : \
		     'vpath[0-9][0-9]*\(c\)'`
		    [ -z "$s" ] && { s=c; }
		    ;;
		*)
		   s="";;
	esac
	if [ $# -gt 1 ]
	then
		eval "${2}=$s"
	else
		echo $s
	fi
}

#
# dogi_is_whole_slice -- return true if the slice names the special "whole"
# slice for the disk
#
dogi_is_whole_slice()
{
	if not dogi_name_is_slice "$1"
	then
		return 1
	fi

	dogi_slice_num $1 slice_name
	[ "$slice_name" = "${VOL_FULL_SLICE}" ] || [ "$slice_name" = "${VOL_FULL_VSLICE}" ]
}

#
# dogi_device_slice -- create a slice name out of the given device
# name and slice
dogi_device_slice()
{
	eval "${3}=${1}s${2}"
}

#
# dogi_slice_to_device -- given a slice name, return the device it names
#
#
dogi_slice_to_device()
{
	name=`basename $1`
	if not dogi_name_is_slice $name
	then
		eval "${2}=$1"
		return 1
	fi

	case "$name" in
		*_*) d=`expr $name : \
		      '\(.*_[0-9][0-9]*\)'`
		     ;;
		mc*)
		     d=`expr $name : \
		      '\(mc[0-9][0-9]*t\{0,1\}[0-9]*d[0-9][0-9]*\)s[0-9][0-9]*'`
		     ;;
		c*)
			if not dogi_is_efi_wd $name
			then
		    		d=`expr $name : \
		     		'\(c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*\)s[0-9][0-9]*'`
				if [ X = X"$d" ]
				then
					d=`expr $name : \
					    '\(c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*\)s[0-9][0-9]*'`
				fi
			else
				d=`expr $name : \
				'\(c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*\)'`
				if [ X = X"$d" ]
				then
					d=`expr $name : \
					'\(c[0-9][0-9]*t[0-9|A-F]*d[0-9][0-9]*\)'`
				fi
			fi
		    ;;
		vpath*)
		    d=`expr $name : \
		     '\(vpath[0-9][0-9]*\)c'`
		    ;;
		*)
		   dmpnode=`vxdmpadm getdmpnode nodename=$name 2> /dev/null | \
			tail +3 | awk '{print $1}'`
		   d=""
		   if [ ! -z "$dmpnode" ]
		   then
			vxdisk list "$dmpnode" > /dev/null 2>&1
			status=$?
			if [ "$status" -ne 0 ]
			then
				vxdisk scandisks
			fi
			devicetag=`vxdisk list $dmpnode | grep '^devicetag' | \
				sed 's/ //g' | awk -F: '{print $2}'`
			d=`expr $name : \
				'\('${devicetag}'\)s[0-9][0-9]*'`
		   fi
		   ;;
	esac

	if [ $# -gt 1 ]
	then
		eval "${2}=$d"
	else
		echo $d
	fi
}

#
# dogi_name_to_device -- return the device name, given a name
#
dogi_name_to_device()
{
	name=`basename $1`
	if dogi_name_is_device $name
	then
		d=$name
	else
		dogi_slice_to_device $name d
	fi

	if [ $# -gt 1 ]
	then
		eval "${2}=$d"
	else
		echo $d
	fi
}

#
# dogi_slice_from_path -- given a disk pathname, strip off everything
# but the slice name.
#
dogi_path_to_slice()
{
	d=
	case $1 in
		"/dev/dsk/"*s*) d=`expr "$1" : '^/dev/dsk/\(.*s.\)\$'`;;
		"/dev/rdsk/"*s*) d=`expr "$1" : '^/dev/rdsk/\(.*s.\)\$'`;;
		"/dev/vx/dmp/"*s*) d=`expr "$1" : '^/dev/vx/dmp/\(.*s.\)\$'`;;
		"/dev/vx/rdmp/"*s*) d=`expr "$1" : '^/dev/vx/rdmp/\(.*s.\)\$'`;;
		"/dev/ap/dsk/"*s*)
			d=`expr "$1" : '^/dev/ap/dsk/\(.*s.\)\$'`;;
		"/dev/ap/rdsk/"*s*)
			d=`expr "$1" : '^/dev/ap/rdsk/\(.*s.\)\$'`;;
	esac

	if [ $# -gt 1 ]
	then
		eval "${2}=$d"
	else
		echo $d
	fi
}

dogi_is_efi_wd()
{
	name=`basename $1`
	if not dogi_name_is_device $name
	then
		return 1
	fi

	path="${VOL_DISK_RAWPATH}/${name}"
	STAT_EFI=
	STAT_SDI_NAME=
	eval `vxparms -s $path 2>/dev/null`
	if [ -z "$STAT_EFI" -a -z "$STAT_SDI_NAME" ]
	then
		return 1
	elif [ "X$STAT_EFI" != "X$STAT_SDI_NAME" ]
	then
		return 1
	fi
	return 0
}
#
# dogi_whole_slice -- given a device, return the slice name of the
# "whole slice"
#
# This is not necessarily a real DOGI interface function; it's a shortcut
# for solaris since the whole path is the device manipulation interface (i.e.,
# for prtvtoc, disk ioctls, etc.)
#
dogi_whole_slice()
{
	name=`basename $1`
	if not dogi_name_is_device $name
	then
		return 1
	fi

	case $name in
		*_*)
			p="${name}";;
		c*)
			if not dogi_is_efi_wd $name
			then
				p="${name}${VOL_FULL_SLICE}"
			else
				p=${name}
			fi;;
		mc*)
			p="${name}${VOL_FULL_SLICE}";;
		vpath*)
			p="${name}${VOL_FULL_VSLICE}";;
		*)	
			if not dogi_is_efi_wd $name
			then
				p="${name}${VOL_FULL_SLICE}"
			else
				p=${name}
			fi;;
	esac

	if [ $# -gt 1 ]
	then
		eval "${2}=${p}"
	else
		echo $p
	fi
}

#
# dogi_device_vxpath -- return the path from the dmp directory to the 
# character node for the device
#

dogi_device_vxpath()
{
	if not dogi_name_is_device $1
	then
		return 1
	fi
	
	dogi_whole_slice "$1" w || return 1
	dogi_slice_vxpath $w pth

	if [ $# -gt 1 ]
	then
		eval "${2}=${pth}";
	else
		echo "$pth";
	fi
}

#
# dogi_device_rawpath -- return the path to the character node for the device
#

dogi_device_rawpath()
{
	if not dogi_name_is_device $1
	then
		return 1
	fi
	
	dogi_whole_slice "$1" w || return 1
	dogi_slice_rawpath $w pth

	if [ $# -gt 1 ]
	then
		eval "${2}=${pth}";
	else
		echo "$pth";
	fi
}

dogi_slice_rawpath()
{
	name=`basename $1`

	if not dogi_name_is_slice $name
	then
		return 1
	fi

	case $name in
		*_*)
		     p="${VOL_DISK_RAWPATH}/${name}";;
		mc*)
		     p="${VOL_AP_DISK_RAWPATH}/${name}";;
		c*)
		     p="${DISK_RAWPATH}/${name}";;
		*)
		     p="${VOL_DISK_RAWPATH}/${name}";;
	esac

	if [ $# -gt 1 ]
	then
		eval "${2}=${p}";
	else
		echo "$p";
	fi
}

dogi_slice_vxpath()
{
	name=`basename $1`

	if not dogi_name_is_slice $name
	then
		return 1
	fi

	case $name in
		*_*)
		     p="${VOL_DISK_RAWPATH}/${name}";;
		mc*)
		     p="${VOL_AP_DISK_RAWPATH}/${name}";;
		c*)
		     p="${VOL_DISK_RAWPATH}/${name}";;
		*)
		     p="${VOL_DISK_RAWPATH}/${name}";;
	esac

	if [ $# -gt 1 ]
	then
		eval "${2}=${p}";
	else
		echo "$p";
	fi
}

#
# dogi_device_blkpath -- return the path to the block node for the device
#
dogi_device_blkpath()
{
	if not dogi_name_is_device $1
	then
		return 1
	fi
	
	dogi_whole_slice "$1" w || return 1
	dogi_slice_blkpath $w pth
	if [ $# -gt 1 ]
	then
		eval "${2}=${pth}"
	else
		echo "$pth"
	fi
}


dogi_slice_blkpath()
{
	name=`basename $1`

	if not dogi_name_is_slice $name
	then
		return 1
	fi

	case $name in
		*_*)
		    p="${VOL_DISK_BLKPATH}/${name}";;
		mc*)
		    p="${VOL_AP_DISK_BLKPATH}/${name}";;
		c*)
		    p="${DISK_BLKPATH}/${name}";;
		*) 
		    p="${VOL_DISK_BLKPATH}/${name}";;
	esac

	if [ $# -gt 1 ]
	then
		eval "${2}=${p}";
	else
		echo "$p";
	fi
}

#
# dogi_name_daname -- return an appropriate DA record name for the
# given device name
#
dogi_name_daname()
{
	name=`basename $1`

	if not dogi_name_is_device $name
	then
		return 1
	fi

	dogi_whole_slice "$name" daname || return 1

	if [ $# -gt 1 ]
	then
		eval "${2}=${daname}"
	else
		echo "$daname"
	fi
}

#
# dogi_root_device -- return true if the named device is the boot disk
#
dogi_root_device()
{
	dname=$1
	if not dogi_name_is_device $dname
	then
		return 1
	fi
	
	set_rootdisk
	[ "$dname" = "$rootdisk" ]
}

#
# dogi_name_is_cntrlr -- returns true if the argument is in the form
# of a controller name.
#
dogi_name_is_cntrlr()
{
	name=`basename $1`
	case "$name" in
		mc*)
		     name_len=`strlen $name`;
		     match_len=`expr $name : \
		      'mc[0-9][0-9]*'`
		     [ "$name_len" -eq "$match_len" ]
		     ;;
		c*)
		    name_len=`strlen $name`;
		    match_len=`expr $name : \
		     'c[0-9][0-9]*'`
		    [ "$name_len" -eq "$match_len" ]
		    ;;
		pseudo*)
		    name_len=`strlen $name`;
		    match_len=`expr $name : \
		     'pseudo*'`
		    [ "$name_len" -eq "$match_len" ]
		    ;;
		*) false;;
	esac
}

#
# dogi_darecs_on_device -- list any DA records that are defined on
# the named device. The form of the list is "<da> <tm> <type>". The
# list is put into the variable named by the second argument. If any
# are found, return true.
#
dogi_darecs_on_device()
{
	> $tmpfile3
	name=`basename $1`
	if [ `echo "$name" | grep _` ]; then
		env LANG=C vxdisk list | grep "^\<${name}\>" > $tmpfile4
	else
		env LANG=C vxdisk list | grep "^${name}s" > $tmpfile4
	fi

	xlist=`cat $tmpfile4`
	rm $tmpfile4

	if [ $# -gt 1 ]
	then
		eval "${2}=\"${xlist}\""
	else
		if [ "$xlist" != "" ]
		then
			echo "$xlist"
		fi
	fi
	[ `list_count $xlist` -ne 0 ]
}

#
# dogi_valid_vtoc -- return false if the disk does not have a valid
# table of contents.
#
dogi_valid_vtoc()
{
	name=`basename $1`
	if dogi_name_is_slice $name
	then
		dogi_slice_to_device "$name" name
	elif not dogi_name_is_device $name
	then
		return 1
	fi

	dogi_device_rawpath $name diskpath

	#
	# Get the vtoc - if error then return immediately
	#
	# Return 0 if there is a valid vtoc, else return 1.
	# There is a valid vtoc if the flags value indicates
	# there is a mountable partition, or there is a VM private region.
	#

	$PRTVTOC -f /dev/stdout $diskpath | egrep "0x200|0x210" > /dev/null 2>&1
	[ $? -eq 0 ] && return 0
	$PRTVTOC -f /dev/stdout $diskpath | egrep "0xf" > /dev/null 2>&1
	[ $? -eq 0 ] && return 0
	return 1
}

#
# dogi_is_boot_device -- return true if the named device or disk media 
#                      record is the boot disk
#                      disk names are in the cNtNdN form
#
dogi_is_boot_device()
{
      dname=`basename $1`
      dgname=$2
      get_mountdevice / root

      #
      # If the aegument is a real disk device then check 
      # to see if it matches the root_disk var set by the
      # get_mountdevice call
      #
      if dogi_name_is_device $dname
      then
              [ "$dname" = "$rootdisk" ] && return 1
      fi

      #
      # The argument must be a dm name
      #
      dm=$dname

      dmlist=`vxprint -g $dgname -F %dm -e as.as=\"$root_vol\" | sort -u`

      for bootdm in $dmlist
      do
              [ "$bootdm" = "$dm" ] && return 1
      done

      return 0
}
#
# dogi_get_prompath -- For systems that support setting their eeproms,
# determine correct path independent of disk name spaces.  Given a
# slice name for the new root device return the appropriate block path
# name.  
#
dogi_get_prompath()
{
	name=`basename $1`

	if not dogi_name_is_slice $name
	then
		return 1
	fi

	# 
	# Return the block path for slices that are NOT from AP
	# metadisks.  
	#
	# Otherwise, here's the deal with AP metadisk devices:  What
	# the eeprom needs is the raw disk path name of the active
	# alternate path.
	#
	# EXACT METHOD: Get alternate path group for metadisk and 
	# then determine which of the two is the active alternate.
	# 
	# GUESS METHOD: AP metadisks map to at most two alternate 
	# paths and the naming is coordinated:  e.g., mc3t4d0 refers to
	# alternate path c3t4d0 and cXt4d0 where "X" is unknown at this
	# point. So, just strip the 'm' off and you have at least one
	# alternate path.
	#

	case $name in
		*_*)
			if [ -f /usr/sbin/vxdmpadm ]
			then
				path=`/usr/sbin/vxdmpadm getsubpaths dmpnodename=$name 2>/dev/null | tail +3 | grep ENABLED | head -1 | awk '{print $1}'`
				if [ -z $path ]
				then
					return 1
				fi
				dogi_slice_to_device "$path" path_device
				dogi_device_slice $path_device 0 path
				dogi_slice_blkpath $path blkpath
			else
				return 1
			fi
			;;
		mc*)
			#
			# EXACT METHOD -- not implemented
			#
			# dogi_get_ap $name ap1 ap2
			# One of these two should be the primary ap
			# if dogi_is_ap_primary $ap1
			# then
			# ap_primary=$ap1
			# else if dogi_is_ap_primary $ap2
			# then
			# ap_primary=$ap2
			# else
			# return 1        
			# fi

			#
			# GUESS METHOD
			#
			ap_primary=`expr $name : '^m\(.*\)'`

			# Get the block node for this primary ap
			dogi_slice_blkpath $ap_primary blkpath
			;;
		c*)
			dogi_slice_blkpath $name blkpath
			;;
		*)
			return 1
			;;
	esac

	# Return information to caller
	if [ $# -gt 1 ]
	then
		eval "${2}=${blkpath}"
	else
		echo "$blkpath"
	fi
}
#
# voladm_get_disk_layout_defaults -- get entries from a given defaults file
#
# SYNTAX: voladm_get_disk_layout_defaults which-file which-attr ret-val
#		where
#			which-file = vxdisk | vxencap
#			which-attr = "" | format | privlen
#				"" => all
#
# RETURNS: 0 on success
#	   1 on failure
#
#	NOTE: success might not return any value since the existence of
#		the file and any particular attribute is optional; that is,
#		upon success ret-val may be ""
#
voladm_get_disk_layout_defaults()
{
        if [ $# -le 2 ]
        then
                echo "voladm_get_disk_layout_defaults: missing arguments"
                return 1
        fi
        if [ "$1" = "vxdisk" ]
	then
                _file=$VOL_VXDISK_DFLT_FILE

        elif [ "$1" = "vxencap" ]
	then
                _file=$VOL_VXENCAP_DFLT_FILE
        else
                eval "${3}="

                echo "voladm_get_disk_layout_defaults: bad first argument: $1"
                return 1
        fi
        #
        # Get default format and/or privlen from the default file if it exists.
        # If there is more than one entry in the file, return the last one.
        #
        if [ ! -f $_file ]
        then
                eval "${3}="
		return 0
        fi
	#
	# Parse the file.
	#
	_privlen=
	_format=
	_valid_entries=0
	_def=`egrep '^privlen=[0-9][0-9]*$' $_file | tail -1`
	[ -n "$_def" ] && {
		eval _$_def

		if [ -z "$_privlen" -o $_privlen -lt 1 -o $_privlen -gt 524288 ]
		then
			export _file _privlen; ewritemsg -s ERROR -M vxvmshm:2572 \
	"Private region length out of range in $_file: privlen=$_privlen."
			return 1
		fi
		_valid_entries=`expr $_valid_entries + 1`
	}
	_def=`egrep '^format=[^ 	#][^ 	#]*$' $_file | tail -1`
	[ -n "$_def" ] && {
		eval _$_def

		if [ "$_format" = "cdsdisk" ] ||
		   [ "$_format" = "sliced" ] ||
		   [ "$_format" = "simple" -a "$1" != "vxencap" ]
		then
			_valid_entries=`expr $_valid_entries + 1`
		else
			export _file _format; ewritemsg -s ERROR -M vxvmshm:2598 \
				"Syntax error in $_file: format=$_format."
			return 1
		fi
	}
	_entries=`grep -v "^#" $_file | wc -l | awk '{print $1}'`

	if [ "$_valid_entries" != "$_entries" ]
	then
		export _file; ewritemsg -s ERROR -M vxvmshm:2027 \
			"Syntax error or duplicate entries in $_file."
		return 1
	fi
	#	
	# Set the requested info.
	#
	if [ -z "$2" ]
	then
        	[ -n "$_privlen" -a -n "$_format" ] && _privlen="$_privlen "

        	eval "${3}=${_privlen}${_format}"

 	elif [ "$2" = "privlen" ]
	then
        	eval "${3}=${_privlen}"

	elif [ -z "$2" -o "$2" = "format" ]
	then
        	eval "${3}=${_format}"
	else
                eval "${3}="

                echo "voladm_get_disk_layout_defaults: bad second argument: $2"
                return 1
	fi
        return 0
}
#
# voladm_set_disk_layout_default -- set an entry in a given defaults file
#
# SYNTAX: voladm_set_disk_layout_defaults which-file which-attr val
#		where
#			which-file = vxdisk | vxencap
#
# RETURNS: None
#
voladm_set_disk_layout_default()
{
        if [ $# -le 2 ]
        then
                echo "voladm_set_disk_layout_default: missing argument"
                return 1
        fi
        if [ "$1" = "vxdisk" ]
	then
                _file=$VOL_VXDISK_DFLT_FILE

        elif [ "$1" = "vxencap" ]
	then
                _file=$VOL_VXENCAP_DFLT_FILE
        else
                echo "voladm_set_disk_layout_default: bad argument"
                return 1
        fi
	_dir=`dirname $_file`
	[ ! -d $_dir ] && mkdir -p $_dir

        _attr=$2
        _val=$3
        #
        # remove any existing lines with the specified attribute and then
        # write a line of the form "attribute=value" to the file.
        #
        _tmp_file=${_file}.$$

        [ -f $_file ] && grep -v "$_attr" $_file > $_tmp_file 2>/dev/null

        [ -n "$_val" ] && echo "${_attr}=${_val}" >> $_tmp_file

        mv $_tmp_file $_file
}
#
# voladm_query_disk_format -- query the user for a disk format
#
# SYNTAX: voladm_query_disk_format [-e] [-p] ret-format [y]
#	where
#		-e - indicates the private region will be used as part of disk
#			encapsulation (w/o -e disk initialization is assumed).
#		-p - indicates that the DG for the disk is precds
#		-f - indicates that the disk is an Solaris EFI disk
#		y - indicates to also query the user about using
#			the default format
# RETURNS: 0 on success
#	   otherwise failure (see also voladm_yorn, read_input)
#
voladm_query_disk_format()
{
 	_kind="vxdisk"
 	_precds="no"
	_efi="no"
 	while [ $# -gt 1 ]
 	do
 		case $1 in
 		-e)	_kind="vxencap"	;;
 		-p)	_precds="yes"	;;
		-f)	_efi="yes"	;;
 		*)	break		;;
 		esac
 		shift
 	done
 	if [ "$_kind" = "vxencap" ]
	then
 		if [ "$_precds" = "yes" ]
 		then
         		eval "${1}=sliced"
         		return 0
 		else
 			_input_list="cdsdisk,sliced"
 		fi
	else
 		if [ "$_precds" = "yes" ]
 		then
 			_input_list="sliced,simple"
 		else
 			_input_list="cdsdisk,sliced,simple"
 		fi
	fi

	voladm_get_disk_layout_defaults $_kind "format" _default_format

	if [ $? -ne 0 -o -z "$_default_format" ]
	then
		_default_format="cdsdisk"
	fi
 	if [ "$_precds" = "yes" -a "$_default_format" = "cdsdisk" ] ||
 	   [ "$_kind" = "vxencap" -a "$_default_format" = "simple" ]
 	then
 		_default_format="sliced"
 	fi

	if [ "$_efi" = "yes" ]
	then
		_input_list="sliced,simple"
 		_default_format="sliced"
	fi

        if [ $# -eq 0 ]
        then
                echo "voladm_query_disk_format: missing argument"
                return 1
        fi
        voladm_help_push disk.format.help

	if [ -n "$2" -a "$2" = "y" ]
	then
		voladm_yorn -M vxvmshm:1545 \
                "Use the default disk format for the disk?" y

		if [ $? -ne 1 ]
                then
        		voladm_help_pop
        		eval "${1}="
        		return 0
		fi
	fi
        _riput=`egettxt "Enter the desired format" vxvmshm:1514`

        while true ; do

		read_input "$_riput" "$_input_list" $_default_format

		if [ "$input" = "cdsdisk" -a "$_precds" = "no" -a "$_efi" = "no" ] ||
		   [ "$input" = "simple" -a "$_kind" != "vxencap"  ] ||
                   [ "$input" = "sliced"  ]
                then
                        break
                fi
                ewritemsg -M vxvmshm:1515 \
                        "Entered format is not valid. Please re-enter."
        done
        voladm_help_pop

        eval "${1}=$input"
        return 0
}
#
# voladm_query_disk_privlen -- query the user for a (disk's) priv region length 
#
# SYNTAX: voladm_query_disk_privlen [-e] ret-privlen [y]
#	where
#		-e - indicates the private region will be used as part of disk
#			encapsulation (w/o -e disk initialization is assumed).
#		 y - indicates to also query the user about using the default
#			private region length
# RETURNS: 0 on success
#	   otherwise failure (see also voladm_yorn, voladm_new_disk_privlen)
#
voladm_query_disk_privlen()
{
	if [ $# -ge 1 -a "$1" = "-e" ]
	then
		shift
		_eopt="-e"
	else
		_eopt=
	fi
        if [ $# -eq 0 ]
        then
                echo "voladm_query_disk_privlen: missing argument"
                return 1
        fi
        voladm_help_push disk.privlen.help

	if [ -n "$2" -a "$2" = "y" ]
	then
		voladm_yorn -M vxvmshm:1548 \
                "Use the default private region length for the disk?" y

		if [ $? -ne 1 ]
                then
        		voladm_help_pop
        		eval "${1}="
        		return 0
		fi
	fi
        voladm_new_disk_privlen $_eopt
        voladm_help_pop
        eval "${1}=$dmprivlen"
        return 0
}
#
# voladm_query_disk_layout - query the user for a disk layout
#
# syntax: voladm_query_disk_layout [-e] fmt_ret plen_ret [y] [-f]
#	where
#		-e - indicates the private region will be used as part of disk
#			encapsulation (w/o -e disk initialization is assumed).
#		-f - efi disk
#		y - indicates to also query the user about using
#			the default disk layout
# RETURNS: 0 on success
#	   otherwise failure (see also voladm_yorn, voladm_query_disk_format
#				voladn_query_disk_privlen)
#
voladm_query_disk_layout()
{
	if [ $# -ge 1 -a "$1" = "-e" ]
	then
		shift
		_eopt="-e"
	else
		_eopt=
	fi
	if [ $# -lt 2  ]
        then
                echo "voladm_query_disk_layout: missing argument"
                return 1
        fi
	if [ -n "$3" -a "$3" = "y" ]
	then
		voladm_help_push disk.layout.help

		voladm_yorn -M vxvmshm:1546 \
	                "Use the default disk layout for the disk?" y
		yorn=$?

        	voladm_help_pop

		if [ $yorn -ne 1 ]
                then
        		eval "${1}="
        		eval "${2}="
        		return 0
		fi
	fi

	efi=0
	if [ "$4" = "-f" -o "$3" = "-f" ]
	then
		efi=1
	fi

	if [ $efi -eq 1 ]
	then
		voladm_query_disk_format -f $_eopt _fmt $3
	else 
		voladm_query_disk_format $_eopt _fmt $3
	fi

	voladm_query_disk_privlen $_eopt _plen $3

        eval "${1}=$_fmt"
        eval "${2}=$_plen"
        return 0
}
#
# voladm_get_disk_reinit_size - determine the public region size for a disk
#				that would exist after 'initializing' the disk
#
# Usage: voladm_get_disk_reinit_size <ret size> <da name> <setup attrs>
#
# This routine returns (in <ret size>) the size of the public region that
# would exist on the disk (indicated by <da name>) after calling vxdisksetup
# with the attributes, if any, indicated by <setup attrs>.
#
# Currently: this routine only supports the setup attrs of:
#		format, privlen and noreserve
#
# Returns -1 on error; otherise, 0
#
voladm_get_disk_reinit_size()
{
	[ $# -ge 1 ] && eval "$1=0"	# set 0 for error returns
	[ $# -lt 3 ] && return -1

	_daname=$2
	_setup_attrs=$3
	#
	# Parse <setup attrs>.
	#
	_fmt=
	_privlen=
	_nores=
	for _attr in $_setup_attrs
	do
		case $_attr in
		format=*)	_fmt=`expr "$_attr" : "format=\(.*\)"` ;;
		privlen=*)	_privlen=`expr "$_attr" : "privlen=\(.*\)"` ;;
		noreserve)	_nores=y ;;
		*)		return -1 ;;
		esac
	done

	efi=0
	if dogi_is_efi_wd $_daname
	then
		efi=1
	fi

	#
	# If no format is specified in <setup attrs>, determine the default
	# value (either from the defaults file or the compiled in default).
	# Same for privlen.
	#
	if [ -z "$_fmt" ]
	then
		voladm_get_disk_layout_defaults vxdisk "format" _fmt
		[ $? -ne 0 ] && return -1
		if [ -z "$_fmt" ]
		then
			if [ $efi -eq 0 ]
			then
				_fmt=cdsdisk
			else
				_fmt=sliced
			fi
		fi
		
	fi
	if [ -z "$_privlen" ]
	then
		voladm_get_disk_layout_defaults vxdisk "privlen" _privlen
		[ $? -ne 0 ] && return -1
		[ -z "$_privlen" ] && _privlen=2048
	fi
	#
	# Determine desired disk info.
	#
	dogi_name_to_device $_daname _device
	dogi_device_rawpath $_device _path
	_devsize=`vxparms -s $_path 2>/dev/null | grep "^STAT_DISKNUMSEC" |\
			cut -d'=' -f2`

	if [ $efi -eq 0 ]	
	then
		_devspc=`vxparms -s $_path 2>/dev/null | grep "^STAT_SECPERCYL" |\
				cut -d'=' -f2`
	else
		_devspc=1
	fi

	[ -z "$_devsize" -o -z "$_devspc" ] && return -1
	#
	# Determine the public region size based on the known info.
	#
	case $_fmt in
	cds*)	_8k=16
		_privlen=`voladm_alignval up $_privlen $_8k`
		_devsize=`voladm_alignval down $_devsize $_8k`
		_res=256
		;;
	simple|sliced)
		_devsize=`voladm_alignval down $_devsize $_devspc`
		if [ -z "$_nores" ]
		then
			_res=`voladm_alignval up 256 $_devspc`
			_privlen=`expr $_privlen + 256`
		else
			_res=0
		fi
		if [ "$_fmt" = "sliced" ]
		then
			_privlen=`voladm_alignval up $_privlen $_devspc`
		fi
		;;
	*)	return -1 ;;
	esac
	#
	# For EFI disks, take into account that the first 34
	# sectors are reserved by the OS
	#
	if [ $efi -eq 0 ]
	then
		_pubsize=`expr $_devsize - $_privlen - $_res`
	else
		_pubsize=`expr $_devsize - $_privlen - $_res - 34`
	fi
	eval "$1=$_pubsize"
	return 0
}

#
# dogi_get_vstab_path - This will get you the boot c#t[wwn]d# for
# a given enclosure name device
#
# Arg# 3 is optional, If this arg is specified it will append that 
#	slice number at the end of arg#2, other wise it will use 
#	the same slice number as like in arg#1
#
dogi_get_vfstab_path()
{

	found=0

	name=`basename $1`
	if not dogi_name_is_slice $name
	then
		return 1
	fi 

	if [ X = X"$3" ] 
	then
		sliceno=`echo $name | awk -Fs '{print $NF}'`
	else
		sliceno=$3
	fi

	dogi_slice_to_device "$name" device

	dmpdev=`/usr/sbin/vxdmpadm getsubpaths dmpnodename=$device \
		2> /dev/null | tail +3 |awk '{print $1}'`
	tpddev=`/usr/sbin/vxdmpadm getsubpaths tpdnodename=$device \
		2> /dev/null | tail +3 | awk '{print $1}'`
	if [ ! -z "$tpddev" ]
	then
		dmpdev="$dmpdev $tpddev"
	fi

	voladm_mount_table $tmpfile4

	for path in $dmpdev
	do
		dogi_slice_to_device "$path" pathdev
		dev=`echo $pathdev\s$sliceno`
		dir=`awk '
			$1=="'$DISK_BLKPATH/$dev'" || $2=="'$DISK_RAWPATH/$dev'" {
				print $3
				exit
			}
			$1=="'$VOL_AP_DISK_BLKPATH/$dev'" || $2 == "'$VOL_AP_DISK_RAWPATH/$dev'" {
				print $3
				exit
			}' $tmpfile4`

		if [ ! -z "$dir" ]
		then
			found=1
			break;
		fi
	done

	eval "${2}=$dev"
}
