#!/sbin/sh -
# $Id: vxcbrlib.sh,v 1.1.2.13.10.1 2004/07/29 21:57:19 hhorn Exp $
# $Source: /project/unixvm-cvs/src/common/cmd/vxvm/voladm/vxcbrlib.sh,v $

# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

CF_BKUP_BASE_PATH=/etc/vx/cbr
CF_BKUP_PATH=$CF_BKUP_BASE_PATH/bk
PRECOMMIT_PRIVREG_PATH=$CF_BKUP_BASE_PATH/volatile
CBR_TMP=/var/tmp
BK_CONFIG=$CF_BKUP_BASE_PATH/bk_config
BK_CONFIG_LOCK=$CF_BKUP_BASE_PATH/bk_config.lock
DEV_PATH=/dev/vx/rdmp
DGINFO=dginfo
CONFIGREC=cfgrec
DISKINFO=diskinfo
DIAG_PATH=/etc/vx/diag.d
CNFFULL=binconfig
CNFBASE=configBase
PRECOMMIT_INITDISK=precommit_initdisk

FLAG_PRECOMMIT="precommit"
FLAG_COMMIT="commit"
FLAG_DECOMMIT="decommit"
FLAG_NOINIT="noinit"     # force restoration without disks initialization
FLAG_DKINIT="diskinit"  # initialize disks as default and complete restoration
PRECOMMIT_INITDISK=precommit_initdisk

REQ_HELP="help"
REQ_DGLIST="dglist"
REQ_INITDISKLIST="initdisklist"
REQ_BADCNFDISKLIST="badcfgdisklist"
REQ_INITDISK="initdisk"
REQ_INITDG="initdg"
REQ_BINPRECOMMIT="binprecommit"
REQ_PRECOMMIT=$FLAG_PRECOMMIT
REQ_COMMIT=$FLAG_COMMIT
REQ_DECOMMIT=$FLAG_DECOMMIT
REQ_FORCE=$FLAG_NOINIT

RET_BYBCNF=1
RET_DEPORTDG=2
RET_INITDISK=3
RET_NODISK=4
RET_FORCE=5

#Exit code for vxdgcfgres
CBR_SUCCESS=0
CBR_FAIL=1
CBR_SYNTAX=2
CBR_BKNOTFOUND=3
CBR_DEPORTDG=4	       # appears to be a deported dg
CBR_REINIT=5
CBR_BINCNFRESTORE=6    # all pda 
CBR_NOPRECOMMITDG=7
CBR_PRECOMMIT_VXMAKE=8
CBR_PRECOMMIT_FORCE=9
CBR_PRECOMMIT_NODK=10
CBR_DGONLINE=11
CBR_AMBIGUOUS=12

#Exit code for vxdgcfgbk
BK_CNFNOTCHANGE=2
BK_DGINPRECOMMIT=3

YES=1
NO=0

OS=`uname`
LN_PLATFORM_OPT=""
BLOCK_SZ=512
ISEEK=iseek
OSEEK=oseek
ECHO_OPT=""
AWK=awk
REQ_DGNAME=
REQ_DGID=

: ${VOLROOT_DIR:=$__VXVM_ROOT_DIR}
. ${VOL_SCRIPTS_LIB:-$VOLROOT_DIR/usr/lib/vxvm/lib}/vxcommon

cmd_supported $0

if [ "$OS" = "SunOS" ] ;then
	LN_PLATFORM_OPT="-n"
	AWK=nawk
fi

if [ "$OS" = "HP-UX" ] ;then
	BLOCK_SZ=1024
fi

if [ "$OS" = "AIX" -o "$OS" = "Linux" ] ;then
	ISEEK=skip
	OSEEK=seek
fi

if [ "$OS" = "Linux" ] ;then
	ECHO_OPT="-e"
fi


vxvm_releaseinfo()
{
	pkginfo -l VRTSvxvm
}

kill_show_prog()
{
	prog_pid=`ps -ef | fgrep inst.show_prog | fgrep $$  | $AWK '{ print $2 }'
`
	if [ "X$prog_pid" != "X" ]
	then
		kill -ABRT $prog_pid 2>/dev/null
	fi
}

acquirelock()
{
        PIDFILE="$1.$$"
        LOCKFILE="$1.lock"

        echo $$ > $PIDFILE 2>/dev/null || {
                return 1
        }

        ln $LN_PLATFORM_OPT $PIDFILE $LOCKFILE 2>/dev/null && {
                rm -f $PIDFILE
                return 0
        }

        STALEPID=`cat $LOCKFILE 2>/dev/null`
        [ "$STALEPID" -gt "0" ] || {
                return 1
        }

        kill -0 $STALEPID 2>/dev/null && {
                rm -f $PIDFILE
                return 1
        }

        rm $LOCKFILE 2>/dev/null && 
        ln $LN_PLATFORM_OPT $PIDFILE $LOCKFILE 2>/dev/null && {
                rm -f $PIDFILE
                return 0
        }

        rm -f $PIDFILE
        return 1
}


# get disk group list - dg (name)
# arg1 : dg list 
# arg2 : 0 = default dg list to backup
# 	 1 = default dg list to restore 
cbr_dflt_dglist()
{
	_dglist=$1

	if [ "X$2" = "Xbackup" -o "X$2" = "Xonline" ]
	then
		list=`vxdg -q list | $AWK '{print $1}' 2>/dev/null`
	elif [ "X$2" = "Xrestore" ]
	then
		list=`ls $CF_BKUP_PATH 2>/dev/null` 
	fi

	eval "$_dglist=\$list"
	return 0
}

# get disk group list - dgid
# arg1 : dg list 
# arg2 : 0 = default dg list to backup
# 	 1 = default dg list to restore 
cbr_dflt_dgidlist()
{
	_dglist=$1

	if [ "X$2" = "Xbackup" -o "X$2" = "Xonline" ]
	then
		list=`vxdg -q list | $AWK '{print $3}' 2>/dev/null`
	elif [ "X$2" = "Xrestore" ]
	then
		list=`ls $CF_BKUP_PATH 2>/dev/null` 
	fi

	eval "$_dglist=\$list"
	return 0
}

# add an item to end of a list
# arg1 		: list 
# arg2,3,.. 	: items
cbr_addlist()
{
	_list=$1
	shift
	eval "set -- \$$_list \$*"
	eval "$_list=\$*"
}

# remove dupicate items from a list
# arg1 		: return list with no duplication
# arg2 		: list with duplicate item to be processed
cbr_rmdupentry()
{
	_uniqlist=$1
	shift
	dglist="$*"

	for item in $dglist; do
		shift
		echo "$*" | grep -w "$item" 2>&1 >/dev/null
		[ $? = 0 ] || cbr_addlist uniqlist $item
	done

	eval "$_uniqlist=\$uniqlist"
}


## get one item from the beginning of input list and remove it from the list
# $1 : first item in the dm list
# $2 : dm list
next_item()
{
	_nextitem=$1
	_finallist=$2
	shift
	eval "set -- \$$_finallist"
	eval "$_nextitem=\$1"
	shift
	eval "$_finallist=\$*"
}


## get selected item from the input list
# $1 : selected item in the input list
# $2 : selected item index in the input list
# $3 : input list
get_selitem()
{
	_selectitem=$1
	_pos=$2
	_list=$3
	shift 2
	eval "set -- \$$_list"
	_pos=`expr $_pos - 1`
	shift $_pos
	eval "$_selectitem=\$1"
}



## get the uniq disk serial numner, vid, ....
# Input:
#       $1 : uuid
#       $2 : da name
getuuid()
{
	_uuid=$1

	wwn=""	
	for i in `/etc/vx/diag.d/vxdmpinq $DEV_PATH/$2 2>/dev/null | tail +3 | \
			$AWK -F":" '{print $2}' 2>/dev/null`
	do
		wwn="$wwn$i-"
	done

	[ "X$wwn" = "X" ] && wwn="----"

	eval "$_uuid=\$wwn"
}


## get the uniq vxvm disk id
# Input:
#       $1 : diskid
#       $2 : da name
getdiskid()
{
	_diskid=$1

	_tmpdkid=`vxdisk list $2 | grep "^disk" | $AWK -F" " '{print $3}' | \
			$AWK -F"=" '{print $2}'`

	eval "$_diskid=\$_tmpdkid"
}

## get the uniq vxvm diskgroup id
# Input:
#       $1 : diskgroupid
#       $2 : da name
getgrpid()
{
	_groupid=$1

	_tmpgpid=`vxdisk list $2 | grep "^group" | $AWK -F" " '{print $3}' | \
			$AWK -F"=" '{print $2}'`

	eval "$_groupid=\$_tmpgpid"
}

## get the private region device path
# Input:
#       $1 : private region device path
#       $2 : da name
getpripath()
{
	_prip=$1

	_dkattr="`vxdisk list $2`"
	_tmpprip=`echo "$_dkattr" | grep "^privpaths:" |$AWK -F"=" '{print $3}'` 

	[ -z "$_tmpprip" ] && {
		_tmpprip=`echo "$_dkattr" | grep "^pubpaths:" | \
			$AWK -F"=" '{print $3}'` 
	}

	eval "$_prip=\$_tmpprip"
}

getprioff()
{
	_privoffset=$1

	_tmprivoffset=`vxdisk list $2 | $AWK '/^private:/ {print substr($3,8)}'`

	eval "$_privoffset=\$_tmprivoffset"
}

## get configdb info 
# Input:
#       $1 : _privp - private region path
#       $2 : _privo - private region offset
#       $3 : _privl - private region length 
#	$4 : _coffs - config region offset list
#	$5 : _cszs - config region size list
#	$6 : _dg
getdiskcfginfo()
{
	_privp=$1
	_privo=$2
	_privl=$3
	_confl=$4
	_coffs=$5
	_cszs=$6
	_cfgdisk=$7
	_flag=$8

	BKFULLPATH=$CF_BKUP_PATH/$REQ_DGNAME.$REQ_DGID

	if [ "$_flag" = "temp" ] ; then
		DGFILE=$BKFULLPATH/temp.$REQ_DGID.$DGINFO
	else
		DGFILE=$BKFULLPATH/$REQ_DGID.$DGINFO
	fi

	DISK_ATTR_F=$CBR_TMP/vxcbr.tmp1.$$
	CNF_INFO_F=$CBR_TMP/vxcbr.tmp2.$$
	CNF_INFO_F1=$CBR_TMP/vxcbr.tmp3.$$

	[ -z "$_cfgdisk" ] && {
		_cfglen=`grep "^config_region_len" $DGFILE | \
			$AWK -F"=" '{print $2}'`
	
		_cfgdisk=`grep "^config_disk" $DGFILE | grep "enabled" |\
			grep -w "len=$_cfglen" | head -1 | $AWK '{print $3}'`
	}

	vxdisk list $_cfgdisk 2>/dev/null > $DISK_ATTR_F
	ret=$?
	[ $ret = 3 -o $ret = 9 ] && {
		rm -rf $DISK_ATTR_F $CNF_INFO_F $CNF_INFO_F1
		return 1
	}

	_privpath=`cat $DISK_ATTR_F | grep -w "privpaths:" | \
		$AWK -F"=" '{print $3}'` 

	[ -z "$_privpath" ] && {
		_privpath=`cat $DISK_ATTR_F | grep -w "pubpaths:" | \
			$AWK -F"=" '{print $3}'` 
	}

	_privoff=`cat $DISK_ATTR_F | grep -w "private:" | \
		$AWK -F" " '{print $3}' | $AWK -F"=" '{print $2}'`

	_privlen=`cat $DISK_ATTR_F | $AWK '/^private:/ {print substr($4,5)}'`

	_configlen=`cat $DISK_ATTR_F |grep -w "configs:"|$AWK -F"=" '{print $3}'`

	# get config offsets and size of each config region
	if [ "$_flag" = "temp" ] ; then
		cat $DISK_ATTR_F | grep -w "config" | grep enabled > $CNF_INFO_F
	else
		cat $DISK_ATTR_F | grep -w "config" > $CNF_INFO_F
	fi

	_cfg_copyno=`cat $CNF_INFO_F | head -1 | $AWK -F" " '{print $4}'`

	cat $CNF_INFO_F | grep -w "$_cfg_copyno" > $CNF_INFO_F1

	_cfg_offs=`cat $CNF_INFO_F1 | $AWK -F" " '{print $3}' | \
		$AWK -F"-" '{print $1}'`

	_cfg_szs=`cat $CNF_INFO_F1 | $AWK -F"[" '{print $2}' | \
		$AWK -F"]" '{print $1}'`

	eval "$_privp=\$_privpath"
	eval "$_privo=\$_privoff"
	eval "$_privl=\$_privlen"
	eval "$_confl=\$_configlen"
	eval "$_coffs=\$_cfg_offs"
	eval "$_cszs=\$_cfg_szs"

	rm -rf $DISK_ATTR_F $CNF_INFO_F $CNF_INFO_F1
	return 0
}

cbr_read_input()
{
	echo ""

	_prompt="$1 "
	[ -z "$2" ] || _prompt="${_prompt}[$2,q] "
	case $_prompt in
????????????????????????????????????????????????????????????????????*)
		if [ ! -z "$2" ] 
		then
			_prompt="$1\n"
			[ -z "$2" ] || _prompt="${_prompt}[$2,q] "
		fi
		;;
	esac

	while true
	do
		echo $ECHO_OPT "$_prompt\c"
		read input <&0
		case $input in
		y|yes)  return 0;;
		n|no)   return 1;;
		"")	continue;;
		q|quit)	exit 101;;
		*)      ewritemsg -s NOTICE -M vxvmshm:788 \
				"Please respond with \\\"y\\\" or \\\"n\\\""
			continue;;
		esac
	done
}
