#
# $Id: vxcds_conv.sh,v 1.1.2.16.4.6 2005/01/19 03:26:22 dstowe Exp $
#ident "$Source: /project/unixvm-cvs/src/common/cmd/vxvm/voladm/vxcds_conv.sh,v $ $Revision: 1.1.2.16.4.6 $"
#
# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

#
# vxcdsconvert -g diskgroup <options> <keyword> [<attribute>=<value>]...
#    where,
#	options: [-A] [-D] [-H] [-V] [-d <file> ] -g diskgroup [-o novolstop ]
#	keyword: alignment|alldisks|disk <name>| group
#
# OPTIONS:
#	-A	Analyze and print the steps that would be taken to do the
#			conversion, but do not take them.
#	-D	Run with "debugging" on. Not documented in usage.
#	-H	Output the full-usage message and exit.
#	-V	Run with "verbose" on. Not documented in usage.
#			Will indicate VxVM commands executed.
#	-X	Run with "set -x". Not documented in usage.
#	-d	Use the specified <file> to get additional attributes.
#			Also, attributes can be specified in the system default
#			file: /etc/default/vxcdsconvert
#	-g	Use the specified <group> for conversion.
#	-o	Operational options:
#		novolstop - indicates that the conversion is to be performed
#			    without stopping any active volume
# KEYWORDS:
#	alignment	Convert the disk group alignment of the specified group
#				(-g option) to have a CDS compatible alignment
#
#	alldisks	Convert all of the non-CDS disks in the specified disks
#				group (-g option) into CDS disks
#
#	disk <name>	Convert the non-CDS disk indicated by disk media <name>
#				into a CDS disk
#
#	group		Convert the specified non-CDS disk group (-g option)
#				into a CDS disk group. This conversion also
#				performs the alignment and alldisks keywords.
#
# ATTRIBUTES
#	evac_disk_list		Provides an ordered, comma-separated list of
#				disk media to use for evacuating subdisks.
#				Default: to use any disk in the disk group
#				with preference given to CDS disks and those
#				with the most free space.
#				
#	evac_subdisks_ok	Indicated whether (yes) or not (no) subdisks
#				can be evacuated to a different disk media if
#				needed for the conversion. Preference will be
#				given to moving subdisks if allowed.
#				Default: no
#
#	min_split_size		Indicates the minimum size to split a subdisk
#				into. This attribute is only meaningful if
#				split_subdisks_ok is not no.
#				Default: 64KB (unit size is _BSIZE)
#
#	move_subdisks_ok	Indicates whether (yes) or not (no) subdisks
#				can be moved to a different location on the
#				same disk media if needed for the conversion.
#				Default: yes
#
#	privlen			Indicates the size of the private region (as
#				would be given to the vxdisk init command)
#				that should exist after converting a non-CDS
#				disk into a CDS disk. The value must be an
#				integral multiple of 8k and at least 512k.
#				Default: calculate one based on current disk
#				and disk group attributes but with a minimum
#				of 1m (or the privlen value, if one exists, in
#				the /etc/defaul/vxdisk file)
#
#	split_subdisks_ok	Indicates whether (yes|move|evac) or not (no)
#				subdisks can be split when moved or evacuated.
#				This allows for a portion of a subdisk to be
#				relocated instead of the entire subdisk. The
#				values of move and evac only allow for a split
#				for the indicated action.
#				Default: move
#

: ${VOLROOT_DIR:=$__VXVM_ROOT_DIR}
: ${VOL_DIAG_DIR:=$VOLROOT_DIR/usr/lib/vxvm/diag.d} 
. ${VOL_SCRIPTS_LIB:-$VOLROOT_DIR/usr/lib/vxvm/lib}/vxcommon
exec 4>&2
set_x=
## set -u
#
# BSIZE numeric constants
#
_BSIZE=`expr 1024 \* 1024 / \`strtovoff -- "1m"\``
_1K=`expr 1024 / $_BSIZE`
_8K=`expr 8 \* $_1K`
_128K=`expr 128 \* $_1K`
_1M=`expr 1024 \* $_1K`
#
# file variables
#
sys_dflt_file=$SYSROOT/etc/default/vxcdsconvert
usr_dflt_file=""	# specified via -d flag on command-line

lock_file=${VOL_LOCK_DIR}/.VXCDSCONVERT.LOCK
#
# attribute variables (declaration & default settings)
#
evac_disk_list=""
evac_subdisks_ok="no"
min_split_size=`expr 64 \* $_1K`	# variable has units of _BSIZE 
move_subdisks_ok="yes"
split_subdisks_ok="move"
privlen=				# variable has units of _BSIZE
					# default set later

attr_list="evac_disk_list evac_subdisks_ok min_split_size move_subdisks_ok"
attr_list="$attr_list privlen split_subdisks_ok"
#
# misc. global variables set via command-line flags, etc.
#
debug="off"
verbose="off"
analyze_only="n"
dg=""
keyword=""
disk_name=""
novolstop="n"
bad_evac_list=""
platform=`uname -s`
#
#-------------
# FUNCTIONS
#-------------
#
# local_cleanup - cleanup function called when script exits
#
local_cleanup() {
	rm -f $lock_file
}
#
# remove_list <list> <item>
#
remove_list() {
	_list=$1
	_item=$2
	_newlist=""

	eval `set -- \$$_list`
	while [ $# -gt 0 ]
	do
		[ "$1" != "$_item" ] && _newlist="$_newlist $1"
	done
	eval "$_list=\"$_newlist\""
}
#
# list_find <list> <item> <idx-ret>
#
list_find()
{
	_list_name=$1
	_list_item=$2
	_list_idx=$3
	
	cnt=1
	eval "set -- \$$_list_name"
	for _item in $*
	do
		[ "$_item" = "$_list_item" ] && {
			eval "$_list_idx=$cnt"
			return 0
		}
		cnt=`expr $cnt + 1`
	done
	eval "$_list_idx=0"
	return 1
}
#
# usage_exit - print a short usage message
#
# Usage: usage_exit [errinfo]
#
usage_exit() {
	[ $# -ge 1 ] && print_err "$1"

	cat <<! >&2
`egettxt "\
Usage: vxcdsconvert -g diskgroup <options> <keyword> [attribute=value ...]
	options: [-A] [-d file] -g diskgroup [-o novolstop]
	keyword: alignment | alldisks | disk <name> | group

For detailed help use: vxcdsconvert -H
" vxvmshm:4163`
!
	quit 1
}
#
# fullusage_exit - print a verbose usage message
#
# Usage: fullusage_exit
#
fullusage_exit() {
	cat <<! >&2
`egettxt "\
vxcdsconvert - volume manager CDS converter

Usage: vxcdsconvert -g diskgroup <options> <keyword> [attribute=value ...]
	options: [-A] [-d file] -g diskgroup [-o novolstop]
	keyword: alignment | alldisks | disk <name> | group

Keyword:
    alignment   Convert the alignment of the disk group to 8k.
    alldisks    Convert all non-CDS disks in the disk group into CDS disks.
    disk        Convert the specified non-CDS disk into a CDS disk.
    group       Convert the non-CDS disk group into a CDS disk group.

Options:
    -A  Analyze and print the steps that would be taken to do the conversion,
       	but do not take them.

    -d  Use the specified file to get additional attributes.

    -g  Use the specified disk group for conversion.

    -o  novolstop	Perform conversion without stopping any active volume.

Attributes:
    evac_disk_list		Provides an ordered, comma-separated list of
				    disk media to use when evacuating subdisks
				    when evac_subdisks_ok is yes.
				    Default: any disk in the disk group

    evac_subdisks_ok=<yes|no>	Indicates if evacuating subdisk space to
				    another disk in the disk group is allowed.
				    Default: no

    min_split_size=<number>	Minimum size to use when splitting a subdisk.
				    Default: 64k

    move_subdisks_ok=<yes|no>	Indicates if moving subdisk space to another
				    location on the same disk is allowed.
				    Default: yes

    privlen=<number>		Private region size for initializing CDS disks.
				    Must be at least 512k and a multiple of 8k.
				    Default: 1m

    split_subdisks_ok=<yes|no|move|evac>
				Indicates if a subdisk may be split in order
				    to evacuate or move it.
				    Default: move
" vxvmshm:4164`
!
	quit 1
} 
#
# xerrmsg - translate an error message
#
# Usage: xerrmsg xmsg-ret catalog-num english-msg
#
xerrmsg() {
	_err_cat="$2"
	_err_def="$3"
	shift 3
	if [ $# -gt 0 ]
	then
		ugettxt -n vxcdsconvert -s ERROR -- "$_err_def" "$_err_cat" "$@" >&2
	else
		ugettxt -n vxcdsconvert -s ERROR -- "$_err_def" "$_err_cat" "" >&2
	fi
}

#
# print_debug - print debug information
#
# Usage: print_debug arg...
#
print_debug() {
	[ "$debug" = "on" ] && {
		cat <<-% >&4
			DEBUG: $*
		%
	}
}
#
# print_verbose - print verbose information
#
# Usage: print_verbose arg...
#
print_verbose() {
	[ "$verbose" = "on" ] && {
		cat <<-% >&4
			$*
		%
	}
}
#
# print_anal- print analysis information
#
# Usage: print_anal arg...
#
print_anal() {
	cat <<-% >&1
		$*
	%
}
#
# mk_print_list - turn a list into a printable list
#
# Usage mk_print_list <list> <print_list>
#
mk_print_list()
{
	_maxlpos=70
	_lpos=0
	_plist=
	for _i in $1
	do
		_isz=`echo "$_i" | wc -m`
		_lpos=`expr $_isz + $_lpos`

		if [ -z "$_plist" ]
		then
			_plist="$_i"

		elif [ $_lpos -gt $_maxlpos ]
		then
			#_plist="$_plist \n      $_i"
			_plist=`echo "$_plist       $_i" | tr '' '\n'`
			_lpos=$_isz
		else
			_plist="$_plist $_i"
		fi
	done
	eval "$2=\"$_plist\""
}
#
# print_err - print out an error message
# print_err_exit - print out an error message and exit
#
# Usage: print_err errinfo
#	 print_err_exit errinfo
#		where errinfo is a string of the form: error-code [arg ...]
#
print_err() {

	set -- $1
	case $1 in
	E_INTERNAL)
		xerrmsg -M vxvmshm:3174 "Internal Error"
		;;
	E_ABORT)
		xerrmsg -M vxvmshm:3120 "Conversion process aborted"
		;;
	E_UNSUPPORTED_BSIZE)
		export _BSIZE; xerrmsg -M vxvmshm:3124 \
			"Converter cannot be run on a system whose block size is $_BSIZE"
		;;
	E_NO_DG)
		xerrmsg -M vxvmshm:3186 \
			"No disk group specified (use -g option)"
		;;
	E_DG_CDS)
		export dg; xerrmsg -M vxvmshm:2834 \
			"$dg: Unable to set CDS attribute"
		;;
	E_DG_ALIGN)
		export dg; xerrmsg -M vxvmshm:2836 \
			"$dg: Unable to set diskgroup alignment"
		;;
	E_DG_UTRANS)
		_state=$2; export dg _state; xerrmsg -M vxvmshm:2837 \
			"$dg: Unable to turn user transactions $_state"
		;;
	E_DG_INFO)
		export dg; xerrmsg -M vxvmshm:2833 \
			"$dg: Unable to get information about the disk group"
		;;
	E_DG_MAXDEV)
		_maxdev=$2; export dg _maxdev; xerrmsg -M vxvmshm:2835 \
			"$dg: Unable to set device quota to $_maxdev"
		;;
	E_DG_NOEXIST)
		export dg; xerrmsg -M vxvmshm:2832 \
			"$dg: Disk group not found"
		;;
	E_DG_ONSLAVE)
		export dg; xerrmsg -M vxvmshm:3512 \
			"$dg: Cannot convert shared disk group from slave node"
		;;
	E_DG_ONE_DM)
		export dg; xerrmsg -M vxvmshm:2831 \
			"$dg: Cannot do disk conversion in a disk group with only one disk"
		;;
	E_DG_UPGRADE)
		export dg; xerrmsg -M vxvmshm:2838 \
			"$dg: Unable to upgrade the disk group to the latest version"
		;;
	E_DM_ADJ)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2768 \
			"$_disk: Unable to adjust the disk's public region"
		;;
	E_DM_INFO)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2775 \
			"$_disk: Unable to get information about the disk"
		;;
	E_DM_NOAUTO_DA|E_DM_DOS_DA)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:3457 \
			"$_disk: Requires the '-o novolstop' option to convert the disk"
		;;
	E_DM_NO_DA)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2762 \
			"$_disk: No associated disk access record"
		;;
	E_DM_NOEXIST)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2761 \
			"$_disk: Disk media not found in configuration"
		;;
	E_DM_RM)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2781 \
			"$_disk: Unable to remove the disk from the disk group"
		;;
	E_DM_USAGE)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2760 \
			"$_disk: Contains a volume with use-type of 'root' or 'swap'"
		;;
	E_DM_VOL_START)
		_disk=$2; _vol=$3
		export _disk _vol; xerrmsg -M vxvmshm:3379 \
			"$_disk: Contains volume $_vol that could not be started."
		;;
	E_DM_VOL_STARTABLE)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2759 \
			"$_disk: Contains a volume that is unstartable or unusable."
		;;
	E_DM_VOL_STOP)
		_disk=$2; _vol=$3
		export _disk _vol; xerrmsg -M vxvmshm:3380 \
			"$_disk: Contains volume $_vol that could not be stopped."
		;;
	E_DA_ADD)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2767 \
			"$_disk: Unable to add the disk back to the disk group"
		;;
	E_DA_REINIT)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2777 \
			"$_disk: Unable to initialize the disk as a CDS disk"
		;;
	E_BAD_OPTION)
		xerrmsg -M vxvmshm:3313 \
			"Unrecognized option or missing argument"
		;;
	E_BAD_KEYWORD)
		_keyword=$2; export _keyword; xerrmsg -M vxvmshm:2790 \
			"$_keyword: Unrecognized keyword"
		;;
	E_NO_KEYWORD)
		xerrmsg -M vxvmshm:3189 "No keyword specified"
		;;
	E_NO_DISKNAME)
		xerrmsg -M vxvmshm:3187 "No disk name specified"
		;;
	E_BAD_ATTRVAL)
		shift
		_attrval="$*"; export _attrval; xerrmsg -M vxvmshm:2749 \
			"$_attrval: Invalid attribute specification"
		;;
	E_BADVAL_ATTR)
		shift
		_attrval="$*"; export _attrval; xerrmsg -M vxvmshm:2750 \
			"$_attrval: Invalid value"
		;;
	E_MAXINT_ATTR)
		_attrval=$2; _max=$3
		export _attrval _max; xerrmsg -M vxvmshm:2751 \
			"$_attrval: Numeric value too large; maximum is $_max"
		;;
	E_MININT_ATTR)
		_attrval=$2; _min=$3
		export _attrval _min; xerrmsg -M vxvmshm:2752 \
			"$_attrval: Numeric value too small; minimum is $_min"
		;;
	E_POSINT_ATTR)
		_attrval=$2; export _attrval; xerrmsg -M vxvmshm:2753 \
			"$_attrval: Positive numeric attribute value expected"
		;;
	E_UNKOWN_ATTR)
		_attrval=$2; export _attrval; xerrmsg -M vxvmshm:2755 \
			"$_attrval: Unknown attribute"
		;;
	E_PRIVLEN_8K)
		_attrval=$2; export _attrval; xerrmsg -M vxvmshm:3378 \
			"$_attrval: privlen must be a multiple of 8k"
		;;
	E_PRIVLEN_SMALL)
		export privlen; xerrmsg -M vxvmshm:3218 \
			"privlen=$privlen: Value too small for use with disk group"
		;;
	E_BAD_FILE)
		_file=$2; export _file; xerrmsg -M vxvmshm:2787 \
			"$_file: File not found or is not a regular file"
		;;
	E_FILE_CREATE)
		_file=$2; export _file; xerrmsg -M vxvmshm:2786 \
			"$_file: Cannot create"
		;;
	E_OOPT_INV)
		_opt=$2; export _opt; xerrmsg -M vxvmshm:2844 \
			"-o $_opt: Invalid option"
		;;
	E_SYNTAX)
		_file=$2; export _file; xerrmsg -M vxvmshm:2788 \
			"$_file: Syntax error"
		;;
	E_SYNTAX_DUP)
		_file=$2; export _file; xerrmsg -M vxvmshm:2789 \
	       		"$_file: Syntax error or duplicate entries"
		;;
	E_DISK_GEOM)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2772 \
			"$_disk: Unable to determine disk geometry"
		;;
	E_SECTSZ)
		_disk=$2; _sz=$3; export _disk _sz; xerrmsg -M vxvmshm:2765 \
			"$_disk: Sector size of $_sz is not supported"
		;;
	E_OVERLAP)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2763 \
			"$_disk: Public and private regions overlap"
		;;
	E_MAX_COMMENT)
		_comm=$2; export _comm; xerrmsg -M vxvmshm:2756 \
			"$_comm: Comment exceeds maximum length"
		;;
	E_OBJ_EDIT)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2774 \
			"$_disk: Unable to edit the object in the disk group"
		;;
	E_VOL_ADD_LOG)
		_vol=$2; _type=$3; export _vol _type;
		case $_type in
		RAID)
		    xerrmsg -M vxvmshm:3490 \
"$_vol: Unable to add RAID log.
  There must be sufficient disks to add an additional log to the volume."
		    ;;
		*)
		    xerrmsg -M vxvmshm:2800 \
			"$_vol: Unable to add log of type $_type"
		    ;;
		esac
		;;
	E_VOL_CLR_NOSTART)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2770 \
			"$_disk: Unable to clear 'nostart' flag for volumes"
		;;
	E_VOL_INFO)
		_disk=$2; _v=$3; export _disk _v; xerrmsg -M vxvmshm:2776 \
			"$_disk: Unable to get information about volume $_v"
		;;
	E_VOL_MIRROR)
		_v=$2; export _v; xerrmsg -M vxvmshm:2795 \
			"$_v: Unable to add mirror to volume"
		;;
	E_VOL_MIRROR_RM)
		_v=$2; export _v; xerrmsg -M vxvmshm:2798 \
			"$_v: Unable to remove mirror from volume"
		;;
	E_VOL_MOVE_FROM)
		_disk=$2; _v=$3; export _disk _v; xerrmsg -M vxvmshm:2780 \
			"$_disk: Unable to move volume $_v off of the disk"
		;;
	E_VOL_MOVE_TO)
		_disk=$2; _v=$3; export _disk _v; xerrmsg -M vxvmshm:2779 \
			"$_disk: Unable to move volume $_v back to the disk"
		;;
	E_VOL_OLDLOG)
		_v=$2; export _v; xerrmsg -M vxvmshm:2796 \
			"$_v: Unable to clear oldlog flag on volume"
		;;
	E_VOL_RECOVER)
		_v=$2; export _v; xerrmsg -M vxvmshm:2797 \
			"$_v: Unable to recover volume"
		;;
	E_VOL_REV_RELAYOUT)
		_disk=$2; _v=$3; export _disk _v; xerrmsg -M vxvmshm:2783 \
			"$_disk: Unable to reverse volume relayout on volume $_v after relayout failure"
		;;
	E_VOL_MAX_EVAC)
		_vol=$2; export _vol;  xerrmsg -M vxvmshm:3458 \
			"$_vol: Maximum disk evacuations reached for this object"
		;;
	E_VOL_RM_LOG)
		_vol=$2; _type=$3; export _vol _type; xerrmsg -M vxvmshm:2802 \
			"$_vol: Unable to remove log of type $_type"
		;;
	E_VOL_BAD_LOGTYPE)
		_vol=$2; _type=$3; export _vol _type; xerrmsg -M vxvmshm:3929 \
"$_vol: Invalid log type $_type
   An invalid log type was encountered during conversion.
   Remove the log or move the volume out of the diskgroup before
   running vxcdsconvert."
		;;
	E_VOL_SET_LENGTH)
		_v=$2; _len=$3; export _v _len; xerrmsg -M vxvmshm:2799 \
			"$_v: Unable to set volume length to $_len"
		;;
	E_VOL_SET_LOG_LENGTH)
		_v=$2; _len=$3; export _v _len; xerrmsg -M vxvmshm:2799 \
			"$_v: Unable to set volume length to $_len"
		;;
	E_VOL_SET_NOSTART)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2784 \
			"$_disk: Unable to set 'nostart' flag for volumes"
		;;
	E_VOL_STW_RELAYOUT)
		_vol=$2; export _vol; xerrmsg -M vxvmshm:2801 \
			"$_vol: Unable to relayout stripe width"
		;;
	E_VOL_START)
		_vol=$2; export _disk _vol; xerrmsg -M vxvmshm:3459 \
			"$_vol: Volume could not be started."
		;;
	E_VOL_STOP)
		_vol=$2; export _disk _vol; xerrmsg -M vxvmshm:3460 \
			"$_vol: Volume could not be stopped."
		;;
	E_UNRELOC_TO)
		_disk=$2; export _disk xerrmsg -M vxvmshm:2778 \
			"$_disk: Unable to move all volumes back to the disk"
		;;
	E_PLEX_ATT)
		_pl=$2; _v=$3; export _pl _v; xerrmsg -M vxvmshm:2792 \
			"$_pl: Unable to attach plex to volume $_v"
		;;
	E_PLEX_DIS)
		_pl=$2; _v=$3; export _pl _v; xerrmsg -M vxvmshm:2793 \
			"$_pl: Unable to dissociate plex from volume $_v"
		;;
	E_PLEX_LOG_ASSOC)
		_log=$2; _pl=$3; export _log _pl; xerrmsg -M vxvmshm:2791 \
			"$_log: Unable to associate log with $_pl"
		;;
	E_PLEX_RM)
		_pl=$2; _v=$3; export _pl _v; xerrmsg -M vxvmshm:2794 \
			"$_pl: Unable to remove the plex from volume $_v"
		;;
	E_CACHE_GROWTO)
		_cache=$2; _size=$3; export _cache _size; xerrmsg -M vxvmshm:3436 \
			"$_cache: Unable to grow cache to size $_size"
		;;
	E_CACHE_MAXAUTOGROW)
		_cache=$2; _size=$3; export _cache _size; xerrmsg -M vxvmshm:3489 \
			"$_cache: Unable to set maxautogrow attr to $_size"
		;;
	E_CACHE_AUTOGROWBY)
		_cache=$2; _size=$3; export _cache _size; xerrmsg -M vxvmshm:3488 \
			"$_cache: Unable to set autogrowby attr to $_size"
		;;
	E_SD_ASSCOL)
		_disk=$2; _pl=$3; _col=$4
		export _disk _pl _col; xerrmsg -M vxvmshm:2769 \
			"$_disk: Unable to associate the subdisk with column $_col of plex $_pl"
		;;
	E_SD_COLOFF)
		_disk=$2; _coff=$3
		export _disk _coff; xerrmsg -M vxvmshm:2766 \
			"$_disk: Subdisk has invalid plex column offset $_coff"
		;;
	E_SD_CREAT)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2771 \
			"$_disk: Unable to create the subdisk in the disk group"
		;;
	E_SD_DIS)
		_disk=$2; _pl=$3 export _disk _pl; xerrmsg -M vxvmshm:2773 \
			"$_disk: Unable to dissociate subdisks from plex $_pl"
		;;
	E_SD_EVAC_NEEDED)
		xerrmsg -M vxvmshm:3255 \
			"Subdisk evacuation is needed, but it is not allowed"
		;;
	E_SD_JOIN)
		_disk=$2; _disk2=$3
		export _disk _disk2; xerrmsg -M vxvmshm:3307 \
			"Unable to join subdisks $_disk and $_disk2 in the disk group"
		;;
	E_SD_MV)
		_disk=$2; _disk2=$3
		export _disk _disk2; xerrmsg -M vxvmshm:3308 \
			"Unable to move subdisk $_disk to $_disk2 in the disk group"
		;;
	E_SD_RELOC_NEEDED)
		xerrmsg -M vxvmshm:3256 \
			"Subdisk relocation is needed, but it is not allowed"
		;;
	E_SD_RENAME)
		_disk=$2; _disk2=$3
		export _disk _disk2; xerrmsg -M vxvmshm:3309 \
			"Unable to rename subdisk $_disk to $_disk2"
		;;
	E_SD_RM)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2782 \
			"$_disk: Unable to remove the subdisk from the disk group"
		;;
	E_SD_SPLIT)
		_disk=$2; export _disk; xerrmsg -M vxvmshm:2785 \
			"$_disk: Unable to split the subdisk in the disk group"
		;;
	E_SD_SPLIT_NEEDED)
		xerrmsg -M vxvmshm:3257 \
			"Subdisk splitting is needed, but it is not allowed"
		;;
	E_SD_MOVE_NEEDED)
		xerrmsg -M vxvmshm:3446 \
			"Subdisk move is needed, but it is not allowed"
		;;
	E_CONFIG_DISOBJ)
		_file=$2; xerrmsg -M vxvmshm:3466 \
"Dissociated objects present in disk group.
  The vxcdsconvert program requires that all objects in the disk group be
  associated, enabled, and active. The following objects are not associated
  and must be reassociated or removed before running vxcdsconvert:
"
		cat $_file | sed -e 's/^/	/' >&2
		echo "" >&2
		return
		;;
	E_CONFIG_PLSTATE)
		_file=$2; xerrmsg -M vxvmshm:3465 \
"Bad plex states present in disk group.
  The vxcdsconvert program requires that plex states be
  ACTIVE CLEAN EMPTY or LOG, and the plex kernel state must be ENABLED.
  The following list contains volumes with plexes that are not
  in a valid state and must be corrected before running vxcdsconvert.
  Try starting all volumes and performing any needed volume recovery.
  List itmes are in the form vol_name plex_name plex_state plex_kernel_state.
"
		cat $_file | sed -e 's/^/	/' >&2
		echo "" >&2
		return
		;;
	E_CONFIG_PLSPARSE)
		_file=$2; xerrmsg -M vxvmshm:3474 \
"Sparse plexes present in disk group.
  The vxcdsconvert program requires that no sparse plexes exist in the
  diskgroup. The following list contains volumes with plexes that are
  sparse. These plexes must be either removed or made complete before 
  running vxcdsconvert.
  List itmes are in the form vol_name plex_name.
"
		cat $_file | sed -e 's/^/	/' >&2
		echo "" >&2
		return
		;;
	E_CONFIG_DIS_KSTATE)
		_file=$2; xerrmsg -M vxvmshm:3472 \
"Objects with invalid kernel state present in disk group.
  The vxcdsconvert program requires objects in the disgroup be usable.
  The following list contains objects that are disabled.
  These objects must be enabled before running vxcdsconvert.
  Try starting any top level objects.
  List itmes are in the form obj_name obj_type kernel_state.
"
		cat $_file | sed -e 's/^/	/' >&2
		echo "" >&2
		return
		;;
	E_CONFIG_VOLNOTACTIVE)
		_file=$2; xerrmsg -M vxvmshm:3480 \
"VOLUMES with invalid state present in disk group.
  The vxcdsconvert program requires volumes in the diskgroup be ACTIVE.
  The following list contains volumes that are not active.
  These volumes must be made active before running vxcdsconvert.
  Try performing any needed resynchronization.
  List itmes are in the form vol_name vol_state.
"
		cat $_file | sed -e 's/^/	/' >&2
		echo "" >&2
		return
		;;
	E_CONFIG_R5LOGLEN)
		_file=$2; export dg; xerrmsg -M vxvmshm:3473 \
"RAID volumes with incompatible log length present in disk group.
  The vxcdsconvert program requires that RAID volumes have a log length
  that is a multiple of 8K.
  The following list contains volumes that do not meet this prerequisite.
  These volumes must be stopped and the log length set to a suitable value.
  List itmes are in the form vol_name new_loglen.
  Set the RAID log length for each volume in the list by performing
  the following commands after stopping applications using the volumes.

    vxvol -g $dg stop vol_name
    vxassist -g $dg remove log vol_name
    vxassist -g $dg addlog vol_name loglen=new_loglen
    vxvol -g $dg start vol_name
"
		cat $_file | sed -e 's/^/	/' >&2
		echo "" >&2
		return
		;;
	E_PROG_RUNNING)
		xerrmsg -M vxvmshm:3292 \
"The vxcdsconvert program is already being run.  Please retry this at a
  later time."
		;;
	E_LOCKFILE_EXISTS)
		_file=$2; export _file; xerrmsg -M vxvmshm:3085 \
"A previous invocation of the vxcdsconvert program appears to have
  terminated abnormally.  If you are certain that this program is not currently
  running, you can remove the file:

       $_file

  Removing this file will allow you to run vxcdsconvert. You can examine the
  contents of the file to find out what the previous invocation was."
		;;
	E_CONFIG_EFIDISKS)
		_file=$2; xerrmsg -M vxvmshm:4135 \
"EFI disk can not be included in conversion.
  Following are the efi disk.
"
		cat $_file
		echo "" >&2
		return
		;;
	*)	internal_err_exit "Unknown Error Code $1"
		;;
	esac
}
err_exit() {
	print_err "E_ABORT"
	quit 1
}
print_err_exit() {
	print_err "$1"
	err_exit
}
#
# internal_error_exit - exit after printing out an internal error
#
# Usage: internal_error_exit debug-info
#
internal_err_exit() {
	print_debug "$@"
	print_err_exit "E_INTERNAL"
}
#
# doit - execute a command; printing the command in verbose mode, exit on fail
#	 if requested
#
# Usage: doit [-n] einfo cmd [arg ...]
#	where einfo is a string containing info to be passed to print_err; a
#		null-string indicates nothing to print
#	-n indicates not to exit on error
#
# NOTE: for the exit on error to work, the function cannot be called in a
# subshell. So for instance, don't do: var=`doit "E_xxx" cmd`.
#
doit() {
	if [ "-n" = "$1" ]
	then
		_noexit=1
		shift
	else
		_noexit=
	fi
	_einfo="$1"
	shift
	print_verbose EXEC: "$@"
	if [ "on" = "$verbose" -o "on" = "$debug" ]
	then
		"$@"
		res=$?
	else
		"$@" 2>/dev/null
		res=$?
	fi
	if [ $res -ne 0 ]
	then
		if [ -z "$_noexit" ]
		then
			[ -z "$_einfo" ] && exit 1
			print_err_exit "$_einfo"
		fi
		[ -n "$_einfo" ] && print_err "$_einfo"
	fi
	return $res
}
#
# attrval_num - extract the numeric value for a given attribute from a given
#		 attribute=value string where value is based on strtovoff
#
# Usage: attrval_num ret-variable attrval-exp attribute min-val max-val
#		use of "" for min-val or max-val indicates "none"
#
attrval_num() {
	$set_x
	[ $# -ne 5 ] && {
		internal_err_exit "attrval_num arg count ($*)"
	}
	_attrval=$2
	_attr=$3
	_min=$4
	_max=$5

	_val=`echo "$_attrval" | sed -e "s/^${_attr}=\([a-zA-Z0-9]*\)$/\1/"`
	if [ -z "$_val" -o "$_val" = "$_attrval" ]; then
		print_err "E_BADVAL_ATTR $_attrval"
		return 1
	fi
	if [ "$debug" = "on" ]; then
		_num=`strtovoff -- "$_val" 2>&4`
	else
		_num=`strtovoff -- "$_val" 2>/dev/null`
	fi

	if [ -z "$_num" ]; then
		print_err "E_POSINT_ATTR $_attrval"
		return 1
	fi
	if [ -n "$_min" ]; then
		_mv=`strtovoff -- "$_min"`

		if [ $_num -lt $_mv ]; then
			print_err "E_MININT_ATTR $_attrval $_min"
			return 1
		fi
	fi
	if [ -n "$_max" ]; then
		_mv=`strtovoff -- "$_max"`

		if [ $_num -gt $_mv ]; then
			print_err "E_MAXINT_ATTR $_attrval $_max"
			return 1
		fi
	fi
	eval "$1=$_num"
	return 0
}
#
# attrval_enum - extract the value for a given attribute from a given
#		 attribute=value string for an enumerated-type attribute
#
# Usage: attrval_enum ret-variable attrval-exp attribute enum-egrep-regexp
#
attrval_enum() {
	$set_x
	[ $# -ne 4 ] && internal_err_exit "attrval_enum arg count ($*)"

	_attrval=$2
	_attr=$3
	_val=`echo "$_attrval" | sed -e "s/^${_attr}=\([a-zA-Z0-9]*\)$/\1/"`
	if [ -z "$_val" -o "$_val" = "$_attrval" ]; then
		print_err "E_BADVAL_ATTR $_attrval"
		return 1
	fi
	_good=`egrep "$4" <<-!
		$_val
		!`
	if [ "$_val" != "$_good" ]; then
		print_err "E_BADVAL_ATTR $_attrval"
		return 1
	fi
	eval "$1=$_val"
	return 0
}
#
# attrval_list - extract the list of (comma separated items) for a given
#		 attribute from a given attribute=value string 
#
# Usage: attrval_list ret-variable attrval-exp attribute
#
attrval_list() {
	$set_x
	[ $# -ne 3 ] && {
		internal_err_exit "attrval_list arg count ($*)"
	}
	_var=$1
	_attrval="$2"
	_attr=$3
	#
	# catch any use of quotes or white-space
	#
	echo "$_attrval" | grep "['\" 	]" > /dev/null
	if [ $? -eq 0 ] ; then
		print_err "E_BADVAL_ATTR \"$_attrval\""
		return 1
	fi
	#
	# catch missing "=" or <value>
	#
	_val=`echo "$_attrval" | sed -e "s/^${_attr}=\(.*\)$/\1/"`
	if [ -z "$_val" -o "$_val" = "$_attrval" ]; then
		print_err "E_BADVAL_ATTR \"$_attrval\""
		return 1
	fi
	#
	# catch any use of commans at begin/end
	#
	echo $_val | egrep "^,|,$" >/dev/null
	if [ $? -eq 0 ] ; then
		print_err "E_BADVAL_ATTR \"$_attrval\""
		return 1
	fi
	#
	# remove commas
	#
	_val=`echo $_val | sed -e 's/,/ /g'`
	#
	# Remove any DM not in the DG for evac_disk_list
	#
	[ "evac_disk_list" = "$_attr" ] && {
		_list=
		for _dm in $_val
		do
			vxprint -g $dg -d $_dm >/dev/null 2>&1
			if [ $? -ne 0 ]; then
				echo "$bad_evac_list" | grep $_dm >/dev/null
				[ 0 -ne $? ] && {
					print_debug "Discarding $_dm from evac_disk_list"
					bad_evac_list="$bad_evac_list $_dm"
				}
				continue
			fi
			if [ -z "$_list" ]
			then
				_list="$_dm"
			else
				_list="$_list $_dm"
			fi
		done
	}
	eval "$_var=\"$_list\""
	return 0
}
#
# attrval - extract the value for an attribute from a given attribute=value
#		string and store the value in the attributes associated
#		global variable
#
# Usage: attrval attrval-exp
#
attrval() {
	$set_x
	[ $# -ne 1 ] && internal_err_exit "attrval: arg count ($*)"
	#
	# Extract attribute from string.
	#
	_av=$1
	_attr=`echo "$_av" | sed -e 's/^\([a-z_]*\)=.*$/\1/'`
	if [ -z "$_attr" -o "$_attr" = "$_av" ]; then
		print_err "E_BAD_ATTRVAL $_av"
		return 1
	fi
	#
	# Currently the global variable names are the same as thier
	# corresponding attribute name.
	#
	_var=$_attr
	#
	# Determine which attribute was specified and its associated
	# validation info, and validate the specified value.
	#
	case $_attr in
	evac_subdisks_ok) 
		attrval_enum $_var "$_av" "$_attr" "yes|no" 
		return $?
		;;
	min_split_size)
		attrval_num $_var "$_av" "$_attr" 1 ""
		return $?
		;;
	move_subdisks_ok)
		attrval_enum $_var "$_av" "$_attr" "yes|no" 
		return $?
		;;
	privlen)
		attrval_num $_var "$_av" "$_attr" "512k" "256m" || return $?
		[ 0 -ne `expr $privlen % $_8K` ] && {
			print_err "E_PRIVLEN_8K $_av"
			return 1
		}
		return 0
		;;
	split_subdisks_ok)
		attrval_enum $_var "$_av" "$_attr" "yes|move|evac|no" 
		return $?
		;;
	evac_disk_list)
		bad_evac_list=""
		attrval_list $_var "$_av" "$_attr"
		return $?
		;;
	*)	print_err "E_UNKOWN_ATTR $_av" 
		return 1
		;;
	esac
	[ $# -ne 1 ] && internal_err_exit "attrval: unreachable"
}
#
# parse_defaults_file - parse attributes from a defaults file
#
# Usage: parse_defaults_file filename
#
parse_defaults_file() {
	$set_x
	_file=$1

	if [ ! -f $_file ]; then
		[ "$_file" = "$sys_dflt_file" ] && return

		print_err_exit "E_BAD_FILE $_file"
	fi
	_valid=0
	for _df_attr in $attr_list
	do
		_df_attrval=`egrep "^${_df_attr}" \$_file | tail -1`

		[ -z "$_df_attrval" ] && continue

		attrval "$_df_attrval"
		[ $? -ne 0 ] && print_err_exit "E_SYNTAX $_file"

		_valid=`expr $_valid + 1`
	done
	_entries=`grep -v "^#" $_file | wc -l | awk '{print $1}'`

	[ "$_valid" != "$_entries" ] && print_err_exit "E_SYNTAX_DUP $_file"
}
#
# debug_print_attributes - print out settings for all attributes if debug=on
#
# Usage: debug_print_attributes
#
debug_print_attributes() {

	[ "$debug" != "on" ] && return

	print_debug "Begin: ATTRIBUTE SETTINGS"
	for _attr in $attr_list
	do
		eval "_val="\$$_attr
		print_debug "${_attr}=${_val}"
	done
	print_debug "End: ATTRIBUTE SETTINGS"
}
#
# check_and_init_env - check and init the operational environment
#
# Usage: check_and_init_env "cmd-line"
#
check_and_init_env() {
	#
	# _BSIZE must be less than 1K (for script)
	#
	[ $_BSIZE -gt 1024 ] && print_err_exit "E_UNSUPPORT_BSIZE $_BSIZE"
	#
	# For ease of "use", make sure PATH contains /etc/vx/diag.d
	# NOTE: /usr/bin, /usr/sbin, and /etc/vx/bin are done in common.sh
	#
	inpath=`echo "${PATH}:" | grep "${VOL_DIAG_DIR}:"`
	[ -z "$inpath" ] && PATH="$PATH:${VOL_DIAG_DIR}"
	#
	# Make sure another copy of the converter is not running. Then set
	# the lock file to prevent others from running.
	#
	if [ -f $lock_file ]
	then
		pid=`head -1 $lock_file 2>&1 | awk '{print $1}'`
		if [ "SunOS" = "$platform" ]
		then
			ps_opt="-f -p $pid"
		else
			ps_opt="-lf -p $pid"
		fi
		ps $ps_opt 2>/dev/null | grep "vxcds_co" >/dev/null 2>&1
		if [ $? -eq 0 ]
		then
			print_err_exit "E_PROG_RUNNING"
		fi
		print_err_exit "E_LOCKFILE_EXISTS $lock_file"
	fi
	add_cleanup local_cleanup
##	trap "quit 1" INT HUP QUIT TERM

	#echo "$$\n$1" > $lock_file 2>/dev/null
	{ echo "$$"; echo "$1"; }  > $lock_file 2>/dev/null
	[ $? -ne 0 ] && print_err_exit "E_FILE_CREATE $lock_file"
}
#
#------
# MAIN
#------
. $VOLADM_LIB/vxcds_diskconv.sh
. $VOLADM_LIB/vxcds_dgconv.ksh

cmd_supported $0

#
[ $# -lt 1 ] && usage_exit "E_NO_DG"
check_and_init_env "$0 $*"
#
# process flags
#
while getopts :ADHVXd:g:o: c
do
	case $c in
	A)	analyze_only="y" ;;
	D)	debug="on" ;;
	X)	set -x; set_x="set -x" ;;
	H)	fullusage_exit ;;
	V)	verbose="on" ;;
	d)	usr_dflt_file=$OPTARG ;;
	g)	dg=$OPTARG ;;
	o)	[ "novolstop" != "$OPTARG" ] && usage_exit "E_OOPT_INV $OPTARG"
		novolstop="y"
		;;
	?)	usage_exit "E_BAD_OPTION $c"
		;;
	esac
done
shift `expr $OPTIND - 1`

[ -z "$dg" ] && print_err_exit "E_NO_DG"
dgvers=`vxprint -g $dg -G -F %version 2>/dev/null`
res=$?
if [ $res -ne 0 ]
then
	[ $res -eq 11 ] && print_err_exit "E_DG_NOEXIST"
	print_err_exit "E_DG_INFO"
fi
dgshared=
[ X"`vxdg list $dg | grep '^flags:.*shared'`" != X ] && dgshared=true
#
# Verify the DG version of the DG. This command only supports DG of rev 110
# and later. If the DG version is less, try to upgrade the DG version first.
#
if [ $dgvers -lt 110 ]
then
	if [ "y" = "$analyze_only" ]
	then
		out=`egettxt "Disk group will be upgraded to latest version" \
			 vxvmshm:3138`
		print_anal "$out"
	else
		vxdg upgrade $dg 2>/dev/null
		dgvers=`vxprint -g $dg -G -F %version 2>/dev/null`
		[ -z "$dgvers" -o $dgvers -lt 110 ] && {
			print_err_exit "E_DG_UPGRADE"
		}
	fi
fi
#
# process keyword
#
[ $# -lt 1 ] && usage_exit "E_NO_KEYWORD"
keyword=$1
shift
case $keyword in
alignment|alldisks|group)
	;;
disk)	[ $# -lt 1 ] && usage_exit "E_NO_DISKNAME"
	disk_name=$1
	shift
	;;
*)	usage_exit "E_BAD_KEYWORD $keyword"
	;;
esac
#
# process attributes
#
# 	Command-line attribute specs should supercede defaults file ones.
#	User specified defaults file attribute specs should supercede system
#	defaults file ones.  If have non-empty bad_evac_list, tell user
#	Also, disallow evac if have empty list and non-empty bad_evac_list,
#	and inform user.
#
parse_defaults_file $sys_dflt_file
[ -n "$usr_dflt_file" ] && parse_defaults_file $usr_dflt_file

while [ $# -gt 0 ]
do
	attrval "$1"
	[ $? -ne 0 ] && err_exit
	shift
done
if [ "yes" = "$evac_subdisks_ok" -a -n "$bad_evac_list" ]
then
	for disk in $bad_evac_list
	do
		export disk; ugettxt -n vxcdsconvert -s NOTICE "Ignoring $disk in 'evac_disk_list' since disk not in disk group" "vxvmshm:3382" "" >&2

	done
	[ -z "$evac_disk_list" ] && {

		export disk; ugettxt -n vxcdsconvert -s NOTICE "Setting 'evac_subdisks_ok' to 'no': invalid disks in 'evac_disk_list'" "vxvmshm:3385" "" >&2
		evac_subdisks_ok="no"
	}
fi
debug_print_attributes
#
# If shared diskgroup, validate proper role
#
if [ -n "$dgshared" ]
then
	case `vxdctl -c mode` in
		*SLAVE*)
			print_err_exit "E_DG_ONSLAVE" ;;
	esac
fi
#
# Initialize things as needed.
#
dgconv_init
#
# Do the requested operation.
#
case $keyword in
alignment)	dgconv alignment || quit 1
		;;
alldisks)	diskconv all || quit 1
		;;
disk)		diskconv one $disk_name || err_exit
		;;
group)		#
		diskconv group || err_exit
		dgconv group || quit 1
		;;
*)		usage_exit "E_BAD_KEYWORD $keyword"
		;;
esac
#
quit $?
