#
# $Id: vxcds_dgconv.ksh,v 1.1.2.19.10.3 2004/09/27 23:31:59 hhorn Exp $
#ident "$Source: /project/unixvm-cvs/src/common/cmd/vxvm/voladm/vxcds_dgconv.ksh,v $ $Revision: 1.1.2.19.10.3 $"
#
# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

#-----------------
# Global Variables 
#-----------------
# See converter.sh for additional globals.
#

# comment for new sd 
vxc_comment=
vxc_comment_sdlog=
vxc_comment_pllog=
# prefixes/suffixes for files created by dg converter
tprefix=
tsuffix=
# temporary files used
tmp1=
tmp2=
# files used for temporary storage only - should not be expected to remain
# intact across function calls
tmp9=
tmp10=
# used for 'read' support (required for bash)
# Use temp file to workaround the fact that bash can't
# do "read var ..." at the end of a pipeline and have
# var values which are read be persistent (due to the fact
# that the the read is done in a subshell)
tmp11=  # used for 'read' support (required for bash)
tmp12=	# used for 'read' support (required for bash)
tmp13=	# used for 'read' support (required for bash)
tmp14=	# used for 'read' support (required for bash)
tmp15=	# used for 'read' support (required for bash)
# files used to hold info regarding the starting configuration
# plex info
dis_objs=
badstate_pls=
state_objs=
recov_vols=
dis_kstate_any=
sparse_pls=
badr5_loglen=
# sd info
active_sds=
raidlog_sds=
active_sds_tmp=
relayout_altstdin=
active_sdlba=
action_vols=
startstop_vols=
# dm info
# in this dir are ${dm}.info files and ${dm}.bounds files
dir_dms=
array_names=	# array containing names of DM free space arrays
# EFI disks
efi_disks=
#
# Variables associated with the subdisk currently being processed
#
rl_realign=
rl_restrict=
rl_resize=
#
# Variables associated with the plex column currently being processed
#
rl_copylen_in=
rl_copylen_out=
rl_copyop_in=
rl_copyop_out=
rl_copyterm_in=
rl_copyterm_out=
rl_sd_in=
rl_sd_out=
rl_sdjoin_in=
rl_sdjoin_out=
#
# Other variables associated with subdisk relayout
#
rl_nsd=0
rl_use_allocator=
rl_skip=
rl_skip_match=
rl_skip_err=0
rl_reterr=0

#
# Other dgconv variables
#
dgconv_enable_copyfwd=
p_list=	# for making a printable list
a_list=	# list for use in making a printable list

#
#---------------------------------------
# Functions for use with disk conversion
#
# NOTE: main entry point is dgconv()
#---------------------------------------
#

#
# extent_overlap - test for overlapping extents
# Usage: extent_overlap off1 len1 off2 len2
#	testing for contiguous extexts must be done separately
#	identical extents are overlapping
#	first contained in second are overlapping
#	second contained in first are overlapping
#
function extent_overlap
{
    $set_x
    typeset x1 y1 x2 y2 ret
    
    [[ $# -eq 4 ]] || {
	internal_err_exit "extext_overlap: arg count ($*)"
    }
    
    x1=$1
    y1=$2
    x2=$3
    y2=$4

    ret=$(( x1 < x2 + y2 && x2 < x1 + y1 ))
    return $((! ret))
}

#
# extent_contig - test for contiguous extents
# Usage: extent_contig off1 len1 off2 len2
#
function extent_contig
{
    $set_x
    typeset x1 y1 x2 y2 ret
    
    [[ $# -eq 4 ]] || {
	internal_err_exit "extext_contig: arg count ($*)"
    }
    
    x1=$1
    y1=$2
    x2=$3
    y2=$4

    ret=$((x1 == x2 + y2 || x2 == x1 + y1))
    return $((! ret))
}

#
# extent_bounded - test if the first extent is completely
#	bounded by the second extent.
#	"bounded" can also be "identical"
# Usage: extent_bounded off1 len1 off2 len2
#
function extent_bounded
{
    $set_x

    typeset x1 y1 x2 y2 ret
    
    [[ $# -eq 4 ]] || {
	internal_err_exit "extext_bounded: arg count ($*)"
    }
    
    x1=$1
    y1=$2
    x2=$3
    y2=$4

    ret=$((x1 >= x2 && x1 + y1 <= x2 + y2))
    return $((! ret))
}


# alignup - align a value to the next alignment boundary
# Usage: alignup value align
function alignup
{
#	$set_x
	typeset value align

	[[ $# -eq 2 ]] || {
	    internal_err_exit "alignup: arg count ($*)"
	}
	value=$1
	align=$2

	echo $(( ((value + align - 1) / align) * align ))
}

# alignup_len - return a number which when added to
#	value will cause it to be aligned
# Usage: alignup_len value align
function alignup_len
{
#	$set_x
	typeset value align

	[[ $# -eq 2 ]] || {
	    internal_err_exit "alignup_len: arg count ($*)"
	}
	value=$1
	align=$2

	value=$((value % align))
	[[ $value -gt 0 ]] && value=$((align - value))
	echo $value
}

#
# aligndown_len - return a number which when subtracted from
#	value will cause it to be aligned
#
# Usage: aligndown_len value align
#
#
function aligndown_len
{
#	$set_x
	typeset value align

	[[ $# -eq 2 ]] || {
	    internal_err_exit "aligndown_len: arg count ($*)"
	}
	value=$1
	align=$2

	echo $((value % align))
}

#
# sdlba - find LBA offset of sdnames provided on stdin
#	and pint them on stdout
# OUTPUT: lines of the form:
#	sd_name dm_name da_name sd_lbaoff sd_len
#
function sdlba
{
    $set_x
    typeset disk_offset sd_lbaoff name dev_offset dm_name da_name len
    typeset ret=0

    # this will include subvolumes
    xargs vxprint -g $dg -sF '%name %dev_offset %dm_name %da_name %len' >$tmp14
    while ((1))
    do
	read name dev_offset dm_name da_name len
	[[ $? -eq 0 ]] || break
	[[ X$da_name = X- ]] && continue

	# disk_offset is offset of public slice
	# dev_offset is offset of sd within public slice
	# on non-sliced platforms disk_offset is always 0
	disk_offset=`grep '^public:' $dir_dms/${dm_name}.info | awk '{ print $5 }'`
	[[ -n "$disk_offset" ]] || {
	    print_err "E_DM_INFO $dm_name"
	    ret=$EINTERNAL
	    continue
	}
	sd_lbaoff=$((disk_offset + dev_offset))
	echo $name $dm_name $da_name $sd_lbaoff $len
    done < $tmp14
    return $ret
}

#
# dm_config - configure DM properties
#	create the ${dm}.info and ${dm}.bounds file.
#	if publba/publen are provided they will define the
#	new public region and on-disk relocation will be contrained
#	to this region
# Usage: dm_config dm [publba publen]
#
function dm_config
{
    $set_x
    typeset dm publba publen
    typeset slice offset len disk_offset
    typeset adj

    [[ $# -eq 1 || $# -eq 3 ]] || internal_err_exit "dm_config: arg count ($*)"

    dm=$1
    [[ $# -eq 3 ]] && {
	publba=$2
	publen=$3
    }

    _da=`vxprint -g $dg -F %da_name -d $dm 2>/dev/null`
    [[ $? -eq 0 ]] || {
	print_err "E_DM_INFO $dm"
	return 1
    }
    vxdisk list $_da > $dir_dms/${dm}.info

    slice= offset= len= disk_offset=
    eval `cat $dir_dms/${dm}.info | grep '^public:' | sed -e 's/^public://'`
    [[ -n "$disk_offset" ]] || {
	print_err "E_DM_INFO $dm"
	return 1
    }

    if [[ -n "$publba" ]]
    then
	# if publba/publen are not constrained within disk_offset/len adjust
	extent_bounded $publba $publen $((disk_offset + offset)) $len
	if [[ $? -ne 0 ]]
	then
	    print_debug "dm_config: public region extent provided is not bounded by current public region"
	    print_debug "           the public region extent used will be the intersection of the two"
	    if extent_overlap $publba $publen $((disk_offset + offset)) $len
	    then
		if ((disk_offset + offset > publba))
		then
		    adj=$((disk_offset + offset - publba))
		    ((publba += adj))
		    ((publen -= adj))
		fi
		if ((disk_offset + offset + len < publba + publen))
		then
		    publen=$((disk_offset + offset + len - publba))
		fi
	    else
		publba=0
		publen=0
	    fi
	fi
    else
	publba=$((disk_offset + offset))
	publen=$len
    fi
    
    echo $dm $offset $disk_offset $((disk_offset + offset)) $len \
	$publba $publen \
	$((publba != disk_offset + offset || publen != len)) > $dir_dms/${dm}.bounds

    # get free space based on new disk bounds
    free_scan $dm
}
	  
#
# configure all DM properties in diskgroup
#
function dm_configall
{
    $set_x
    typeset ret=0 dm da

    vxprint -g $dg -Qqd | awk '{ print $2, $3 }' >$tmp12
    while ((1))
    do
	read dm da
	[[ $? -eq 0 ]] || break
	[[ X$da = X- ]] && {
	    # disabled disks are not an error in general.
	    # if they have disabled objects linked to them
	    # we error out in dgconv_config().
	    # print_err "E_DM_NO_DA $dm"
	    continue
	}
	dm_config $dm
	[[ $? -eq 0 ]] || ret=$?
    done < $tmp12
    return $ret
}

#
# recover - cleanup objects created by the diskgroup converter
#	There are 2 types of objects which are cleaned up
#	1) data subdisks
#		(comment = $vxc_comment)
#	2) DRL/DCM log subdisks/plexes
#		(comment = $vxc_comment_sdlog or $vxc_comment_pllog)
#
#	data subdisks:
#         - if the subdisk is not associated, remove it
#         - if the subdisk is associated, clear the comment
#	logs:
#         - if the object is accociated then clear the comment
#         - if the src plex/vol referred to by the comment exists and
#	    src volume has logmap_align >= 8K  then the src volume already
#	    had the logs rebuilt so remove all (unassociated) logs for
#	    referenceing the src volume
#         - if the src plex/vol has logmap_align < 8K reassociate the object
#	    with the src plex/vol (order the associations from smallest
#	    length to largest) then clear the comment
#        Note that reassociating log plexes/subdisks does not permit
#	    setting log length or map length so the resultant values
#	    may be different than the originals
#
# Usage: recover
#
# Note: this function will recover objects even if analyze_only is "yes"
#
function recover
{
    $set_x
    typeset name plex delete uncomment log_uncomment log_rm logs
    typeset log len parent ident rid
    typeset vol logalign cmd
	
    vxprint -g $dg -se 'comment="'$vxc_comment'"' -F '%name %assoc' >$tmp11
    while ((1))
    do
	read name plex
	[[ $? -eq 0 ]] || break
	if [[ X$plex = X- ]]
	then
	    delete="$delete $name"
	else
	    uncomment="$uncomment $name"
	fi
    done < $tmp11
    
    # vxc_comment_sdlog: ident:plex-rid
    # vxc_comment_pllog: ident:vol-rid
    (
	vxprint -g $dg -se 'comment~/^'$vxc_comment_sdlog':/' \
	    -F '%name %len %pl_name %comment' |
	    sed -e 's/:/ /g'
	vxprint -g $dg -pe 'comment~/^'$vxc_comment_pllog':/' \
	    -F '%name %len %v_name %comment' |
		    sed -e 's/:/ /g'
    ) | sort +3 -4 +1n -2 >$tmp12
    # input:
    #	log log-objlen log-parent ident log-fparent-rid
    # input is sorted first by volume name and then by log object length
    # so logging plexes (length 0) will be attached first followed by logging
    # subdisks of increasing size
    while ((1))
    do
	read log len parent ident rid
	[[ $? -eq 0 ]] || break
	[[ X$parent != X- ]] && {
	    # log is still accociated so just clear comment
	    log_uncomment="$log_uncomment $log"
	    continue
	}
	if [[ $ident = $vxc_comment_pllog ]]
	then
	    # associate the logging plex
	    vxprint -g $dg -ve 'rid=='$rid'' -F '%name %logmap_align' >$tmp11
	    read vol logalign < $tmp11
	    [[ $logalign -ge $_8K ]] && {
		# volume already has logs rebuilt so just delete the 
		# dissociated log
		log_rm="$log_rm $log"
		continue
	    }
	    logs="$logs $log"
	    doit -n "E_PLEX_ATT $log $vol" vxplex -g $dg att $vol $log
	    [[ $? -eq 0 ]] || return $EUTIL
	    log_uncomment="$log_uncomment $log"
	else
	    # associate the logging sd
	    vxprint -g $dg -pe 'rid=='$rid'' -F '%v_name %name' >$tmp11
	    read vol plex < $tmp11
	    logalign=`vxprint -g $dg -vF %logmap_align $vol`
	    [[ $logalign -ge $_8K ]] && {
		# volume already has logs rebuilt so just delete the 
		# dissociated log
		log_rm="$log_rm $log"
		continue
	    }
	    logs="$logs $log"
	    doit -n "E_PLEX_LOG_ASSOC $log $plex" \
		vxsd -g $dg aslog $plex $log
	    [[ $? -eq 0 ]] || return $EUTIL
	    log_uncomment="$log_uncomment $log"
	fi
    done < $tmp12

    [[ -n "$logs" ]] && print_debug "RECOV: Reattached logs: $logs"
    [[ -n "$delete" ]] && {	    
	cmd="vxedit -g $dg rm $delete"
	print_verbose ""
	print_verbose "Deleting temporary data subdisks used by vxcdsconvert ..."
	doit -n "E_SD_RM" $cmd
	[[ $? -eq 0 ]] || return $EUTIL
	print_debug "RECOV: Deleted subdisks: $delete"
    }

    [[ -n "$log_rm" ]] && {	    
	cmd="vxedit -g $dg -fr rm $log_rm"
	print_verbose ""
	print_verbose "Deleting logs dissociated by vxcdsconvert ..."
	doit -n "E_SD_RM" $cmd
	[[ $? -eq 0 ]] || return $EUTIL
	print_debug "RECOV: Deleted logs: $log_rm"
    }

    [[ -n "$uncomment" ]] && {
	cmd="vxedit -g $dg set comment= $uncomment"
	print_verbose ""
	print_verbose "Clearing data subdisk comment fields set by vxcdsconvert ..."
	doit -n "E_OBJ_EDIT `echo $uncomment | sed -e 's/ /,/g'`" $cmd
	[[ $? -eq 0 ]] || return $EUTIL
	print_debug "RECOV: Uncommented subdisks: $uncomment"
    }

    [[ -n "$log_uncomment" ]] && {
	cmd="vxedit -g $dg set comment= $log_uncomment"
	print_verbose ""
	print_verbose "Clearing log comment fields set by vxcdsconvert ..."
	doit -n "E_OBJ_EDIT `echo $log_uncomment | sed -e 's/ /,/g'`" $cmd
	[[ $? -eq 0 ]] || return $EUTIL
	print_debug "RECOV: Uncommented logs: $log_uncomment"
    }

    return 0
}
	
#
# dgconv_init_files - init file variables for the converter
#
function dgconv_init_files
{
    $set_x
	    
    dis_objs=${tprefix}dis_objs${tsuffix}		# dissociated objs
    badstate_pls=${tprefix}badstate_pls${tsuffix}	# plexes in invalid states
    state_objs=${tprefix}state_objs${tsuffix}		# object states
    recov_vols=${tprefix}recov_vols${tsuffix}		# enabled/!active vols
    dis_kstate_any=${tprefix}dis_kstate_any${tsuffix}	# disabled objs
    sparse_pls=${tprefix}sparse_pls${tsuffix}		# sparse plexes
    badr5_loglen=${tprefix}badr5_loglen${tsuffix}	# incompat r5 loglen
    efi_disks=${tprefix}efi_disks${tsuffix}		# EFI disks

    # then go through all the sds on the system.
    # If the sd is in the skip list skip it
    # otherwise the sd goes to the active_sds list and will be processed
    active_sds=${tprefix}active_sds${tsuffix}
    raidlog_sds=${tprefix}raidlog_sds${tsuffix}
    active_sds_tmp=${tprefix}active_sds_tmp${tsuffix}
    relayout_altstdin=${tprefix}relayout_altstdin${tsuffix}
    active_sdlba=${tprefix}active_sdlba${tsuffix}

    # list of volume which are modified by relayout
    action_vols=${tprefix}action_vols${tsuffix}
    # list of volume that need to be start/stopped
    startstop_vols=${tprefix}startstop_vols${tsuffix}
}

#
# dgconv_config - analyze the configuration and setup files for the
#	subdisk relayout engine
# 
function dgconv_config
{
    $set_x
    typeset err=0 cmd vol vols contiglen loglen len ret

    print_verbose "Performing object recovery in $dg ..."
    recover

    print_verbose "Inspecting objects in $dg ..."

    # turn off user transactions while gathering config
    cmd="vxdg -g $dg set no_utrans=on"
    doit -n "E_DG_UTRANS off" $cmd
    # it's not fatal if we can't turn off user transactions

    # check for dissociated objects
    vxprint -g $dg -e 'assoc=""' -F %name > $dis_objs
    # check for plexes in invalid states
    vxprint -g $dg -pe '
	assoc!="" &&
	    ( ( pl_v_name!="" && pl_state!="ACTIVE" && 
	        pl_state!="CLEAN" && pl_state!="EMPTY" &&
	        pl_state!="LOG" ) ||
              pl_kstate!=PL_ENABLED )' \
	-F '%v_name %name %state %kstate' > $badstate_pls

    # get all objs
    vxprint -g $dg -F '%name %rectype %state %kstate %assoc' > $state_objs
    # objs w/ disabled/detached kernel state
    cat $state_objs | 
	awk '$4 == "DISABLED" || \
	     $4 == "DETACHED" { print $1,$2,$4; }' > $dis_kstate_any
    # vols w/ ENABLED kstate and !ACTIVE state
    cat $state_objs | 
	awk '$2 == "vol" && $4 == "ENABLED" && $3 != "ACTIVE" \
		{ print $1,$3; }' > $recov_vols

    # check for sparse plexes
    vxprint -g $dg -pe 'pl_contig_len > 0 && 
	pl_contig_len < assoc.len && pl_state!="LOG"' \
	-F '%v_name %name' > $sparse_pls
    # check for raid5 loglen that is not a multiple of 8K
    # it's not worth changing VxVM to allow setting the raid5 loglen
    # for this unlikely case so we simply fail if we encounter it
    vxprint -g $dg -pe 'pl_raidlog' -F '%v_name %pl_name %contig_len' |
	sort +0 -1 +2n -3 > $tmp9
    vols=`cat $tmp9 | awk '{print $1}' | sort -u`
    for vol in $vols
    do
	loglen=`vxprint -g $dg -vF '%loglen' $vol`
	if (( loglen % $_8K == 0 ))
	then
	    continue
	fi
	# raid5 loglenth is not an 8K multiple so we need to fail
	# but first determine what it should be set to based on the smallest
	# contig_len of the associated logs
	contiglen=`grep "^$vol " $tmp9 | awk '{print $3}' | \
	    head -1`	# shortest contig_len
	len=`alignup_len $loglen $_8K`
	if ((loglen + len >  contiglen))
	then
	    ((loglen -= (_8K - len) ))
	else
	    ((loglen += len))
	fi
	echo $vol $loglen
    done > $badr5_loglen

	danames=`cat $state_objs | awk '$2 == "dm" {print $5}'`
	rm $efi_disks 2>/dev/null
    for da_disk in $danames
    do
 		if cdsconv_is_efi_disk $da_disk
		then
			echo "$da_disk" >> $efi_disks
		fi
	done

    # report configuration errors
    [[ -s $dis_objs ]] && {
	print_err "E_CONFIG_DISOBJ $dis_objs"
	err=$ECONFIG
    }
    [[ -s $badstate_pls ]] && {
	print_err "E_CONFIG_PLSTATE $badstate_pls"
	err=$ECONFIG
    }
    [[ -s $sparse_pls ]] && {
	print_err "E_CONFIG_PLSPARSE $sparse_pls"
	err=$ECONFIG
    }
    [[ -s $badr5_loglen ]] && {
	print_err "E_CONFIG_R5LOGLEN $badr5_loglen"
	err=$ECONFIG
    }
    [[ -s $dis_kstate_any ]] && {
	print_err "E_CONFIG_DIS_KSTATE $dis_kstate_any"
	err=$ECONFIG
    }
    [[ -s $recov_vols ]] && {
	print_err "E_CONFIG_VOLNOTACTIVE $recov_vols"
	err=$ECONFIG
    }
    [[ -s $efi_disks ]] && {
	print_err "E_CONFIG_EFIDISKS $efi_disks"
	err=$ECONFIG
    }

    [[ $err -eq 0 ]] || {
	# turn on user transactions before returning
	cmd="vxdg -g $dg set no_utrans=off"
	doit -n "E_DG_UTRANS on" $cmd
	# it's not fatal if we can't turn on user transactions
	# return the original error
	return $err
    }
    
    # generate a list of subdisks to convert which excludes subvolumes
    # and RAID logging sds
    #
    # vxprint -g $dg -ts output lists subvolumes as 'sd' so use
    # vxprint -g $dg -t output instead
    vxprint -g $dg -Qqht | grep '^sd ' | awk '{ print $2 }' > $tmp9
    vxprint -g $dg -se 'assoc.pl_raidlog' -F %name > $raidlog_sds
    cat $raidlog_sds | sed -e 's,.*,/^&$/d,' > $tmp10
    cat $tmp9 | sed -f $tmp10 > $active_sds

    print_verbose "Ascertaining LBA offset of subdisks ..."

    cat $active_sds | sdlba > $active_sdlba

    # turn on user transactions before returning
    cmd="vxdg -g $dg set no_utrans=off"
    doit -n "E_DG_UTRANS on" $cmd
    # it's not fatal if we can't turn on user transactions

    print_verbose "Gathering free space info ..."

    free_scan

    return 0
}

# list manipulation functions
# A list is defined by a shell variable that uses spaces to separate
# items in the list.  These items are numbered starting at 1.

# list_set list_name item ... - set the items in a list
function list_set
{
	$set_x
	typeset _list_set_list

	# translate any dashes in name to underscores
	_list_set_list=`echo $1 | tr "-" "_"`; shift
	eval "set -A $_list_set_list $*"
}

# list_set list_name item ... - set the items in a list
function list_set_bash
{
	$set_x
	typeset _list_set_list

	# translate any dashes in name to underscores
	_list_set_list=`echo $1 | tr "-" "_"`; shift
	eval "$_list_set_list=($*)"
}

# list_append list_name item ... - append items to a list
function list_append
{
	$set_x
	typeset _list_append_list

	# translate any dashes in name to underscores
	_list_append_list=`echo $1 | tr "-" "_"`; shift
	eval "set -A $_list_append_list \${$_list_append_list[*]} $*"
}

# list_append list_name item ... - append items to a list
function list_append_bash
{
	$set_x
	typeset _list_append_list

	# translate any dashes in name to underscores
	_list_append_list=`echo $1 | tr "-" "_"`; shift
	eval "$_list_append_list=(\${$_list_append_list[*]} $*)"
}

#
# list_member - test if item is a member of the list
#	 set $index or $setvar to the index of the item found
#	 array indexes are 1-based
# Usage: list_member list_name item [setvar] 
#
list_member()
{
	$set_x
	typeset _list_member_list _list_member_want _list_member_setvar
	typeset _list_member_vals _list_member_i

	# translate any dashes in name to underscores
	_list_member_list=`echo $1 | tr "-" "_"`
	_list_member_want=$2
	_list_member_setvar=index
	[ $# -gt 2 ] && _list_member_setvar=$3
	eval "$_list_member_setvar="
	eval "set -A _list_member_vals \${$_list_member_list[*]}"
	_list_member_i=0
	while (( _list_member_i < ${#_list_member_vals[*]}))
	do
		if [[ "X$_list_member_want" = \
		      "X${_list_member_vals[$_list_member_i]}" ]]
		then
			eval "$_list_member_setvar=$((_list_member_i+1))"
			return $?
		fi
		(( _list_member_i += 1 ))
	done
	eval "$_list_member_setvar=0"
}

#
# list_member - test if item is a member of the list
#	 set $index or $setvar to the index of the item found
#	 array indexes are 1-based
# Usage: list_member list_name item [setvar] 
#
list_member_bash()
{
	$set_x
	typeset _list_member_list _list_member_want _list_member_setvar
	typeset _list_member_vals _list_member_i

	# translate any dashes in name to underscores
	_list_member_list=`echo $1 | tr "-" "_"`
	_list_member_want=$2
	_list_member_setvar=index
	[ $# -gt 2 ] && _list_member_setvar=$3
	eval "$_list_member_setvar="
	eval "_list_member_vals=(\${$_list_member_list[*]})"
	_list_member_i=0
	while (( _list_member_i < ${#_list_member_vals[*]}))
	do
		if [[ "X$_list_member_want" = \
		      "X${_list_member_vals[$_list_member_i]}" ]]
		then
			eval "$_list_member_setvar=$((_list_member_i+1))"
			return $?
		fi
		(( _list_member_i += 1 ))
	done
	eval "$_list_member_setvar=0"
}

#
# list_count - return the number of items in the list
#	return the count in $count or in $setvar
# Usage: list_count list_name [setvar] 
#
function list_count
{
	$set_x
	typeset _list_count_setvar _list_count_list

	_list_count_setvar=count
	# translate any dashes in name to underscores
	_list_count_list=`echo $1 | tr "-" "_"`
	[[ $# -gt 1 ]] && _list_count_setvar=$2
	eval "$_list_count_setvar=\${#$_list_count_list[*]}"
}

#
# list_fetch - fetch an indexed item from a list
#	return the fetched item in $item, or in $setvar
#	array indexes are 1-based
# Usage: list_fetch list_name item_index [setvar]
#
list_fetch()
{
	$set_x
	typeset _list_fetch_setvar _list_fetch_list
	typeset _list_fetch_index _list_fetch_argc

	_list_fetch_setvar=item
	# translate any dashes in name to underscores
	_list_fetch_list=`echo $1 | tr "-" "_"`
	_list_fetch_index=$2
	[ $# -gt 2 ] && _list_fetch_setvar=$3
	eval "_list_fetch_argc=\${#$_list_fetch_list[*]}"
	if [ "$_list_fetch_index" -gt "$_list_fetch_argc" ] || 
		[ "$_list_fetch_index" -lt 1 ]
	then
		eval "$_list_fetch_setvar="
		p_list=`egettxt "list_fetch: ERROR! index out of bounds" \
		    vxvmshm:3428`
		print_anal "$p_list"
		return $EINTERNAL
	fi
	eval "$_list_fetch_setvar=\${$_list_fetch_list[(($_list_fetch_index-1))]}"
	return 0
}

#
# list_store - store item into indexed element in list
#	array indexes are 1-based
# Usage: list_store list_name item_index item
#
list_store()
{
	$set_x
	typeset _list_store_list _list_store_index _list_store_item
	typeset _list_store_counter _list_store_argc

	# translate any dashes in name to underscores
	_list_store_list=`echo $1 | tr "-" "_"`
	_list_store_index=$2
	_list_store_item=$3
	_list_store_counter=
	eval "_list_store_argc=\${#$_list_store_list[*]}"
	if [ "$_list_store_index" -gt "$_list_store_argc" ] ||
		[ "$_list_store_index" -lt 1 ]
	then
		p_list=`egettxt "list_store: ERROR! index out of bounds" \
		   vxvmshm:0`
		print_anal "$p_list" 
		return $EINTERNAL
	fi
	eval "$_list_store_list[(($_list_store_index-1))]=\"$_list_store_item\""
}

# Usage: free_scan [dm]
function free_scan
{
    $set_x
    typeset dm junk off len list

    [[ -n "$1" ]] && list=$1

    free_free $list
    vxdg -g $dg -q free $list >$tmp13
    while ((1))
    do
	read dm junk junk off len junk
	[[ $? -eq 0 ]] || break
	free_add $dm $off $len
    done < $tmp13
}	

	
#
# free_free - free all (or named) free space list
# Usage: free_free [space]
#
function free_free
{
    $set_x
    typeset namecnt i list match tmplist

    match="$1"

    list_count array_names namecnt
    i=1
    while (( i <= namecnt ))
    do
	list_fetch array_names $i list
	(( i += 1 ))
	if [[ -n "$match" && X$match != X$list ]]
	then
	    list_append tmplist $list
	    continue
	fi
	list_set $list ""
    done
    list_set array_names ${tmplist[*]}
}

#
# free_add - adds free space for disk dm of off/len
# Usage: free_add [-lba] dm off len
#	-lba will cause offset to be interpreted as an LBA
#	otherwise it will be read as priv-region relative
# Note: the off/len of the free space created may not be identical
#	to the parameters provided as they will be adjusted to meet
#	pub region restrictions.
#
function free_add
{
    $set_x
    typeset list off len
    typeset idx adj=0 off_max
    typeset cds_off cds_len pub_lbaoff
    typeset lbaflg=0

    [[ X$1 = X-lba ]] && {
	# interpret offset as LBA
	lbaflg=1
	shift
    }

    list=$1
    off=$2
    len=$3

    [[ -s $dir_dms/${list}.bounds ]] || {
	internal_err_exit free_add: DM boundary file $dir_dms/${list}.bounds missing
    }
    tail -1 $dir_dms/${list}.bounds | awk '{ print $4, $6, $7 }' >$tmp11
    read pub_lbaoff cds_off cds_len < $tmp11

    [[ $lbaflg -eq 0 ]] && {
	((off += $pub_lbaoff))
    }

    list_member array_names $list idx
    if [[ $idx -eq 0 ]]
    then
	# new dm in free space
	list_append array_names $list
    fi

    # validate that off/len do not violate CDS disk restrictions
    # adjust if neccessary
    if extent_bounded $off $len $cds_off $cds_len
    then
	# the free region can be added as-is
	:
    elif extent_overlap $off $len $cds_off $cds_len
    then
	# adjust off/len to conform to disk restrictions
	[[ $off -lt $cds_off ]] && {
	    adj=$((cds_off - off))
	    off=$cds_off
	    (( len -= adj ))
	}
	off_max=$((cds_off + cds_len))
	[[ $((off + len)) -gt $off_max ]] && {
	    adj=$((off + len - off_max))
	    (( len -= adj ))
	}
    else
	# disjoint - can't add any of the free region
	return $ENOMATCH
    fi

    list_append $list $off $len
    return 0
}


#
# free_test - test availablility of off/len
# usage: free_test [-lba] dm off len
# RETURN:
#	0: space exists
#	!0: space does not exist
#
function free_test
{
    $set_x
    typeset free_off free_len pub_lbaoff i
    typeset lbaflg=0 list off len
    typeset cds_off cds_len dmcnt
    
    [[ X$1 = X-lba ]] && {
	# interpret offset as LBA
	lbaflg=1
	shift
    }
    
    list=$1
    off=$2
    len=$3

    # get the free list for the DM 
    free_scan $list

    [[ -s $dir_dms/${list}.bounds ]] || {
	internal_err_exit free_test: DM boundary file $dir_dms/${list}.bounds missing
    }
    tail -1 $dir_dms/${list}.bounds | awk '{ print $4, $6, $7 }' >$tmp11
    read pub_lbaoff cds_off cds_len < $tmp11

    [[ $lbaflg -eq 0 ]] && {
	((off += $pub_lbaoff))
    }
    # find free region which completely encompasses the off/len
    list_count $list dmcnt
    i=1

    while (( i <= dmcnt ))
    do
	list_fetch $list $i free_off
	list_fetch $list $((i + 1)) free_len
	if extent_bounded $off $len $free_off $free_len
	then
	    # Found it
	    break
	fi
	(( i += 2 ))
    done
    if (( i > dmcnt ))
    then
	return $ENOMATCH
    fi
    
    return 0
}

#
# free_find - find 'len' free space on 'dm' aligned to 'align'
# Note: len is not rounded up to align granule for finding
#	free space match
# Usage: free_find [-lba] dm len align &off &len
# RETURN:
#	0: success
#	!0: failure
#
function free_find
{
    $set_x
    typeset _ff_free_off _ff_free_len pub_lbaoff i j
    typeset lbaflg=0 list _ff_len
    typeset dmcnt
    typeset align
    typeset _ff_off_sv=0 _ff_len_sv=0
    
    [[ X$1 = X-lba ]] && {
	# interpret offset as LBA
	lbaflg=1
	shift
    }
    
    [[ $# -eq 5 ]] || {
	internal_err_exit "free_find: arg count ($*)"
    }
    list=$1
    _ff_len=$2
    align=$3
    # $4: free offset
    # $5: free len

    # get the free list
    free_scan $list

    [[ -s $dir_dms/${list}.bounds ]] || {
	internal_err_exit free_find: DM boundary file $dir_dms/${list}.bounds missing
    }
    tail -1 $dir_dms/${list}.bounds | awk '{ print $4 }' >$tmp11
    read pub_lbaoff < $tmp11

    # find the smallest region which satisfies request
    list_count $list dmcnt
    i=1

    while (( i <= dmcnt ))
    do
	list_fetch $list $i _ff_free_off
	list_fetch $list $((i + 1)) _ff_free_len
	j=`alignup_len $_ff_free_off $align`
	((_ff_free_off += j))
	((_ff_free_len -= j))
	if [[ $_ff_len -le $_ff_free_len ]]
	then
	    # Found one
	    if (( _ff_len_sv == 0 || _ff_free_len < _ff_len_sv))
	    then
		_ff_len_sv=$_ff_free_len
		_ff_off_sv=$_ff_free_off
	    fi
	fi
	(( i += 2 ))
    done
    if (( _ff_len_sv == 0 ))
    then
	return $ENOMATCH
    fi
    [[ $lbaflg -eq 0 ]]&& {
	# adjust output offset to priv-region relative
	(( _ff_off_sv -= pub_lbaoff ))
    }
    eval "$4=$_ff_off_sv"
    [[ ! X$5 = X- ]] && eval "$5=$_ff_len_sv"
    return 0
}

#
# free_dump - dump copy of free space to stdout
# usage: free_dump [-lba] [space]
#
function free_dump
{
    $set_x
    typeset namecnt i list dmcnt j off len
    typeset match lbaflg=0 pub_lbaoff

    [[ X$1 = X-lba ]] && {
	# interpret offset as LBA
	lbaflg=1
	shift
    }
    
    match="$1"

    list_count array_names namecnt
    i=1
    while (( i <= namecnt ))
    do
	list_fetch array_names $i list
	(( i += 1 ))
	
	list_count $list dmcnt
	if [[ -n "$match" && X$match != X$list ]]
	then
	    continue
	fi

	[[ -s $dir_dms/${list}.bounds ]] || {
	    internal_err_exit free_dump: DM boundary file $dir_dms/${list}.bounds missing
	}
	tail -1 $dir_dms/${list}.bounds | awk '{ print $4 }' >$tmp11
	read pub_lbaoff < $tmp11

	# $list contains offset/len pairs
	j=1
	while (( j <= dmcnt ))
	do
	    list_fetch $list $j off
	    list_fetch $list $((j + 1)) len
	    (( j += 2 ))
	    [[ $lbaflg -eq 0 ]] && {
		# adjust output offset to priv-region relative
		(( off -= pub_lbaoff))
	    }
	    echo "$list	$off $len"
	done
    done
}

#
# Usage: newsdname sd &newsd &nextseq order
#	 creates unique sd name
#	 if 4th arg is given it is used as the seqence number
#	 otherwise it is 1 greater than the largest currently used
#
function newsdname
{
    $set_x
    typeset _nsd_seq _nsd_sd _nsd_prefix

    [[ $# -eq 4 ]] || {
	internal_err_exit "newsdname: arg count ($*)"
    }
    _nsd_sd=$1

    # in case we're handed a temporary subdisk 
    # cut off the suffix
    _nsd_sd=${_nsd_sd%-UC-[0-9]*}
    
    if [[ X$4 != X- ]]
    then
	_nsd_seq=$4
    else
	_nsd_seq=`vxprint -g $dg -sF %name | grep "${_nsd_sd}-UC-[0-9]*" | \
	    sed -e 's/.*-UC-//' | tail -1`
	if [[ X$_nsd_seq = X ]]
	then
	    _nsd_seq=0
	else
	    # pull off leading 0s so it's not confused with octal
	    _nsd_seq=`expr $_nsd_seq : '0*\(.*\)'`
	    [[ X$_nsd_seq = X ]] && _nsd_seq=0
	    ((_nsd_seq += 1))
	fi
    fi
    if [[ $_nsd_seq -lt 10 ]]
    then
	#_nsd_seq=00$_nsd_seq
	_nsd_prefix=00
    elif [[ $_nsd_seq -lt 100 ]]
    then
	#_nsd_seq=0$_nsd_seq
	_nsd_prefix=0
    fi
    eval "$2=${_nsd_sd}-UC-$_nsd_prefix$_nsd_seq"
    [[ X$3 != X- ]] && eval "$3=$((_nsd_seq + 1))"
}

#
# sdr_resize_only - relayout a subdisk which requires resize only
#	
# Usage: sdr_resize_only sd sddm sdlba sdlen align
#
function sdr_resize_only
{
    $set_x
    typeset sd newsd align
    typeset sddm sdlba sdlen
    typeset chunk ret=0

    [[ $# -eq 5 ]] || {
	internal_err_exit "sdr_resize_only: arg count ($*)"
    }
    sd=$1
    sddm=$2
    sdlba=$3
    sdlen=$4
    align=$5

    [[ $align = 1 || $align = $_8K ]] || {
	internal_err_exit "sdr_resize_only: align parameter must be 1 or $_8K"
    }

    [[ $rl_copylen_in -eq 0 && $rl_realign -eq 0 && $rl_restrict -eq 0 ]] || {
	print_debug "sdr_resize_only: ERROR! invalid operations state"
	return $ECOLSTATE
    }

    # Attempt to find contiguous space on the dm that is free
    # allocate an sd there and use it for a copyback location
    # of sd data.
    # If space cannot be found, split a fragment off the end of the
    # subdisk and use it as a copyfwd locattion for the next iteration
    chunk=`alignup_len $sdlen $align`
    if [[ $chunk -eq 0 ]]
    then
	return 0
    fi

    free_test -lba $sddm $((sdlba + sdlen)) $chunk
    ret=$?
    [[ $ret -eq 0 || $ret -eq $ENOMATCH ]] || return $ret
	
    newsdname $sd newsd - -
    # test for success of free_test above
    if [[ $ret -eq 0 ]]
    then
	# space exists
	# create sd for subsequent iteration
	sd_creat -lba $newsd $sddm $((sdlba + sdlen)) $chunk $align
	ret=$?
	[[ $ret -eq 0 ]] || return $ret

	# configure new state and return
	rl_sd_out=$newsd
	rl_copyop_out=copyback
	rl_copylen_out=$chunk
	rl_sdjoin_out=$sd
	rl_copyterm_out=sdr_resize_only:1
    elif [[ X$dgconv_enable_copyfwd = Xyes &&
	    $split_subdisks_ok != no && $split_subdisks_ok != evac ]]
    then
	# NOTE: this is the only place that a copyfwd operation is set
	# The copyfwd op is used when the subdisk needs to grow and
	# the alignment is already good, but contiguous space on the
	# disk is not available.
	#
	# no contiguous space found split off trailing fragment

	if [[ $sdlen -lt $align ]]
	then
	    # configure new state and return
	    rl_sd_out=$sd
	    rl_copyop_out=copyfwd
	    rl_copylen_out=$sdlen
	    rl_sdjoin_out=
	    rl_copyterm_out=sdr_resize_only:3

	    return 0
	fi

	#
	# split off a fragment from the tail end of sd to make
	# sd 8K sized. honor min_split_size.
	# if min_split_size is greater than 1/2 the size of the sd then
	# we use a value close to min_split_size
	#
	if (( min_split_size <= align ))
	then
	    chunk=`aligndown_len $sdlen $align`
	elif (( sdlen < min_split_size * 2 ))
	then
	    chunk=$((sdlen / 2))
	    chunk=`alignup $chunk $align`
	    chunk=$((sdlen - chunk))
	else
	    chunk=$((sdlen - min_split_size))
	    (( chunk -= (chunk % align) ))
	    chunk=$((sdlen - chunk))
	fi
	sd_split -lba -s $((sdlen - chunk)) $sd $sd $newsd
	ret=$?
	[[ $ret -eq 0 ]] || return $ret
	((sdlen -= chunk))

	# configure new state and return
	rl_sd_out=$newsd
	rl_copyop_out=copyfwd
	rl_copylen_out=$chunk
	rl_sdjoin_out=
	rl_copyterm_out=sdr_resize_only:2
    else
	# splitting subdisks is disallowed so relocate the whole thing
	rl_realign=1
	sdr_realign $sd $sddm $sdlba $sdlen $align
	ret=$?
	rl_realign=0
    fi

    return $ret
}
    

#
# sdr_restrict_only - relayout a subdisk which requires moving
#	out of restricted areas only. This is done for disk conversion
#
# Usage: sdr_restrict_only sd sddm sdlba sdlen align
#
function sdr_restrict_only
{
    $set_x
    typeset newsd1 newsd2 newsd3 align 
    typeset sd sddm sdlba sdlen
    typeset ret=0 seq
    typeset dmlba_cds dmlen_cds
    typeset newlen1 newlen2 tmpoff tmplen
    typeset allocator=$rl_use_allocator

    [[ $# -eq 5 ]] || {
	internal_err_exit "sdr_restrict_only: arg count ($*)"
    }

    sd=$1
    sddm=$2
    sdlba=$3
    sdlen=$4
    align=$5

    [[ $align = 1 ]] || {
	internal_err_exit "sdr_restrict_only: align parameter must be 1"
    }

    [[ $rl_copylen_in -eq 0 && $rl_realign -eq 0 && ! $rl_restrict -eq 0 ]] || {
	print_debug "sdr_restrict_only: ERROR! invalid operations state"
	return $ECOLSTATE
    }

    [[ -s $dir_dms/${sddm}.bounds ]] || {
	internal_err_exit sdr_restrict_only: DM boundary file $dir_dms/${sddm}.bounds missing
    }
    tail -1 $dir_dms/${sddm}.bounds | awk '{ print $6, $7 }' >$tmp11
    read dmlba_cds dmlen_cds < $tmp11

    if extent_overlap $dmlba_cds $dmlen_cds $sdlba $sdlen
    then
	# portion of sd in the reserved area at start of disk
	# min_split_size is honored
	newlen1=$((dmlba_cds - sdlba))
	[[ $newlen1 -lt 0 ]] && newlen1=0
	if [[ $newlen1 -gt 0 ]]
	then
	    if [[ $sdlen -lt $min_split_size ]]
	    then
		newlen1=$sdlen
	    elif [[ $newlen1 -lt $min_split_size ]]
	    then
		newlen1=$min_split_size
	    fi
	fi

	# portion of sd in reserved area at end of disk and
	newlen2=$((sdlba + sdlen - dmlba_cds - dmlen_cds))
	[[ $newlen2 -lt 0 ]] && newlen2=0
	if [[ $newlen2 -gt 0 ]]
	then
	    if [[ $sdlen -lt $min_split_size ]]
	    then
		newlen2=$sdlen
	    elif [[ $newlen2 -lt $min_split_size ]]
	    then
		newlen2=$min_split_size
	    fi
	fi
    else
	newlen1=$sdlen
	newlen2=0
    fi

    newsdname $sd newsd1 seq -
    newsdname $sd newsd2 seq $seq
    newsdname $sd newsd3 seq $seq

    if [[ $sdlen -eq $newlen1 ||
	  $split_subdisks_ok = no || $split_subdisks_ok = evac ]]
    then
	# entire sd is outside CDS disk restrictions
	# treat as if we weren't aligned and relocate
	rl_realign=1
	sdr_realign $sd $sddm $sdlba $sdlen $align
	ret=$?
	rl_realign=0
	return $ret
    fi
    
    if [[ $newlen1 -gt 0 ]]
    then
	# allocator volumes require 8K size granules
	[[ X$allocator != X ]] && newlen1=`alignup $newlen1 $_8K`

	# look for space on-disk
	free_find -lba $sddm $newlen1 $align tmpoff tmplen
	ret=$?
	[[ $ret -eq $ENOMATCH ]] && {
	    # use vxassist to relocate entire volume off-disk
	    return $ENEEDEVAC
	}
	[[ $ret -eq 0 ]] || return $ret
	
	# space found on-disk create a replacement sd
	sd_creat -lba $newsd1 $sddm $tmpoff $newlen1 $align
	ret=$?
	[[ $ret -eq 0 ]] || return $ret

	if [[ $newlen1 -eq $sdlen ]]
	then
	    # the entire sd is being relocated
	    sd_mv -rm -rename -ccom $sd $newsd1
	    ret=$?
	    return $ret
	fi

	# split off newlen1 from front the current sd
	# newlen1 is align or 8K multiple
	# newlen1 honors min_split_size if possible
	sd_split -s $newlen1 $sd $newsd3 $sd
	ret=$?
	[[ $ret -eq 0 ]] || return $ret
	((sdlba += newlen1))
	((sdlen -= newlen1))
    
	# move the split-off sd to the replacement
	# newsd1 is a new permanent subdisk so we clear
	# the comment field
	sd_mv -rm -ccom $newsd3 $newsd1
	ret=$?
	[[ $ret -eq 0 ]] || return $ret
    fi

    if [[ $newlen2 -gt 0 ]]
    then
	# allocator volumes require 8K size granules
	[[ X$allocator != X ]] && newlen2=`alignup $newlen2 $_8K`
	
	# look for space on-disk
	free_find -lba $sddm $newlen2 $align tmpoff tmplen
	ret=$?
	[[ $ret -eq $ENOMATCH ]] && {
	    # use vxassist to relocate entire volume off-disk
	    return $ENEEDEVAC
	}
	[[ $ret -eq 0 ]] || return $ret

	# space found on-disk create new sd
	sd_creat -lba $newsd2 $sddm $tmpoff $newlen2 $align
	ret=$?
	[[ $ret -eq 0 ]] || return $ret

	if [[ $newlen2 -eq $sdlen ]]
	then
	    # the entire sd is being relocated
	    sd_mv -rm -rename -ccom $sd $newsd2
	    ret=$?
	    return $ret
	fi

	# split off a fragment from the end of the existing sd
	# Note: sdlen may have already been decremented
	#	by newlen1
	tmplen=$((sdlen - newlen2))
	sd_split -s $tmplen $sd $sd $newsd3
	ret=$?
	[[ $ret -eq 0 ]] || return $ret
	((sdlen = tmplen))

	# move the split-off sd to the replacement
	# newsd2 is a new permanent subdisk so we clear
	# the comment field
	sd_mv -rm -ccom $newsd3 $newsd2
	ret=$?
    else
	# we've now been reduced to the
	# sdr_resize_only case.

	rl_restrict=0
	rl_resize=1
	sdr_resize_only $sd $sddm $sdlba $sdlen $align
	rl_resize=0
	rl_restrict=1
	ret=$?
    fi
    return $ret
}
	
#
# sdr_realign - relayout a subdisk which requires realignment
#	and possibly resize too.
# Usage: sdr_realign sd sddm sdlba sdlen align
#
function sdr_realign
{
    $set_x
    typeset sd newsd align 
    typeset sddm sdlba sdlen
    typeset chunk ret=0 newsd1 seq

    [[ $# -eq 5 ]] || {
	internal_err_exit "sdr_realign: arg count ($*)"
    }

    sd=$1
    sddm=$2
    sdlba=$3
    sdlen=$4
    align=$5

    [[ $align = 1 || $align = $_8K ]] || {
	internal_err_exit "sdr_realign: align parameter must be 1 or $_8K"
    }

    [[ $rl_copylen_in -eq 0 && ! $rl_realign -eq 0 ]] || {
	print_debug "sdr_realign: ERROR! invalid operations state"
	return $ECOLSTATE
    }

    chunk=`alignup_len $sdlen $align`
    
    # look for space on-disk
    free_find -lba $sddm $((sdlen + chunk)) $align tmpoff tmplen
    ret=$?
    [[ $ret -eq $ENOMATCH ]] && {
	# use vxassist to relocate entire volume off-disk
	return $ENEEDEVAC
    }
    [[ $ret -eq 0 ]] || return $ret

    newsdname $sd newsd seq -
    newsdname $sd newsd1 seq $seq

    # space found on-disk create new sd large enough for sd and chunk
    sd_creat -lba $newsd $sddm $tmpoff $((sdlen + chunk)) $align
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    if [[ $chunk -gt 0 ]]
    then
	# the sd is growing so we split the newsd,
	# move the original over and rename it

	# split newly created sd
	sd_split -s $sdlen $newsd $newsd1 $newsd
	ret=$?
	[[ $ret -eq 0 ]] || return $ret

	# move orig sd to newsd1
	sd_mv -rm -rename -ccom $sd $newsd1
	ret=$?
	[[ $ret -eq 0 ]] || return $ret
	
	# configure new state and return
	rl_sd_out=$newsd
	rl_copyop_out=copyback
	rl_copylen_out=$chunk
	rl_sdjoin_out=$sd
	rl_copyterm_out=sdr_realign:1
    else
	# the sd is not growing
	# move the original over and rename it

	# move orig sd to newsd
	sd_mv -rm -rename -ccom $sd $newsd
	ret=$?
	[[ $ret -eq 0 ]] || return $ret

	# configure new state and return
	rl_sd_out=
	rl_copyop_out=none
	rl_copylen_out=0
	rl_sdjoin_out=
	rl_copyterm_out=sdr_realign:2
    fi
    return $ret
}

#
# sdr_realign2 - relayout a subdisk which requires realignment 
#	and possibly resize too. This relayout involves a pending
#	copyback operation.
#    
# Usage: sdr_realign2 sd sddm sdlba sdlen align
#
# We are doing an sd realign with pending copyback operation
# to rl_sd_in of len rl_copylen_in.
#
# First we split sd into rl_copylen_in and (sdlen-rl_copylen_in) pieces
# and move the rl_copylen_in piece rl_sd_in and join it w/ rl_sd_join to
# complete processing of the previous sd in this plex column.
# Next we calculate the extra length needed to make the remaining sd
# conform to the specified alignment and allocate a properly aligned sd
# of that length.
# If the extra length was 0, then we simply move sd to the new one and
# rename it. Otherwise we pre-split the destination before the move and
# setup a copyback operation for the next interation
#
function sdr_realign2
{
    $set_x
    typeset sd newsd newsd1 align 
    typeset sddm sdlba sdlen
    typeset chunk ret=0
    typeset tmpoff tmplen seq

    [[ $# -eq 5 ]] || {
	internal_err_exit "sdr_realign2: arg count ($*)"
    }

    sd=$1
    sddm=$2
    sdlba=$3
    sdlen=$4
    align=$5

    [[ $align = 1 || $align = $_8K ]] || {
	internal_err_exit "sdr_realign2: align parameter must be 1 or $_8K"
    }

    [[ ! $rl_copylen_in -eq 0 && $rl_copyop_in = copyback ]] || {
	print_debug "sdr_realign2: ERROR! invalid operations state"
	return $ECOLSTATE
    }

    newsdname $sd newsd seq -
    newsdname $sd newsd1 seq $seq

    if [[ $sdlen -le $rl_copylen_in ]]
    then

	newsdname $rl_sd_in newsd1 - -

	if [[ $sdlen -eq $rl_copylen_in ]]
	then
	    # current sd exactly fits into rl_sd_in
	    # rl_sd_in will utilmately be joined w/ rl_sdjoin_in
	    # and sd will cease to exist

	    # move sd to rl_sd_in
	    sd_mv -rm $sd $rl_sd_in
	    ret=$?
	    [[ $ret -eq 0 ]] || return $ret

	    # join the sds
	    sd_join $rl_sdjoin_in $rl_sd_in $rl_sdjoin_in
	    ret=$?
	    [[ $ret -eq 0 ]] || return $ret

	    # configure new state and return
	    rl_sd_out=
	    rl_copyop_out=none
	    rl_copylen_out=0
	    rl_sdjoin_out=
	    rl_copyterm_out=sdr_realign2:4
	else
	    # current sd is smaller than rl_sd_in
	    # split rl_sd_in, move current sd to head
	    # fragment and join with rl_sdjoin_in
	    # leaving (a smaller) rl_sd_in for next iteration
	    # rl_sd_in will utilmately be joined w/ rl_sdjoin_in
	    # and sd will cease to exist

	    # split off sdlen from rl_sd_in
	    sd_split -s $sdlen $rl_sd_in $newsd1 $rl_sd_in
	    ret=$?
	    [[ $ret -eq 0 ]] || return $ret

	    # move sd to newsd1
	    sd_mv -rm $sd $newsd1
	    ret=$?
	    [[ $ret -eq 0 ]] || return $ret
	
	    # join the sds
	    sd_join $rl_sdjoin_in $newsd1 $rl_sdjoin_in
	    ret=$?
	    [[ $ret -eq 0 ]] || return $ret

	    # configure new state and return
	    rl_sd_out=$rl_sd_in
	    rl_copyop_out=copyback
	    rl_copylen_out=$((rl_copylen_in - sdlen))
	    rl_sdjoin_out=$rl_sdjoin_in
	    rl_copyterm_out=sdr_realign2:3
	fi

	return 0
    fi

    # split off rl_copylen_in from the current sd
    sd_split -s $rl_copylen_in $sd $newsd $sd
    ret=$?
    [[ $ret -eq 0 ]] || return $ret
    ((sdlba += rl_copylen_in))
    ((sdlen -= rl_copylen_in))

    # move the split-off sd to rl_sd_in
    sd_mv -rm $newsd $rl_sd_in
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # join the moved sd to its joinee
    sd_join $rl_sdjoin_in $rl_sd_in $rl_sdjoin_in
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # Note: sdlen was modified above
    chunk=`alignup_len $sdlen $align`

    # look for space on-disk
    free_find -lba $sddm $((sdlen + chunk)) $align tmpoff tmplen
    ret=$?
    [[ $ret -eq $ENOMATCH ]] && {
	# use vxassist to relocate entire volume off-disk
	return $ENEEDEVAC
    }
    [[ $ret -eq 0 ]] || return $ret

    # space found on-disk  create new sd large enough for sd and chunk
    sd_creat -lba $newsd $sddm $tmpoff $((sdlen + chunk)) $align
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # we don't bother to check if sd is now magically aligned
    # we always relocate it.

    if [[ $chunk -gt 0 ]]
    then
	# split newly created sd
	sd_split -s $sdlen $newsd $newsd1 $newsd
	ret=$?
	[[ $ret -eq 0 ]] || return $ret

	sd_mv -rm -rename -ccom $sd $newsd1
	ret=$?
	[[ $ret -eq 0 ]] || return $ret

	# configure new state and return
	rl_sd_out=$newsd
	rl_copyop_out=copyback
	rl_copylen_out=$chunk
	rl_sdjoin_out=$sd
	rl_copyterm_out=sdr_realign2:1

    else

	sd_mv -rm -rename -ccom $sd $newsd
	ret=$?
	[[ $ret -eq 0 ]] || return $ret

	# configure new state and return
	rl_sd_out=
	rl_copyop_out=none
	rl_copylen_out=0
	rl_sdjoin_out=
	rl_copyterm_out=sdr_realign2:2
    fi

    return $ret
}

# sdr_realign3 - relayout a subdisk which requires realignment 
#	and possibly resize too. This relayout involves a pending
#	copyfwd operation.
#    
# Usage: sdr_realign3 sd sddm sdlba sdlen align
#
# We are doing an sd realign with pending copyfwd operation
# from rl_sd_in of len rl_copylen_in.
#
# First we calculate the length needed to make (sdlen+rl_copylen_in)
# conform to the specified alignment and allocate a properly aligned sd
# of that length. Next we split the newly allocated sd into 2 or 3
# pieces of size rl_copylen_in,sdlen,rem where rem is the additional length
# needed to make (sdlen+rl_copylen_in) conform to the alignment.
# Next rl_sd_in is moved to the first piece, sd is moved to the second piece
# and the last piece becomes the copyback location for the next iteration
#
function sdr_realign3
{
    $set_x
    typeset sd newsd newsd1 newsd2 align 
    typeset sddm sdlba sdlen
    typeset chunk ret=0
    typeset newlen tmpoff tmplen seq

    [[ $# -eq 5 ]] || {
	internal_err_exit "sdr_realign3: arg count ($*)"
    }

    sd=$1
    sddm=$2
    sdlba=$3
    sdlen=$4
    align=$5

    [[ $align = 1 || $align = $_8K ]] || {
	internal_err_exit "sdr_realign3: align parameter must be 1 or $_8K"
    }

    [[ ! $rl_copylen_in -eq 0 && $rl_copyop_in = copyfwd ]] || {
	print_debug "sdr_realign3: ERROR! invalid operations state"
	return $ECOLSTATE
    }

    newlen=$((sdlen + rl_copylen_in))
    chunk=`alignup_len $newlen $align`

    
    # look for space on-disk
    free_find -lba $sddm $((newlen + chunk)) $align tmpoff tmplen
    ret=$?
    [[ $ret -eq $ENOMATCH ]] && {
	# use vxassist to relocate entire volume off-disk
	return $ENEEDEVAC
    }
    [[ $ret -eq 0 ]] || return $ret

    newsdname $sd newsd seq -
    newsdname $sd newsd1 seq $seq
    newsdname $sd newsd2 seq $seq

    # space found on-disk create new sd
    sd_creat -lba $newsd $sddm $tmpoff $((newlen + chunk)) $align
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    if [[ $chunk -gt 0 ]]
    then
	# split off rl_copylen_in from front and chunk from
	# back of newsd
	sd_split -s $rl_copylen_in $newsd $newsd1 $newsd
	ret=$?
	[[ $ret -eq 0 ]] && {
	    sd_split -s $sdlen $newsd $newsd2 $newsd
	    ret=$?
	}
	[[ $ret -eq 0 ]] && {
	    sd_mv -rm $rl_sd_in $newsd1
	    ret=$?
	}
	[[ $ret -eq 0 ]] && {
	    sd_mv -rm $sd $newsd2
	    ret=$?
	}
	[[ $ret -eq 0 ]] && {
	    sd_join -ccom $newsd1 $newsd2 $sd
	    ret=$?
	}
	[[ $ret -eq 0 ]] && {
	    # configure new state and return
	    rl_sd_out=$newsd
	    rl_copyop_out=copyback
	    rl_copylen_out=$chunk
	    rl_sdjoin_out=$sd
	    rl_copyterm_out=sdr_realign3:1
	
	}
    else
	# split off rl_copylen_in from front of newsd
	sd_split -s $rl_copylen_in $newsd $newsd1 $newsd2
	ret=$?
	[[ $ret -eq 0 ]] && {
	    sd_mv -rm $rl_sd_in $newsd1
	    ret=$?
	}
	[[ $ret -eq 0 ]] && {
	    sd_mv -rm $sd $newsd2
	    ret=$?
	}
	[[ $ret -eq 0 ]] && {
	    sd_join -ccom $newsd1 $newsd2 $sd
	    ret=$?
	}

	[[ $ret -eq 0 ]] && {
	    # configure new state and return
	    rl_sd_out=
	    rl_copyop_out=none
	    rl_copylen_out=0
	    rl_sdjoin_out=
	    rl_copyterm_out=sdr_realign3:2
	}
    fi

    return $ret
}

#
# Usage: sdr_relocate_sds dm publba publen sds ...
#
# This function used by the disk conversion code to move sds out of
# restricted regions of the disk
#
function sdr_relocate_sds
{
    $set_x
    typeset dm publba publen sd lba len junk sds ret vol pl matchvol

    [[ $# -eq 4 ]] || internal_err_exit "sdr_relocate_sds: arg count ($*)"

    dm=$1
    publba=$2
    publen=$3
    sds="$4"

    # configure the new pub region location
    dm_config $dm $publba $publen

    # setup state variables
    rl_copylen_in=0
    rl_copylen_out=0
    rl_restrict=1
    rl_resize=0
    rl_realign=0
    
    # perform any needed relocation
    for sd in $sds
    do
	echo $sd | sdlba >$tmp11
	read junk junk junk lba len < $tmp11
	vxprint -g $dg -sF '%v_name %pl_name' $sd >$tmp11
	read vol pl < $tmp11
	[[ $matchvol = *" $vol "* ]] && {
	    # skip this sd as it was in a volume
	    # that we've already evacuated from this disk
	    continue
	}
	if [[ X`vxprint -g $dg -F %stp_rid $vol` != X- ]]
	then
	    # Allocator volume
	    # the rl_use_allocator global will be used to add
	    # the =o allocator flags to any subdisk split/move
	    # operations
	    rl_use_allocator=true
	    sdr_restrict_only $sd $dm $lba $len 1
	    ret=$?
	    rl_use_allocator=
	else
	    sdr_restrict_only $sd $dm $lba $len 1
	    ret=$?
	fi
	if [[ ! $ret -eq 0 ]]
	then
	    # need to evac the disk
	    relayout_seterr sdr_relocate_sds $ret $vol $pl $sd $dm
	    ret=$?
	    [[ $ret -eq $ENEEDEVAC ]] || return $ret
	    # subdisk was successfully evacuated
	    # now process the rest of the sds but skipping
	    # any that were in this volume
	    matchvol="$matchvol $vol "
	fi
    done

    return 0
}

function error_code_init
{
    EOK=0
    EINTERNAL=100	# internal error
    ESDCREATE=101	# creation of subdisk failed
    ENOMATCH=102	# no match
    EINVAL=103		# invalid function args
    EUTIL=104		# error in support utility
    EOVERLAP=105	# error due to extent overlap
    ECOLSTATE=106	# invalid/unexpected column state
    ENOCPLEX=108	# cannot ascertain plex association
    ENOCOLUMN=109	# cannot ascertain plex column association
    ENOSPLIT=110	# operation requires subdisk splitting
			# which has been disallowed by the user
			# (split_subdisks_ok=no)
    ENOEVAC=111		# operation requires off-disk relocation
			# which has been disallowed by the user
    EBADLOGS=112
    EMAXEVAC=113
    ENOCONVERGE=114
    EMAXCOMMENT=115	# tried to create a comment > 40 chars
    ECONFIG=116		# the configuration has objects which will
			# prevent successful conversion of the diskgroup
    
    # pseudo errors
    ENEEDEVAC=120	# error used internally to indicate the
			# need to use vxassist to relocate subdisks
			# off-disk
}
	
# Usage: sd_creat [-lba] sd dm off len align
function sd_creat
{
    $set_x
    typeset lbaflg=0 sd dm off len align
    typeset lba_flg cmd
    typeset cmt=$vxc_comment

    [[ X$1 = X-lba ]] && {
	# interpret offset as LBA
	lbaflg=1
	lba_flg="$1"
	shift
    }

    [[ $# -eq 5 ]] || {
	internal_err_exit "sd_creat: arg count ($*)"
    }

    sd=$1
    dm=$2
    off=$3
    len=$4
    align=$5

    # if move is disallowed, don't create any subdisks, but
    # return ENEEDEVAC which will cause
    # relayout_seterr to attempt the evac
    #[[ $move_subdisks_ok = yes ]] || return $ENEEDEVAC
    #
    # We don't tie creation of sds to ability to do subdisk moves
    # temporary sds will be cleaned up as a side effect of
    # subdisk relayout failure or subdisk relocation failure
    # so if the move operation is disallowed, the destination subdisk
    # may get created but the subsequent move op will fail.
    # This means if move is disallowed and evac is allowed
    # a subdisk which need resize (assuming single subdisk plexcolumn)
    # will succeed using evac+sd_create+join

    [[ -s $dir_dms/${dm}.bounds ]] || {
	internal_err_exit sd_creat: DM boundary file $dir_dms/${fm}.bounds missing
    }
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4 }' >$tmp11
    read pub_lbaoff < $tmp11

    [[ $lbaflg -eq 0 ]] && {
	((off += $pub_lbaoff))
    }

    cmd="vxmake -g $dg sd $sd dm_name=$dm dm_offset=$((off - pub_lbaoff)) len=$len comment=$cmt"
    doit -n "E_SD_CREAT $sd" $cmd
    ret=$?

    return $ret
}


# Usage: sd_mv [-rm] [-force] [-rename] [-ccom] sd newsd
function sd_mv
{
    $set_x
    typeset rmflg forceflg renameflg ccomflg sd newsd cmd
	
    while [[ -n "$1" ]]
    do
	case $1 in
	-rm)		rmflg=true ;;
	-force)		forceflg=true ;;
	-rename)	renameflg=true ;;
	-ccom)		ccomflg=true ;;
	*)		break ;;
	esac
	shift
    done
    [[ $# -eq 2 ]] || {
	internal_err_exit "sd_mv: arg count ($*)"
    }
    [[ -n "$renameflg" && -z "$rmflg" ]] && {
	internal_err_exit "sd_mv: invalid args ($*) -rename requires -rm"
    }
    [[ -n "$ccomflg" && -z "$rmflg" ]] && {
	internal_err_exit "sd_mv: invalid args ($*) -ccom requires -rm"
    }

    sd=$1
    newsd=$2

    # if move is disallowed, return ENEEDEVAC which will cause
    # relayout_seterr to attempt the evac
    # if however there is a pending copy length (need to copy data between
    # adjacent subdisks in a plex column), we will need to do
    # a subdisk move at some point (which is disabled) so return failure
    [[ $move_subdisks_ok = yes ]] || {
	if [[ $rl_copylen_in -gt 0 ]]
	then
	    print_err "E_SD_MOVE_NEEDED"
	    return $ENOSPLIT
	else
	    return $ENEEDEVAC
	fi
    }

    # note that we retain hot-reloc info
    cmd="vxsd -g $dg ${forceflg:+-f} ${rmflg:+-o rm} ${rl_use_allocator:+-o allocator} -r mv $sd $newsd"
    doit -n "E_SD_MV $sd $newsd" $cmd
    ret=$?

    [[ $ret -eq 0 && -n "$renameflg" ]] && {
	cmd="vxedit -g $dg rename $newsd $sd"
	doit -n "E_SD_RENAME $newsd $sd" $cmd
	ret=$?
    }

    [[ $ret -eq 0 && -n "$ccomflg" ]] && {
	if [[ -n "$renameflg" ]] 
	then
	    cmd="vxedit -g $dg set comment= $sd"
	    doit -n "E_OBJ_EDIT $sd" $cmd
	    ret=$?
	else
	    cmd="vxedit -g $dg set comment= $newsd"
	    doit -n "E_OBJ_EDIT $newsd" $cmd
	    ret=$?
	fi
    }

    return $ret
}

#
# sd_assoc -  associate subdisk at the end of a plex or plex column
# Usage: sd_assoc sd newsd
#
function sd_assoc
{
    $set_x
    typeset sd newsd cmd sdpl sdcol

    [[ $# -ge 2 ]] || {
	internal_err_exit "sd_assoc: arg count ($*)"
    }
    
    sd=$1
    newsd=$2

    [[ X`vxprint -g $dg -sF %assoc $newsd 2>/dev/null` = X- ]] || return 0

    sdpl=`vxprint -g $dg -sF %plex $sd 2>/dev/null`
    sdcol=`vxprint -g $dg -sF %column $sd 2>/dev/null`

    [[ X$sdpl != X- && X$sdcol = X[0-9]* ]] || {
	export sd sdpl sdcol
	p_list=`egettxt "sd_assoc: $sd: invalid plex/column $sdpl/$sdcol" \
	    vxvmshm:3435`
	print_anal "$p_list"
	return $EINTERNAL
    }
    
    cmd="vxsd -g $dg assoc $sdpl ${newsd}:$sdcol"
    doit -n "E_SD_ASSCOL $newsd $sdpl $sdcol" $cmd
    ret=$?

    return $ret
}

# Usage: sd_join [-ccom] sd1 sd2 newsd
function sd_join
{
    $set_x
    typeset src1 src2 dst ccomflg cmd

    while [[ -n "$1" ]]
    do
	case $1 in
	-ccom)		ccomflg=true ;;
	*)		break ;;
	esac
	shift
    done
    [[ $# -ge 3 ]] || {
	internal_err_exit "sd_join: arg count ($*)"
    }
    
    src1=$1
    src2=$2
    dst=$3

    cmd="vxsd -g $dg join $src1 $src2 $dst"
    doit -n "E_SD_JOIN $src1 $src2" $cmd
    ret=$?

    [[ $ret -eq 0 && -n "$ccomflg" ]] && {
	cmd="vxedit -g $dg set comment= $dst"
	doit -n "E_OBJ_EDIT $dst" $cmd
	ret=$?
    }

    return $ret
}

# Usage: sd_split [-lba] -s size sd newsd1 newsd2
function sd_split
{
    $set_x
    typeset size=0 sd newsd1 newsd2
    typeset lbaflg=0 lba_flg cmd

    while [[ -n "$1" ]]
    do
	case $1 in 
	-lba)
	    # interpret offset as LBA
	    lbaflg=1
	    lba_flg="$1"
	    ;;
	-s)
	    size=$2; shift;
	    ;;
	*)
	    break ;;
	esac
	shift
    done
    [[ $size -gt 0 ]] || {
	internal_err_exit "sd_split: $size: invalid split size"
    }
    [[ $# -eq 3 ]] || internal_err_exit "sd_split: invalid sd params: $*"

    sd=$1
    newsd1=$2
    newsd2=$3

    # If splitting is disallowed, check to see if the subdisk which
    # is being split is associated. If it's not, then we're splitting a new
    # subdisk which the converter has created and that operation
    # is not disallowed
    if [[ X`vxprint -g $dg -sF '%assoc' $sd` != X- ]]
    then
	# if move is disallowed, don't split any subdisks
	# if there is no pending copy length 
	# return ENEEDEVAC which will cause
	# relayout_seterr to attempt the evac
	# if there is a pending copy length (need to copy data between
	# adjacent subdisks in a plex column), we will need to do
	# a subdisk move at some point (which is disabled) so return failure
	[[ $move_subdisks_ok = yes ]] || {
	    if [[ $rl_copylen_in -gt 0 ]]
	    then
		print_err "E_SD_MOVE_NEEDED"
		return $ENOSPLIT
	    else
		return $ENEEDEVAC
	    fi
	}

	case $split_subdisks_ok in
	no|evac)
	    print_err "E_SD_SPLIT_NEEDED"
	    return $ENOSPLIT
	    ;;
	move|yes)
	    ;;
	*)
	    internal_err_exit "sd_split: invalid split_subdisks_ok value ($split_subdisks_ok)"
	    ;;
	esac
    fi

    cmd="vxsd -g $dg ${rl_use_allocator:+-o allocator} -s $size split $sd $newsd1 $newsd2"
    doit -n "E_SD_SPLIT $sd" $cmd
    ret=$?

    return $ret
}

  
function plexcolumn_state_init
{
    # setup state for next plex column
    rl_copylen_in=0
    rl_copylen_out=0
    rl_copyop_in=none
    rl_copyop_out=none
    rl_copyterm_in=
    rl_copyterm_out=
    rl_sd_in=
    rl_sd_out=
    rl_sdjoin_in=
    rl_sdjoin_out=
}

function plexcolumn_state_roll
{
    # setup state for next sd in plex column
    rl_copylen_in=$rl_copylen_out
    rl_copyop_in=$rl_copyop_out
    rl_sd_in=$rl_sd_out
    rl_sdjoin_in=$rl_sdjoin_out
    rl_copyterm_in=$rl_copyterm_out
    rl_copylen_out=0
    rl_copyop_out=none
    rl_sd_out=
    rl_sdjoin_out=
    rl_copyterm_out=
}

# sdr_plexcolumn_term_fwd - perform actions for
#	sdr_plexcolumn_term in the case where the pending
#	copy operation is "copyfwd"
# Usage: sdr_plexcolumn_term_fwd sd dm sdlen align
function sdr_plexcolumn_term_fwd
{
    $set_x
    typeset sd sddm sdlen align
    typeset newlen tmpoff tmplen newsd ret

    [[ $# -eq 4 ]] || {
	internal_err_exit "sdr_plexcolumn_term_fwd: arg count ($*)"
    }
    sd=$1
    sddm=$2
    sdlen=$3
    align=$4

    # need to migrate sd to a properly aligned and sized location
    newlen=`alignup $sdlen $align`
    # look for space on-disk
    free_find -lba $sddm $newlen $align tmpoff tmplen
    ret=$?
    [[ $ret -eq $ENOMATCH ]] && {
	# use vxassist to relocate entire volume off-disk
	return $ENEEDEVAC
    }

    newsdname $sd newsd - -
    # test for success of free_find above
    [[ $ret -eq 0 ]] && {
	# space found on-disk create new sd
	sd_creat -lba $newsd $sddm $tmpoff $newlen $align
	ret=$?
    }
    [[ $ret -eq 0 ]] && {
	sd_mv -rm -force $sd $newsd
	ret=$?
	# newsd is a permanent new record
    }

    return $ret
}

#
# sdr_plexcolumn_term - perform termination processing at the end of a
#	plex column. This processing will occur when the volume changes,
#	the plex changes, or the plex column changes.
#	
# Usage: sdr_plexcolumn_term align
#
function sdr_plexcolumn_term
{
    $set_x
    typeset align 
    typeset ret=0
    typeset sdvol sdpl sddm sdcol sdcoloff junk

    [[ $# -eq 1 ]] || {
	internal_err_exit "sdr_plexcolumn_term: arg count ($*)"
    }
    align=$1

    [[ $rl_copylen_in -gt 0 ]] || return 0

    [[ $align = 1 || $align = $_8K ]] || {
	internal_err_exit "sdr_plexcolumn_term: align parameter must be 1 or $_8K"
    }

    [[ $align -gt 1 ]] || return 0


    case $rl_copyop_in in
    copyback)
	# need to join rl_sdjoin_in and rl_sd_in
	sd_assoc $rl_sdjoin_in $rl_sd_in
	ret=$?

	[[ $ret -eq 0 ]] && {	    
	    sd_join $rl_sdjoin_in $rl_sd_in $rl_sdjoin_in
	    ret=$?
	}
	;;

    copyfwd)
	vxprint -g $dg -sF '%v_name %pl_name %dm_name %column_pl_offset' $rl_sd_in |
	    sed -e 's/\// /' >$tmp11
	read sdvol sdpl sddm sdcol sdcoloff < $tmp11
	[[ -z "$sdcoloff" ]] && {
	    sdcoloff=$sdcol
	    sdcol=0
	    [[ X$sdcoloff = XLOG ]] && sdcoloff=0
	}
	sdr_plexcolumn_term_fwd $rl_sd_in $sddm $rl_copylen_in $align
	ret=$?
	if [[ $ret -eq $ENEEDEVAC ]]
	then
	    relayout_seterr sdr_plexcolumn_term \
		$ret $sdvol $sdpl $rl_sd_in $sddm
	    ret=$?
	    [[ $ret -eq $ENEEDEVAC ]] && {
		# relocation was successful
		# find subdisk and grow it
		rl_sd_in=`vxprint -g $dg -se 'assoc.name="'$sdpl'" && \
		    sd_column=='$sdcol' && sd_pl_offset=='$sdcoloff'' \
		    -F %name`
		echo $rl_sd_in | sdlba 2>/dev/null >$tmp11
		read junk sddm junk sdlba sdlen junk < $tmp11
		sdr_plexcolumn_term_fwd $rl_sd_in $sddm $sdlen $align
		ret=$?
		[[ $ret -eq 0 ]] && ret=$ENEEDEVAC
	    }
	fi
	;;

    none)
	;;
    *)
	internal_err_exit "sdr_plexcolumn_term: invalid copy operation $rl_copyop_in"
	;;
    esac
	
    [[ $ret -eq 0 || $ret -eq $ENEEDEVAC ]] && rl_copylen_in=0
    return $ret
}
	
#
# relayout_seterr - set error after relayout function failure and possibly
#	perform off-disk subdisk relocation.
#
#	Off-disk relocation will be attempted only if the function
#	error is $ENEEDEVAC and is done by vxassist "move" operation for
#	non-allocator volumes and by vxvoladm "evac subdisk" operation
#	for allocator volumes. The later is only valid if the calling
#	function is sdr_relocate_sds. Successful off-disk relocation
#	will return $ENEEDEVAC which has special meaning to relayout()
#	and is not treated as a true error.
#	All other return values will be treated as error conditions by
#	relayout().
# Usage: relayout_seterr func err vol pl sd dm
function relayout_seterr
{
    $set_x
    typeset func err vol pl sd dm ret pl_layout layvol topvol
    typeset plsds cmd

    [[ $# -eq 6 ]] || internal_err_exit "relayout_seterr: arg count ($*)"
    func=$1
    err=$2
    vol=$3
    pl=$4
    sd=$5
    dm=$6

    # cleanup any temporary subdisks we've created so they
    # don't get in the way of the subsequent relayout ops or
    # are left over after failure
    recover

    if [[ $err -eq $ENEEDEVAC ]]
    then
	[[ $evac_subdisks_ok = yes ]] || {
	    print_err "E_SD_EVAC_NEEDED"
	    print_debug "Skipping rest of plex $pl due to evac failure"
	    rl_skip=true
	    rl_skip_match=$vol
	    rl_skip_err=$ENOEVAC
	    rl_reterr=$ENOEVAC
	    return $rl_skip_err
	}

	print_debug "Unable to relocate $sd within $dm"

	pl_layout=`vxprint -g $dg -pF %layout $pl`
	if [[ X$pl_layout = XRAID ]]
	then
	    print_debug "Evacuating RAID subdisks for volume $vol from disk $dm"
	    # Subdisks created by the move operation will be resized later
	else
	    if [[ X`vxprint -g $dg -vF '%layered' $vol` = Xon ]]
	    then
		# Layered volume
		print_debug "Evacuating subdisks for layered volume $vol from disk $dm"
	    else
		print_debug "Evacuating subdisks for volume $vol from disk $dm"
	    fi
	    # Subdisks created by the move operation will be resized later
	fi	    
	# use vxassist to move the subdisks off-disk and retain
	# original dm and offset info
	if [[ X`vxprint -g $dg -F %stp_rid $vol` != X- ]]
	then
	    [[ X$func = Xsdr_relocate_sds ]] || {
		internal_err_exit \
			"relayout_seterr: $vol: Allocator volume encountered"
	    }
	    # relocate subdisk using the allocator
	    vxvoladm -g $dg evacuate subdisk $vol sd=$sd
	    ret=$?
	else
	    if [[ X$func = Xsdr_relocate_sds ]]
	    then
		cmd="vxassist -g $dg -m move $vol !dm:$dm layout=nodiskalign $evac_disk_list"
	    else
		cmd="vxassist -g $dg -m -a $_8K move $vol !dm:$dm $evac_disk_list"
	    fi
	    rl_skip=true
	    rl_skip_match=$vol
	    rl_skip_err=$err
	    doit -n "E_VOL_MOVE_FROM $dm $vol" $cmd
	    ret=$?
	fi
	[[ $ret -eq 0 ]] || {
	    # Arrange for relayout to fail
	    rl_skip_err=$EUTIL
	    rl_reterr=$EUTIL
	}
    else
	case $func in
	sdr_realign|sdr_realign2|sdr_realign3)
	    print_debug "relayout_seterr: ERROR! $func failed ($err) on sd/plex/vol $sd/$pl/$vol"
	    rl_skip=true
	    rl_skip_match=$vol
	    rl_skip_err=$err
	    rl_reterr=$err
	    ;;
	sdr_resize_only)
	    print_debug "relayout_seterr: ERROR! $func failed ($err) on sd/plex/vol $sd/$pl/$vol"
	    rl_skip=true
	    rl_skip_match=$vol
	    rl_skip_err=$err
	    rl_reterr=$err
	    ;;
	sdr_plexcolumn_term)
	    print_debug "relayout_seterr: ERROR! $func failed ($err) on plex/vol $pl/$vol"
	    rl_skip=true
	    rl_skip_match=$vol
	    rl_skip_err=$err
	    rl_reterr=$err
	    ;;
	sdr_relocate_sds)
	    print_debug "relayout_seterr: ERROR! $func failed ($err) on sd/plex/vol $sd/$pl/$vol"
	    # disk conversion IF, we aren't being called from relayout()
	    # so no need to set rl_ state variables
	    return $err
	    ;;
	*)
	    internal_err_exit "relayout_seterr: invalid error function $func"
	    ;;
	esac
    fi
    return $rl_skip_err
}

#
# relayout_print_sds - read the list of subdisks from stdin and
#	produce output suitable for consumption by the main loop in
#	relayout()
#
function relayout_print_sds
{
    $set_x
    sed -e '/^#/d' |
    xargs vxprint -g $dg -F \
	'%v_name %pl_name %dm_name %name %column_pl_offset' |
	sed -e 's,/, ,g' |
    sort -b +0 -1 +1 -2 +4n -5 +5n -6
}

# dgconv_sdrelayout - perform subdisk relayout
function dgconv_sdrelayout
{
    $set_x
    typeset nlast ret=0

    nlast=0
    while ((1))
    do
	# Gather subdisk information
	dgconv_config
	ret=$?
	[[ $ret -eq 0 ]] || return $ret

	# Perform subdisk relayout
	relayout
	ret=$?
	[[ $ret -eq 0 ]] || return $ret
	[[ $rl_nsd -eq 0 ]] && {
	    print_debug "dgconv: subdisk relayout complete"
	    return 0
	}
	[[ X$analyze_only = Xy* ]] && break
	[[ $nlast -gt 0 && $nlast -ge $rl_nsd ]] && {
	    print_debug "dgconv: subdisk relayout is not converging"
	    return $ENOCONVERGE
	}
	nlast=$rl_nsd
    done
    return $ret
}

#
# relayout - perform subdisk relayout on the diskgroup so that the
#	subdisks conform to 8K alignment restrictions.
#
# This function is provided a list of subdisks to anaylze and possibly
# perform relayout operations on. However the granularity is that of highest
# level object i.e. all the subdisks associated with a higher-level
# object must be provided not just those which require relayout.
#
# Subdisks are processed grouped by volume.
# All the subdisks associated with the lowest plex column are processed
# (in order of plex column offset) before moving to the next highest plex
# column. The resize aspect of subdisk relayout requires subdisks to be
# split and joined, so inter-subdisk state is maintined within a plex column.
# Even single subdisk plexes are handled this way being simply the
# degenerate case of concatenated plex columns.
#
# The list of subdisks provided to this function should not include
# subdisks from raid logging plexes which require relayout.
#
# Subdisks are relocated on-disk if there is room to do so,
# otherwise all the subdisks for the volume are evacuated from the disk
# using vxassist (allocator volume subdisks are already 8K
# aligned and sized). Subdisks are evacuated in way that they will be
# 8K aligned and have enough contiguous space on-disk to be able to
# grow them to an 8K multiple. After evacuation, all the subdisks for a
# volume are reprocessed to perform any needed resize operations.
#
#
# Usage: relayout [-v] [-e] [-i] [-n] [vol ...]
function relayout
{
    $set_x
    typeset e_flag=0 i_flag=0 n_flag=0
    typeset vol pl dm coloff sd lastv lastpl lastsd lastdm
    typeset pl_column pl_coloff pl_column_last pl_coloff_last
    typeset rlfunc
    typeset sdlen sdlba sddm
    typeset dmlba_cds dmlen_cds
    typeset junk
    typeset ret
    typeset align=$_8K
    typeset sentinel="VXCDSCONVERT_SENTINEL_VOLNAME"
    typeset work_sds=$active_sds
    typeset nevac=0 nevac_sds alist
    typeset sv_vol sv_pl sv_sd sv_dm sv_pl_column sv_pl_coloff

    print_verbose "Performing subdisk relayout ..."

    rl_nsd=0
    rl_reterr=0
    while [[ -n "$1" ]]
    do
	case $1 in
	-e)
	    # exit on first error
	    # default is to skip to the next plex column
	    e_flag=1 ;;
	-i)
	    # iterative - ascertain sd characteristics each time
	    #	instead of relaying on active_sdlba
	    i_flag=1 ;;
	-v)
	    # verbose
	    set -x ;;
	-n)
	    n_flag=1 ;;
	-f) # use files provided non $active_sds
	    # this implies -i too
	    work_sds=$2; shift
	    i_flag=1
	    ;;
	*)
	    break ;;
	esac
	shift
    done

    [[ X$analyze_only = Xy* ]] && n_flag=1

    if [[ $# -gt 0 ]]
    then
	# a list of volumes is provided on the command line
	# use those instead of $active_sds and set i_flag
	i_flag=1
	work_sds=$active_sds_tmp
	vxprint -g $dg -vF '%aslist' $* | sed -e 's/,/ /g' |
	    xargs vxprint -g $dg -pF 'log=%raidlog %aslist' |
	    grep -v 'log=on' |
	    sed -e 's/,/ /g' -e '/^-/d' |
	    awk '{ $1=""; print; }' > $work_sds
    fi

    # initialize the alternate input for the case where we
    # have done a vxassist "move" operation and need to re-process
    # all the subdisks for a given volume
    > $relayout_altstdin
    exec 6< $relayout_altstdin
	    
    (
    cat $work_sds |
    relayout_print_sds
    echo $sentinel pl dm sd 0 0
    ) >$tmp15
    while ((1))
    do
	if [[ -s $relayout_altstdin ]]
	then
	    read vol pl dm sd pl_column pl_coloff <&6
	    [[ X$vol = X ]] && {
		> $relayout_altstdin
		vol=$sv_vol
		pl=$sv_pl
		dm=$sv_dm
		sd=$sv_sd
		pl_column=$sv_pl_column
		pl_coloff=$sv_pl_coloff
	    }
	else
	    read vol pl dm sd pl_column pl_coloff
	fi

	if [[ X$pl_coloff = X ]]
	then
	    # %column_pl_offset was not of the form col/off
	    # so we use pl_column as the column offset and
	    # set coplumn to 0
	    pl_coloff=$pl_column
	    pl_column=0
	fi
	
	[[ X$dm = X- ]] && {
	    print_debug "WARNING: $sd: no disk association. Skipped."
	    continue
	}
	[[ X$pl = X- ]] && {
	    print_debug "WARNING: $sd: no plex association. Skipped."
	    continue
	}
	[[ X$vol = X- ]] && {
	    print_debug "WARNING: $sd: no volume association. Skipped."
	    continue
	}
	[[ X$pl_coloff = XLOG ]] && {
	    print_debug "$sd: Logging subdisk skipped."
	    continue
	}
	[[ $pl_coloff = [0-9]* ]] || {
	    print_err "E_SD_COLOFF $sd $pl_coloff"
	    return $EINTERNAL
	}

	# perform skip processing
	if [[ -n "$rl_skip" ]]
	then
	    if [[ X$rl_skip_match = X$vol ]]
	    then
		if [[ $rl_skip_err -eq $ENEEDEVAC ]]
		then
		    print_verbose "$sd: skipping subdisk in volume which was relocated"
		else
		    print_verbose "WARNING: $sd: skipping after error processing volume"
		fi
		[[ $e_flag -eq 0 || $rl_skip_err -eq $ENEEDEVAC ]] && continue
		[[ $rl_skip_err -eq $ENOSPLIT && -n "$debug_offdisk" ]] && continue
		return $rl_reterr
	    fi

	    # The volume has changed.
	    if [[ $rl_skip_err -eq $ENEEDEVAC ]]
	    then
		# volume subdisks were skipped because the volume was
		# evaced from one of its disks
		if [[ $nevac -eq 0 ]]
		then
		    nevac_sds=`vxprint -g $dg -pe \
			'!pl_raidlog && assoc.name="'$lastv'"' \
			-F %aslist | sed -e 's/,/ /g' | wc -w | awk '{print $1}'`
		fi
		if [[ $nevac -le $nevac_sds ]]
		then
		    (( nevac += 1))
		    print_debug "Reanalyzing $lastv (pass $nevac) after evacuating $lastdm"
		    # we've skipped the rest of the subdisks in the volume
		    # after evacuating the volume from one of the disks
		    # now go back an reanalyze the volume
		    #
		    # save the new volume,plex,etc while we reprocess the
		    # one which was evacuated off-disk but only save it if
		    # we're not already reading from $relayout_altstdin
		    [[ ! -s $relayout_altstdin ]] && {
			sv_vol=$vol
			sv_pl=$pl
			sv_dm=$dm
			sv_sd=$sd
			sv_pl_column=$pl_column
			sv_pl_coloff=$pl_coloff
		    }
		    vxprint -g $dg -pe '!pl_raidlog && assoc.name="'$lastv'"' \
			-F %aslist | sed -e 's/,/ /g' | \
			relayout_print_sds > $relayout_altstdin
		    exec 6< $relayout_altstdin

		    # set last state to prevent column termination at start
		    # of reprocessing this volume
		    lastv= lastpl= lastdm= lastsd= pl_column_last= pl_coloff_last=

		    # reset skip state
		    rl_skip=
		    rl_skip_match=
		    rl_skip_err=

		    # setup state for next plex column
		    plexcolumn_state_init

		    continue
		else
		    print_err "E_VOL_MAX_EVAC $lastv"
		    rl_skip_err=$EMAXEVAC
		    rl_reterr=$EMAXEVAC
		fi
	    fi

	fi

	if [[ ! X$vol = X$lastv ]]
	then
	    # the volume name has changed

	    # do column termination processing when the volume
	    # name changes but only if no errors were seen
	    if [[ X$lastv != X && X$rl_skip = X ]]
	    then
		# finish processing for previous plex column
		# then reset the plex state info
		print_debug "column termination (vol $lastv/$vol)"
		sdr_plexcolumn_term $align
		ret=$?
		[[ $ret -eq 0 ]] || {
		    # rl_skip_err and rl_reterr may have been reset
		    if [[ ! $rl_skip_err -eq $ENEEDEVAC && $e_flag -gt 0 ]]
		    then
			if [[ ! $rl_skip_err -eq $ENOSPLIT || -z "$debug_offdisk" ]]
			then
			    return $rl_reterr
			fi
		    fi
		    # Column termination processing failed but it
		    # was due to ENEEDEVAC and the disk was successfully
		    # evacuated
		}
	    fi

	    # break out if we see the sentinel
	    if [[ $vol = $sentinel ]]
	    then
		break
	    fi

	    lastv=$vol
	    lastpl=$pl
	    lastsd=$sd
	    lastdm=$dm
	    [[ $pl_column -ne 0 || $pl_coloff -ne 0 ]] && {
		export sd vol
		p_list=`egettxt "relayout: ERROR! ${sd}: Plex column/offset is not 0/0 for new vol $vol" \
		    vxvmshm:3431`
		print_anal "$p_list"
		return $EINTERNAL
	    }
	    pl_column_last=$pl_column
	    pl_coloff_last=$pl_coloff

	    rl_skip=
	    rl_skip_match=
	    rl_skip_err=
	    
	    nevac=0
	
	    # setup state for next plex column
	    plexcolumn_state_init

	fi
	if [[ ! X$pl = X$lastpl ]]
	then
	    # do column termination processing when the plex
	    # name changes but only if no errors were seen
	    if [[ X$rl_skip = X ]]
	    then
		# finish processing for previous plex column
		# then reset the plex state info
		print_debug "column termination (plex $lastpl/$pl)"
		sdr_plexcolumn_term $align
		ret=$?
		[[ $ret -eq 0 ]] || {
		    # rl_skip_err and rl_reterr may have been reset
		    if [[ ! $rl_skip_err -eq $ENEEDEVAC && $e_flag -gt 0 ]]
		    then
			if [[ ! $rl_skip_err -eq $ENOSPLIT || -z "$debug_offdisk" ]]
			then
			    return $rl_reterr
			fi
		    fi
		    # Column termination processing failed but it
		    # was due to ENEEDEVAC and the disk was successfully
		    # evacuated
		}
	    fi

	    lastpl=$pl
	    lastsd=$sd
	    lastdm=$dm
	    [[ $pl_column -ne 0 || $pl_coloff -ne 0 ]] && {
		export sd pl
		p_list=`egettxt "relayout: ERROR! ${sd}: Plex column/offset is not 0/0 for new plex $pl" \
		    vxvmshm:3430`
		print_anal "$p_list"
		return $EINTERNAL
	    }
	    pl_column_last=$pl_column
	    pl_coloff_last=$pl_coloff

	    # setup state for next plex column
	    plexcolumn_state_init
	fi
	
	if [[ $pl_column -eq $pl_column_last ]]
	then
	    if [[ $pl_coloff -lt $pl_coloff_last ]]
	    then
		export pl_column pl
		p_list=`egettxt "relayout: ERROR! Plex column offset is not strictly increasing for column/plex $pl_column/$pl" \
		    vxvmshm:3434`
		print_anal "$p_list"
		return $EINTERNAL
	    fi
	else
	    if [[ $pl_column -ne $((pl_column_last + 1)) ]]
	    then
		export pl
		p_list=`egettxt "relayout: ERROR! Plex column is not increasing for plex $pl" \
		    vxvmshm:3432`
		print_anal "$p_list"
		return $EINTERNAL
	    fi
	    if [[ $pl_coloff -ne 0 ]]
	    then
		export pl_column pl
		p_list=`egettxt "relayout: ERROR! Plex column offset is not 0 for new column/plex $pl_column/$pl" \
		    vxvmshm:3433`
		print_anal "$p_list"
		return $EINTERNAL
	    fi

	    # do column termination processing when the plex column changes
	    # but only if there were no errors
	    if [[ X$rl_skip = X ]]
	    then
		print_debug "column termination (column $pl_column_last/$pl_column)"
		sdr_plexcolumn_term $align
		ret=$?
		[[ $ret -eq 0 ]] || {
		    # rl_skip_err and rl_reterr may have been reset
		    if [[ ! $rl_skip_err -eq $ENEEDEVAC && $e_flag -gt 0 ]]
		    then
			if [[ ! $rl_skip_err -eq $ENOSPLIT || -z "$debug_offdisk" ]]
			then
			    return $rl_reterr
			fi
		    fi
		    # Column termination processing failed but it
		    # was due to ENEEDEVAC and the disk was successfully
		    # evacuated
		    continue
		}
	    fi
	    pl_column_last=$pl_column
	fi
	pl_coloff_last=$pl_coloff
	lastsd=$sd
	lastdm=$dm

	# sd dm device lbaoff len
	if [[ $i_flag -gt 0 || -s $relayout_altstdin ]]
	then
	    echo $sd | sdlba 2>/dev/null >$tmp11
	    read sd sddm junk sdlba sdlen junk < $tmp11
	else
	    grep "^$sd " $active_sdlba >$tmp11
	    read junk sddm junk sdlba sdlen junk < $tmp11
	fi

	# dm pub_off pub_diskoff pub_lbaoff pub_len pub_lbaoff_cds pub_len_cds not_cds
	tail -1 $dir_dms/${sddm}.bounds | awk '{ print $6, $7 }' >$tmp11
	read dmlba_cds dmlen_cds < $tmp11

	# ascertain sd alignment, length, if not within CDS disk restrictions
	rl_realign=$((sdlba % $align))
	rl_resize=$((sdlen  % $align))
	extent_bounded $sdlba $sdlen $dmlba_cds $dmlen_cds
	rl_restrict=$?

	print_debug "processing $sd/$pl/$vol ($rl_realign $rl_resize $rl_restrict $rl_copyop_in)"
	
	# if alignment, size, and disk restrictions are OK
	# AND we do not have a carryover from the sd in the column,
	# there's nothing to do
	if ((rl_realign == 0 && rl_resize == 0 && rl_restrict == 0 && rl_copylen_in == 0))
	then
	    # nothing to do

	    print_debug "	no relayout needed"

	    plexcolumn_state_roll
	    continue
	fi

	((rl_nsd += 1))
	echo $vol >> $action_vols

	[[ $n_flag -gt 0 ]] && {
	    #echo "$sd	realign=$((rl_realign > 0)) resize=$((rl_resize > 0)) restrict=$((rl_restrict > 0))"
	    alist="$alist $sd"

	    plexcolumn_state_roll
	    continue
	}

	# perform relayout operations for this subdisk
	for junk in 1
	do
	    if [[ $rl_copylen_in -eq 0 ]]
	    then

		if ((rl_realign == 0 && rl_restrict == 0))
		then
		    sdr_resize_only $sd $sddm $sdlba $sdlen $align
		    ret=$?
		    [[ $ret -eq 0 ]] || {
			rlfunc=sdr_resize_only
			break
		    }
		elif ((rl_realign == 0))
		then
		    # restrict only will never occur during
		    # subdisk relayout as the disk converter
		    # is run before we get here which handles 
		    # all cases where a subdisk needs attention
		    # due to CDS boundary issues

		    internal_err_exit "relayout: ERROR! subdisk $sd is outside disk boundaries"
		else
		    sdr_realign $sd $sddm $sdlba $sdlen $align
		    ret=$?
		    [[ $ret -eq 0 ]] || {
			rlfunc=sdr_realign
			break
		    }
		fi

	    else
		case $rl_copyop_in in
		copyback)
		    sdr_realign2 $sd $sddm $sdlba $sdlen $align
		    ret=$?
		    [[ $ret -eq 0 ]] || {
			rlfunc=sdr_realign2
			break
		    }
		    ;;
		copyfwd)
		    sdr_realign3 $sd $sddm $sdlba $sdlen $align
		    ret=$?
		    [[ $ret -eq 0 ]] || {
			rlfunc=sdr_realign3
			break
		    }
		    ;;
		*)
		    internal_err_exit "relayout: ERROR! invalid copy op $rl_copyop_in"
		    ;;
		esac
	    fi
	done

	[[ $ret -eq 0 ]] && {
	    print_debug "state: term=$rl_copyterm_out op=$rl_copyop_out \
len=$rl_copylen_out sd=$rl_sd_out sdjoin=$rl_sdjoin_out"
	    plexcolumn_state_roll
	    continue
	}

	# error processing
	relayout_seterr $rlfunc $ret $vol $pl $sd $dm

    done < $tmp15
    [[ $n_flag -gt 0 && X$alist != X ]] && {
	mk_print_list "$alist" a_list
	export a_list; p_list=`egettxt "\
	
	The following list contains subdisks that need relayout
	to conform to 8K alignment restrictions.

	    $a_list
" vxvmshm:3414`
	print_anal "$p_list"
    }

    print_debug "relayout: made modifications to $rl_nsd subdisks"
    return $rl_reterr
}

#
# dgconv_cleanup - remove all files created by an invocation of the converter
#	for diskgroup $dg
#
function dgconv_cleanup
{
    $set_x
    [[ -z "$tprefix" ]] && {
	internal_err_exit "cleanup: global variable \$tprefix not set"
    }
    print_verbose "Cleaning up conversion files for $dg ..."
    rm -rf ${tprefix}
}

#
# dgconv_init - initialize the diskgroup converter
#
function dgconv_init
{	
    $set_x
    typeset var varval

    print_verbose "Initializing diskgroup converter ..."

    # temp file init
    tprefix=/tmp/vxcds/$dg/	# must be capable of "rm -rf ${tprefix}*"
    tsuffix=-1
    tmp1=${tprefix}dgconv-tmp1
    tmp2=${tprefix}dgconv-tmp2
    # used for temporary storage only - should not be expected to remain
    # intact across function calls
    tmp9=${tprefix}dgconv-tmp9
    tmp10=${tprefix}dgconv-tmp10
    # used for 'read' support (required for bash)
    # Use temp file to workaround the fact that bash can't
    # do "read var ..." at the end of a pipeline and have
    # var values which are read be persistent (due to the fact
    # that the the read is done in a subshell)
    tmp11=${tprefix}dgconv-tmp11
    tmp12=${tprefix}dgconv-tmp12
    tmp13=${tprefix}dgconv-tmp13
    tmp14=${tprefix}dgconv-tmp14
    tmp15=${tprefix}dgconv-tmp15

    for var in dg _8K _BSIZE split_subdisks_ok move_subdisks_ok \
	evac_subdisks_ok novolstop analyze_only min_split_size
    do
	eval "varval=\$$var"
	[[ -n "$varval" ]] ||
	    internal_err_exit "dgconv_init: \$$var is not set"
    done

    # DM boundaries
    # format:
    # dm pub_off pub_diskoff pub_lbaoff len pub_lbaoff_cds len_cds cds
    # if "cds" is !0, then part of the public region exists outside CDS 
    # boundaries location where 'vxdisk list' dm info resides (by DM name)
    dir_dms=${tprefix}dms/dms${tsuffix}
    [[ ! -d $dir_dms ]] && mkdir -p $dir_dms

    # Comment used when creating new subdisks
    vxc_comment=VXCDS-SD-TMP
    vxc_comment_sdlog=VXCDS-LOG-SDSRC
    vxc_comment_pllog=VXCDS-LOG-PLSRC

    error_code_init

    dgconv_init_files
	    
    print_verbose "Gathering disk media info ..."
    dm_configall

    > $action_vols

    dgconv_enable_copyfwd=yes
    #print_debug "dgconv_enable_copyfwd = $dgconv_enable_copyfwd"
}



#
# sdwork - analyze subdisks for conformance to alignment restrictions
#
# Usage: sdwork [-q] sd ... align
# RETURN:
#	0: no relayout required
#	>0: relayout required
#		ret & 1 : restrict issues
#		ret & 2 : resize issues
#		ret & 4 : realign issues
function sdwork
{
    $set_x
    typeset i_flag=0 sd sddm sdlba junk align q_flag=0
    typeset dmlba_cds dmlen_cds sds
    typeset realign resize restrict ret out_sds=
    typeset fret=0

    [[ X$1 = X-q ]] && {
	q_flag=1
	shift
    }
    [[ $# -gt 1 ]] || internal_err_exit "sdwork: arg count ($*)"

    sds="$*"
    align=${sds##* }
    sds=${sds% *}

    [[ $align = 1 || $align = $_8K ]] || {
	internal_err_exit "sdwork: align parameter must be 1 or $_8K"
    }

    echo $sds | sdlba >$tmp13
    while ((1))
    do
	read sd sddm junk sdlba sdlen junk
	[[ $? -eq 0 ]] || break
	tail -1 $dir_dms/${sddm}.bounds | awk '{ print $6, $7 }' >$tmp11
	read dmlba_cds dmlen_cds < $tmp11

	realign=$((sdlba % $align))
	resize=$((sdlen  % $align))
	extent_bounded $sdlba $sdlen $dmlba_cds $dmlen_cds
	restrict=$?
	ret=$(( (realign > 0) * 4 + (resize > 0) * 2 + (restrict > 0) ))
	[[ $ret -eq 0 ]] || {
	    if [[ $q_flag -gt 0 ]]
	    then
		return $ret
	    else
		fret=$ret
		out_sds="$out_sds $sd"
	    fi
	}
    done < $tmp13
    echo $out_sds
    return $fret
}

#
# cleanup_cachevols - grow cache volumes and set autogrowby/maxautogrow
#	so they are 8K multiples in length prior to doing subdisk relayout.
#	We need to use "vxcache grow" on the cache volume to ensure that the
#	cache metadata and cachevol are in sync.
#
function cleanup_cachevols
{
    $set_x
    typeset vol junk cache len newlen cmd growby maxgrow
    typeset alist

    print_verbose "Performing growth operations on cache objects ..."

    # get a list of cachevols and adjust lengths
    vxprint -g $dg -vF '%name %iscachevol %len %cachename' | grep " on " >$tmp11
    alist=
    while ((1))
    do
	read vol junk len cache
	[[ $? -eq 0 ]] || break
	newlen=`alignup $len $_8K`
	[[ $newlen -eq $len ]] && continue
	cmd="vxcache -g $dg growcacheto $cache $newlen"
	[[ X$analyze_only = Xy* ]] && {
	    alist="$alist ${cache}:${newlen}"
	    continue
	}
	doit -n "E_CACHE_GROWTO $cache $newlen" $cmd
	[[ $? -eq 0 ]] || return $EUTIL
    done < $tmp11
    [[ X$analyze_only = Xy* && X$alist != X ]] && {
	mk_print_list "$alist" a_list
	export a_list; p_list=`egettxt "\
	
	The following list contains CACHE objects that need the length adjusted
	to the indicated value to conform to 8K alignment restrictions.
	List items are in the form cache:newlength.

	    $a_list
" vxvmshm:3453`
	    print_anal "$p_list"
    }

    # get a list of cache objects
    vxprint -g $dg -CF '%name %autogrowby %maxautogrow' > $tmp11

    # adjust autogrowby attrs
    alist=
    while ((1))
    do
	read cache growby junk
	[[ $? -eq 0 ]] || break
	newlen=`alignup $growby $_8K`
	[[ $newlen -eq $growby ]] && continue
	cmd="vxcache -g $dg set autogrowby=$newlen $cache"
	[[ X$analyze_only = Xy* ]] && {
	    alist="$alist ${cache}:${newlen}"
	    continue
	}
	doit -n "E_CACHE_AUTOGROWBY $cache $newlen" $cmd
	[[ $? -eq 0 ]] || return $EUTIL
    done < $tmp11
    [[ X$analyze_only = Xy* && X$alist != X ]] && {
	mk_print_list "$alist" a_list
	export a_list; p_list=`egettxt "\
	
	The following list contains CACHE objects that need the autogrowby
	attribute adjusted to the indicated value to conform to 8K alignment
	restrictions.
	List items are in the form cache:newautogrowby.

	    $a_list
" vxvmshm:3495`
	    print_anal "$p_list"
    }

    # adjust maxautogrow attrs
    alist=
    while ((1))
    do
	read cache junk maxgrow
	[[ $? -eq 0 ]] || break
	newlen=`alignup $maxgrow $_8K`
	[[ $newlen -eq $maxgrow ]] && continue
	cmd="vxcache -g $dg set maxautogrow=$newlen $cache"
	[[ X$analyze_only = Xy* ]] && {
	    alist="$alist ${cache}:${newlen}"
	    continue
	}
	doit -n "E_CACHE_MAXAUTOGROW $cache $newlen" $cmd
	[[ $? -eq 0 ]] || return $EUTIL
    done < $tmp11
    [[ X$analyze_only = Xy* && X$alist != X ]] && {
	mk_print_list "$alist" a_list
	export a_list; p_list=`egettxt "\
	
	The following list contains CACHE objects that need the maxautogrow
	attribute adjusted to the indicated value to conform to 8K alignment
	restrictions.
	List items are in the form cache:newmaxautogrow.

	    $a_list
" vxvmshm:3496`
	    print_anal "$p_list"
    }

    return 0
}

#
# cleanup_r5logs - rebuild any RAID logs which contain subdisks
#	which do not conform to 8K alignment restrictions
#
function cleanup_r5logs
{	
    $set_x
    typeset vols i nlogs found log junk type pl dm state sds worksds reterr=0
    typeset alist

    print_verbose "Rebuilding RAID5 logs ..."

    # get a list of all raidlog plexes, and their vols,sds
    vxprint -g $dg -pe 'pl_raidlog' -F '%v_name %pl_name %aslist' |
	sed -e 's/,/ /g' -e 's/$/ /' > $tmp1
    # get a list of the raidlog sds which are not CDS compliant
    sds=`awk '{ $1=""; $2=""; print }' $tmp1`
    [[ -n "$sds" ]] && worksds=`sdwork $sds $_8K`
    [[ -z "$worksds" ]] && return 0

    # operate on only the vols that need logs converted
    vxprint -g $dg -F '%v_name' $worksds | sort -u >$tmp12
    while ((1))
    do
	    read i
	    [[ $? -eq 0 ]] || break
	    [[ X$i = X- ]] && continue
	    [[ X`vxprint -g $dg -F %stp_rid $i` = X- ]] || {
		internal_err_exit "cleanup_r5logs: $i: Allocator volume encountered"
	    }
	    [[ X$analyze_only = Xy* ]] && {
		alist="$alist $i"
		continue
	    }
	    # if any of the sds associated w/ the plex are not cds compliant,
	    # rebuild all the logs for the volume.
	    sds=`grep "^$i " $tmp1 | awk '{ $1=""; $2=""; print }'`
	    nlogs=`grep "^$i " $tmp1 | wc -l`

	    # add a new log and determine its dm record
	    doit -n "E_VOL_ADD_LOG $i RAID" \
		vxassist -g $dg -a $_8K addlog $i $evac_disk_list
	    [[ $? -eq 0 ]] || return $EUTIL

	    found=0
	    log=0
	    vxprint -g $dg -htv $i >$tmp11
	    while ((1))
	    do
		read type junk pl dm state junk
		[[ $? -eq 0 ]] || break
		if [[ $type = pl ]]
		then
		    if [[ $state = LOG ]]
		    then
			log=1
		    else
			log=0
		    fi
		    continue
		fi
		[[ $type = sd && $log -gt 0 ]] || continue
		if [[ X`grep " $pl " $tmp1` = X ]]
		then
		    # this is the new one
		    found=1
		    break
		fi
	    done < $tmp11
	    if [[ $found -eq 0 ]]
	    then
		return $ENOMATCH
	    fi

	    # remove all logs but the one we just created
	    # this will remove all but one log located on $dm
	    doit -n "E_VOL_RM_LOG $i RAID" \
		vxassist -g $dg remove log $i nlog=1 dm:$dm
	    [[ $? -eq 0 ]] || return $EUTIL
	    [[ $nlogs -gt 1 ]] && {
		# add the subsequent logs
		doit -n "E_VOL_ADD_LOG $i RAID" \
		    vxassist -g $dg -a $_8K addlog $i nlog=$((nlogs - 1)) \
			$evac_disk_list
		[[ $? -eq 0 ]] || return $EUTIL
	    }
	    print_verbose "$i: RAID logs rebuilt"
    done < $tmp12
    [[ X$analyze_only = Xy* && X$alist != X ]] && {
	mk_print_list "$alist" a_list
	export a_list; p_list=`egettxt "\
	
	The following list contains RAID volumes that need the log rebuilt
	to conform to 8K alignment restrictions.

	    $a_list
" vxvmshm:3412`
	print_anal "$p_list"
    }
    return 0
}

#
# cleanup_st_width - relayout the stripe width for any RAID and/or
#	STRIPE plexes whose stripe width is not a multiple of 8K
#
# Usage: cleanup_st_width stripe|raid|all
#
function cleanup_st_width
{
    $set_x
    typeset vol stw stwidth reterr=0 do_stripe=0 do_raid=0 alist

    [[ $# -eq 1 ]] || internal_err_exit "cleanup_st_width: arg count ($*)"
    type=$1

    case $1 in
    stripe)	do_stripe=1 ;;
    raid)	do_raid=1 ;;
    all)	do_stripe=1; do_raid=1 ;;
    *)
	internal_err_exit "cleanup_st_width: invalid stripe type $1"
	;;
    esac

    [[ $do_stripe -gt 0 ]] && {
	print_verbose "Stripe width relayout ..."
	alist=
	# pl_st_width expressions do not work for RAID5
	# so we'll handle those volumes separately
	vxprint -g $dg -pe 'pl_layout=STRIPE && pl_st_width%'$_8K'!=0' \
	    -F '%v_name %st_width' | sort -u >$tmp11
	while ((1))
	do
	    read vol stw
	    [[ $? -eq 0 ]] || break
	    [[ X$vol = X- ]] && continue
	    [[ X`vxprint -g $dg -F %stp_rid $vol` = X- ]] || {
		    internal_err_exit "cleanup_st_width: $vol: Allocator volume encountered"
	    }

	    [[ X`vxprint -g $dg -F %layered $vol` = Xon ]] && {
		# For layered volumes we must operate on the topmost object
		continue
	    }
	    stwidth=`alignup $stw $_8K`
	    [[ X$analyze_only = Xy* ]] && {
		alist="$alist ${vol}:${stwidth}"
		continue
	    }
	    doit -n "E_VOL_STW_RELAYOUT $vol" \
		vxassist -g $dg -a $_8K relayout $vol st_width=$stwidth \
		    $evac_disk_list
	    [[ $? -eq 0 ]] || {
		reterr=$EUTIL
		doit -n "E_VOL_REV_RELAYOUT $vol" vxrelayout -g $dg reverse $vol
		continue
	    }
	    print_verbose "$vol: stripe width changed to $stwidth"
	done < $tmp11
	[[ X$analyze_only = Xy* && X$alist != X ]] && {
	    mk_print_list "$alist" a_list
	    export a_list; p_list=`egettxt "\
	
	The following list contains STRIPE volumes that need stripe width
	relayout to the value indicated to conform to 8K alignment
	restrictions.
	List items are in the form volname:stripewidth.

	    $a_list
" vxvmshm:3413`
	    print_anal "$p_list"
	}
    }

    [[ $do_raid -gt 0 ]] && {
	print_verbose "RAID stripe width relayout ..."
	alist=
	vxprint -g $dg -pe 'pl_layout=RAID' -F '%v_name %st_width' | \
	    sort -u >$tmp11
	while ((1))
	do
	    read vol stw
	    [[ $? -eq 0 ]] || break
	    [[ X$vol = X- ]] && continue
	    [[ X`vxprint -g $dg -F %layered $vol` = Xon ]] && {
		# For layered volumes we must operate on the topmost object
		continue
	    }
	    stwidth=`alignup $stw $_8K`
	    [[ $stw -eq $stwidth ]] && continue
	    [[ X$analyze_only = Xy* ]] && {
		alist="$alist ${vol}:${stwidth}"
		continue
	    }
	    doit -n "E_VOL_STW_RELAYOUT $vol" \
		vxassist -g $dg -a $_8K relayout $vol st_width=$stwidth \
		    $evac_disk_list
	    [[ $? -eq 0 ]] || {
		reterr=$EUTIL
		doit -n "E_VOL_REV_RELAYOUT $vol" vxrelayout -g $dg reverse $vol
		continue
	    }
	    print_verbose "$vol: raid stripe width changed to $stwidth"
	done < $tmp11
	[[ X$analyze_only = Xy* && X$alist != X ]] && {
	    mk_print_list "$alist" a_list
	    export a_list; p_list=`egettxt "\
	
	The following list contains RAID volumes that need stripe width
	relayout to the value indicated to conform to 8K alignment
	restrictions.
	List items are in the form volname:stripewidth.

	    $a_list
" vxvmshm:3411`
	    print_anal "$p_list"
	}
    }

    return $reterr
}

#
# cleanup_drl_dcm_logs - make DRL and DCM logs 8K alignment compliant
#
# Usage: cleanup_drl_dcm_logs [-(D1|D2|D3)]
# debug levels:
#	1: only set the comment fields in the logs
#	2: set comment fields and dissociate logs
#	3: set comment fields, dissociate logs, create new logs
#
function cleanup_drl_dcm_logs
{
    $set_x
    typeset vols vol topvol j nlog logsds logtypes found log junk type dm sds
    typeset loglen tmp freeoff ret worksds reterr=0
    typeset mapsz logalign maplen oldlog n pl pls comment rid
    typeset local_debug=0 alist badlogsdlen layered attrs workaround_IR131866

    while [[ -n "$1" ]]
    do
	case $1 in
	-D1)	local_debug=1 ;;
	-D2)	local_debug=2 ;;
	-D3)	local_debug=3 ;;
	*)	break ;;
	esac
	shift
    done

    print_verbose "Rebuilding DRL/DCM logs ..."

    vols=`vxprint -g $dg -se 'sd_is_log' -F '%v_name' | sort -u`
    for vol in $vols
    do
	ret=0
	[[ X$vol = X- ]] && continue
	[[ X`vxprint -g $dg -F %stp_rid $vol` = X- ]] || {
	    internal_err_exit "cleanup_drl_dcm_logs: $vol: Allocator volume encountered"
	}
	vxprint -g $dg -vF \
	    '%layered %loglen %logtype %logmap_align %logmap_len %oldlog' $vol >$tmp11
	read layered loglen logtype logalign maplen oldlog < $tmp11
	    
	logsds=`vxprint -g $dg \
	    -se 'sd_is_log && assoc.assoc.name=="'$vol'"' \
	    -F %name`
	nlog=`echo $logsds | wc -w | awk '{print $1}'`
	[[ $nlog -gt 0 ]] || {
	    export logtype
	    p_list=`egettxt "cleanup_drl_dcm_logs: number of $logtype logs is 0" \
		vxvmshm:3421`
	    print_anal "$p_list"
	    reterr=$EINTERNAL
	    continue
	}
	# logging sds w/ non-compliant length
	badlogsdlen=`vxprint -g $dg \
	    -se 'len%'$_8K'!=0 && sd_is_log && assoc.assoc.name=="'$vol'"' \
	    -F %name`

	# skip if logs are already compliant
	[[ $logalign -ge $_8K && X$oldlog = Xoff && -z "$badlogsdlen" ]] && continue

	[[ X$analyze_only = Xy* ]] && {
	    alist="$alist ${vol}:${logtype}"
	    continue
	}

	topvol=$vol
	workaround_IR131866=
	if [[ X$layered = Xon ]]
	then
	    topvol=`vxprint -g $dg -se 'sd_dmname="'$vol'"' -F %v_name | sort -u`
	    if [[ $logtype = DCM ]]
	    then
		workaround_IR131866=true
	    fi
	fi

	# rebuild all the logs and set the log length

	if [[ $logalign -lt $_8K ]]
	then
	    case $logtype in
	    DCM)
		mapsz=$((loglen / 2))
		loglen=`alignup $mapsz $_8K`
		((loglen *= 2))
		;;
	    REGION)
		# determine current mapsize
		if [[ X`vxdg list | grep "^${dg}.*shared"` = X ]]
		then
		    # private dg
		    mapsz=$((loglen / 2))
		    loglen=`alignup $mapsz $_8K`
		    ((loglen *= 34))
		    mapsz=$((mapsz * _BSIZE - 64))
		else
		    # shared dg
		    mapsz=$((loglen / 33))
		    loglen=`alignup $mapsz $_8K`
		    ((loglen *= 34))
		    mapsz=$((mapsz * _BSIZE - 64))
		fi
		[[ $maplen -eq 0 ]] && maplen=$mapsz
		;;
	    *)
		print_err "E_VOL_BAD_LOGTYPE $vol $logtype"
		reterr=$EINTERNAL
		continue
		;;
	    esac
	fi

	# set comment in logging sds/plexes for recovery purposes
	sds=
	pls=
	for log in $logsds
	do
	    pl=`vxprint -g $dg -sF %pl_name $log`
	    if [[ X`vxprint -g $dg -pF %plex_LOGONLY_title $pl` = XLOGONLY ]]
	    then
		rid=`vxprint -g $dg -vF %rid $vol`
		comment="${vxc_comment_pllog}:${rid}"
		[[ `echo $comment | wc -c` -gt 40 ]] && {
		    print_err "E_MAX_COMMENT $comment"
		    reterr=$EMAXCOMMENT
		    continue
		}
		doit -n "E_OBJ_EDIT $pl" \
		    vxedit -g $dg set comment="$comment" $pl
		[[ $? -eq 0 ]] || {
		    reterr=$EUTIL
		    continue
		}
		pls="$pls $pl"
	    else
		rid=`vxprint -g $dg -pF %rid $pl`
		comment="${vxc_comment_sdlog}:${rid}"
		[[ `echo $comment | wc -c` -gt 40 ]] && {
		    print_err "E_MAX_COMMENT $comment"
		    reterr=$EMAXCOMMENT
		    continue
		}
		doit -n "E_OBJ_EDIT $log" \
		    vxedit -g $dg set comment="$comment" $log
		[[ $? -eq 0 ]] || {
		    reterr=$EUTIL
		    continue
		}
		sds="$sds $log"
	    fi
	done
	[[ $local_debug -eq 1 ]] && continue

	if [[ -n "$workaround_IR131866" ]]
	then
	    # due to a bug/feature, when working with layered vols
	    # we can't use vxplex/vxsd to remove all the logs and "vxassist addlog"
	    # create new ones for some kinds of logs.
	    # So, we need to use vxassist to remove and add
	    # the logs. This will break recovery if we fail between
	    # removing the old logs and building new ones
	    doit -n "E_VOL_RM_LOG $topvol $logtype" \
		vxassist -g $dg remove log $topvol nlog=0 \
		    $evac_disk_list
	    [[ $? -eq 0 ]] || ret=$EUTIL
	else
	    # Dissociate the logging sds/plexes from the parent vols
	    if [[ -n "$sds" ]]
	    then
		doit -n "E_SD_DIS `echo $sds | sed -e 's/ /,/g'`" \
		    vxsd -g $dg dis $sds
		[[ $? -eq 0 ]] || ret=$EUTIL
	    fi
	    [[ $ret -eq 0 ]] && {
		if [[ -n "$pls" ]]
		then
		    doit -n "E_PLEX_DIS `echo $pls | sed -e 's/ /,/g'` $vol" \
			vxplex -g $dg dis $pls
		    [[ $? -eq 0 ]] || ret=$EUTIL
		fi		    
	    }
	fi
	[[ $local_debug -eq 2 ]] && continue

	# At this point the old logging sds/plexes have been dissociated and will
	# be reassociated if need be during recovery if we die before
	# creating the new logs

	# rebuild the new logs
	[[ $ret -eq 0 ]] && {
	    attrs="nlog=$nlog logtype=$logtype loglen=$loglen"
	    [[ $logtype = REGION ]] && attrs="$attrs logmap_len=$maplen"
	    doit -n "E_VOL_ADD_LOG $topvol $logtype" \
		vxassist -g $dg -a $_8K addlog $topvol $attrs \
		    $evac_disk_list
	    [[ $? -eq 0 ]] || ret=$EUTIL
	}
	[[ $local_debug -eq 3 ]] && continue

	if [[ -n "$workaround_IR131866" ]]
	then
	    :
	else
	    # Now delete the original logging sds/plexes
	    [[ $ret -eq 0 && -n "$sds" ]] && {
		doit -n "E_SD_RM `echo $sds | sed -e 's/ /,/g'`" \
		    vxedit -g $dg -fr rm $sds
		[[ $? -eq 0 ]] || ret=$EUTIL
	    }
	    [[ $ret -eq 0 && -n "$pls" ]] && {
		doit -n "E_PLEX_RM `echo $pls | sed -e 's/ /,/g'`" \
		    vxedit -g $dg -fr rm $pls
		[[ $? -eq 0 ]] || ret=$EUTIL
	    }
	fi

	if [[ $ret -eq 0 ]]
	then
	    print_verbose "$vol: $logtype logs rebuilt"
	else
	    reterr=$ret
	fi

    done
    [[ X$analyze_only = Xy* && X$alist != X ]] && {
	mk_print_list "$alist" a_list
	export a_list; p_list=`egettxt "\
	
	The following list contains volumes that need the DRL/DCM log
	rebuilt to conform to 8K alignment restrictions.
	List items are in the form volname:logtype.

	    $a_list
" vxvmshm:3415`
	print_anal "$p_list"
    }
	    
    return $reterr
}

#
# cleanup_lengths - adjust volume lengths to be a multiple of 8K
#
function cleanup_lengths
{
    $set_x
    typeset vol vollen cmd layout width reterr=0 loglen alist
    typeset instant layered nlayer which need_vxassist

    which=$1

    case $which in
    pre-relayout)
	;;
    post-relayout)
	;;
    *)
	internal_err_exit "cleanup_lengths: bad args ($*)"
	;;
    esac
    print_verbose "Adjusting volume lengths ..."

    # sort so sublayers are resized first
    vxprint -g $dg -ve 'len%'$_8K'!=0' -F '%v_name %layered %nlayer %instant %len' |
	sort +1r >$tmp12
    alist=
    while ((1))
    do
	read vol layered nlayer instant vollen
	[[ $? -eq 0 ]] || break
	[[ X$vol = X- ]] && continue
	if [[  X$instant = Xon ]] || [[ X$layered = Xoff && $nlayer -gt 1 ]]
	then
	    # need vxassist to resize instant snapshots (at least SO ones)
	    # also need vxassist to resize layered vols
	    need_vxassist=true
	else
	    need_vxassist=
	fi
	if [[ X$which = Xpre-relayout && X$need_vxassist != Xtrue ]]
	then
	    # for pre-relayout we only resize using vxassist
	    continue
	fi
	if [[ X$which = Xpost-relayout && X$need_vxassist = Xtrue ]]
	then
	    # for post-relayout we only resize using vxvol
	    continue
	fi
	vxprint -g $dg -pe '!pl_raidlog && assoc.name="'$vol'"' \
	    -F '%layout %ncol %st_width' >$tmp11
	read layout ncol width < $tmp11
	case $layout in
	STRIPE)
	    newlen=`alignup $vollen $((width * ncol))`
	    ;;
	RAID)
	    newlen=`alignup $vollen $(( width * (ncol - 1) ))`
	    ;;
	CONCAT)
	    newlen=`alignup $vollen $_8K`
	    ;;
	*)
	    export layout vol
	    p_list=`egettxt "cleanup_lengths: unexpected plex layout $layout in volume $vol" \
		vxvmshm:3424`
	    print_anal "$p_list"
	    return $EINTERNAL
	    ;;
	esac
	[[ X$analyze_only = Xy* ]] && {
	    alist="$alist ${vol}:${newlen}"
	    continue
	}
	# SO instant snapshots require the volume length to be set
	# using vxassist as it will reset the lengths of contained
	# subcaches. Similarly layered volumes may need the same to
	# resize subvolumes. All other volumes are resized using vxvol.
	if [[ X$need_vxassist = Xtrue ]]
	then
	    cmd="vxassist -g $dg growto $vol $newlen layout=nodiskalign"
	else
	    cmd="vxvol -g $dg set len=$newlen $vol"
	fi
	doit -n "E_VOL_SET_LENGTH $vol $newlen" $cmd
	ret=$?
	[[ $ret -eq 0 ]] || reterr=$EUTIL
    done < $tmp12
    [[ X$analyze_only = Xy* && X$alist != X ]] && {
	mk_print_list "$alist" a_list
	export a_list; p_list=`egettxt "\
	
	The following list contains volumes that need the length adjusted
	to the indicated value to conform to 8K alignment restrictions.
	List items are in the form volname:newlength.

	    $a_list
" vxvmshm:3416`
	print_anal "$p_list"
    }

    return $reterr
}

#
# cleanup_oldlogs - clear the V_PLFLAG_OLDLOG flag on any volumes
#	it is present on
#
function cleanup_oldlogs
{
    $set_x
    typeset vol logtype oldlog reterr=0 alist ret=0

    print_verbose "Clearing oldlog flags ..."

    vxprint -g $dg -vF '%v_name %logtype %len %oldlog' | grep ' on$' > $tmp11
    [[ ! -s $tmp11 ]] && return 0
    while ((1))
    do
	read vol logtype len oldlog
	[[ $? -eq 0 ]] || break
	case $logtype in
	REGION|DCM)
	    # DRL/DCM logs should have been rebuilt and so oldlog should have
	    # been cleared by cleanup_drl_dcm_logs()
	    export vol
	    p_list=`egettxt "cleanup_oldlogs: $vol: $logtype log should no longer have oldlog flag set" \
		vxvmshm:3425`
	    print_anal "$p_list"
	    ret=$EINTERNAL
	    ;;
	UNDEF)
	    # verify its a DCO log
	    [[ X`vxprint -g $dg -e 'dco_log_name="'$vol'"' -F %name` != X ]] || {
		export vol
		p_list=`egettxt "cleanup_oldlogs: ERROR! $vol: is not a DCO log" \
		    vxvmshm:3427`
		print_anal "$p_list"
		ret=$EINTERNAL
	    }
	    [[ X$analyze_only = Xy* ]] && {
		alist="$alist $vol"
		continue
	    }
	    [[ $ret -eq 0 ]] && {
		# To clear the oldlog flag on DCO v.0 volumes requires that after
		# the diskgroup upgrade, there is a transaction involving the
		# the log (this will flush the new map hdrs to disk) and then the
		# log length is set (this clears the flag). If the DCO log has never
		# been initialized then 2 transactions are required prior to setting
		# the log length, so we set the length 3 times to cover all cases.
		# Klunky, but we'll only ever have to do this once.
		doit -n "E_VOL_OLDLOG $vol" vxvol -g $dg -o force set len=$len $vol
		ret=$?
		[[ $ret -eq 0 ]] && {
		    doit -n "E_VOL_OLDLOG $vol" vxvol -g $dg set len=$len $vol
		    ret=$?
		}
		[[ $ret -eq 0 ]] && {
		    doit -n "E_VOL_OLDLOG $vol" vxvol -g $dg set len=$len $vol
		    ret=$?
		}
		[[ $ret -eq 0 ]] || ret=$EUTIL
	    }
	    ;;
	*)
	    export vol logtype
	    p_list=`egettxt "cleanup_oldlogs: $vol: oldlog flag is set on unrecognized log type $logtype" \
		vxvmshm:3426`
	    print_anal "$p_list"
	    ret=$EINTERNAL
	    ;;
	esac
    done < $tmp11
    [[ $ret -eq 0 ]] || reterr=$ret
    [[ X$analyze_only = Xy* && X$alist != X ]] && {
	mk_print_list "$alist" a_list
	export a_list; p_list=`egettxt "\
	
	The following list contains volumes that need the oldlog flag
	cleared before they can be present in an 8K aligned diskgroup.
	rebuilt to conform to 8K alignment restrictions.

	    $a_list
" vxvmshm:3417`
	print_anal "$p_list"
    }
    return $reterr
}

#
# get a list of startable volumes and start them in preparation for
# for diskgroup conversion
#
function dgconv_startvols
{
    $set_x
    typeset vol

    vxinfo -g $dg | grep " Startable$" | awk '{ print $1 }' > $startstop_vols
    [[ -s $startstop_vols ]] && {
	print_verbose "Starting additional volumes ..."
	(
	for vol in `cat $startstop_vols`
	do
	    # starting a volume w/ a DCO log will automatically
	    # start the log first (if needed) so don't start DCO logs
	    [[ X`vxprint -g $dg -vF %isdcolog $vol` = Xon ]] && continue

	    doit -n "E_VOL_START $vol" vxvol -g $dg start $vol
	    [[ $? -eq 0 ]] && echo $vol
	done
	) > $tmp9
	cp $tmp9 $startstop_vols
    }
}

#
# stop any volumes that were started by dgconv_startvols
#
function dgconv_stopvols
{
    $set_x
    typeset vol logs dco

    [[ -s $startstop_vols ]] && {
	print_verbose "Stopping started volumes ..."
	for vol in `cat $startstop_vols`
	do
	    doit -n "E_VOL_STOP $vol" vxvol -g $dg stop $vol
	    # stop DCO logs after the data vol
	    [[ X`vxprint -g $dg -vF %hasdcolog $vol` = Xon ]] && {
		dco=`vxprint -g $dg -vF %dconame $vol`
		logs=`vxprint -g $dg -ve 'v_dco_parent_name="'$dco'"' -F %name`
		doit -n "E_VOL_STOP `echo $logs | sed -e 's/ /,/g'`" \
		    vxvol -g $dg stop $logs
	    }
	done
	> $startstop_vols
    }
}

#
# dgconv_cds - convert the diskgroup to a CDS diskgroup. This function
#	should be run only after dgconv_alignment has succeeded
#
function dgconv_cds
{
    $set_x
    typeset max cur cdsmax cmd cds

    # set CDS attribute
    cds=`vxprint -g $dg -F %cds $dg`
    case $cds in
    on)
	;;
    off)
	if [[ X$analyze_only = Xy* ]]
	then
	    p_list=`egettxt "\

	Diskgroup requires CDS attribute be set" vxvmshm:3418`
	    print_anal "$p_list"
	else
	    print_verbose "Setting CDS attribute ..."
	    cmd="vxdg -g $dg set cds=on"
	    doit -n "E_DG_CDS" $cmd
	    [[ $? -eq 0 ]] || return $EUTIL
	fi
	;;
    *)
	internal_err_exit "dgconv_cds: ERROR! unexpected CDS value $cds"
	;;
    esac

    # set device quota if no already set
    cur=0 max=0
    eval `vxprint -lG $dg | grep '^devices:' | awk '{ $1=""; print; }'`
    cdsmax=$((32 * 1024))
    if [[ $cur -gt $cdsmax ]]
    then
	cdsmax=$cur
    fi
    if [[ $max -eq 0 ]]
    then
	if [[ X$analyze_only = Xy* ]]
	then
	    export cdsmax; p_list=`egettxt "\

	Diskgroup requires device quota be set to $cdsmax" vxvmshm:3420`
	    print_anal "$p_list"
	else
	    print_verbose "Setting device quota ..."
	    cmd="vxdg -g $dg set maxdev=$cdsmax"
	    doit -n "E_DG_MAXDEV $cdsmax" $cmd
	    # failure to set device quota is not fatal
	fi
    fi
    
    return 0
}

#
# dgconv_alignment - convert the diskgroup to an 8K alignment
#
function dgconv_alignment
{
    $set_x
    typeset ret=0 cmd align

    # cleanup any needed recovery
    # from previous runs of the converter
    # we do this here so the logs are recovered
    # for the cleanup routines run prior to subdisk relayout
    recover

    # Set length of cache objects so the underlaying
    # cache volume will be properly sized and both remain in sync
    cleanup_cachevols
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # Rebuild RAID log plexes in such a way that
    # the new logs are alignment compliant
    cleanup_r5logs
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # Do any stripe width relayouts
    cleanup_st_width all
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # Rebuild DRL/DCM logs so that
    # the subdisk length, loglength, and logmap alignment
    # are compliant
    # Subdisks may still need to be realigned
    # during relayout
    cleanup_drl_dcm_logs
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # set volume lengths after relayout
    cleanup_lengths pre-relayout
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # do initial sd relayout so all subdisks are properly
    # aligned and sized
    dgconv_sdrelayout
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # set volume lengths after relayout
    cleanup_lengths post-relayout
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # clear oldlog flags
    cleanup_oldlogs
    ret=$?
    [[ $ret -eq 0 ]] || return $ret

    # set diskgroup alignment
    align=`vxprint -g $dg -F %align $dg`
    case $align in
    1)
	if [[ X$analyze_only = Xy* ]]
	then
	    p_list=`egettxt "\

	Diskgroup requires alignment be set to 8K" vxvmshm:3419`
	    print_anal "$p_list"
	else
	    print_verbose "Setting diskgroup alignment ..."
	    cmd="vxdg -g $dg set align=$_8K"
	    doit -n "E_DG_ALIGN" $cmd
	    #vxdg -g $dg set align=$_8K
	    [[ $? -eq 0 ]] || ret=$EUTIL
	fi
	;;
    $_8K)
	;;
    *)
	internal_err_exit "dgconv_alignment: ERROR! unexpected diskgroup alignment $align"
	;;
    esac
    return $ret
}
	    
#
# dgconv - convert the diskgroup to 8K alignment or to a CDS diskgroup
#	which also implies 8K alignment
#
# Usage: dgconv alignment|group
#
function dgconv
{
    $set_x
    typeset ret=0
	
    [[ $# -eq 1 ]] || internal_err_exit "dgconv: arg count ($*)"

    [[ X$VXCDS_DGCONV_DEBUG != X ]] && {
	dgconv_debug
	return $?
    }
    case $1 in
    alignment)
	# init the diskgroup converter	    
	dgconv_init

	# start any startable volumes
	dgconv_startvols

	dgconv_alignment
	ret=$?

	# stop any started volumes
	dgconv_stopvols
	;;
    group)
	# init the diskgroup converter	    
	dgconv_init

	# start any startable volumes
	dgconv_startvols

	dgconv_alignment
	ret=$?
	[[ $ret -eq 0 ]] && {
	    dgconv_cds
	    ret=$?
	}

	# stop any started volumes
	dgconv_stopvols
	;;
    *)
	internal_err_exit "dgconv: bad args ($*)"
	;;
    esac
    if [[ $ret -eq 0 ]]
    then
	# if conversion was successful, cleanup 
	dgconv_cleanup
    else
	# if conversion was not successful, call recover
	recover
    fi	
    # return conversion status
    return $ret
}

#
# Setup for BASH environment
#
set -A bash_env a b c d 2>/dev/null || {
    function list_set { list_set_bash "$@"; }
    function list_append { list_append_bash "$@"; }
    function list_member { list_member_bash "$@"; }
} 

#
# DEBUG/TEST support functions
#
function dgconv_debug
{
    $set_x
    typeset test ret

    set -- $VXCDS_DGCONV_DEBUG
    test="$1"
    shift

    echo "Executing internal test $test ..."
    case "$test" in
    "")			return 0 ;;
    test_space1)	test_space1 $*; return $? ;;
    test_space2)	test_space2 $*; return $? ;;
    test_dmconv_if)	test_dmconv_if $*; return $? ;;
    test_dmconv_if2)	test_dmconv_if2 $*; return $? ;;
    test_recov1)
	# leaves sd unassociated
	dgconv_init && dgconv_config
	ret=$?
	[[ $ret -eq 0 ]] && {
	    newsdname $2 newsd - -
	    sd_creat $newsd $1 0 1024 $_8K
	    ret=$?
	}
	return $ret
	;;
    test_recov2)
	# only sets comment fields in drl/dcm logs
	dgconv_init && dgconv_config
	ret=$?
	[[ $ret -eq 0 ]] && {
	    cleanup_drl_dcm_logs -D1
	    ret=$?
	}
	return $ret
	;;
    test_recov3)
	# test_recov2 + dissociates logs
	dgconv_init && dgconv_config
	ret=$?
	[[ $ret -eq 0 ]] && {
	    cleanup_drl_dcm_logs -D2
	    ret=$?
	}
	return $ret
	;;
    test_recov4)
	# test_recov3 + creates new logs
	dgconv_init && dgconv_config
	ret=$?
	[[ $ret -eq 0 ]] && {
	    cleanup_drl_dcm_logs -D3
	    ret=$?
	}
	return $ret
	;;
    *)			echo "dgconv_debug: unknown debug test $test"
			return $EINVAL ;;
    esac
}

function test_space1
{
    $set_x
    typeset dm da publba publen newoff newlen
    typeset slice offset=7 len disk_offset reterr=0

    vxprint -g $dg -Qqd | awk '{print $2,$3}' >$tmp11
    while ((1))
    do
	read dm da
	[[ $? -eq 0 ]] || break
	[[ X$da = X- ]] && continue
	slice= offset=0 len= disk_offset=0
	eval `vxdisk list $da | grep ^public: | sed -e 's/^public://'`
	((offset += disk_offset))
	[[ $offset -ge 128 ]] && break
    done < $tmp11
    [[ $offset -lt 128 ]] && {
	echo test_space1: no usable DM available
	return 1
    }
    dm_config $dm
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5, $6, $7 }' >$tmp11
    read publba publen newoff newlen < $tmp11
    [[ $publba = $newoff && $publen = $newlen ]] || {
	echo test_space1: INTERNAL ERROR 1a
	return 1
    }
    # modify pub region restrictions in various ways 
    # and validate results on the space management data

    # test1: superset
    n=1
    newoff=$((publba - 11))
    newlen=$((publen + 13))
    dm_config $dm $newoff $newlen
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5, $6, $7 }' >$tmp11
    read publba publen newoff newlen < $tmp11
    if [[ $newoff = $publba && $newlen = $publen ]]
    then
	: 
    else
	echo test_space1: Test${n}: FAILED
	reterr=1
    fi

    # test2: subset
    n=2
    newoff=$((publba + 11))
    newlen=$((publen - 13))
    dm_config $dm $newoff $newlen
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5, $6, $7 }' >$tmp11
    read publba publen newoff newlen < $tmp11
    if [[ $newoff = $((publba + 11)) && $newlen = $((publen - 13)) ]]
    then
	:
    else
	echo test_space1: Test${n}: FAILED
	reterr=1
    fi

   # test3: head overlap
    n=3
    newoff=$((publba - 11))
    newlen=$((publen))
    dm_config $dm $newoff $newlen
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5, $6, $7 }' >$tmp11
    read publba publen newoff newlen < $tmp11
    if [[ $newoff = $((publba)) && $newlen = $((publen - 11)) ]]
    then
	:
    else
	echo test_space1: Test${n}: FAILED
	reterr=1
    fi

    # test4: tail overlap
    n=4
    newoff=$((publba + 11))
    newlen=$((publen))
    dm_config $dm $newoff $newlen
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5, $6, $7 }' >$tmp11
    read publba publen newoff newlen < $tmp11
    if [[ $newoff = $((publba + 11)) && $newlen = $((publen - 11)) ]]
    then
	:
    else
	echo test_space1: Test${n}: FAILED
	reterr=1
    fi

    # test5: head contig
    n=5
    newoff=0
    newlen=$publba
    dm_config $dm $newoff $newlen
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5, $6, $7 }' >$tmp11
    read publba publen newoff newlen < $tmp11
    if [[ $newoff = 0 && $newlen = 0 ]]
    then
	:
    else
	echo test_space1: Test${n}: FAILED
	reterr=1
    fi

    # test6: tail contig
    n=6
    newoff=$((publba + publen))
    newlen=128
    dm_config $dm $newoff $newlen
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5, $6, $7 }' >$tmp11
    read publba publen newoff newlen < $tmp11
    if [[ $newoff = 0 && $newlen = 0 ]]
    then
	:
    else
	echo test_space1: Test${n}: FAILED
	reterr=1
    fi

    # test7: head dissjoint
    n=7
    newoff=0
    newlen=$(( (publba - 10) / 2 ))
    dm_config $dm $newoff $newlen
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5, $6, $7 }' >$tmp11
    read publba publen newoff newlen < $tmp11
    if [[ $newoff = 0 && $newlen = 0 ]]
    then
	:
    else
	echo test_space1: Test${n}: FAILED
	reterr=1
    fi

    # test8: tail disjoin
    n=8
    newoff=$((publba + publen + 10))
    newlen=128
    dm_config $dm $newoff $newlen
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5, $6, $7 }' >$tmp11
    read publba publen newoff newlen < $tmp11
    if [[ $newoff = 0 && $newlen = 0 ]]
    then
	:
    else
	echo test_space1: Test${n}: FAILED
	reterr=1
    fi

    return $reterr
}

function test_space2
{
    $set_x
    typeset dm da publba publen newoff newlen
    typeset slice offset len disk_offset reterr=0

    vxprint -g $dg -Qqd | awk '{print $2,$3}' >$tmp11
    while ((1))
    do
	read dm da
	[[ $? -eq 0 ]] || break
	[[ X$da = X- ]] && continue
	slice= offset=0 len= disk_offset=0
	eval `vxdisk list $da | grep ^public: | sed -e 's/^public://'`
	((offset += disk_offset))
	[[ $offset -ge 128 ]] && break
    done < $tmp11
    [[ $offset -lt 128 ]] && {
	echo test_space2: no usable DM available
	return 1
    }
    dm_config $dm
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5, $6, $7 }' >$tmp11
    read publba publen newoff newlen < $tmp11
    [[ $publba = $newoff && $publen = $newlen ]] || {
	echo test_space2: INTERNAL ERROR 1a
	return 1
    }

    # given defined disk constraints, add free space in various ways
    # and validate results on the space management data

    # test1: superset
    n=1
    newoff=$((publba - 11))
    newlen=$((publen + 13))
    free_free $dm
    free_add -lba $dm $newoff $newlen
    free_dump -lba $dm >$tmp11
    read junk newoff newlen < $tmp11
    if [[ $newoff = $publba && $newlen = $publen ]]
    then
	:
    else
	echo Test${n}: FAILED
	reterr=1
    fi

    # test2: subset
    n=2
    newoff=$((publba + 11))
    newlen=$((publen - 13))
    free_free $dm
    free_add -lba $dm $newoff $newlen
    free_dump -lba $dm >$tmp11
    read junk newoff newlen < $tmp11
    if [[ $newoff = $((publba + 11)) && $newlen = $((publen - 13)) ]]
    then
	:
    else
	echo Test${n}: FAILED
	reterr=1
    fi

   # test3: head overlap
    n=3
    newoff=$((publba - 11))
    newlen=$((publen))
    free_free $dm
    free_add -lba $dm $newoff $newlen
    free_dump -lba $dm >$tmp11
    read junk newoff newlen < $tmp11
    if [[ $newoff = $((publba)) && $newlen = $((publen - 11)) ]]
    then
	:
    else
	echo Test${n}: FAILED
	reterr=1
    fi

    # test4: tail overlap
    n=4
    newoff=$((publba + 11))
    newlen=$((publen))
    free_free $dm
    free_add -lba $dm $newoff $newlen
    free_dump -lba $dm >$tmp11
    read junk newoff newlen < $tmp11
    if [[ $newoff = $((publba + 11)) && $newlen = $((publen - 11)) ]]
    then
	:
    else
	echo Test${n}: FAILED
	reterr=1
    fi

    # test5: head contig
    n=5
    newoff=0
    newlen=$publba
    free_free $dm
    free_add -lba $dm $newoff $newlen
    free_dump -lba $dm >$tmp11
    read junk newoff newlen < $tmp11
    if [[ X$newoff = X && X$newlen = X ]]
    then
	:
    else
	echo Test${n}: FAILED
	reterr=1
    fi

    # test6: tail contig
    n=6
    newoff=$((publba + publen))
    newlen=128
    free_free $dm
    free_add -lba $dm $newoff $newlen
    free_dump -lba $dm >$tmp11
    read junk newoff newlen < $tmp11
    if [[ X$newoff = X && X$newlen = X ]]
    then
	:
    else
	echo Test${n}: FAILED
	reterr=1
    fi

    # test7: head dissjoint
    n=7
    newoff=0
    newlen=$(( (publba - 10) / 2 ))
    free_free $dm
    free_add -lba $dm $newoff $newlen
    free_dump -lba $dm >$tmp11
    read junk newoff newlen < $tmp11
    if [[ X$newoff = X && X$newlen = X ]]
    then
	:
    else
	echo Test${n}: FAILED
	reterr=1
    fi

    # test8: tail disjoin
    n=8
    newoff=$((publba + publen + 10))
    newlen=128
    free_free $dm
    free_add -lba $dm $newoff $newlen
    free_dump -lba $dm >$tmp11
    read junk newoff newlen < $tmp11
    if [[ X$newoff = X && X$newlen = X ]]
    then
	:
    else
	echo Test${n}: FAILED
	reterr=1
    fi

    return $reterr
}

function test_dmconv_if
{
    $set_x

    function cleanup
    {
	# remove the temp object created
	vxedit -g $dg -fr rm $vol
	# don't leave a corrupted DM bounds file
	dm_config $dm
    }

    typeset sd dm lba len sdlen junk ret newlen1 publba publen freelba
    typeset align newsd vol totlen newlba newlen newlen2
    typeset me=test_dmconv_if newdm sdpl sdcol sdcoloff 

    [[ $# -eq 1 ]] || {
	echo "${me}: arg count ($*)"
	return 1
    }
    dm=$1
    tail -1 $dir_dms/${dm}.bounds | awk '{ print $4, $5 }' >$tmp11
    read publba publen < $tmp11
    align=1
    vol=sdr1



    # create a volume, pick a subdisk to move
    # use the space management to arramge free space and
    # restricted areas and then call sdr_relocate_sds to
    # move the subdisk.

    # case 1: head move w/ split
    split_subdisks_ok=move
    move_subdisks_ok=yes
    evac_subdisks_ok=no
    evac_disk_list=""
    n=1
    sdlen=1111
    newlen1=17
    if ((min_split_size > newlen1))
    then
	newlen2=$min_split_size
    else
	newlen2=$newlen1
    fi
    totlen=$((sdlen + newlen2))
    free_find -lba $dm $totlen 1 freelba -
    # create subdisk at start of freelba (w/ newlen2 space after)
    vxassist -g $dg make $vol $sdlen dm:$dm,$((freelba - publba)),$sdlen layout=nodiskalign
    sd=`vxprint -g $dg -pF %aslist ${vol}-01`
    echo $sd | sdlba >$tmp11
    read junk junk junk lba len < $tmp11
    if ((lba != freelba || len != sdlen))
    then
	echo ${me}: ${vol}: getting lba attrs of volume failed
	cleanup
	return 1
    fi
    # relocate sd w/ new disk bounds
    sdr_relocate_sds $dm $((lba + newlen1)) $((sdlen + newlen2 - newlen1)) $sd
    ret=$?
    [[ $ret -eq 0 ]] || {
	echo $me: relocation failed
	cleanup
	return 1
    }
    # a new volume subdisk should exist of len newlen2
    # which is contiguous and after sd
    newsd=`vxprint -g $dg -se 'assoc.assoc.v_name="'$vol'"' -F %name | grep -v "^$sd\$"`
    if [[ -z "$newsd" ]]
    then
	echo ${me}: $sd: relocation failed to associate subdisk with volume
	cleanup
	return 1
    fi
    echo $newsd | sdlba >$tmp11
    read junk junk junk lba len < $tmp11
    if ((lba != freelba + sdlen || len != newlen2))
    then
	echo ${me}: $sd: relocation failed to relocate subdisk properly
	cleanup
	return 1
    fi
    cleanup

    # case 2: head move w/o split
    split_subdisks_ok=no
    move_subdisks_ok=yes
    evac_subdisks_ok=no
    evac_disk_list=""
    n=2
    sdlen=1111
    newlen1=17
    totlen=$((2 * sdlen))
    free_find -lba $dm $totlen 1 freelba -
    # create subdisk at start of freelba (w/ sdlen space after)
    vxassist -g $dg make $vol $sdlen dm:$dm,$((freelba - publba)),$sdlen layout=nodiskalign
    sd=`vxprint -g $dg -pF %aslist ${vol}-01`
    echo $sd | sdlba >$tmp11
    read junk junk junk lba len < $tmp11
    if ((lba != freelba || len != sdlen))
    then
	echo ${me}: ${vol}: getting lba attrs of volume failed
	cleanup
	return 1
    fi
    # relocate sd w/ new disk bounds
    sdr_relocate_sds $dm $((lba + newlen1)) $((2 * sdlen - newlen1)) $sd
    ret=$?
    [[ $ret -eq 0 ]] || {
	echo $me: relocation failed
	cleanup
	return 1
    }
    # sd should now be the same length but lba should start
    # where old sd location terminated
    echo $sd | sdlba >$tmp11
    read junk junk junk newlba newlen < $tmp11
    if (( newlen != len || newlba != lba + len ))
    then
	echo ${me}: $sd: relocation failed to relocate subdisk properly
	cleanup
	return 1
    fi
    cleanup

    # case 3: evac of sd
    # same setup conditions as test 2 but 
    # with move_subdisks_ok=no
    split_subdisks_ok=no
    move_subdisks_ok=no
    evac_subdisks_ok=yes
    evac_disk_list=""
    n=3
    sdlen=1111
    newlen1=17
    totlen=$((2 * sdlen))
    free_find -lba $dm $totlen 1 freelba -
    # create subdisk at start of freelba (w/ sdlen space after)
    vxassist -g $dg make $vol $sdlen dm:$dm,$((freelba - publba)),$sdlen layout=nodiskalign
    sd=`vxprint -g $dg -pF %aslist ${vol}-01`
    echo $sd | sdlba >$tmp11
    read junk junk junk lba len < $tmp11
    if ((lba != freelba || len != sdlen))
    then
	echo ${me}: ${vol}: getting lba attrs of volume failed
	cleanup
	return 1
    fi
    # save pl, sdcol, sdcoloff for after relocation
    vxprint -g $dg -sF '%pl_name %column_pl_offset' $sd |
	    sed -e 's/\// /' >$tmp11
    read sdpl sdcol sdcoloff < $tmp11
    [[ -z "$sdcoloff" ]] && {
	sdcoloff=$sdcol
	sdcol=0
	[[ X$sdcoloff = XLOG ]] && sdcoloff=0
    }
    # relocate sd w/ new disk bounds
    sdr_relocate_sds $dm $((lba + newlen1)) $((2 * sdlen - newlen1)) $sd
    ret=$?
    [[ $ret -eq 0 ]] || {
	echo $me: relocation failed
	cleanup
	return 1
    }
    # sd should now be the same length but have a different name, lba and dm
    newsd=`vxprint -g $dg -se 'assoc.name="'$sdpl'" && \
		    sd_column=='$sdcol' && sd_pl_offset=='$sdcoloff'' \
		    -F %name`
    echo $newsd | sdlba >$tmp11
    read junk newdm junk newlba newlen < $tmp11
    if [[ ! $newlen -eq $len || X$newdm = X$dm ]]
    then
	echo ${me}: $sd: relocation failed to relocate subdisk properly
	cleanup
	return 1
    fi
    cleanup

    return 0
}

function test_dmconv_if2
{
    $set_x

    function cleanup
    {
	# remove the temp object created
	vxedit -g $dg -fr rm $vol
	# don't leave a corrupted DM bounds file
	dm_config $dm1
    }

    typeset sd dm1 dm2 lba len sdlen junk ret newlen1 publba publen freelba
    typeset align newsd vol totlen newlba newlen newlen2
    typeset me=test_dmconv_if2 newdm sdpl sdcol sdcoloff 
    typeset n sdlba mss blba cdslba cdslen relocsz space

    [[ $# -eq 2 ]] || {
	echo "${me}: arg count ($*)"
	return 1
    }
    dm1=$1
    dm2=$2
    tail -1 $dir_dms/${dm1}.bounds | awk '{ print $4, $5 }' >$tmp11
    read publba publen < $tmp11
    align=1
    vol=sdr1

    # case 1: head move w/ split enabled
    # we fix the sd's lba and len then let
    # min_split_size and cdslba vary to test boundary
    # conditions
    #
    split_subdisks_ok=yes
    move_subdisks_ok=yes
    evac_subdisks_ok=yes
    evac_disk_list=""
    for space in 0 1
    do
	for mss in 0 1
	do
	    for blba in 0 1 2
	    do
		# create subdisk of len 100 and get sdlba
		vxassist -g $dg make $vol 100 $dm1 layout=nodiskalign
    		sd=`vxprint -g $dg -pF %aslist ${vol}-01`
    		echo $sd | sdlba >$tmp11
    		read junk junk junk sdlba sdlen < $tmp11

		case $mss in
		0)
		    # min_split_size < sdlen
		    min_split_size=$((sdlen / 2))
		    ;;
		1)
		    # min_split_size > sdlen
		    min_split_size=$((sdlen + sdlen / 2))
		    ;;
		esac

		case $blba in
		0)
		    # cdslba > sdlba + max(sdlen,min_split_size)
		    cdslba=$((sdlba + min_split_size + sdlen + 10))
		    relocsz=$sdlen
		    ;;
		1)
		    # cdslba < sdlba + max(sdlen,min_split_size)
		    # and cdslba > sdlba + min(sdlen,min_split_size)
		    cdslba=$((sdlba + (sdlen + min_split_size) / 2))
		    if (( sdlen > min_split_size ))
		    then
			relocsz=$((cdslba - sdlba))
		    else
			relocsz=$sdlen
		    fi
		    ;;
		2)
		    # cdslba < sdlba + min(sdlen,min_split_size)
		    # and cdslba > sdlba
		    if (( sdlen > min_split_size ))
		    then
			cdslba=$((sdlba + min_split_size / 2))
			relocsz=$min_split_size
		    else
			cdslba=$((sdlba + sdlen / 2))
			relocsz=$sdlen
		    fi
		esac

		# calculate length of destination pubregion
		newlen2=$((sdlba + sdlen - cdslba))
		[[ $newlen2 -gt 0 ]] || newlen2=0
		cdslen=$((newlen2 + relocsz))
		if [[ ! $space -gt 0 ]]
		then
			((cdslen -= 1))
		fi
		: space=$space mss=$mss blba=$blba
		: sdlba=$sdlba sdlen=$sdlen min_split_size=$min_split_size \[$sdlba,$((sdlba+sdlen))\]
		: cdslba=$cdslba cdslen=$cdslen relocsz=$relocsz
		# relocate sd w/ new disk bounds
		sdr_relocate_sds $dm1 $cdslba $cdslen $sd
		ret=$?
		[[ $ret -eq 0 ]] || {
			echo $me: relocation failed
			cleanup
			return 1
		}

		# analyze results
		newsd=`vxprint -g $dg \
			-se 'assoc.assoc.v_name="'$vol'"' -F %name`
		n=`echo $newsd | wc -w | awk '{print $1}'`
		if [[ $space -eq 0 ]]
		then
			: sd should exist on !dm:$dm1 w/ same length
			if [[ X$n != X1 ]]
			then
				echo ${me}: $sd: relocation failed to associate subdisk with volume
				cleanup
				return 1
			fi
			echo $newsd | sdlba >$tmp11
			read junk newdm junk newlba newlen < $tmp11
			[[ $newdm != $dm1 && $newlen = $sdlen ]] || {
				echo ${me}: $sd: relocation failed to relocate subdisk properly
				echo "	!($newdm != $dm1 && $newlen = $sdlen)"
				cleanup
				return 1
			}
		elif [[ $relocsz = $sdlen ]]
		then
			: sd should have migrated within $dm1 w/ same length
			if [[ X$n != X1 ]]
			then
				echo ${me}: $sd: relocation failed to associate subdisk with volume
				cleanup
				return 1
			fi
			echo $newsd | sdlba >$tmp11
			read junk newdm junk newlba newlen < $tmp11
			[[ $newdm = $dm1 && $newlen = $sdlen &&
			      $newlba != $sdlba ]] || {
				echo ${me}: $sd: relocation failed to relocate subdisk properly
				echo "	!($newdm = $dm1 && $newlen = $sdlen &&$newlba != $sdlba)"
				cleanup
				return 1
			}
		else
			: sd should have been split and migrated within $dm1
			if [[ X$n != X2 ]]
			then
				echo ${me}: $sd: relocation failed to associate subdisk with volume
				cleanup
				return 1
			fi
			echo $newsd | head -1 | sdlba >$tmp11
			# find the sd w/ pl_offset==0
			while ((1))
			do
			    read newsd newdm junk newlba newlen
			    [[ $? -eq 0 ]] || break
			    sdcoloff=`vxprint -g $dg -sF '%pl_offset' $newsd`
			    [[ X$sdcoloff = X0 ]] && break
			done < $tmp11
			[[ X$newsd != X && $newdm = $dm1 && $newlen = $relocsz &&
			   $newlba != $sdlba ]] || {
				echo ${me}: $sd: relocation failed to relocate subdisk properly
				echo "	!(X$newsd != X && $newdm = $dm1 && $newlen = $relocsz && $newlba != $sdlba)"
				cleanup
				return 1	
			}
		fi
		cleanup
	    done
	done
    done

}

. $VOLADM_LIB/vxcds_dgconv_sys.ksh
