# $Id: vxcds_diskconv.sh,v 1.1.2.12.24.5.36.1 2006/12/07 01:46:18 mchester Exp $
# #ident $Source: /project/unixvm-cvs/src/common/cmd/vxvm/voladm/vxcds_diskconv.sh,v $
# $Copyrights: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

#-----------------
# Global Variables 
#-----------------
# See converter.sh for additional globals.
#
# Variables associated with disk group (dg variable in converter.sh)
#
dg_log_sz=	# current KLOG size for the DG
dg_log_min=	# currnet KLOG minimum size
dg_cfg_sz=	# current config DB size for the DG
dg_cfg_use=	# current config DB size used by the DG
#
#
# Variables associated with the disk currently being processed
#
dm=		# DM name for the disk
da=		# DA name for the disk
da_type=	# DA type for the disk
rpath=		# path to a raw device file for the disk
useable_disksz=	# useable size of the disk as a CDS disk	
free_at_start=	# end of free space at start of the disk media
free_at_end=	# start of free space at the end of the disk media
free_total=	# total amount of free space on the disk media
curr_prv_off=	# current private region offset for the disk (from LBA 0)
curr_prv_len=	# current private region length for the disk
curr_pub_off=	# current public region offset for the disk (from LBA 0)
curr_pub_len=	# current public region length for the disk
init_prv_off=	# private region offset for re-initializing the disk
init_prv_len=	# private region length for re-initializing the disk
init_pub_off=	# public region offset for re-initializing the disk
init_pub_len=	# public region length for re-initializing the disk
init_attrs=	# attributes for re-initializing the disk

full_reloc_sd=	# list of subdisks that must be entirely relocated 
part_reloc_sd=	# list of subdisks that must be partially relocated
size_reloc_sd=	# size of subdisk space that needs to be relocated

perdm_varlist="da da_type rpath useable_disksz \
	free_at_start free_at_end free_total \
	curr_prv_off curr_prv_len curr_pub_off curr_pub_len \
	init_prv_off init_prv_len init_pub_off init_pub_len \
	init_attrs full_reloc_sd part_reloc_sd size_reloc_sd"

dmlist=		# disks being processed
todo_dmlist=	# has components <dm> <reloc-size> <free-reloc> <free-init>
#
#---------------------------------------
# Functions for use with disk conversion
#
# NOTE: main entry point is diskconv()
#---------------------------------------
#

#
# set_dg_cfglog - determine the config and log usage of the DG
#
# usage: set_dg_cfglog
#
# globals used: dg, tmpfile1
# globals set: dg_log_sz, dg_log_min, dg_cfg_sz, dg_cfg_use
#
set_dg_cfglog() {
	$set_x
	doit "E_DG_INFO" vxdg list $dg > $tmpfile1
	eval `grep "^config:" $tmpfile1 | sed -e 's/.*\(perm.*\)/\1/'`
	#
	# eval should have just set permlen, free, loglen
	#
	[ -z "$permlen" -o -z "$free" -o -z "$loglen" ] && {
		internal_err_exit "init_privlen: $permlen : $free : $loglen"
	}
	dg_log_sz=$loglen
	dg_cfg_sz=$permlen
	dg_cfg_use=`expr $permlen - $free`
	if [ $dg_cfg_use -lt $_8K ]
	then
		dg_cfg_use=$_8K		# 8K min
	else
		_rem=`expr $dg_cfg_use % $_8K`
		[ "0" != "$_rem" ] && {
			dg_cfg_use=`expr $dg_cfg_use + $_8K - $_rem` # round-up
		}
	fi
	#
	# Minimum log is 1 + num-vols + num-plex + num-RAID5-sds entries
	# with 20 entries per 512 bytes. Convert to BSIZE with min of 8K.
	#
	doit "E_DG_INFO" vxprint -g $dg -qvpG > $tmpfile1
	_num_vp=`wc -l $tmpfile1 | awk '{print $1}'`
	_num_vp=`expr $_num_vp - 1`	# discount 'dg' line
	doit -n "" vxprint -g $dg -qse 'assoc.pl_layout=RAID' > $tmpfile1
	_num_rs=`wc -l $tmpfile1 | awk '{print $1}'`
	_num_blks=`expr \( $_num_vp + $_num_rs + 1 + 19 \) / 20`
	dg_log_min=`expr \( $_num_blks \* 512 + $_BSIZE - 1 \) / $_BSIZE`
	if [ $dg_log_min -lt $_8K ]
	then
		dg_log_min=$_8K		# min 8K
	else
		_rem=`expr $dg_log_min % $_8K`
		[ "0" != "$_rem" ] && {
			dg_log_min=`expr $dg_log_min + $_8K - $_rem` # round-up
		}
	fi
}
#
# set_useable_disksz - determine the useable area for a (CDS) disk
#
# usage: set_useable_disksz
#
# globals used: da, rpath, tmpfile1, platform
# globals set: useable_disksz
#
set_useable_disksz() {
	$set_x
	[ -z "$da" -o -z "$rpath" ] && {
		internal_err_exit "set_useable_disksz: bad globals ($da:$rpath)"
	}
	#
	# The useable area for a CDS disk is from address 128K until the second
	# to last cylinder.
	#
	# note:
	#	1. we only support disks with 512-byte sector size currently
	#	2. we really only support SCSI disks
	#
	case $platform in
	SunOS)	#
		# Since Solaris already doesn't use the last two sectors,
		# STAT_DISKNUMSEC from vxparms gives us what we want.
		#
		doit "E_DISK_GEOM $da" vxparms -s $rpath > $tmpfile1
		_disksz=`grep "^STAT_DISKNUMSEC" $tmpfile1 | cut -f 2 -d '='`
		[ -z "$_disksz" ] && print_err_exit "E_DISK_GEOM $da"

		sectsz=`grep "^STAT_SECSIZ" $tmpfile1 | cut -f 2 -d '='`
		[ "$sectsz" != "512" ] && print_err_exit "E_SECSZ $da $sectsz"
		;;
	*)	#
		# For non-Solaris platforms, use vxscsi to get the disk geom.
		#
		doit "E_DISK_GEOM $da" vxscsi -g $rpath > $tmpfile1
		nhead=; nsect=; ncyl=; sectsz=; cap=
		eval "`sed -e 's/.*://' $tmpfile1`"
		#
		# The above eval should have set:
		#	nhead, nsect, ncyl, sectsz, cap
		#
		[ -z "$nhead" -o -z "$nsect" -o -z "$ncyl" -o -z "$cap" ] && {
			print_err_exit "E_DISK_GEOM $da"
		}
		[ "$sectsz" != "512" ] && print_err_exit "E_SECTSZ $da $sectsz"
		#
		# Solaris disk size is determined by reserving at least 2
		# full cylinders at the end of the disk.
		#
		_cylsize=`expr $nhead \* $nsect`
		_capcyls=`expr $cap / $_cylsize`
		[ $ncyl -gt $_capcyls ] && ncyl=$_capcyls
		_disksz=`expr $_cylsize \* \( $ncyl - 2 \)`
		;;
	esac
	#
	# _disksz must be a multiple of 8K and not include the first 128K.
	# If _disksz is not currently a multiple of 8K, round down.
	#
	_disksz=`expr $_disksz - \( $_disksz % 16 \) - 256`
	#
	# Convert to BSIZE units (note divide unit conversion first to
	# prevent overflow).
	#
	useable_disksz=`expr $_disksz / \( $_BSIZE / 512 \)`

	return 0
}
#
# set_curr_privpub - determine the private/public region offset/length
#
# usage: set_curr_privpub
#
# globals used: dg, da, da_type, dm, rpath, platform, tmpfile1
# globals set: curr_prv_off, curr_prv_len, curr_pub_off, curr_pub_len
#
set_curr_privpub() {
	$set_x
	[ -z "$da" -o -z "$dm" -o -z "$rpath" ] &&  {
		internal_err_exit "set_curr_privpub: bad globals ($dm:$da:$rpath)"
	}
	#
	# Get the public and private offset and length info for the DM, and
	# then adjust it if need be to take into account any disk partitioning
	# such that the values are relative to LBA 0.
	#
	set -- `doit "E_DM_INFO $dm" vxprint -g $dg \
			-F "%privoffset %privlen %puboffset %publen" -d $dm`
	curr_prv_off=$1
	curr_prv_len=$2
	curr_pub_off=$3
	curr_pub_len=$4

	doit "E_DM_INFO $dm" vxdisk list $da > $tmpfile1 
	_pubst=`grep "^public:" $tmpfile1 | \
			sed -e 's/.*disk_offset=\([0-9][0-9]*\).*/\1/'`

	[ -z "$_pubst" ] && print_err_exit "E_DM_INFO $dm"

	curr_pub_off=`expr $curr_pub_off + $_pubst`

	[ "nopriv" = "$da_type" ] && return 0

	doit "E_DM_INFO $dm" vxdisk list $da > $tmpfile1
	_prvst=`grep "^private:" $tmpfile1 | \
			sed -e 's/.*disk_offset=\([0-9][0-9]*\).*/\1/'`
 
	[ -z "$_prvst" ] && print_err_exit "E_DM_INFO $dm"
 
	curr_prv_off=`expr $curr_prv_off + $_prvst`
	#
	# There should only be three possible cases:
	#	1. the private region comes before the public region
	#	2. the private region comes after the public region
	#	3. the private region is wholely contained within the public
	#		region; this should only occur for root encapsulated
	#		disks, which we can't convert to CDS disks anyways.
	#
	_privend=`expr $curr_prv_off + $curr_prv_len`
	_pubend=`expr $curr_pub_off + $curr_pub_len`

	if [ $curr_prv_off -lt $curr_pub_off ]
	then
		[ $_privend -gt $curr_pub_off ] && {
			print_err_exit "E_OVERLAP $da"
		}
	elif [ $curr_prv_off -lt $_pubend ]
	then
		print_err_exit "E_OVERLAP $da"
	fi
	return 0
}
#
# set_dm_free - determine the free space attributes for the disk media
#
# usage: set_dm_free
#
# globals used: dg, dm, tmpfile1
# globals set: free_at_start, free_at_end, free_total
#
set_dm_free() {
	$set_x
	[ -z "$dm" ] && internal_err_exit "set_dm_free: bad globals ($dm)"

	free_at_start=0
	free_at_end=0
	free_total=0
	#
	# Get a list of free space for the DM. Loop through the list to
	# sum up the free space, as well as look for any free space at the
	# start and end of the DM.
	#
	doit "E_DM_INFO $dm" vxprint -g $dg -F "%publen" -d $dm > $tmpfile1
	_pubsz=`cat $tmpfile1`
	doit "E_DM_INFO $dm" vxdg -g $dg -qa free $dm > $tmpfile1

	exec 3<&0 < $tmpfile1
	while read _disk _device _tag _offset _length _flags
	do
		if [ 0 -eq $_offset ]
		then
			free_at_start=$_length

		elif [ $_pubsz -eq `expr $_offset + $_length` ]
		then
			free_at_end=$_length
		fi
		free_total=`expr $free_total + $_length`
	done
	exec 0<&3 3<&-
	return 0
}
#
# calc_privlen - calculate the private region length for a CDS disk
# calc_rgn_lens - calculate config and log region info for a CDS disk
#
# usage: calc_privlen privlen-ret cfg-region-len log-region-len
# usage: calc_rgn_lens privlen cfg-len-ret log-len-ret
#
# globals used: none
#
# The size of a private region is the sum of:
#	A. 16 KB (for primary and alternate header)
#	B. 32 KB (for 4 TOCs)
#	C. 72 KB (for lock disk region)
#	D. config region, rounded to 8KB multiple if needed
#	E. log region, rounded to 8KB multiple if needed
#	F. privlen * 0.15 (for free space), rounded up
#		to 8KB multiple if needed; min free is 8KB
#
# when calculating config/log regions from privlen the idea is to keep the
# log size approx. 15% of config size
#
calc_privlen() {
	$set_x
	[ $# -ne 3 ] &&  internal_err_exit "calc_privlen: arg count ($*)"

	_plen_ret=$1
	_cfglen=$2
	_loglen=$3
	#
	# Since we don't want to shrink the cfg/log regions, round up if needed.
	#
	_rem=`expr $_cfglen % $_8K`
	[ "0" != "$_rem" ] && _cfglen=`expr $_cfglen + $_8K - $_rem

	_rem=`expr $_loglen % $_8K`
	[ "0" != "$_rem" ] && _loglen=`expr $_loglen + $_8K - $_rem

	_privlen=`expr \( 16 + 32 + 72 \) \* $_1K + $_cfglen + $_loglen`
	_free=`expr $_privlen \* 15 / 85`
	if [ 0 -eq $_free ]
	then
		_free=$_8K
	else
		_rem=`expr $_free % $_8K`
		[ "0" != "$_rem" ] && _free=`expr $_free + $_8K - $_rem`
	fi
	eval "$_plen_ret=`expr $_privlen + $_free`"
}
calc_rgn_lens() {
	$set_x
	[ $# -ne 3 ] &&  internal_err_exit "calc_rgn_lens: arg count ($*)"

	_privlen=$1
	_clen_ret=$2
	_llen_ret=$3

	_free=`expr $_privlen \* 15 / 100`
	if [ 0 -eq $_free ]
	then
		_free=$_8K
	else
		_rem=`expr $_free % $_8K`
		[ "0" != "$_rem" ] && _free=`expr $_free + $_8K - $_rem`
	fi
	_plen=`expr \( 16 + 32 + 72 \) \* $_1K + $_free`
	_plen=`expr $_privlen - $_plen`

	if [ $_plen -lt 0 ]
	then
		_cflglen=0
		_loglen=0
	else
		_cfglen=`expr $_plen \* 100 / 115`
		_loglen=`expr $_cfglen \* 15 / 100`
	fi
	#
	# Since the cfg/log regions need to fit, round down if needed.
	#
	_cfglen=`expr $_cfglen - \( $_cfglen % $_8K \)`
	_loglen=`expr $_loglen - \( $_loglen % $_8K \)`
	#
	# Now apply max restrictions. We don't apply min because min privlen
	# allowed by the script is 512K which can't hit the min of 8K for
	# each region. Also, given the relationship between the max config
	# and max log region, max config cannot be exceeded unless max log is.
	#
	if [ $_loglen -gt `expr 8 \* $_1M` ]
	then
		_loglen= `expr 8 \* $_1M` 
		_cfglen=`expr $_plen - $_loglen`
		_cfglen=`expr $_cfglen - \( $_cfglen % $_8K \)`

		if [ $_cfglen -gt `expr 64 \* $_1M` ]
		then
			_cfglen=`expr 64 \* $_1M`
		fi
	fi
	eval "$_llen_ret=$_loglen"
	eval "$_clen_ret=$_cfglen"
}
#
# determine_privoff - determine where to put the private region
#
# usage: determine_privoff privlen privoff-ret
#
# globals used: curr_pub_off, curr_pub_len, useable_disksz
# globals cleared: init_pub_off and init_pub_len
#
determine_privoff() {
	$set_x
	#
	# Check that privlen and useable_disksz are 8K aligned.
	#
	_rem=`expr $1 % $_8K`
	[ "0" != "$_rem" ] && {
		internal_err_exit "determine_privoff: bad privlen $1"
	}
	_rem=`expr $useable_disksz % $_8K`
	[ "0" != "$_rem" ] && {
		internal_err_exit "determine_privoff: bad disksz $useable_disksz"
	}
	init_pub_off=
	init_pub_len=
	#
	# The location of the private region will be either at the start or
	# end of the disk depending on which results in the least amount of
	# subdisk relocation. In case both would cause no (or the same amount
	# of) relocation, the start of the disk is given preference.
	#
	_use_to=`expr $_128K + $1`
	_free_from=`expr $curr_pub_off + $free_at_start`
	_free_from=`expr $_free_from - \( $_free_from % $_8K \)` # round down
	_start_relo=`expr $_use_to - $_free_from`
	[ $_start_relo -le 0 ] && {
		# 
		# Since nothing relocated if use start and it has preference,
		# use it.
		#
		eval "$2=$_128K"
		return 0
	}
	_use_to=`expr $_128K + $useable_disksz - $1`
	_free_from=`expr $curr_pub_off + $curr_pub_len - $free_at_end`
	_rem=`expr $_free_from % $_8K`
	[ "0" != "$_rem" ] && _free_from=`expr $_free_from + $_8K - $_rem`
	_end_relo=`expr $_free_from - $_use_to`
	[ $_end_relo -le 0 ] && {
		# 
		# Since nothing relocated if use end and we will at start,
		# use end.
		#
		eval "$2=$_use_to"
		return 0
	}
	#
	# Relo at both start or end, so choose the one with the least. In
	# case of a tie, go with start. Instead of just looking at the
	# gross-level free_xxx variables, actually see how much relocation
	# is needed.
	#
	init_pub_len=`expr $useable_disksz - $1`
	init_pub_off=`expr $_128K + $1`
	set_reloc_sds
	_start_relo=$size_reloc_sd

	init_pub_off=$_128K
	set_reloc_sds
	_end_relo=$size_reloc_sd
	init_pub_off=
	init_pub_len=

	if [ $_start_relo -le $_end_relo ]
	then
		eval "$2=$_128K"
	else
		eval "$2=$_use_to"
	fi
	return 0
}
#
# set_init_attrs - set the attributes for re-initializing a disk as a CDS disk
#			after determining desired priv/pub locations
#
# usage: set_init_attrs
#
# globals used: dg, da, tmpfile1, dg_log_sz, dg_cfg_sz, dg_cfg_use, dg_log_min,
#		curr_prv_len
# globals set: init_prv_off, init_prv_len, init_pub_off, init_pub_len,
#		init_attrs
#
set_init_attrs() {
	$set_x
	#
	# Verify that dg_cfg_use and dg_log_min are 8K aligned.
	#
	_rem=`expr $dg_cfg_use % $_8K`
	[ "0" != "$_rem" ] && {
		internal_err_exit "set_init_attrs: bad dg_cfg_use $dg_cfg_use"
	}
	_rem=`expr $dg_log_min % $_8K`
	[ "0" != "$_rem" ] && {
		internal_err_exit "set_init_attrs: bad dg_log_min $dg_log_min"
	}
	#
	# As part of conversion, we only create disks with one config region
	# and one log region.
	#
	init_attrs="format=cdsdisk nconfig=1 nlog=1"
	#
	# The size of the private region is dependent on whether the user has
	# specified one via the privlen attribute to the vxcdsconvert utility.
	# If one was not specified, we try to pick one.
	#
	if [ -n "$privlen" ]
	then
		_privlen_spec=y
	else
		_privlen_spec=n
		#
		# The user did not specify a privlen, so first see if any
		# privlen value is specified in the vxdisk default file. If so,
		# round-up if need be to 8KB boundary; otherwise, use 1MB.
		# Then see if this privlen value is less than current size
		# of the private region (rounded-down to 8KB if need be); if
		# so, set privlen to current size.
		#
		voladm_get_disk_layout_defaults vxdisk "privlen" privlen \
			>/dev/null 2>&1
		[ $? -ne 0 ] && {
			print_err_exit "E_SYNTAX_DUP $VOL_VXDISK_DFLT_FILE"
		}
		if [ -n "$privlen" ]
		then
			_rem=`expr $privlen % $_8K`
			[ "0" != "$_rem" ] && {
				privlen=`expr $privlen + $_8K - $_rem`
			}
		else
			privlen=`expr 1 \* $_1M`
		fi
		_val=`expr $curr_prv_len - \( $curr_prv_len % $_8K \)`
		[ $privlen -lt $_val ] && privlen=$_val
	fi
	#
	# Verify that privlen by default creates config and log regions
	# that are "big enough".
	#
	calc_rgn_lens $privlen cfglen loglen
	if [ $cfglen -lt $dg_cfg_use -o $loglen -lt $dg_log_min ]
	then
		#
		# See if use of needed config and log gives us a useable
		# private region.
		#
		_avail=`expr $cfglen + $loglen`
		_need=`expr $dg_cfg_use + $dg_log_min`
		if [ $_avail -lt `expr $_need + $_8K` ]
		then
			[ "n" != $_privlen_spec ] && {
				print_err_exit "E_PRIVLEN_SMALL"
			}
			#
			# Increase privlen to what is compatible with the
			# current DG.
			#
			if [ $cfglen -lt $dg_cfg_sz -o $cfglen -lt $dg_cfg_use ]
			then
				if [ $dg_cfg_use -gt $dg_cfg_sz ]
				then
					cfglen=$dg_cfg_use
				else
					cfglen=`expr $dg_cfg_sz - \
							\( $dg_cfg_sz % $_8K \)`
				fi
			fi
			if [ $loglen -lt $dg_log_sz -o $loglen -lt $dg_log_min ]
			then
				if [ $dg_log_min -gt $dg_log_sz ]
				then
					loglen=$dg_log_min
				else
					loglen=`expr $dg_log_sz - \
						 	\( $dg_log_sz % $_8K \)`
				fi
			fi
			calc_privlen privlen $cfglen $loglen
		else
			#
			# Reapportion space 85:15% config:log. Note we don't
			# need to verify if we exceed max. values of 65M and 8M
			# respectively since vxconfigd will silently truncate
			# things for us.
			#
			_extra=`expr $_avail - $_need`
			_le=`expr $_extra \* 15 / 100`
			_le=`expr $_le - \( $_le % $_8K \)`
			if [ $_le -lt $_8K ]
			then
				_le=$_8K
			else
				_rem=`expr $_le % $_8K`
				[ "0" != "$_rem" ] && {
					_le=`expr $_le + $_8K - $_rem`
				}
			fi
			loglen=`expr $dg_log_min + $_le`
			cfglen=`expr $dg_cfg_use + $_extra - $_le`
		fi
		#
		# Set init_attrs for loglen and configlen.
		#
		init_attrs="$init_attrs loglen=$loglen"
		init_attrs="$init_attrs configlen=$cfglen"
		#
		# Sanity check.
		#
		_rem=`expr $cfglen % $_8K`
		[ "0" != "$_rem" ] && {
			internal_err_exit "set_init_attrs: bad cfglen $cfglen"
		}
		_rem=`expr $loglen % $_8K`
		[ "0" != "$_rem" ] && {
			internal_err_exit "set_init_attrs: bad loglen $loglen"
		}
	fi
	init_prv_len=$privlen
	#
	# Now determine priv offset.
	#
	determine_privoff $privlen init_prv_off 
	#
	# Now determine the value for puboff, which is either 128K or after
	# the private region.
	#
	if [ $_128K -eq $init_prv_off ]
	then
		init_pub_off=`expr $init_prv_off + $init_prv_len`
		init_pub_len=`expr $_128K + $useable_disksz - $init_pub_off`
	else
		init_pub_off=$_128K
		init_pub_len=`expr $init_prv_off - $init_pub_off`
	fi
	#
	# Although not strictly necessary, set all private and public offset
	# and length info in init_attrs.
	#
	init_attrs="$init_attrs privoffset=$init_prv_off"
	init_attrs="$init_attrs privlen=$init_prv_len"
	init_attrs="$init_attrs puboffset=$init_pub_off"
	init_attrs="$init_attrs publen=$init_pub_len"

	return 0
}
#
# append_reloc_sds - append sds to reloc lists based on reloc area
#
# usage: append_reloc_sds min_addr max_addr
#
# globals used: dg, dm, tmpfile1
# globals set: full_reloc_sd, part_reloc_sd, size_reloc_sd
#
append_reloc_sds() {
	$set_x
	[ $# -ne 2 ] &&  internal_err_exit "append_reloc_sds: arg count ($*)"
	[ $1 -gt $2 ] &&  internal_err_exit "append_reloc_sds: bad args ($*)"
	_min_addr=$1
	_max_addr=$2
	#
	# Get a list of subdisks on the disk. For each subdisk, determine if
	# any or all of the subdisk lies withing the area being relocated. If
	# so, add the subdisk to the appropriate list.
	#
	doit -n "" vxprint -g $dg -qse 'sd_dmname="'$dm'"' \
					-F "%name %offset %len" > $tmpfile1
	exec 3<&0 < $tmpfile1
	while read _name _offset _len
	do
		_end=`expr $_offset + $_len`
		if [ $_offset -lt $_max_addr -a $_end -gt $_min_addr ]
		then
			if [ $_offset -ge $_min_addr -a $_end -le $_max_addr ]
			then
				append_list full_reloc_sd $_name $_len
			else
				if [ $_offset -lt $_min_addr ]
				then
					_start=$_min_addr
				else
					_start=$_offset
				fi
				if [ $_end -lt $_max_addr ]
				then
					_size=`expr $_end - $_start`
				else
					_size=`expr $_max_addr - $_start`
				fi
				append_list part_reloc_sd $_name $_start $_size $_len
				[ "no" != "$split_subdisks_on" ] && _len=$_size
			fi
			size_reloc_sd=`expr $size_reloc_sd + $_len`
		fi
	done
	exec 0<&3 3<&-
}
#
# set_reloc_sds - place sds in reloc lists based on changes to public region
#
# usage: set_reloc_sds
#
# globals used: init_pub_off, init_pub_len, curr_pub_off, curr_pub_len,
#		free_at_start, free_at_end (dg, dm, tmpfile1)
# globals set: full_reloc_sd, part_reloc_sd, size_reloc_sd
#
set_reloc_sds() {
	$set_x
	[ -z "$init_pub_off" -o -z "$init_pub_len -o -z "$free_at_start" -o \
	  -z "$curr_pub_off" -o -z "$curr_pub_len -o -z "$free_at_end" ] && {
		internal_err_exit "set_reloc_sds: bad globals"
	}
	full_reloc_sd=
	part_reloc_sd=
	size_reloc_sd=0

	_init_pubend=`expr $init_pub_off + $init_pub_len`
	_curr_pubend=`expr $curr_pub_off + $curr_pub_len`
	_first_sd=`expr $curr_pub_off + $free_at_start`
	_last_sd=`expr $_curr_pubend - $free_at_end`

	if [ $_first_sd -lt $init_pub_off ]
	then
		append_reloc_sds 0 `expr $init_pub_off  - $curr_pub_off`
	fi
	if [ $_last_sd -gt $_init_pubend ]
	then
		append_reloc_sds `expr $_init_pubend - $curr_pub_off` \
				  $curr_pub_len
	fi
	[ -z "$full_reloc_sd" -a -z "$part_reloc_sd" -a 0 -eq $size_reloc_sd ] \
		&& return 0
	[ 0 -eq $size_reloc_sd ] && {
		internal_err_exit "set_reloc_sds: size_reloc_sd not set"
	}
	[ -z "$full_reloc_sd" -a -z "$part_reloc_sd" ] && {
		internal_err_exit "set_reloc_sds: full/part_reloc_sd not set"
	}
}
#
# save_disk_parms - save the "per-disk" global variables and disk state
# restore_disk_parms - restore the "per-disk" global variables
#
# usage: save_disk_parms
#	 restore_disk_parms <disk>
#
# globals used: dmlist, and "per-disk" globals for "save"
# globals set: "per-disk" globals for "restore"
#
# MPW: what about overflow of full_reloc_sd and part_reloc_sd (e.g., >2048)
#
save_disk_parms() {
	$set_x
	dogi_name_to_device $da _dev
	_savedir=$DISKDIR/$_dev

	[ -d $_savedir ] || mkdir -p $_savedir

	doit "E_DM_INFO $dm" vxdisk list $da > $_savedir/dainfo
	case $platform in
	SunOS|Linux)
		dogi_device_rawpath $_dev _rawpath
		$PRTVTOC -f $_savedir/vtoc $_rawpath 2>/dev/null
		;;
	*)	;;
	esac
	_f=$_savedir/convinfo
	rm -f $_f; touch $_f

	list_find dmlist $dm idx
	[ 0 -ne $? ] && internal_err_exit "save_disk_parms: no $dm in $dmlist"

	for var in $perdm_varlist
	do
		eval "_val=\"\$$var\""
		eval "save${idx}_${var}=\"$_val\""
		echo "$var=\"$_val\"" >> $_f
	done
}
restore_disk_parms() {
	$set_x
	[ $# -ne 1 ] && internal_err_exit "restore_disk_parms: arg count ($*)"

	dm=$1
	list_find dmlist $dm idx
	[ 0 -ne $? ] && internal_err_exit "restore_disk_parm: no $dm in $dmlist"

	for var in $perdm_varlist
	do
		eval "${var}=\"\$save${idx}_${var}\""
	done
}
#
# order_todolist - order the dm_todolist by criteria described below
#
# usage: order_todolist
#
# globals used: todo_dmlist, novolstop
# globals set: todo_dmlist
#
# Ordering criteria:
#	1. first comes any DM that needs no subdisk relocation
#
#	2. next comes any DM that can do on-disk subdisk relocation (i.e.,
#		and the amount of needed space is less than the amount of
#		free space on the disk)
#
#	3. next comes any DM that needs off-disk subdisk relocation
#
order_todolist() {
	$set_x
	no_sdr_list=
	ondisk_list=
	offdisk_list=

	set -- $todo_dmlist
	while [ $# -ge 4 ]
	do
		# args 1 - 4 are: <dm> <reloc-size> <free-reloc> <free-init>
		
		if [ "0" = "$2" ]
		then
			no_sdr_list="$no_sdr_list $1 $2 $3 $4"

		elif [ $2 -lt $3 ]
		then
			ondisk_list="$ondisk_list $1 $2 $3 $4"

		elif [ $2 -lt $4 -a "y" = "$novolstop" ]
		then
			ondisk_list="$ondisk_list $1 $2 $3 $4"
		else
			offdisk_list="$offdisk_list $1 $2 $3 $4"
		fi
		shift 4
	done
	[ $# -gt 0 ] && internal_err_exit "order_todolist: bad todo_dmlist"

	todo_dmlist="$no_sdr_list $ondisk_list $offdisk_list"
}
#
# output_analysis_info - output info associated with the -A option
#
# usage: output_analysis_info <op>
#		where op is group|all|one
#
output_analysis_info() {
	$set_x
	_op=$1
	#
	# Handle the no conversion case.
	#
	if [ -z "$todo_dmlist" ]
	then
		if [ "one" = "$_op" ]
		then
			out=`egettxt "The disk is already a CDS disk" vxvmshm:3270`
		else
			out=`egettxt "No disk needs to be converted to a CDS disk" vxvmshm:3188`
		fi
		print_anal "$out"
		return 0
	fi
	export dm; out=`egettxt "\

Disk $dm needs to be converted to a CDS disk" vxvmshm:2920`
	print_anal "$out"
	#
	# Determine which volumes need to be stopped/started.
	#
	vlist=`vxprint -g "$dg" -vn -e "aslist.aslist.sd_dm_name=\"$dm\""`

	start_list=
	stop_list=
	if [ -n "$vlist" ]
	then
 		if [ "y" = "$novolstop" ]
		then
			#
			# For online convert, all volumes must be started.
			#
			vol_startstop_list start slist $vlist
			mk_print_list "$slist" start_list
		else
			#
			# For offline convert, all volumes must be stopped;
			# however, we still may need to start a volume for
			# subdisk relocation.
			#
			vol_startstop_list stop slist $vlist
			mk_print_list "$slist" stop_list 

			vlist=
			get_list_group 4 1 part_reloc_sd part_sdlist
			get_list_group 2 1 full_reloc_sd full_sdlist
			for sd in $part_sdlist $full_sdlist
			do
				doit "E_DG_INFO" vxprint -g "$dg" -v -F %name \
				   -e "aslist.aslist.sd_name=\"$sd\"" >$tmpfile1
				vlist="$vlist `cat $tmpfile1`"
			done
			[ -n "$vlist" ] && {
				vol_startstop_list start slist $vlist
				mk_print_list "$slist" start_list
			}
		fi
	fi
	if [ -z "$start_list" -a -z "$stop_list" ]
	then
		out=`egettxt "\

	No volumes will be started or stopped in order to convert the disk.
" vxvmshm:2859`
		print_anal "$out"
	else
		if [ -n "$start_list" ]
		then
			export start_list; out=`egettxt "\

	The following volumes will be started in order to convert the disk:

		$start_list
" vxvmshm:2869`
			print_anal "$out"
		fi
		if [ -n "$stop_list" ]
		then
			export stop_list; out=`egettxt "\

	The following volumes will be stopped in order to convert the disk:

		$stop_list
" vxvmshm:2870`
			print_anal "$out"
		fi
	fi
	#
	# Output if subdisk relocation is needed and can be done.
	#
	if [ -z "$part_reloc_sd" -a -z "$full_reloc_sd" ]
	then
		out=`egettxt "\

	No subdisk relocation is needed in order to convert the disk.
" vxvmshm:2858`
		print_anal "$out"
		return
	fi
	if [ "yes" != "$move_subdisks_ok" -a "yes" != "$evac_subdisks_ok" ]
	then
		get_list_group 4 1 part_reloc_sd part_sdlist
		get_list_group 2 1 full_reloc_sd full_sdlist
		mk_print_list "$part_sdlist $full_sdlist" plist

		export plist; out=`egettxt "\

	Subdisk relocation is not enabled because both 'move_subdisks_ok' and
	'evac_subdisks_ok' are set to 'no'. The disk cannot be converted because
	the following subdisks need to be relocated:

		$plist
" vxvmshm:2868`
		print_anal "$out"
		return
	fi
	#
	# Determine which subdisks need to be moved/evac'ed.
	#
	_part_on=	# disks that can be paritially relocated on-disk
	_part_off=	# disks that can be partially relocated off-disk
	_full_on=	# disks that need to be fully relocated on-disk
	_full_off=	# disks that need to be fully relocated off-disk
	_move_err=	# disks that can't be relocated (not enough space)

	_disk_free=$free_total		# remaining free space on disk
	_disk_free_mv=$free_total	# remaining free space if move ok
	_disk_free_mvsp=$free_total	# remaining free space if move&split ok

	_num_part_full=0	# num of full relocs because split not yes
				#	or split size too large
	_num_mv_evac=0		# num of evacs because move not yes
	_num_part_evac=0	# num of evacs because split not yes
				#	or split size too large
	#
	# NOTE: currently the subdisk relocation code does not support splitting
	# subdisks during evacuation. Hopefully, this'll change in the future,
	# so we'll leave _num_part_full calculations in place and force things
	# with _force_evac_full.
	#
	_force_evac_full=y

	set -- $part_reloc_sd
	while [ $# -ge 4 ]
	do
		# $_name $_start $_size $_len

		if [ $4 -lt `expr 2 \* $min_split_size` ]
		then
			split_move=n; split_evac=n
		else
			case $split_subdisks_ok in
			yes)	split_move=y; split_evac=y;;
			no)	split_move=n; split_evac=n;;
			move)	split_move=y; split_evac=n;;
			evac)	split_move=n; split_evac=y;;
			*)	internel_err_exit \
					"output_anal_info:bad split_subdisks_ok"
			esac
		fi
	
		_mvsp_fit="n"
		if [ $3 -le $_disk_free ]
		then
			#
			# The partial subdisk can be relocated on disk if we
			# are allowed to.
			#
			if [ "yes" = "$move_subdisks_ok" -a \
			     $min_split_size -le $_disk_free -a \
			     "y" = "$split_move" ]
			then
				_part_on="$_part_on $1"
				_disk_free=`expr $_disk_free - $3`
				shift 4
				continue
			fi
			[ $3 -le $_disk_free_mv -a "y" = "$split_move" ] && {
				_disk_free_mv=`expr $_disk_free_mv - $3`
			}
			if [ $3 -le $_disk_free_mvsp ]
			then
				_disk_free_mvsp=`expr $_disk_free_mvsp - $3`
				_mvsp_fit="y"
			fi
			#
			# We weren't allowed to relocate the partial subdisk
			# on-disk. See if we can, and are allowed to, relocate
			# the entire subdisk on-disk.
			#
			if [ $4 -le $_disk_free -a "yes" = "$move_subdisks_ok" ]
			then
				_full_on="$_full_on $1"
				_disk_free=`expr $_disk_free - $4`

				[ "y" = "$_mvsp_fit" ] && {
					_num_part_full=`expr $_num_part_full + 1`
				}
				shift 4
				continue
			fi
			if [ "yes" = "$move_subdisks_ok" ]
			then
				[ "y" = "$_mvsp_fit" ] && {
					_num_part_evac=`expr $_num_part_evac + 1`
				}
			elif [ $4 -le $_disk_free ]
			then
				[ $4 -le $_disk_free_mv ] && {
					_disk_free_mv=`expr $_disk_free_mv - $4`
					_num_mv_evac=`expr $_num_mv_evac + 1`
				}
			fi
		fi
		#
		# We either couldn't, or weren't allowed to, relocate on-disk.
		# See about off-disk.
		#
		if [ "yes" != "$evac_subdisks_ok" ]
		then
			_move_err="$_move_err $1"

		elif [ "y" = "$_force_evac_full" ]
		then
			_full_off="$_full_off $1"

		elif [ "y" != "$split_evac" ]
		then
			[ "yes" != "$move_subdisks_ok" -o \
			  "y" != "$_mvsp_fit" ] && {

				_num_part_full=`expr $_num_part_full + 1`
			}
			_full_off="$_full_off $1"
		else
			_part_off="$_part_off $1"
		fi
		shift 4
	done
	[ $# -ne 0 ] && internal_err_exit "output_anal_info: bad part_reloc_sd"

	set -- $full_reloc_sd
	while [ $# -ge 2 ]
	do
		# $_name $_len

		if [ $2 -le $_disk_free ]
		then
			#
			# The full subdisk can be relocated on disk if we
			# are allowed to.
			#
			if [ "yes" = "$move_subdisks_ok" ]
			then
				_full_on="$_full_on $1"
				_disk_free=`expr $_disk_free - $2`
				shift 2
				continue
			fi
			[ $2 -le $_disk_free_mv ] && {
				_disk_free_mv=`expr $_disk_free_mv - $2`
				_num_mv_evac=`expr $_num_mv_evac + 1`
			}
		fi
		#
		# We either couldn't, or weren't allowed to, relocate on-disk.
		# See about off-disk.
		#
		if [ "yes" != "$evac_subdisks_ok" ]
		then
			_move_err="$_move_err $1"
		else
			_full_off="$_full_off $1"
		fi
		shift 2
	done
	[ $# -ne 0 ] && internal_err_exit "output_anal_info: bad full_reloc_sd"
	#
	# Output subdisk relocation info for converting the disk.
	#
	if [ -n "$_move_err" ]
	then
		mk_print_list "$_move_err" plist
		export plist; out=`egettxt "\

	Subdisk evacuation is not enabled ('evac_subdisks_ok' is set to 'no').
	The disk cannot be converted because the following subdisks need to be
	evacuated as there is not enough space to move them elsewhere on the
	disk:

		$plist
" vxvmshm:2867`
		print_anal "$out"

		[ -n "$_num_part_evac" -a "0" != "$_num_part_evac" ] && {

			if [ "1" = "$_num_part_evac" ]
			then
				out=`egettxt "\

	Note: one subdisk needs to be evacuated because either
	'split_subdisks_ok' is not set to 'move' or 'yes' or
	'min_split_size' is too large.
" vxvmshm:3398`
			else
				export _num_part_evac; out=`egettxt "\

	Note: $_num_part_evac subdisks need to be evacuated because
	either 'split_subdisks_ok' is not set to 'move' or 'yes' or
	'min_split_size' is too large.
" vxvmshm:3395`
			fi
			print_anal "$out"
		}
		return
	fi
	[ -n "$_part_on" ] && {

		mk_print_list "$_part_on" plist
		export plist; out=`egettxt "\

	To convert the disk, the following subdisks will be split and the split
	portion moved elsewhere on the same disk:

		$plist
" vxvmshm:2874`
		print_anal "$out"
	}
	[ -n "$_part_off" ] && {

		mk_print_list "$_part_off" plist
		export plist; out=`egettxt "\

	To convert the disk, the following subdisks will be split and the split
	portion evacuated to a different disk:

		$plist
" vxvmshm:2873`
		print_anal "$out"
	}
	[ -n "$_full_on" ] && {

		mk_print_list "$_full_on" plist
		export plist; out=`egettxt "\

	To convert the disk, the following subdisks will be moved in their
	entirety elsewhere on the same disk:

		$plist
" vxvmshm:2872`
		print_anal "$out"
	}
	[ -n "$_full_off" ] && {

		mk_print_list "$_full_off" plist
		export plist; out=`egettxt "\

	To convert the disk, the following subdisks will be evacuated in their
	entirety to a different disk:

		$plist
" vxvmshm:2871`
		print_anal "$out"
	}
	[ -n "$_num_mv_evac" -a "0" != "$_num_mv_evac" ] && {
		if [ "1" = "$_num_mv_evac" ]
		then
			out=`egettxt "\

	Note: one subdisk needs to be evacuated because 'move_subdisks_ok'
	is not set to 'yes'.
" vxvmshm:3397`
		else
			export _num_mv_evac; out=`egettxt "\

	Note: $_num_mv_evac subdisks need to be evacuated because
	'move_subdisks_ok' is not set to 'yes'.
" vxvmshm:2860`
		fi
		print_anal "$out"
	}
	[ -n "$_num_part_evac" -a "0" != "$_num_part_evac" ] && {
		if [ "1" = "$_num_part_evac" ]
		then
			out=`egettxt "\

	Note: one subdisk needs to be evacuated because either
	'split_subdisks_ok' is not set to 'move' or 'yes' or
	'min_split_size' is too large.
" vxvmshm:3398`
		else
			export _num_part_evac; out=`egettxt "\

	Note: $_num_part_evac subdisks need to be evacuated because
	either 'split_subdisks_ok' is not set to 'move' or 'yes' or
	'min_split_size' is too large.
" vxvmshm:3395`
		fi
		print_anal "$out"
	}
	[ -n "$_num_part_full" -a "0" != "$_num_part_full" ] && {
		if [ "1" = "$_num_part_full" ]
		then
			out=`egettxt "\

	Note: one subdisk needs to be relocated in its entirety because
	either 'split_subdisks_ok' is not set to 'yes' or 'min_split_size'
	is too large.
" vxvmshm:3399`
		else
			export _num_part_full; out=`egettxt "\

	Note: $_num_part_full subdisks need to be relocated in their
	entirety because either 'split_subdisks_ok' is not set to 'yes'
	or 'min_split_size' is too large.
" vxvmshm:3396`
		fi
		print_anal "$out"
	}
}
#
# do_sd_reloc - do subdisk relocation for conversion of a disk to a CDS disk
#
# usage: do_sd_reloc
#
# globals used: dm, init_pub_off, init_pub_len, full_reloc_sd, part_reloc_sd,
#		move_subdisks_ok, evac_subdisks_ok, 
#
do_sd_reloc() {
	$set_x
	[ -z "$part_reloc_sd" -a -z "$full_reloc_sd" ] && return 0

	[ "no" = "$move_subdisks_ok" -a "no" = "$evac_subdisks_ok" ] && {
		print_err_exit "E_SD_RELOC_NEEDED"
	}
	#
	# Get the list of partial and full subdisks to be relocated. We keep
	# separate lists so that we can try to relocate the ones we might
	# split first (to try and keep the split subdisks on the same disk).
	#
	if [ -n "$part_reloc_sd" ] ; then
		get_list_group 4 1 part_reloc_sd _sdlist_part
	else
		_sdlist_part=""
	fi
	if [ -n "$full_reloc_sd" ] ; then
		get_list_group 2 1 full_reloc_sd _sdlist_full
	else
		_sdlist_full=""
	fi
	#
	# Start any volumes that are not already started that contain one or
	# more of the subdisks to be relocated. We'll remember the ones we
	# started here so that we can stop them later.
	#
	_vol_list=
	for _sd in $_sdlist_part $_sdlist_full
	do
		doit "E_DG_INFO" vxprint -g "$dg" -v -F %name \
			   -e "aslist.aslist.sd_name=\"$_sd\"" >$tmpfile1
		_vol_list="$_vol_list `cat $tmpfile1`"
	done
	_start_list=
	[ -n "$_vol_list" ] && {
		vol_startstop_list start _start_list $_vol_list || return 1

		[ -n "$_start_list" ] && {
			vxvol -g "$dg" start $_start_list >/dev/null 2>&1
		}
		_efound=0
		for _vol in $_vol_list
		do
			[ "ACTIVE" != "`vxprint -g $dg -F %state $_vol`" ] && {
				print_err "E_DM_VOL_START $dm $_vol"
				_efound=1
			}
		done
		[ "$_efound" != "0" ] && return 1
	}
	#
	# Now that the volumes are started attempt to relocate the subdisks.
	#
	[ -n "$_sdlist_part" ] && {
		sdr_relocate_sds $dm $init_pub_off $init_pub_len \
				"$_sdlist_part" || _efound=1
	}
	[ -n "$_sdlist_full" -a "$_efound" = "0" ] && {
		sdr_relocate_sds $dm $init_pub_off $init_pub_len \
				"$_sdlist_full" || _efound=1
	}
	#
	# Stop any volumes we started.
	#
	[ -n "$_start_list" ] && {
		vxvol -g "$dg" stop $_start_list >/dev/null 2>&1
	}
	return $_efound
}
#
# reformat_disk - take a disk out of the DG, reformat it, and put it back
#
# usage: refomat_disk
#
# Returns 0 for success; 1 for failure
#
# globals used: dg, da, da_type, dm, init_attrs, novolstop
#
reformat_disk() {
	$set_x
	#
	# Steps:
	# 	1. Remove the disk from the DG
	#	2. Destroy any existing format on the disk
	#	3. Remove any existing DA record for the disk
	#	4. Create a new auto DA record for the disk
	#	5. Reinit the disk (keeping current disk id if doing
	#		off-line init).
	#	6. Add the disk back to the DG
	#
	doit -n "E_DM_RM $dm" vxdg -g "$dg" -k rmdisk $dm || return 1

	dogi_name_to_device $da dev

	[ "nopriv" != "$da_type" -a "y" = "$novolstop" ] && {
		doit -n "E_DA_REINIT $da" vxdisk destroy $da || return 1
	}
	doit -n "E_DA_REINIT $da" vxdisk rm $da || return 1
	doit -n "E_DA_REINIT $dev" vxdisk scandisks device=$dev || return 1

	if [ "y" =  "$novolstop" ]
	then
		#
		# Do destroy again just in case the auto-config DA record is
		# still not auto:none (e.g., like pub/priv tags are still in
		# VTOC). We need to do this so we can call vxdisksetup w/o -i.
		#
		# Note: we call vxdisksetup w/o -i since some of the init_attrs
		# are not supported (on all platforms).
		#
		doit -n "E_DA_REINIT $da" vxdisk destroy $dev || return 1
		doit -n "E_DA_REINIT $dev" vxdisksetup $dev format=cdsdisk || \
			return 1
		doit -n "E_DA_REINIT $dev" vxdisk -f init $dev $init_attrs || \
			return 1
	else
		#
		# Adjust the sd offsets for the DM.
		#
		_diskoff_adj=`expr $curr_pub_off - $init_pub_off`
		[ 0 -ne $_diskoff_adj ] && {
			doit -n "E_DM_ADJ $dm" vxedit -g $dg set \
				last_disk_offset=$init_pub_off $dm || return 1
		}
		#
		# Note: vxdisk internally already does the equiavlent of
		# "vxdisk destroy" and "vxdisksetup" when creating, except
		# for DOS to SUN label conversion on Linux (which we require
		# elsewhere in this script to go through the novolstop path).
		#
		doit -n "E_DA_REINIT $dev" vxdisk -k -f init $dev $init_attrs \
			|| return 1
	fi
	doit -n "E_DA_ADD $dev" vxdg -g "$dg" -k adddisk $dm=$dev || return 1

	return 0
}
#
# find_toplevel_vol - find the top-level volume for a given volume
#
# usage: find_toplevel_vol vol topvol-ret
#
# Returns: none (exits upon error detection)
#
find_toplevel_vol() {

	[ $# -ne 2 ] && internal_err_exit "find_toplevel_vol: bad arg cnt ($*)"
	[ -z "$1" ] && internal_err_exit "find_toplevel_vol: no volume name"

	_tl_vol=$1
	while [ 1 -eq 1 ]
	do
		doit "E_VOL_INFO $dm $_tl_vol" \
				vxprint -g $dg -F %layered $_tl_vol >$tmpfile1
		_layered=`cat $tmpfile1`

		[ "off" = "$_layered" ] && break

		doit "E_DG_INFO" vxprint -g $dg -v -F %name \
			-e "aslist.aslist.sd_dmname=\"$_tl_vol\"" >$tmpfile1
		_tl_vol=`cat $tmpfile1`

		[ -z "$_tl_vol" ] && print_err_exit "E_DG_INFO"
	done
	eval "$2=$_tl_vol"
}
#
# vol_startstop_list - create a list of volumes that need to be started/stoppped
#
# usage: vol_startstop_list start|stop list-ret vol [vol]...
#	where: start|stop indicates whether the volumes should be started or not
#		list-ret is a variable name in which to return a list of volumes
#			to be started/stopped
#		vol is a volume to consider
#
# Depending on the state of the volume (vol) and what type of volume it is,
# the volume and/or some related volume (e.g., a DCO log's data volume) will
# be appended to the list returned.
#
# globals used: dg, dm, tmpfile1, tmpfile2
#
# Returns 0 for success; 1 for failure
# Routine may cause an exit instead of a return depending on the error.
#
vol_startstop_list() {
	$set_x

	[ $# -le 2 ] &&  internal_err_exit "vol_startstop_list: arg count ($*)"

	_action=$1
	_ret_list=$2
	shift 2
	_vollist=
	_sslist=
	#
	# Determine which of the volumes need to be started/stopped.
	#
	doit "E_DG_INFO" vxinfo -g "$dg" $* > $tmpfile1

	if [ "start" = "$_action" ]
	then
		egrep -v "Started$|Startable$" $tmpfile1 >/dev/null 2>&1
		[ 1 -ne $? ] && {
			print_err "E_DM_VOL_STARTABLE $dm"
			return 1
		}
		_vollist=`grep "Startable$" $tmpfile1 2>/dev/null | \
			awk '{print $1}'`

	elif [ "stop" = "$_action" ]
	then
		_vollist=`grep "Started$" $tmpfile1 2>/dev/null | \
			awk '{print $1}'`
	else
		internal_err_exit "vol_startstop_list: bad action: $_action"
	fi
	#
	# For every volume that needs to be started/stopped, decide which
	# volumes should really be started/stopped. In particular,
	#
	#	1. A DCO log volume cannot be stopped unless the data volume
	#		is also stopped. So, if need be add the data volume
	#		to the start/stop list prior to the log volume.
	#		Note: the converse is not true (i.e., the DCO log
	#		volume can be started eventhough the data volume isn't).
	#
	#	2. Always return the toplevel volume (for layered volumes).
	#		It should be noted that starting/stopping the toplevel
	#		volume may indicate a failure eventhough it has
	#		started/stopped the volume we are intersted in.
	#
	set -- $_vollist
	while [ $# -gt 0 ]
	do
		#
		# Determine if volume is a DCO log volume.
		# If so,
		#	find the associated data volume
		#	if the data volume is a layered volume
		#		find its toplevel volume (DCO log volume should
		#			already be a toplevel volume)
		# If not,
		#	find the volume's toplevel volume
		#
		# For the "stop" action, if the toplevel volume is a VVR SRL,
		# it cannot be stopped so we return a 1.
		#
		doit "E_VOL_INFO $dm $1" vxprint -g $dg \
			-F "%isdcolog %dco_parent_name" -v $1 >$tmpfile1

		if [ "on" = `awk '{print $1}' $tmpfile1` ]
		then
			_dvol=`awk '{print $2}' $tmpfile1`

			doit "E_VOL_INFO $dm $_dvol" \
				vxprint -g $dg -F %parent $_dvol >$tmpfile1
			_dvol=`cat $tmpfile1`

			find_toplevel_vol $_dvol _dvol
		else
			find_toplevel_vol $1 _dvol
		fi
		if [ "$_dvol" = "$1" ]
		then
			_dvol=""	# don't add it twice
		else
			#
			# See if the desired volume is already in the
			# correct state. If not, don't add to the list.
			#
			doit "E_DG_INFO" vxinfo -g "$dg" $_dvol > $tmpfile1
			_state=`awk '{print $3}' $tmpfile1`

			if [ "start" = "$_action" ]
			then
				case $_state in
				Started)
					_dvol=""
					;;
				Startable)
					;;
				*)	print_err "E_DM_VOL_STARTABLE $dm"
					;;
				esac
			elif [ "Started" != "$_state" ]
			then
				_dvol=""
			else
				#
				# Started volume that needs to be stopped. Check
				# if it is an SRL log.
				#
				doit "E_VOL_INFO $dm $_dvol" vxprint -g $dg \
					-F "%rvg_name" -v $_dvol >$tmpfile1
				_name=`cat $tmpfile1`

				[ -n "$_name" -a "-" != "$_name" ] && {
					doit "E_VOL_INFO $dm $_name" vxprint \
						-g $dg -F %srl $_name >$tmpfile1
					_srl=`cat $tmpfile1`

					[ "$_srl" = "$_dvol" ] && return 1
				}
			fi
		fi
		for _item in $_dvol $1
		do
			#
			# Add to list if not already there.
			#
			echo "$_sslist" | \
				egrep " $_item | $_item$" >/dev/null 2>&1
			[ 0 -ne $? ] && _sslist="$_sslist $_item"
		done
		shift 1
	done
	rm -f $tmpfile1
	eval "$_ret_list=\"$_sslist\""
	return 0
}
#
# do_disk_reinit - do (off-line or on-line) disk re-initialization
#
# usage: do_disk_reinit
#
# Returns 0 for success; 1 for failure
# Routine may cause an exit instead of a return depending on the error.
#
# globals used: dg, dm, novolstop (and callees use da and init_attrs)
#
do_disk_reinit() {
	$set_x
	#
	# Find all volumes with a subdisk on the disk. If there are no volumes,
	# then just reformat the disk.
	#
	vlist=`vxprint -g "$dg" -vn -e "aslist.aslist.sd_dm_name==\"$dm\""`

	[ -z "$vlist" ] && {
		reformat_disk || return 1
		return 0
	}
	slist=
	if [ "y" = "$novolstop" ]
	then
		#
		# On-line re-initialization is being requested. So, we will
		# perform the following steps:
		#
		# 1. Check that all volumes with a subdisk on the disk are 
		#	started. If any are not started but startable, try
		#	to start them.
		#
		# 2. Relocate the volumes off of the disk using vxassist with
		#	the appropriate options so that they can unrelocated
		#	back.
		#
		# 3. Remove the disk from the DG (keeping usage info), re-init
		#	the disk, and add it back.
		#
		# 4. Move the relocated subdisks back. Stop any volumes started
		#	in [1].
		#
		vol_startstop_list start slist $vlist || return 1
		[ -n "$slist" ] && {
			vxvol -g "$dg" start $slist >/dev/null 2>&1
		}
		for vol in $vlist
		do
			#
			# Verify volume is started.
			#
			[ "ACTIVE" != "`vxprint -g $dg -F %state $vol`" ] && {
				print_err "E_DM_VOL_START $dm $vol"
				return 1
			}
			#
			# Call vxalloc to move allocator volumes; otherwise,
			# call vxassist.
			#
			doit "E_VOL_INFO $dm $vol" \
				 vxprint -g $dg -F %stp_rid -v $vol >$tmpfile1
			v_stp_rid=`cat $tmpfile1`

			if [ "$v_stp_rid" != "-" ]
			then
				#
				# vxvoladm accepts only top volume names
				#
				volname=`echo $vol | awk -F"-" '{ print $1 }'`
				doit -n "" vxvoladm -m -g "$dg" evacuate disk \
					$volname !$dm
			else
				doit -n "" vxassist -m -g "$dg" move \
					$vol !$dm layout=nodiskalign
			fi
			[ $? -ne 0 ] && {
				print_err "E_VOL_MOVE_FROM $dm $vol"
				return 1
			}
		done
		reformat_disk || return 1

		doit -n "" vxunreloc -f -g "$dg" $dm
		[ $? -ne 0 ] && print_err "E_UNRELOC_TO $dm"

		[ -n "$slist" ] && {
			vxvol -g "$dg" stop $slist >/dev/null 2>&1
		}
	else
		#
		# Off-line re-initialization is allowed. So, we will perform
		# the following steps:
		#
		# 1. Stop all volumes with a subdisk on the disk (whose state
		#	is Started).
		#
		# 2. Mark all volumes with a subdisk on the disk as "nostart".
		#
		# 3. Remove the disk from the DG (keeping usage info), re-init
		#	the disk, and add it back.
		#
		# 4. Clear the "nostart" indicator [for volumes in 2] and
		#	restart any volumes stopped in [1].
		#
		vol_startstop_list stop slist $vlist || return 1

		[ -n "$slist" ] && {
			vxvol -g "$dg" stop $slist >/dev/null 2>&1

			doit "E_DG_INFO" vxinfo -g "$dg" $slist > $tmpfile1
			vol=`grep "Started$" $tmpfile1 | awk '{print $1}'`
			[ -n "$vol" ] && {
				print_err "E_DM_VOL_STOP $dm $vol"
				return -1
			}
		}
		doit -n "E_VOL_SET_NOSTART $dm" \
			vxvol -g "$dg" set nostart=on $vlist || return 1
		reformat_disk || return 1

		doit -n "E_VOL_CLR_NOSTART $dm" \
			vxvol -g "$dg" set nostart=off $vlist || return 1
		[ -n "$slist" ] && {
			vxvol -g "$dg" start $slist >/dev/null 2>&1
		}
	fi
	return 0
}
#
# diskconv - main routine for doing non-CDS to CDS disk conversion
#
# usage: diskconv group|all|one <name>
#
diskconv() {
	$set_x
	[ -z "$dg" ] && internal_err_exit "diskconv: bad globals ($dg)"
	if [ $# -lt 1 -o $# -gt 2 ]
	then
			internal_err_exit "diskconv: bad args ($*)"
	elif [ $# -eq 1 ]
	then
		[ "$1" != "all" -a "$1" != "group" ] && {
			internal_err_exit "diskconv: bad args ($*)"
		}
	else
		[ "$1" != "one" ] && {
			internal_err_exit "diskconv: bad args ($*)"
		}
	fi
	op=$1
	#
	# Set up DG related info (globals). Then determine the set of disks to
	# be (possibly) converted and set up the disk related info (globals).
	#
	set_dg_cfglog
	if [ "$op" = "one" ]
	then
		dm=$2
		doit "E_DM_NOEXIST $dm" vxprint -g $dg -d $dm > /dev/null
	fi
	dmlist=`vxprint -q -g $dg -d 2>/dev/null | grep -v "REMOVED" | \
		awk '{print $2}'`
	[ -z "$dmlist" ] && print_err_exit "E_DG_INFO"

	set -- $dmlist
	[ $# -eq 1 ] && {
		#
		# Only one disk in DG. If it isn't already a CDS disk, we
		# can't convert it.
		#
		dm=$1
		doit "E_DM_INFO $dm" vxprint -g $dg -F %da_name -d $dm >$tmpfile1
		da=`cat $tmpfile1`
		[ "-" = "$da" ] && print_err_exit "E_DM_NO_DA $dm"

		doit "E_DM_INFO $dm" vxdisk list $da > $tmpfile1
		grep "^info:.*format=cds.*" $tmpfile1 > /dev/null
		res=$?
		rm -f $tmpfile1
		[ $res -eq 0 ] && return 0
		print_err_exit "E_DG_ONE_DM"
	}
	[ "$op" = "one" ] && dmlist="$dm"
	#
	# Determine what needs to be done for each disk.
	#
	efound=0
	rm -f $tmpfile3
	for dm in `echo $dmlist`
	do
		#
		# See if the DM contains any subdisks associated with a volume
		# whose use-type is not allowed on a CDS disk (i.e., root/swap).
		#
		nocds=`vxprint -g $dg -F "%vname" -s \
			-e 'sd_dm_name="'$dm'" &&
			    (assoc.assoc.v_use_type="root" ||
			     assoc.assoc.v_use_type="swap")' | head -1`

		[ -n "$nocds" ] && {
			print_err "E_DM_USAGE $dm"
			[ "$op" = "one" -o "$op" = "group" ] && err_exit
			efound=1
			continue
		}
		#
		# Get the DA name for the DM. Make sure there is an association.
		#
		doit "E_DM_INFO $dm" vxprint -g $dg -F %da_name -d $dm >$tmpfile1
		da=`cat $tmpfile1`
		[ "-" = "$da" ] && {
			[ "$op" = "one" ] && print_err_exit "E_DM_NO_DA $dm"
			continue
		}
 		if cdsconv_is_efi_disk $da
		then
			echo "$da" >> $tmpfile3
			continue
		fi

		#
		# Determine if the current DA is a CDS disk. If so, there is
		# nothing to do. Also, determine if the disk has a persistent
		# DA record (in which case novolstop must be set unless the
		# DA record can be converted to an auto-config record). Also,
		# for Linux, disk must have SUN label to not use novolstop.
		#
		doit "E_DM_INFO $dm" vxdisk list $da > $tmpfile1 
		grep "^info:.*format=cds.*" $tmpfile1 > /dev/null
		[ $? -eq 0 ] && continue

		da_type=`grep "^type:" $tmpfile1 | awk '{print $2}'`

		grep "^flags:.*autoconfig.*" $tmpfile1 > /dev/null
		if [ $? -ne 0 -a "y" != "$novolstop" ]
		then
			poff=`grep "^private:" $tmpfile1 | \
					sed -e 's/.*offset=\(.*\) len.*/\1/'`
			bad=0
			if [ "auto" != "$da_type" ]
			then
				case $platform in
				AIX)	[ "simple" != "$da_type" -o \
					  "256" != "$poff" ] && bad=1
					;;
				HP-UX)	[ "simple" != "$da_type" -o \
					  "128" != "$poff" ] && bad=1
					;;
				SunOS)
					if [ "`uname -p`" = "i386" ]
					then
						[ "sliced" != "$da_type" -o \
							"4" != "$poff" ] && bad=1
					else
						[ "sliced" != "$da_type" -o  \
							"1" != "$poff" ] && bad=1
					fi
					;;
				Linux)
					[ "sliced" != "$da_type" -o \
					  "1" != "$poff" ] && bad=1
					;;
				*)	internal_err_exit "diskconv: unknown plat $platform"
					;;
				esac
			fi
			[ 0 -ne $bad ] && {
				print_err "E_DM_NOAUTO_DA $dm $da"
				[ "$op" = "one" -o "$op" = "group" ] && err_exit
				efound=1
				continue
			}
		fi

		if [ "$platform" = "SunOS" -a "`uname -p`" = "i386" -a \
			\( "sliced" = "$da_type" -o "auto" = "$da_type" \) -a "y" != "$novolstop" ]
		then
			print_err "E_DM_DOS_DA $dm $da"
			[ "$op" = "one" -o "$op" = "group" ] && err_exit
			efound=1
			continue
		fi

		if [ "$platform" = "Linux" -a "y" != "$novolstop" ]
		then
			suntag=`vxparms | grep "SUN_PART_TYPE" | \
					 awk -F= '{print $2}'`

			ltag=`vxparms -s $da | grep "part_type" | \
					 awk -F= '{print $2}'`

			[ -z "$ltag" -o "$suntag" != "$ltag" ] && {
				print_err "E_DM_DOS_DA $dm $da"
				[ "$op" = "one" -o "$op" = "group" ] && err_exit
				efound=1
				continue
			}
		fi
		#
		# Setup other info related to the disk.
		#
		doit "E_DM_INFO $dm" vxprint -g $dg -F %cpath -d $dm >$tmpfile1
		rpath=`cat $tmpfile1`
		
		set_useable_disksz
		set_curr_privpub
		set_dm_free
		#
		# Determine how to re-initialize the disk and what subdisks
		# need to be relocated.
		#
		set_init_attrs
		set_reloc_sds
		#
		# Determine how much free space is on the DM to do reloc and
		# how much free space will exist after re-init.
		#
		ipe=`expr $init_pub_off + $init_pub_len`
		cpe=`expr $curr_pub_off + $curr_pub_len`

		f_init=`expr $free_total + \( $ipe - $cpe \) + \
				\( $curr_pub_off - $init_pub_off \)`
		f_reloc=$free_total
		if [ $init_pub_off -gt $curr_pub_off ]
		then
			f_reloc=`expr $f_reloc - \
					\( $init_pub_off - $curr_pub_off \)`
		fi
		if [ $ipe -lt $cpe ]
		then
			f_reloc=`expr $f_reloc - \( $cpe - $ipe \)`
		fi

		todo_dmlist="$dm $size_reloc_sd $f_reloc $f_init $todo_dmlist"
		#
		# Need to save stuff for next pass so we'll write info out
		# to a temp file for later use.
		#
		save_disk_parms
	done


    [[ -s $tmpfile3 ]] && {
		print_err "E_CONFIG_EFIDISKS $tmpfile3"
		efound=1
		rm -f $tmpfile1 $tmpfile2 $tmpfile3
		return $efound
	}

	if [ -z "$todo_dmlist" ]
	then
		#
		# No disks to convert. If not outputing analysis, just return.
		#
		[ "n" != "$analyze_only" -a "0" = "$efound" ] && {
			output_analysis_info $op
		}
		rm -f $tmpfile1 $tmpfile2 $tmpfile3
		return $efound
	fi
	order_todolist		# order the DM's into a desired order

	set -- $todo_dmlist
	while [ $# -ge 4 ]
	do
		restore_disk_parms $1
		if [ 0 -lt $2 -a \( "y" != "$novolstop" -o $2 -gt $4 \)  ]
		then
			need_sd_reloc='y'
		else
			need_sd_reloc='n'
		fi
		if [ "n" != "$analyze_only" ]
		then
			output_analysis_info $op
		else
			res=0
			if [ "y" = "$need_sd_reloc" ]
			then
				do_sd_reloc
				res=$?
			fi
			if [ 0 -eq $res ]
			then
				do_disk_reinit
				res=$?
			fi
			[ 0 -ne $res ] && efound=1
		fi
		shift 4
	done
	rm -f $tmpfile1 $tmpfile2
	return $efound
}
